### 🤖 Assistant



# PI_SYS 全平台接口统一库

[![版本](https://img.shields.io/badge/版本-1.0.0-blue.svg)]()
[![许可协议](https://img.shields.io/badge/license-MIT-green.svg)]()

## 文档目录
- [功能概述](#功能概述)
- [主要特性](#主要特性)
- [核心API](#核心api)
  - [基础模块](#基础模块)
  - [资源管理](#资源管理)
  - [媒体处理](#媒体处理)
  - [网络交互](#网络交互)
- [风险提示](#风险提示)
- [常见问题](#常见问题)
- [使用示例](#使用示例)

## 功能概述
PI_SYS 是一个面向全平台的游戏启动引擎核心库，提供以下核心能力：
- 跨平台游戏初始化与生命周期管理
- 统一的资源加载与管理系统
- 多媒体（音视频/字体/图像）处理
- 平台适配与屏幕响应式布局
- 本地存储与网络通信
- 基础工具库（编码/时间/日志等）

## 主要特性
🏗 **全平台支持** - 通过 Native 模块实现多平台适配  
📦 **资源智能加载** - 支持资源预加载/批量加载/缓存管理  
🎮 **多媒体能力** - 提供音视频播放、图像处理、字体渲染等核心能力  
🔄 **响应式布局** - 自动屏幕适配与尺寸重置功能  
📡 **网络通信** - 封装 HTTP 请求与文件下载能力  
📝 **日志系统** - 支持多级别日志记录与日志器定制

## 核心API

### 基础模块
| 模块 | 功能描述 | 主要接口 |
|------|---------|---------|
| **Env** | 环境变量管理 | 运行时环境检测与配置 |
| **Time** | 时间管理 | 帧计时器、时间戳获取 |
| **BaseTools** | 基础工具库 | 常用工具函数集合 |
| **Logger** | 日志系统 | log/info/warn/error 多级别输出 |

### 资源管理
```typescript
// 资源加载接口
ResLoadTool.loadImage(url: string): Promise<PiImage>
ResLoadTool.loadFont(name: string): Promise<PiFont>
ResLoadTool.loadVideo(url: string): Promise<PiVideo>

// 批量加载示例
BatchLoad.load([
  {type: 'image', url: 'bg.png'},
  {type: 'audio', url: 'bgm.mp3'}
])
```

### 媒体处理
| 类型 | 创建方法 | 功能说明 |
|------|---------|---------|
| **Canvas** | `createCanvas()` | 创建绘制画布 |
| **Image** | `createImage()` | 图像对象操作 |
| **Audio** | `PiAudio.create()` | 音频资源播放控制 |

### 网络交互
```typescript
HTTP.get(url: string): Promise<Response>
HTTP.post(url: string, body: object): Promise<Response>
```

## 风险提示
⚠️ **资源加载失败** - 使用 Promise 异常捕获处理加载错误  
⚠️ **内存泄漏** - 音频/视频对象需要手动执行 destroy()  
⚠️ **屏幕适配** - 图片资源建议提供多分辨率版本  
⚠️ **过时API** - Log 模块已弃用，请使用 Logger 替代


## 使用示例
```typescript
// 应用初始化
await PISYS.ResLoadTool.loadFont('defaultFont')

// 创建游戏画布
const canvas = PISYS.createCanvas()
const ctx = canvas.getContext('2d')

// 加载游戏资源
const [playerImg, bgm] = await Promise.all([
  PISYS.ResLoadTool.loadImage('player.png'),
  PISYS.ResLoadTool.loadAudio('bgm.mp3')
])

// 启动游戏循环
PISYS.Time.createFrameLoop((delta) => {
  ctx.drawImage(playerImg, 100, 100)
  bgm.play()
})
```
