# 设计

## 每个平台：虚拟机底层都会提供的函数

无需封装到 pi_sys

* 打印，高层可以重载关掉：console.log|warn|error
* ES6标准的所有api
    + Promise
    + ArrayBuffer, TypedArray
    + Date, Math, Number, String, Object, Array, Function
    + Worker
    + 定时器：setTimeout，clearTimeout, setInterval, clearInterval
* 高精度计时器：performance.now()，精度：百纳秒；
* 渲染相关
    + requstAnimationFrame, cancelAnimationFrame
    + Canvas，web版本的canvas相关的api，包括：webgl和canvas2d的context
* navigation.userAgent -- 判断系统信息；
    + 没有找到 "PI_"，就是 Web平台
    + "PI_MINIGAME;" 小游戏平台；
    + "PI_ANDROID;" Android 平台
    + "PI_IOS;" iOS平台
    + "PI_WINDOWS;" Windows平台
    + "PI_WEBVIEW;" WebView版本
    + "PI_NATIVE;"  原生开发版本

## 目录结构

* app/ App相关的模块，包括：系统信息、app信息、事件封装、生命周期、性能相关（比如：gc）；
* device/ 底层硬件设备，比如：陀螺仪，加速器，蓝牙，振动，网络改变，电量改变，定位，...
* load/ 加载，入口处：BatchLoad, loadRes；
* media/ 媒体：相册，相机，音频，视频，录音，实时语音；
* net/ 网络：http，websocket；
* platform/ 平台：登录，支付，分享，上传，上报；
* render/ 渲染：图片，字体；
* setup/ 初始化：模块加载；
* store/ 存储：文件，kv，资源包；
* ui/ 底层UI：比如各种对话框
* util/ 工具模块，时间、日志、env、depend、utf8、base64、...

### 平台

* index.html 的 代码部分需要挪到 index.js，或者通过构建插入进来；
* 根据平台加载 sys.js, depend.json 

表格说明：

- 加载
    * web版是标配
        + 代码
        + 资源：blob-url
        + 配置
        + 对象：声音，视频，图像，字体；
    * 除了web版本，都有：assets包；
- 渲染：是指底层曝露的渲染接口；

|平台名|宿主|加载|（本地）硬件|渲染|
|--|--|--|--|--|
|pi_sys|浏览器|web|无|webgl|
|web|浏览器|同pi_sys|同pi_sys|同pi_sys|
|minigame|微信小游戏|web+assets+code分包|有|webgl|
|webview:windows|electron|web+assets|无|webgl|
|native:windows|带v8的exe|web+assets|无|rendy-dx12-opengl|
|native:android|带v8的app|web+assets|有|gcanvas/rendy-valkan-gles3-gles2|
|webview:android|webview的app|web+assets|有|webgl|
|ios:webview/native|webview的app|web+assets|有|webgl|