# 加载策略

## 需求

* 默认：需要快速加载必要文件，以便于快速进入界面
* 需要单个资源加载
* 需要加载指定目录的所有资源文件

## 加载 类型

``` js

env = {
    // 代码：同步加载
    code_suffixs: [debug ? "js"],
    // 配置：同步加载，配置资源加载处理器；
    cfg_suffixs: [release ? "js", "wasm", "widget.json", "merge.json", ...],
    // 二进制资源后缀，加载成ArrayBuffer
    res_suffixs: ["gltf", "bin", "dds", "bin", "anim", ...],
    // 对象后缀，对象特指：声音，视频，字体，（需要CPU解码的）图片；
    object_suffixs: ["mp3", "mp4", "woff", "png", "jpg", ...],
}

```

## loadRes / loadObject：单 资源/对象 加载

``` js

// 加载：资源 和 对象
(async () => {
    
    // 对象需要泛型；
    let img = await loadObject<Image>("a/b/c.png");

    // 资源永远返回 ArrayBuffer；
    let ab = await loadRes("a/b/c.anim");

    // 用它...
}());

```

## BatchLoad 批处理

BatchLoad(路径数组)，每个路径以斜杠结尾，代表需要加载目录；

### 加载策略

``` js

enum LoadPolicy {
    No = 1,
    DownOnly = 2,
    All = 3,
}

```

### 接口

``` js

(async () => {

    let load = new BatchLoad(["app_b/", "app_c/"]);
    
    // 可以针对特殊文件添加过滤器；
    // 字符串以!开头代表不需要该模式的文件；
    load.addFilter("!**/*.png");

    // 资源和对象的加载策略，如果不填，默认：不下载、不加载
    // 代码和配置的加载策略，内部定死，永远都是：无论如何都 同步 加载到内存
    let resLoadPolicy = LoadPolicy.No;
    let objectLoadPolicy = LoadPolicy.All;

    await load.start(resLoadPolicy, objectLoadPolicy);
}())

```