/**
 * 系统信息
 */

import { AppName, BenchLevel, SystemInfo, TgetOAID, TgetDeviceID, TgetSystemInfo } from "pi_sys/native/base/system_info";
import { Native } from "../../../webview/native_impl/core/index";
import { Env } from "pi_sys/util/env";

/**
 * 获取系统信息
 */
export const getSystemInfo: TgetSystemInfo = (): Promise<SystemInfo> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getInfo(
                async function (res) {
                    let left = Math.ceil(res.safearea_left);
                    let right = Math.floor(res.safearea_right);
                    let top = Math.ceil(res.safearea_top);
                    let bottom = Math.floor(res.safearea_bottom);
                    let screenWidth = Math.ceil(res.screenWidth);
                    let screenHeight = Math.ceil(res.screenHeight);
                    let statusBarHeight = Math.ceil(res.statusBarHeight);

                    let ua = navigator.userAgent;
                    ua = ua.toLowerCase();

                    // 一定是 pi_native pi_android
                    let app: AppName = AppName.Native;;
                    let device = "mobile";
                    let platform = "android";;

                    let benchLevel = BenchLevel.Unkown;

                    let isSupportWebGL = true;
                    // native 平臺，總是支持webgl；因爲底層就是 OpenGLES 模擬的；


                    if (!isSupportWebGL) {
                        benchLevel = BenchLevel.CantRun;
                    }

                    if (isSupportWebGL) {
                        // 不支持 WebAssembly的，统统是CantRun
                        let isSupportWasm = false;
                        try {
                            if (window["WebAssembly"] && window["WebAssembly"].instantiate) {
                                isSupportWasm = true;
                            }
                        } catch (_e) {
                            isSupportWasm = false;
                        }
                        if (!isSupportWasm) {
                            benchLevel = BenchLevel.CantRun;
                        }
                    }
                    /**
                     * 内存 >=7G高端 >=5G中端 其他低端
                    */
                    if (res.memory > 0) {
                        let memory = res.memory / 1024 / 1024 / 1024;
                        if (memory >= 7) {
                            benchLevel = BenchLevel.High;
                        } else if (memory >= 5) {
                            benchLevel = BenchLevel.Middle;
                        } else {
                            benchLevel = BenchLevel.Low;
                        }
                    }


                    let r: SystemInfo = {
                        safeArea: {
                            left,
                            right,
                            top,
                            bottom,
                            width: right - left,
                            height: bottom - top,
                        },
                        memory: (res.memory || 0) / 1024 / 1024 / 1024,
                        screenWidth,
                        screenHeight,
                        statusBarHeight,

                        brand: res.brand,
                        model: res.model,
                        pixelRatio: 1,

                        language: res.language,
                        cameraAuthorized: (<any>res).cameraAuthorized,
                        locationAuthorized: (<any>res).locationAuthorized,
                        microphoneAuthorized: (<any>res).microphoneAuthorized,
                        notificationAuthorized: (<any>res).notificationAuthorized,
                        bluetoothEnabled: (<any>res).bluetoothEnabled,
                        locationEnabled: (<any>res).locationEnabled,
                        wifiEnabled: (<any>res).wifiEnabled,

                        colorDepth: 24,

                        app,
                        platform,
                        device,
                        system: res.system,
                        benchLevel,
                    };
                    const rewriteSystemInfo = Env.get("rewriteSystemInfo") || window["winit"].env.rewriteSystemInfo;
                    if (rewriteSystemInfo) {
                        await rewriteSystemInfo(r, res);
                    }
                    resolve(r);
                },
                reject
            );
        })
            .catch((e) => {
                reject(e);
            });
    });
}



/**
 * 取手机的设备ID
 */
export const getDeviceID: TgetDeviceID = (): Promise<string> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getDeviceID(
                function (pName) {
                    resolve(pName);
                },
                reject
            );
        })
            .catch((e) => {
                reject(e);
            });
    });
}

export const getOAID: TgetOAID = () => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getOAID(
                function (pName) {
                    resolve(pName);
                },
                reject
            );
        })
            .catch((e) => {
                reject(e);
            });
    });
}