
/**
 * 小游戏键盘输入
 */

import { Native } from "../../../webview/native_impl/core/index";
import { KeyboardConfig, Thide, ToffComplete, ToffInput, ToffKeyboardHeightChange, ToffKeyDown, ToffKeyUp, ToffSelection, TonComplete, TonInput, TonKeyboardHeightChange, TonKeyDown, TonKeyUp, TonSelection, TsetSelection, Tshow, TsupportSelection, Tupdate } from "../../../pi_sys/native/device/keyboard";

import { Logger } from "../../../pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

// // 通知键盘监听函数
// const keyboardNotify = (notifier: Function, params: string) => {
//     if (notifier && notifier instanceof Function) {
//         notifier(params);
//     }
// };

// 维护么个输入字符串的光标位置
let CURSOR_INDEX_LIST: { [index: string]: object } = {};
let CURSOR_INDEX = {};
let keyboardValue = "";

const DEFAULT_CFG: any = {
    multiple: false,
    maxLength: 0,
    confirmHold: false
};


export type ListenFunction = (value: string) => void;

// 唤起键盘
export const show: Tshow = (config: KeyboardConfig) => {
    keyboardValue = config.defaultValue;
    let inputType = config.attribute["type"] || "text";
    globalThis.Window.showSoftInput(keyboardValue, inputType);
    CURSOR_INDEX_LIST[config.attribute["type"]] = { selectionStart: config.defaultValue.length, selectionEnd: config.defaultValue.length };
    CURSOR_INDEX = CURSOR_INDEX_LIST[config.attribute["type"]];
    if (!isShow) {
        config = {
            ...DEFAULT_CFG,
            ...config,
        };
        delete config.attribute;
        // wx.showKeyboard(config);
        logger.debug(config);

        bindListener();
    }
}

// 隐藏键盘
export const hide: Thide = (p: any) => {
    globalThis.Window.hideSoftInput(true);
    keyboardValue = "";
    unbindListener();
};


// 更新键盘
export const update: Tupdate = (p) => {
    logger.warn("update");
};

export const onInput: TonInput = (callback: ({ value: string }) => void) => {
    if (!inputListenerList.includes(callback)) {
        inputListenerList.push(callback);
    }
}

/**
 * 取消监听键盘输入事件.参数为空，则取消所有的事件监听。
 */
export const offInput: ToffInput = (callback?: ({ value: string }) => void) => {
    if (callback) {
        let index = inputListenerList.indexOf(callback);
        if (index >= 0) {
            inputListenerList.splice(index, 1)
        }
    } else {
        inputListenerList.length = 0;
    }
}


export const onComplete: TonComplete = (callback: (obj: { value: string }) => void) => {
    if (!completeListenerList.includes(callback)) {
        completeListenerList.push(callback);
    }
}

export const offComplete: ToffComplete = (callback?: ({ value: string }) => void) => {
    if (callback) {
        let index = completeListenerList.indexOf(callback);
        if (index >= 0) {
            completeListenerList.splice(index, 1)
        }
    } else {
        completeListenerList.length = 0;
    }
}

export const supportSelection: TsupportSelection = () => {
    return true;
}

/**
 * 设置光标位置
 * @param index
 */
export const setSelection: TsetSelection = (index: number): boolean => {
    if (typeof index != typeof 0) {
        return true;
    }
    CURSOR_INDEX['selectionStart'] = index;
    CURSOR_INDEX['selectionEnd'] = index;
    globalThis.Window.setSelection(index);
    return true;
}

/**
 * 监听光标改变
 * @param callback 
 * @param 
 * @returns 
 */
export const onSelection: TonSelection = (callback: (value: number) => void) => {
    selectionListenerList.push(callback);
}

export const offSelection: ToffSelection = (callback?: (value: number) => void) => {
    if (callback) {
        let index = selectionListenerList.indexOf(callback);
        if (index >= 0) {
            selectionListenerList.splice(index, 1)
        }
    } else {
        selectionListenerList.length = 0;
    }
}

let inputFuncs: Set<Function>;         // registerSoftInputChangedListener监听回调
let inputRunFunc: Function;            // registerSoftInputChangedListener实际注册回调函数
let inputCbID: number;           // NativeHook监听回调ID

/**
 * 监听 虚拟键盘 高度，仅限于 支持 虚拟键盘 的 平台 
 */
export const onKeyboardHeightChange: TonKeyboardHeightChange = (callback: ({ height: number }) => void) => {
    if (!inputFuncs) {
        inputFuncs = new Set;
    }
    inputFuncs.add(callback);
    if (inputRunFunc) return;
    inputRunFunc = (height: number) => {
        inputFuncs && inputFuncs.forEach(func => func && func(height));
    }
    Native.App.init().then(() => {
        const id = Native.App.registerSoftInputChangedListener(inputRunFunc as any, undefined);
        inputCbID = id;
    });
}

/**
 * 取消监听 虚拟键盘 高度，仅限于 支持 虚拟键盘 的 平台.参数为空，则取消所有的事件监听。
 */
export const offKeyboardHeightChange: ToffKeyboardHeightChange = (callback?: ({ height: number }) => void) => {
    if (!inputFuncs) return;
    if (callback) {
        inputFuncs.delete(callback);
    } else {
        inputFuncs.clear();
    }
    if (!inputFuncs.size) {
        inputFuncs = undefined;
        Native.App.init().then(() => {
            if (inputCbID) {
                Native.App.unregisterSoftInputChangedListener(inputCbID);
                inputCbID = undefined;
            }
        });
        inputRunFunc = undefined;
    }
}

/**
 * 监听键盘按键按下事件，仅适用于 PC 平台
 * @param callback 
 */
export const onKeyDown: TonKeyDown = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    console.warn("android platform not supported!")
}

/**
 *  取消监听键盘按键按下事件，仅适用于 PC 平台
 * @param callback 
 */
export const offKeyDown: ToffKeyDown = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    console.warn("android platform not supported!")
};


/**
 * 监听键盘按键弹起事件，仅适用于 PC 平台
 * @param callback 
 */
export const onKeyUp: TonKeyUp = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    console.warn("android platform not supported!")
}

/**
 *  取消监听键盘按键弹起事件，仅适用于 PC 平台
 * @param callback 
 */
export const offKeyUp: ToffKeyUp = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    console.warn("android platform not supported!")
};


const inputListener = (obj) => {
    let index = Math.min(obj.value.length, Math.max(CURSOR_INDEX['selectionStart'] + (obj.value.length - keyboardValue.length), 0)); // 钳制在0 到 obj.value.length 之间
    keyboardValue = obj.value;
    CURSOR_INDEX['selectionStart'] = index;
    CURSOR_INDEX['selectionEnd'] = index;
    for (let li of inputListenerList) {
        (li instanceof Function) && li(obj);
    }
}

const completeListener = (obj) => {
    console.log("====== onComplete: text = " + obj.value);
    let index = Math.min(obj.value.length, Math.max(CURSOR_INDEX['selectionStart'] + (obj.value.length - keyboardValue.length), 0)); // 钳制在0 到 obj.value.length 之间
    keyboardValue = obj.value;
    CURSOR_INDEX['selectionStart'] = index;
    CURSOR_INDEX['selectionEnd'] = index;
    for (let li of completeListenerList) {
        (li instanceof Function) && li(obj);
    }
}

const resultListener = (r: { value: string }) => {
    for (let li of completeListenerList) {
        (li instanceof Function) && li(r);
    }
    unbindListener();
}

const bindListener = () => {
    globalThis.onKeyboardInput(inputListener);
    globalThis.onKeyDown(decodeOnKeyDown);
    globalThis.onComplete(completeListener);
    isShow = true;
}
const unbindListener = () => {
    globalThis.offKeyboardInput(inputListener);
    globalThis.offKeyDown(decodeOnKeyDown);
    globalThis.offComplete(completeListener);
    isShow = false;
}



const decodeOnKeyDown = (obj) => {
    switch (obj.key) {
        // 左方向(<-)键 
        case "ArrowLeft":
            if (CURSOR_INDEX['selectionStart'] === CURSOR_INDEX['selectionEnd']) {
                CURSOR_INDEX['selectionStart'] = CURSOR_INDEX['selectionEnd'] = Math.min(keyboardValue.length, Math.max(CURSOR_INDEX['selectionStart'] - 1, 0))
            } else {
                CURSOR_INDEX['selectionEnd'] = CURSOR_INDEX['selectionStart'];
            }
            for (let li of selectionListenerList) {
                (li instanceof Function) && li(CURSOR_INDEX['selectionStart']);
            }
            break;
        // 右方向(->)键 
        case "ArrowRight":
            if (CURSOR_INDEX['selectionStart'] === CURSOR_INDEX['selectionEnd']) {
                CURSOR_INDEX['selectionStart'] = CURSOR_INDEX['selectionEnd'] = Math.min(keyboardValue.length, Math.max(CURSOR_INDEX['selectionStart'] + 1, 0))
            } else {
                CURSOR_INDEX['selectionEnd'] = CURSOR_INDEX['selectionStart'];
            }
            for (let li of selectionListenerList) {
                (li instanceof Function) && li(CURSOR_INDEX['selectionStart']);
            }
            break;
    }
}

const inputListenerList = [];
const completeListenerList = [];
const selectionListenerList = [];
let isShow = false;



