/**
 * 华为的适配
 */

import "./platform";
import { touchEventHandlerFactory } from "./screen_event";
import { reDefinedCanvas } from './define_global_canvas';

window["canvas"] = document.getElementById("canvas");

// 华为小游戏的全局canvas的宽高如果修改了，并没有修改到canvas实际的像素宽高，但在当前帧，使用width、heigth读到的canvas宽高是修改后的数值（经过测试，在下一个帧回调时读取，数值又与实际宽高相等了）
// 该方法重载了全局canvas的width和height的setter、getter方法，使之在设置指的时候直接返回，保证canvas的width、height在读取数值上与实际宽高保持一致
// 2023.1.6  实测不能重载
// reDefinedCanvas(window["canvas"]);



// 华为快游戏没有setStorage相关接口,这里使用localStorage模拟
wx.setStorage = (obj: any) => {
    try {
        localStorage.setItem(obj.key, obj.data);
        if (obj.success) {
            obj.success();
        }
    } catch (e) {
        if (obj.fail) {
            obj.fail(e);
        }
    }
    if (obj.complete) {
        obj.complete();
    }
}

wx.getStorage = (obj: any) => {
    try {
        const data = localStorage.getItem(obj.key);
        if (obj.success) {
            obj.success({ data });
        }
    } catch (e) {
        if (obj.fail) {
            obj.fail(e);
        }
    }
    if (obj.complete) {
        obj.complete();
    }
}

wx.clearStorage = (obj: any) => {
    try {
        localStorage.clear();
        if (obj.success) {
            obj.success();
        }
    } catch (e) {
        if (obj.fail) {
            obj.fail(e);
        }
    }
    if (obj.complete) {
        obj.complete();
    }
}


wx.getStorageInfo = (obj: any) => {
    try {
        let keys = [];
        for (let i = 0; i < localStorage.length; i++) {
            keys.push(localStorage.key(i));
        }
        if (obj.success) {
            obj.success({ keys });
        }
    } catch (e) {
        if (obj.fail) {
            obj.fail(e);
        }
    }
    if (obj.complete) {
        obj.complete();
    }
}



document.addEventListener("touchstart", touchEventHandlerFactory('pointerdown'));
document.addEventListener("touchmove", touchEventHandlerFactory('pointermove'));
document.addEventListener("touchend", touchEventHandlerFactory('pointerup'));
document.addEventListener("touchcancel", touchEventHandlerFactory('pointercancel'));
