import "./platform"; // 屏蔽各小游戏平台差异
// 将微信的touch事件转换为pointer事件,并派发到pi_sys的事件系统
import { dispatchEvent } from "pi_sys/native/base/event";
import { AppName } from "pi_sys/native/base/system_info";
import { Env } from "pi_sys/util/env";

const CLONE_PROPS = [

    // MouseEvent
    'bubbles',
    'cancelable',
    'view',
    'detail',
    'screenX',
    'screenY',
    'clientX',
    'clientY',
    'ctrlKey',
    'altKey',
    'shiftKey',
    'metaKey',
    'button',
    'relatedTarget',

    // PointerEvent
    'pointerId',
    'width',
    'height',
    'pressure',
    'tiltX',
    'tiltY',
    'pointerType',
    'hwTimestamp',
    'isPrimary',

    // event instance
    'pageX',
    'pageY',
    'timeStamp'
];

const CLONE_DEFAULTS = [

    // MouseEvent
    false,
    false,
    null,
    null,
    0,
    0,
    0,
    0,
    false,
    false,
    false,
    false,
    0,
    null,

    // DOM Level 3
    0,

    // PointerEvent
    0,
    0,
    0,
    0,
    0,
    0,
    '',
    0,
    false,

    // event instance
    0,
    0,
    0
];

const POINTER_TYPE = 'touch';

export const noop = () => { }

const touchToPointer = (type: string, touch: Touch) => {
    const e = {} as any;

    for (let i = 0; i < CLONE_PROPS.length; i++) {
        const p = CLONE_PROPS[i];
        e[p] = touch[p] || CLONE_DEFAULTS[i];
    }
    // 华为快游戏中 事件响应坐标乘上了pixelRatio,这里恢复,并且不支持screenX,screenY

    const systemInfo = Env.get("system_info");
    const pixelRatio = systemInfo.pixelRatio;
    if ((<any>window).qg) {
        e.clientX = e.clientX / pixelRatio;
        e.clientY = e.clientY / pixelRatio;
    }

    e.type = type;
    e.target = window["canvas"];
    e.currentTarget = window["canvas"];
    e.timeStamp = Date.now();
    e.buttons = typeToButtons(type);
    e.which = e.buttons;

    e.pointerId = (touch.identifier || 0) + 2;
    e.bubbles = true;
    e.cancelable = true;
    // e.detail = this.clickCount;
    e.button = 0;

    e.width = (touch.radiusX || 0.5) * 2;
    e.height = (touch.radiusY || 0.5) * 2;
    e.pressure = touch.force || 0.5;
    e.isPrimary = isPrimaryPointer(touch);
    e.pointerType = POINTER_TYPE;

    e.preventDefault = noop;
    e.stopPropagation = noop;


    // // forward modifier keys
    // e.altKey = rawEvent.altKey;
    // e.ctrlKey = rawEvent.ctrlKey;
    // e.metaKey = rawEvent.metaKey;
    // e.shiftKey = rawEvent.shiftKey;

    // if (rawEvent.preventDefault) {
    //     e.preventDefault = function() {
    //         rawEvent.preventDefault();
    //     };
    // }

    return e;
}

function typeToButtons(type) {
    let ret = 0;
    if (type === 'touchstart' || type === 'touchmove' || type === 'pointerdown' || type === 'pointermove') {
        ret = 1;
    }
    return ret;
}

let firstPointer = null;

function isPrimaryPointer(touch) {
    return firstPointer === touch.identifier;
}

function setPrimaryPointer(touch) {
    if (firstPointer === null) {
        firstPointer = touch.identifier;
    }
}

function removePrimaryPointer(touch) {
    if (firstPointer === touch.identifier) {
        firstPointer = null;
    }
}


const downIds = new Set();

// 如果监听了触摸事件， 需要该方法转换为
export const touchEventHandlerFactory = (type) => {
    return (rawEvent) => {
        const changedTouches = rawEvent.changedTouches;
        for (let i = 0; i < changedTouches.length; i++) {
            const touch = changedTouches[i];
            if ('pointerdown' == type) {
                /**
                 * 抖音小游戏IOS平台双指三指操作的时候changedTouches会把之前的down事件也带上,
                 * 且在数组中的顺序不确定,这里如果相同id的down事件已经触发过,忽略
                 * 
                 */
                if (downIds.has(touch.identifier)) {
                    continue;
                } else {
                    downIds.add(touch.identifier);
                }
            }

            if (i === 0 && type === 'pointerdown') {
                setPrimaryPointer(touch);
            } else if (type === 'pointerup' || type === 'pointercancel') {
                downIds.delete(touch.identifier);
                removePrimaryPointer(touch);
            }
            const event = touchToPointer(type, touch);
            dispatchEvent(event.type, event);
        }
    }
}
