
import { FileLoadParam } from "pi_sys/load/fileload_base";
import { Store } from "../store/store";
import { Env } from "pi_sys/util/env";
import { fileCache } from "../wx/file_cache";
import { StoreType } from "pi_sys/store/store_base";
import * as Sign from "pi_sys/load/sign";
import { utf8Decode, utf8Encode } from "pi_sys/util/utf8";
import { Depend } from "pi_sys/util/depend";
import { saveDbSign } from "pi_sys/load/down_load_hook";


export namespace Load {
    export function getLocalStore(pname: string) {
        if (FileLoadParam.localStore) {
            return Promise.resolve(FileLoadParam.localStore);
        } else {
            let maxMainAllSize = Env.get('platform').minigame.MaxMainAllSize;
            maxMainAllSize = maxMainAllSize ? maxMainAllSize : 200 * 1024 * 1024;
            let sdfTextSize = Env.get('platform').minigame.SdfTextSize;
            sdfTextSize = sdfTextSize ? sdfTextSize : 15 * 1024 * 1024;
            return fileCache.init(maxMainAllSize, sdfTextSize).then(() => {
                return Store.createStore(pname, '', StoreType.File).then((store) => {
                    FileLoadParam.localStore = store;
                    store.immediateWrite = false;
                    return store;
                })
            });
        }
    }

    export function init(pname: string): Promise<any> {
        FileLoadParam.dbSign = {};
        return getLocalStore(pname).then((store) => {
            FileLoadParam.localStore = store;
            store.immediateWrite = false;
            return store.read(Sign.SIGN_KEY).then(uint8 => {
                let signs = {};
                try {
                    const str = utf8Decode(uint8);
                    signs = JSON.parse(str);
                    localInitCheck(store, signs, false);
                    // console.log("========== signs ", signs);
                } catch {

                }
            });
        })


    }
    /**
     * @description 获取本地存储
     * @example
     */
    export function getStore() {
        return FileLoadParam.localStore;
    };

    /**
     * 删除store
     */
    export function deleteStore() {
        if (FileLoadParam.localStore) {
            return Store.deleteStore(FileLoadParam.localStore.dbName);
        } else {
            return Promise.resolve();
        }
    }

    // 本地加载检查
    function localInitCheck(store: Store, signs: any, save: boolean) {
        FileLoadParam.dbSign = signs;
        // 删除不存在或签名不正确的文件
        for (let k in signs) {
            if (!signs.hasOwnProperty(k)) {
                continue;
            }
            let info = Depend.getFile(k);
            if (info && signs[k] === info.sign && fileCache.has(store.formatStorePath(info.path))) {
                continue;
            }
            if (!info) {
                store.delete(k);
            }

            delete signs[k];
            save = true;
        }
        if (save) {
            saveDbSign();
        }
    };


}