import { FileLoad } from 'pi_sys/load/file_load';
import { Env } from 'pi_sys/util/env';

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

export class PackageLoad extends FileLoad {
    // 多个加载文件
    public packages: Set<string> = new Set();
    public packageList: string[] = [];
    public loadedSizeMap: Map<string, number> = new Map;
    public singleSize = 400 * 1024;        // 单个文件默认400K,太大太小都不合适,大概一个平均值
    /**
     * @description 添加下载文件
     * @example
     */
    public addPackage(pkgName: string) {
        const platform = Env.get('platform');

        if (platform.minigame && platform.minigame.subpackages) {
            const pk = (<Array<{ name: string, root: string }>>platform.minigame.subpackages).find((el) => el.root === pkgName);
            if (pk) {
                if (!this.packages.has(pk.name)) {
                    this.packages.add(pk.name);
                    this.packageList.push(pk.name);
                    this.total += this.singleSize;
                    this.loadedSizeMap.set(pk.name, 0);
                }
            }
        }
    }
    /**
     * @description 开始
     * @example
     */
    public start(): Promise<any> {
        const arr = [];
        // this.packages.forEach(pkgName => {
        //     arr.push(new Promise((resolve, reject) => {
        //         loadSubPkg(this, pkgName, resolve, reject);
        //     }));
        // });

        // return this.loadPackage();

        for (let pkgName of this.packageList) {
            arr.push(new Promise((resolve, reject) => {
                loadSubPkg(this, pkgName, this.loadedSizeMap, this.singleSize, resolve as any, reject);
            }));
        }
        return Promise.all(arr);
    }

    private loadPackage() {
        if (this.packageList.length > 0) {
            const pkName = this.packageList.shift();
            return new Promise((resolve, reject) => {
                loadSubPkg(this, pkName, this.loadedSizeMap, this.singleSize, () => {  // 华为快游戏这里分包加载需要暂停一会,否则会失败
                    setTimeout(() => {
                        this.loadPackage().then(resolve);
                    }, 0);
                }, reject);
            });
        }
        else {
            return Promise.resolve(true)
        }
    }
}

const loadSubPkg = (
    load: PackageLoad,
    pkgName: string,
    loadedSizeMap: Map<string, number>,
    totalSize: number,
    callback: () => void,
    errorCallback: (err: string) => void,
    errText?: string,
    i?: number
) => {
    logger.debug("Start of loading subcontracting : ", pkgName);
    const task = wx.loadSubpackage({
        name: pkgName,   // 微信小游戏分包名称
        subpackage: pkgName,  // 华为快游戏分包名称
        success: () => {
            logger.debug("The subcontracting is successfully loaded:", pkgName);
            loadedSizeMap.set(pkgName, totalSize);
            let loaded = 0;
            loadedSizeMap.forEach(p => {
                loaded += p;
            });
            load.loaded = loaded;
            load.onProcess(pkgName, "codeLoad", load.total, load.loaded);
            callback();
        },
        fail: (err: { errMsg: string }) => {
            loadSubPkg(
                load, pkgName, loadedSizeMap, totalSize, callback, errorCallback,
                err.errMsg, i === undefined ? 0 : i + 1
            );
        },
        complete: () => {

        }
    });

    // 华为快游戏没有onProgressUpdate,文档上说有onprogress,但实际测试并没有
    if (task.onprogress) {
        task.onProgressUpdate = task.onprocess;
    }

    if (task.onProgressUpdate) {
        // 分包加载进度
        task.onProgressUpdate((res: {
            /** 分包下载进度百分比*/
            progress: number
            /** 已经下载的数据长度，单位 Bytes	*/
            totalBytesWritten: number
            /** 预期需要下载的数据总长度，单位 Bytes*/
            totalBytesExpectedToWrite: number
        }) => {
            if (res.totalBytesWritten && res.totalBytesExpectedToWrite) {
                const p = Math.floor(res.totalBytesWritten / res.totalBytesExpectedToWrite * totalSize);
                loadedSizeMap.set(pkgName, p);
            }
            let loaded = 0;
            loadedSizeMap.forEach(p => {
                loaded += p;
            });
            load.loaded = loaded;
            load.onProcess(pkgName, "codeLoad", load.total, load.loaded);
        });
    }

};
