/**
 * 媒体：音频
 */

import { IPiInnerAudio } from "pi_sys/media/inner_audio";


export class PiInnerAudio implements IPiInnerAudio {
    private context: WechatMinigame.InnerAudioContext = null;

    constructor(src: string) {
        this.context = wx.createInnerAudioContext();
        this.context.src = src;
    }

    public get startTime() {
        return this.context.startTime;
    }

    public set startTime(startTime: number) {
        this.context.startTime = startTime;
    }

    public get loop() {
        return this.context.loop;
    }

    public set loop(loop: boolean) {
        this.context.loop = loop;
    }

    public get volume() {
        return this.context.volume;
    }

    public set volume(volume: number) {
        this.context.volume = volume;
    }

    public get playbackRate() {
        return this.context.playbackRate;
    }

    public set playbackRate(rate: number) {
        this.context.playbackRate = rate;
    }


    public get duration() {
        return this.context.duration;
    }

    public get currentTime() {
        return this.context.currentTime;
    }

    public get paused() {
        return this.context.paused;
    }

    play() {
        this.context.play();
    }

    /**
     * 暂停；音频再播放会从暂停处开始播放
     */
    pause() {
        this.context.pause();
    }

    /**
     * 停止。停止后的音频再播放会从头开始播放
     */
    stop() {
        this.context.stop();
    }

    /**
     * 跳转到指定位置
     * @param position 跳转的时间，单位 s。精确到小数点后 3 位，即支持 ms 级别精确度
     */
    seek(position: number) {
        this.context.seek(position);
    }

    /**
     * 销毁当前实例
     */
    destroy() {
        this.context.destroy();
    }

    /**
     * 监听音频进入可以播放状态的事件。但不保证后面可以流畅播放
     * @param callback 音频进入可以播放状态的事件的回调函数
     */
    onCanplay(callback: () => void) {
        this.context.onCanplay(callback);
    }

    /**
     * 取消监听音频进入可以播放状态的事件
     * @param callback 音频进入可以播放状态的事件的回调函数
     */
    offCanplay(callback: () => void) {
        this.context.offCanplay(callback);
    }

    /**
     * 监听音频播放事件
     * @param callback 音频播放事件的回调函数
     */
    onPlay(callback: () => void) {
        this.context.onPlay(callback);
    }

    /**
     * 取消监听音频播放事件
     * @param callback 音频播放事件的回调函数
     */
    offPlay(callback: () => void) {
        this.context.offCanplay(callback);
    }

    /**
     * 监听音频暂停事件
     * @param callback 音频暂停事件的回调函数
     */
    onPause(callback: () => void) {
        this.context.onPause(callback);
    }

    /**
     * 取消监听音频暂停事件
     * @param callback 音频暂停事件的回调函数
     */
    offPause(callback: () => void) {
        this.context.offCanplay(callback);
    }

    /**
     * 监听音频停止事件
     * @param callback 音频停止事件的回调函数
     */
    onStop(callback: () => void) {
        this.context.onStop(callback);
    }

    /**
     * 取消监听音频停止事件
     * @param callback 音频停止事件的回调函数
     */
    offStop(callback: () => void) {
        this.context.offStop(callback);
    }

    /**
     * 监听音频自然播放至结束的事件
     * @param callback 音频自然播放至结束的事件的回调函数
     */
    onEnded(callback: () => void) {
        this.context.onEnded(callback);
    }

    /**
     * 取消监听音频自然播放至结束的事件
     * @param callback 音频自然播放至结束的事件的回调函数
     */
    offEnded(callback: () => void) {
        this.context.offEnded(callback);
    }

    /**
     * 监听音频播放进度更新事件
     * @param callback 音频播放进度更新事件的回调函数
     */
    onTimeUpdate(callback: () => void) {
        this.context.onTimeUpdate(callback);
    }

    /**
     * 取消监听音频播放进度更新事件
     * @param callback 音频播放进度更新事件的回调函数
     */
    offTimeUpdate(callback: () => void) {
        this.context.offTimeUpdate(callback);
    }

    /**
     * 监听音频进行跳转操作的事件
     * @param callback 音频进行跳转操作的事件的回调函数
     */
    onSeeking(callback: () => void) {
        this.context.onSeeking(callback);
    }

    /**
     * 取消监听音频进行跳转操作的事件
     * @param callback 音频进行跳转操作的事件的回调函数
     */
    offSeeking(callback: () => void) {
        this.context.offSeeking(callback);
    }

    /**
     * 监听音频完成跳转操作的事件
     * @param callback 音频完成跳转操作的事件的回调函数
     */
    onSeeked(callback: () => void) {
        this.context.onSeeked(callback);
    }

    /**
     * 取消监听音频完成跳转操作的事件
     * @param callback 音频完成跳转操作的事件的回调函数
     */
    offSeeked(callback: () => void) {
        this.context.offSeeked(callback);
    }

}