/**
 * App 相关接口
 */

import { FlagAppLife, ISdkInfo, TexitApp, TgetFlavorName, TgetProcessId, TgetPackageName, TgetSdkInfo, TgetVersionCode, TgetVersionName, ThideFullScreenPopupWindow, ToffDestroy, ToffHide, ToffShow, TonDestroy, TonHide, TonShow, TtriggerGC, TgetApiVersion, TopenWebView, TcloseWebView, TonBlur, ToffBlur, TgetRunconfigValue, TregisterAppLifeListener, TunregisterAppLifeListener, TregisterSoftInputChangedListener, TunregisterSoftInputChangedListener, TgetChannelInfo, ChannelInfo, TrequestPointerLock, TexitPointerLock, TonPointerLockError, ToffPointerLockError, TonPointerLockChange, ToffPointerLockChange, TgetTemperature, TgetChannelQuick, TprivacyAgreement, TclearCache, TshowWithEntranceId, TupdateUserInfo, TresetUserInfo, TshowWithEntranceIdLogin, TgetIdfv, TgetIpAdress, TgetIdfa } from "pi_sys/native/base/app";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

// 退出当前小程序。必须有点击行为才能调用成功
export const exitApp: TexitApp = () => {
    wx.exitMiniProgram({
        success: () => {
            console.log("======= exitMiniProgram success")
        },
        fail: () => {
            console.log("======= exitMiniProgram fail")
        },
        complete: () => {
            console.log("======= exitMiniProgram complete")
        }
    });
}

export const getProcessId: TgetProcessId = () => {
    let now = new Date().getTime();
    return Promise.resolve(now.toString());
}

export const getApiVersion: TgetApiVersion = () => {
    return Promise.resolve("");
}

export const getPackageName: TgetPackageName = () => {
    return Promise.resolve("");
}

export const getFlavorName: TgetFlavorName = () => {
    return Promise.resolve("");
}

export const getVersionName: TgetVersionName = () => {
    return Promise.resolve("");
}

export const getVersionCode: TgetVersionCode = () => {
    return Promise.resolve(0);
}

export const getRunconfigValue: TgetRunconfigValue = (key: string): Promise<string> => {
    return Promise.resolve("");
}

export const getChannelInfo: TgetChannelInfo = (): Promise<ChannelInfo> => {
    let info = {
        channel: "",
        param: {}
    };
    return Promise.resolve(info);
}

export const getSdkInfo: TgetSdkInfo = (): Promise<ISdkInfo> => {
    return Promise.resolve({
        list: [],
    });
}

export const triggerGC: TtriggerGC = () => {
    // 加快触发 JavaScriptCore 垃圾回收（Garbage Collection）。GC 时机是由 JavaScriptCore 来控制的，并不能保证调用后马上触发 GC。
    wx.triggerGC();
}

export const openWebView: TopenWebView = (url: string, params?: Object) => {
    return Promise.resolve("");
};

export const closeWebView: TcloseWebView = (tag: string) => {
    return Promise.resolve();
};

// 获取CPU温度,单位摄氏度,保留小数点后一位
export const getTemperature: TgetTemperature = () => {
    return Promise.resolve(0);
};


// 获取idfv
export const getIdfv: TgetIdfv = () => {
    return Promise.reject("getIdfv not impl");
};

// 获取IpAdress
export const getIpAdress: TgetIpAdress = () => {
    return Promise.reject("getIpAdress not impl");
};

// 获取Idfa
export const getIdfa: TgetIdfa = () => {
    return Promise.reject("getIpAdress not impl");
};



/**
 * 获取quick渠道分发出来的渠道
 */
export const getChannelQuick: TgetChannelQuick = () => {
    return Promise.resolve(null);
};


export const privacyAgreement: TprivacyAgreement = (isAccepted: boolean) => {
    return Promise.resolve(null);
}


export const clearCache: TclearCache = () => {
    return Promise.resolve(null);
};

let funcs: Set<Function>;         // registerAppLifeListener监听回调
let showFuncs: Set<Function>;     // onShow监听回调
let hideFuncs: Set<Function>;     // onHide监听回调
let destroyFuncs: Set<Function>;  // onDestroy监听回调
let runShowFunc: Function;            // 实际注册onShow回调函数
let runHideFunc: Function;            // 实际注册onHide回调函数
let runDestroyFunc: Function;            // 实际注册onDestroy回调函数

export const registerAppLifeListener: TregisterAppLifeListener = (callback: (flag: FlagAppLife) => void) => {
    if (!funcs) {
        funcs = new Set;
    }
    funcs.add(callback);
    __registerShow();
    __registerHide();
}

export const unregisterAppLifeListener: TunregisterAppLifeListener = (callback: (flag: FlagAppLife) => void) => {
    if (!funcs) return;
    if (callback) {
        funcs.delete(callback);
    } else {
        funcs.clear();
    }
    if (!funcs.size) {
        funcs = undefined;
        __unRegisterShow();
        __unRegisterHide();
    }
}



export const onShow: TonShow = (callback: () => void) => {
    if (!showFuncs) {
        showFuncs = new Set;
    }
    showFuncs.add(callback);
    __registerShow();
};

export const onHide: TonHide = (callback: () => void) => {
    if (!hideFuncs) {
        hideFuncs = new Set;
    }
    hideFuncs.add(callback);
    __registerHide();
};

export const onDestroy: TonDestroy = (callback: () => void) => {

    logger.warn("WeChat mini games do not support");
};

export const offShow: ToffShow = (callback?: () => void) => {
    if (!showFuncs) return;
    if (callback) {
        showFuncs.delete(callback);
    } else {
        showFuncs.clear();
    }
    if (!showFuncs.size) {
        showFuncs = undefined;
        __unRegisterShow();
    }
};

export const offHide: ToffHide = (callback?: () => void) => {
    if (!hideFuncs) return;
    if (callback) {
        hideFuncs.delete(callback);
    } else {
        hideFuncs.clear();
    }
    if (!hideFuncs.size) {
        hideFuncs = undefined;
        __unRegisterHide();
    }
};

export const offDestroy: ToffDestroy = (callback?: () => void) => {

    logger.warn("WeChat mini games do not support");
};


const __registerShow = () => {
    if (runShowFunc) return;
    runShowFunc = () => {
        showFuncs && showFuncs.forEach(func => func && func());
        funcs && funcs.forEach(func => func && func("onResume"));
    }
    wx.onShow(runShowFunc as any);
}

const __unRegisterShow = () => {
    if (funcs || showFuncs) return;
    wx.offShow(runShowFunc as any);;
    runShowFunc = undefined;
}

const __registerHide = () => {
    if (runHideFunc) return;
    runHideFunc = () => {
        hideFuncs && hideFuncs.forEach(func => func && func());
        funcs && funcs.forEach(func => func && func("onPause"));
    }
    wx.onHide(runHideFunc as any);
}

const __unRegisterHide = () => {
    if (funcs || hideFuncs) return;
    wx.offHide(runHideFunc as any);;
    runHideFunc = undefined;
}


/**
 * 注册软件盘弹出监听
 * @param callback 
 */
export const registerSoftInputChangedListener: TregisterSoftInputChangedListener = (callback: (height: number) => void) => { };

/**
 * 取消软件盘弹出监听
 * @param callback 
 */
export const unregisterSoftInputChangedListener: TunregisterSoftInputChangedListener = (callback?: (height: number) => void) => { };

/**
 * 隐藏NativeActivity的全屏开屏图PopupWindow
 */
export const hideFullScreenPopupWindow: ThideFullScreenPopupWindow = () => { }



/**
 * 监听APP失焦事件
 * @note 仅web和exe支持
 */
export const onBlur: TonBlur = (callback: () => void) => { };


/**
 * 取消 监听APP失焦事件
 * @note 仅web和exe支持
 */
export const offBlur: ToffBlur = (callback: () => void) => { };

export const requestPointerLock: TrequestPointerLock = () => {
};

export const exitPointerLock: TexitPointerLock = () => {
};

export const onPointerLockError: TonPointerLockError = (callback: () => void) => {
};

export const offPointerLockError: ToffPointerLockError = (callback?: () => void) => {

};

export const onPointerLockChange: TonPointerLockChange = (callback: (locked: boolean) => void) => {

};

export const offPointerLockChange: ToffPointerLockChange = (callback?: (locked: boolean) => void) => {

};


/**
 * 唤起客服
 */
export const showWithEntranceId: TshowWithEntranceId = () => {
}

/**
 * 客服后台用户信息
 * @returns 
 */
export const updateUserInfo: TupdateUserInfo = (str: string) => {
}

/**
 * 清空用户信息
 */
export const resetUserInfo: TresetUserInfo = () => {
}

/**
 * 唤起客服
 */
export const showWithEntranceIdLogin: TshowWithEntranceIdLogin = () => {
}
