import { AppName } from "pi_sys/native/base/system_info";
import { ScreenInfo, TgetBrightness, ToffUserCaptureScreen, TonUserCaptureScreen, TsetBrightness, TsetKeepOn } from "pi_sys/native/device/screen";

export const setBrightness: TsetBrightness = (value: number) => {
    // vivo平台差异
    value = window["_$platform_app_name"] === AppName.VVMinigame ? value / 100 : value;
    return new Promise((resolve, reject) => {
        wx.setScreenBrightness({
            value,
            success: resolve as any,
            fail: reject
        })
    });
}

export const setKeepOn: TsetKeepOn = (keepScreenOn: boolean) => {
    return new Promise((resolve, reject) => {
        wx.setKeepScreenOn({
            keepScreenOn,
            success: resolve as any,
            fail: reject
        })
    });
}

export const getBrightness: TgetBrightness = (): Promise<ScreenInfo> => {
    return new Promise((resolve, reject) => {
        wx.getScreenBrightness({
            success: (res) => {
                // vivo平台差异
                if (res.value) {
                    res.value = window["_$platform_app_name"] === AppName.VVMinigame ? res.value / 100 : res.value;
                }

                resolve(res)
            },
            fail: reject
        })
    });
}

export const onUserCaptureScreen: TonUserCaptureScreen = (callback: (path: string) => void) => {
    wx.onUserCaptureScreen(callback as any);
}

export const offUserCaptureScreen: ToffUserCaptureScreen = () => {
    // 微信文档上没有明确写明有此接口,但实测能用,这里判断一下
    if ((<any>wx).offUserCaptureScreen) {
        (<any>wx).offUserCaptureScreen();
    }
}