import { IParamPiFont, IPiFont } from "pi_sys/render/font";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

export class PiFont implements IPiFont {
    /**
     * 加载参数
     */
    public arg: IParamPiFont;
    /**
     * @hidden
     */
    private _family: string;
    /**
     * 字体名
     * @access
     */
    public get family() {
        return this._family;
    }
    private _src: string;
    /**
     * 字体文件路径
     * @access
     */
    public get src(): string {
        return this._src;
    }
    /**
     * value: localPath
     * 约束：小游戏环境下：高层的 arg.family 必须和 字体本身的 family 一致，才算加载成功；
     */
    public set src(value: string) {
        this._src = value;
        try {

            if (!this.arg || !this.arg.family) {
                const reason = "PiFont set src failed, arg.family not set yet";
                logger.warn(reason);
                this.onerror && this.onerror(reason);
                return;
            }

            // 华为小游戏需要两个参数， 另外vivo文档存在问题，文档上要求两参数，实际上只需要一个参
            const family = window["hbs"] ? (<Function>wx.loadFont)(this.arg.family, `url('${value}')`) : wx.loadFont(value);

            if (family === null) {
                // 根据 wx.loadFont 文档，返回null代表错误；
                const reason = "wx.loadFont return null for font-family";
                logger.warn(reason);
                this.onerror && this.onerror(reason);
            } else if (family !== this.arg.family) {
                // 字体文件的fontfamily 和 高层想用的 不一致，直接报错；
                // 注：小游戏不支持FontFace，不支持多个font-family；
                const reason = `wx.loadFont return ${family}, but app expectd ${this.arg.family}`;
                logger.warn(reason);
                this.onerror && this.onerror(reason);
            } else {
                // 成功
                this._family = family;
                this.onload && this.onload();
            }
        } catch (err) {
            this.onerror && this.onerror(err);
        }
    }

    public onload: Function;
    public onerror: (err) => void;

}

