import { IStats, Taccess, TappendFile, TcopyFile, Tftruncate, Tmkdir, TreadAssets, Treaddir, TreadFile, Trename, Trmdir, Tstat, Tunlink, TwriteFile } from "pi_sys/store/file_system";

let fs: wx.FileSystemManager = wx.getFileSystemManager();
let exists: Set<string> = new Set;        // 存在的文件或者目录

export const access: Taccess = (path: string) => {
    return new Promise((resolve, reject) => {
        fs.access({
            path,
            success(res) {
                resolve(true);
            },
            fail(res) {
                resolve(false);
            }
        })
    });
}

export const appendFile: TappendFile = (path: string, data: string | ArrayBuffer) => {
    let encoding = "utf8";
    if (typeof data !== "string") {
        encoding = "binary";
    }
    const __append = (): Promise<null> => {
        return new Promise((resolve, reject) => {
            fs.appendFile({
                filePath: path,
                data,
                encoding,
                success(res) {
                    resolve(null);
                },
                fail(res) {
                    reject(res);
                }
            })
        });
    }
    // 小游戏当文件不存在的时候不能直接append
    if (exists.has(path)) {
        return __append();
    } else {
        return access(path).then(exist => {
            if (!exist) {
                return writeFile(path, data);
            } else {
                exists.add(path);
                // 抖音小游戏appendFile Buffer数据的时候有BUG,没有正确append,字符串可以正常处理
                if ((<any>window).tt && encoding == "binary") {
                    return readFile(path, "binary").then((r) => {
                        const oldBuffer = r as ArrayBuffer;
                        const newU8 = new Uint8Array(oldBuffer.byteLength + (<ArrayBuffer>data).byteLength);
                        newU8.set(new Uint8Array(oldBuffer));
                        newU8.set(new Uint8Array(data as ArrayBuffer), oldBuffer.byteLength);
                        return writeFile(path, newU8.buffer);
                    });
                } else {
                    return __append();
                }
            }
        });
    }


}

export const copyFile: TcopyFile = (srcPath: string, destPath: string) => {
    return new Promise((resolve, reject) => {
        fs.copyFile({
            srcPath,
            destPath,
            success(res) {
                resolve(null);
            },
            fail(res) {
                reject(res);
            }
        })
    });
}

export const mkdir: Tmkdir = (path: string) => {
    return new Promise((resolve, reject) => {
        fs.mkdir({
            dirPath: path,
            recursive: true,
            success(res) {
                resolve(null);
            },
            fail(res) {
                reject(res);
            }
        })
    });
}

export const readFile: TreadFile = (path: string, encoding: "utf8" | "binary", offset: number, size: number) => {

    if (encoding === "binary") {
        // 经测试,如果传入binary返回值反而是utf8格式的字符串,不传以二进制格式返回
        encoding = undefined;
    }
    return new Promise((resolve, reject) => {
        let option = {
            filePath: path,
            encoding,
            success(res) {
                if (res.data instanceof ArrayBuffer) {
                    if ((<any>window).tt) {   // 抖音小游戏不支持读文件片段,自己处理
                        if (offset !== undefined) {
                            let end = size !== undefined ? offset + size : (<ArrayBuffer>res.data).byteLength;
                            const data = new Uint8Array(res.data.slice(offset, end))
                            resolve(data as any);
                        } else {
                            resolve(new Uint8Array(res.data) as any);
                        }
                    } else {
                        resolve(new Uint8Array(res.data) as any);
                    }

                } else {
                    resolve(res.data);
                }

            },
            fail(res) {
                reject(res);
            }
        }
        if (offset !== undefined) {
            (<any>option).position = offset
        }
        if (size) {
            (<any>option).length = size;
        }
        fs.readFile(option);
    });
}

export const readAssets: TreadAssets = (path: string) => {
    return Promise.resolve(null);
}


export const readdir: Treaddir = (path: string) => {
    return new Promise((resolve, reject) => {
        fs.readdir({
            dirPath: path,
            success(res) {
                resolve(res.files);
            },
            fail(res) {
                reject(res);
            }
        })
    });
}

export const rename: Trename = (oldPath: string, newPath: string) => {
    return new Promise((resolve, reject) => {
        fs.rename({
            oldPath,
            newPath,
            success(res) {
                resolve(null);
            },
            fail(res) {
                reject(res);
            }
        })
    });
}

export const rmdir: Trmdir = (path: string) => {
    return new Promise((resolve, reject) => {
        fs.rmdir({
            dirPath: path,
            recursive: true,
            success(res) {
                resolve(null);
            },
            fail(res) {
                reject(res);
            }
        })
    });
}


export const stat: Tstat = (path: string) => {
    return new Promise((resolve, reject) => {
        fs.stat({
            path,
            success: res => {
                resolve(res.stats);
            },
            fail(res) {
                reject(res);
            }
        })
    });
}

export const unlink: Tunlink = (path: string) => {
    return new Promise((resolve, reject) => {
        fs.unlink({
            filePath: path,
            success(res) {
                resolve(null);
            },
            fail(res) {
                reject(res);
            }
        })
    });
}

export const writeFile: TwriteFile = (path: string, data: string | ArrayBuffer) => {
    return new Promise((resolve, reject) => {
        let encoding = "utf8";
        if (typeof data !== "string") {
            encoding = "binary";
        }
        fs.writeFile({
            filePath: path,
            data,
            encoding,
            success(res) {
                resolve(null);
            },
            fail(res) {
                reject(res);
            }
        })
    });
}


export const ftruncate: Tftruncate = (path: string, length: number) => {
    return new Promise((resolve, reject) => {
        // 打开文件
        fs.open({
            filePath: path,
            flag: 'a+',
            success(res) {
                // 对文件内容进行截断操作
                fs.ftruncate({
                    fd: res.fd,
                    length, // 从第length个字节开始截断文件
                    success(res) {
                        resolve(res);
                    },
                    fail(res) {
                        reject(res);
                    }
                })
            },
            fail(res) {
                reject(res);
            }
        })
    })
}