import { ICfg } from "pi_sys/update/update_mgr";
import { getUpdateLevel, UpdateLevel } from "pi_sys/update/version";
import { fileCache } from "../wx/file_cache";


/**
 * 这里只处理大版本更新,中小版本更新交由项目处理
 */
export namespace UpdateMgr {
    let cfg: ICfg;
    let oldAppVersion: string;

    export const init = (_cfg: ICfg) => {
        cfg = _cfg;
        oldAppVersion = localStorage.getItem("appVersion");
    }

    /**
     * 是否是大版本更新
     * @returns 
     */
    export const isMajorUpdate = () => {
        if (!oldAppVersion) return false;
        if (!cfg) {
            return false;
        } else {
            return getUpdateLevel(oldAppVersion, cfg.appVersion) === UpdateLevel.Major;
        }
    }

    /**
     * 清除本地数据
     */
    export const clearLocalData = () => {
        const p1 = new Promise((resolve, reject) => {
            wx.clearStorage({
                success: resolve,
                fail: reject
            })
        });

        const p2 = fileCache.clear();
        return Promise.all([p1, p2]);
    }

    /**
     * 开始更新
     */
    export const startUpdate = () => {
        localStorage.setItem("appVersion", cfg.appVersion);
        return Promise.resolve();
    }


}
