import { Env as _Env } from "./util/env";
import { Depend as _Depend } from "./util/depend";
import { BaseTools as _BaseTools } from "./util/base_tools";
import { loadRes } from "./load/res_load";
import { loadAudio, loadFile, loadFont, loadImage, loadAudioBuffer, loadVideo } from "./load/object_load";
import { AdaptiveResult as _AdaptiveResult } from "./util/screen_adapter";
import { setCfgHandler as _setCfgHandler } from "./load/cfg_tools";
import * as _Time from './util/time';
import * as _HTTP from "./net/http";
import * as _Store from "./store/index";
import * as _UTF8 from "./util/utf8";
import * as _Base64 from "./util/base64js";
import * as _BatchLoad from "./load/batch_load";
import * as _PiAudio from "./media/audio";
import * as _PiInnerAudio from "./media/inner_audio";
import * as _PiFont from "./render/font";
import * as _PiImage from "./render/image";
import * as _PiVideo from "./render/video";
import * as _PiCanvas from "./render/canvas";
import * as _PiCursor from "./render/cursor";
import * as _Module from "./util/mod";
import * as _Log from "./util/log";
import * as _Logger from "./util/logger";
import * as _event from "./native/base/event";
import { Native as _Native } from "./native/index";
import { resetSize as _resetSize } from "./setup/main";
import { switchLang as _switchLang } from "./load/cfg_tools"
import { Asset as _Asset } from "./store/asset"
import * as _updateLoad from "./native/base/update";
import * as _fs from "./store/file_system";


export namespace PISYS {
    /**
     * 文件系统模块
     */
    export const FileSystem = _fs;
    /**
     * 更新下载模块
     */
    export const UpdateLoad = _updateLoad;
    /**
     * 日志库
     */
    export const Logger = _Logger;

    /**
     * 创建 Image
     */
    export const createImage = _PiImage.createImage;
    /**
     * 创建canvas
     */
    export const createCanvas = _PiCanvas.createCanvas;

    export const Cursor = _PiCursor;

    export const event = _event;
    /**
     * 对 _$pi 模块的封装
     */
    export const Module = _Module;

    /**
     * 屏幕宽高发生变化，重新适配
     */
    export const resetSize = _resetSize;

    /**
     * 语言切换
     */
    export const switchLang = _switchLang;

    /**
     * 框架环境
     */
    export const Env = _Env;
    /**
     * Depend 信息
     */
    export const Depend = _Depend;
    /**
     * 基础工具函数库
     */
    export const BaseTools = _BaseTools;
    /**
     * 资源加载接口
     */
    export const ResLoadTool = {
        loadRes, loadAudio, loadAudioBuffer, loadFile, loadFont, loadImage, loadVideo
    };

    /**
     * 底层资源接口
     */
    export const Asset = {
        getSystemPath: _Asset.getSystemPath,
        copyAssetToData: _Asset.copyAssetToData,
        read: _Asset.read
    }
    /**
     * 屏幕适配
     */
    export type AdaptiveResult = _AdaptiveResult;
    /**
     * Cfg 类型资源加载后处理流程注册接口
     */
    export const setCfgHandler = _setCfgHandler;
    /**
     * 时间
     */
    export const Time = _Time;
    /**
     * 网络
     */
    export const HTTP = _HTTP;
    /**
     * 本地存储
     */
    export const Store = _Store;

    export type IUserConfig = _Store.IUserConfig;

    /**
     * Base64编码解码
     * @note 项目请用 pi_utils/util/base64.ts 的 封装方法
     */
    export const Base64 = _Base64;
    /**
     * 文本编解码
     */
    export const UTF8 = _UTF8;
    /**
     * 批下载
     */
    export const BatchLoad = _BatchLoad;
    /**
     * 音频
     */
    export type PiAudio = _PiAudio.PiAudio;

    export type PiAudioBuffer = _PiInnerAudio.PiAudioBuffer;
    /**
     * 图片
     */
    export type PiImage = _PiImage.PiImage;
    export type PiVideo = _PiVideo.PiVideo;
    /**
     * 字体
     */
    export type PiFont = _PiFont.PiFont;
    /**
     * 原生运行平台底层接口
     */
    export const Native = _Native;

    /**
     * 日志库
     * @deprecated 已过时，请用 Logger 代替
     */
    export const Log = _Log;


}
