import { DownloadBase } from "pi_sys/load/down_load_base";
import { saveDbSign, savefile } from "pi_sys/load/down_load_hook";
import { FileLoadParam, Result } from "pi_sys/load/fileload_base";
import { IDirDependInfo, IFileDependInfo } from "pi_sys/util/depend_base";
import { Logger } from "pi_sys/util/logger";
import { formatError } from "pi_sys/util/util";
import { CollectType, doCollect } from "pi_sys/util/collect";
import { showRestartModel } from "pi_sys/setup/main";
import { ErrorCode } from "pi_sys/util/constant";
declare var module;
let logger = new Logger(module.name, "sys");

export class Download extends DownloadBase {
    /**
     * @description 添加下载文件
     * @example
     */
    public add(info: IFileDependInfo) {
        super.add(info);
    }
    /**
     * @description 开始
     * @example
     */
    public start() {
        const loadedMap: Map<string, number> = new Map;
        const ps: Promise<Uint8Array>[] = [];
        let localSignWait: Promise<void>[] = [];
        for (let info of this.files.values()) {
            doCollect(info, CollectType.RES);
            const p = new Promise<Uint8Array>((resolve, reject) => {
                FileLoadParam.netWeightQueuePool.put(FileLoadParam.downFileWeight, info.path, (data: Uint8Array, downloadData: Uint8Array, err: Error) => {
                    if (err) {
                        reject(err)
                    } else {
                        localSignWait.push(savefile(info.path, downloadData, info ? info.sign : "", this.waitWrite));
                        resolve(data);
                    }
                }, (url: string, type: string, total: number, loaded: number) => {
                    loadedMap.set(info.path, loaded);
                    this.loaded = 0;
                    loadedMap.forEach(l => this.loaded += l);
                    this.onProcess(url, 'fileDownload', this.total, this.loaded);
                });
            });
            ps.push(p);
        }

        return Promise.all(ps).then((datas) => {
            const fileMap: Map<string, Uint8Array> = new Map;

            let i = 0;
            for (let info of this.files.values()) {
                let data = datas[i++];
                fileMap.set(info.path, data);
            }
            this.onResult(fileMap);


            // 所有文件写本地结束
            let p = Promise.all(localSignWait).then(() => {
                saveDbSign();
            });
            if (this.waitWrite) {
                return p.then(() => {
                    return fileMap;
                })
            } else {
                return fileMap;
            }
        }).catch((reason) => {
            const message = `pi_sys, Download failed, reason = ${formatError(reason)}`;
            showRestartModel(ErrorCode.RES_DOAWLOAD_FAILED, message);
            this.onResult(null, reason);
        });

    }

}