import { BaseTools } from "pi_sys/util/base_tools";
import { EncryptType } from "pi_sys/util/constant";
import { IDirDependInfo } from "pi_sys/util/depend_base";
import { Env } from "pi_sys/util/env";
import { utf8Encode } from "pi_sys/util/utf8";
import { FileLoadParam, Result } from "./fileload_base";
import * as Sign from "./sign";


// 保存文件
export const savefile = (path: string, data: Uint8Array, sign: string, immediate: boolean = false) => {
    if (!data) return Promise.resolve();
    if (data.byteLength !== data.buffer.byteLength) {
        data = new Uint8Array(data.slice().buffer);
    }
    return FileLoadParam.localStore.write(path, data, undefined, undefined, undefined, undefined, immediate).then((_) => {
        FileLoadParam.netWeightQueuePool.removeDataCache(path);
        FileLoadParam.dbSign[path] = sign;
    }).catch(e => {
        FileLoadParam.netWeightQueuePool.removeDataCache(path);
        throw e;
    });
};

// 保存索引
export const saveDbSign = () => {
    let platform = Env.get("platform_type");
    if (platform === "minigame") {
        const uint8 = utf8Encode(JSON.stringify(FileLoadParam.dbSign));
        FileLoadParam.localStore.write(Sign.SIGN_KEY, uint8);
    } else {
        FileLoadParam.localStore.write(Sign.SIGN_KEY, FileLoadParam.dbSign);
    }

}

/**
 * 上行消息结构： (后缀1(文件名1:文件名2)后缀2(文件名1:文件名2):目录1()目录2())
 * 现在批量加载协议并没有验证文件大小是否正确 (windows和linux换行符的原因会导致文件大小不一致)
 * 会导致如下情况也是正常返回:
 * 批量下载a.js,b.js,c.js。但是服务器只有a.js,b.js,接口也会正常返回
 * @param tree
 * @param limit
 * @param result
 */
export const stringify = (tree: IDirDependInfo, limit: number, result: Result, sizeLimit: number) => {
    // 先写入本目录下的文件，按后缀归类
    if (tree.suffixMap) {
        for (let [k, arr] of tree.suffixMap) {
            let i = arr.length - 1;
            let f = arr[i--];
            let rk = BaseTools.replace(k);
            let rs = BaseTools.replace(BaseTools.fileBasename(f.path));
            let fileSize = f.encryptSize || f.size;
            if (result.url.length + rk.length + rs.length > limit || result.size + fileSize > sizeLimit && result.files.length) {
                result.next = true;
                // result.size = 0;
                return;
            }
            result.files.push(f);
            result.size += fileSize;
            result.url += rk + "(" + rs;
            for (; i >= 0; i--) {
                f = arr[i];
                fileSize = f.encryptSize || f.size;
                rs = BaseTools.replace(BaseTools.fileBasename(f.path));
                if (result.url.length + rs.length > limit || result.size + fileSize > sizeLimit && result.files.length) {
                    arr.length = i + 1;
                    result.next = true;
                    result.url += ")";
                    // result.size = 0;
                    return;
                }
                result.url += ":" + rs;
                result.files.push(f);
                result.size += fileSize;
            }
            result.url += ")";
            tree.suffixMap.delete(k);
        }
    }
    result.url += ":";
    if (!tree.children) {
        return;
    }
    let dir: IDirDependInfo;
    while (dir = tree.children[tree.children.length - 1]) {
        let i = dir.path.lastIndexOf("/", dir.path.length - 2);
        let rk = BaseTools.replace(dir.path.slice(i + 1, dir.path.length - 1));
        result.url += rk + "(";
        stringify(dir, limit, result, sizeLimit);
        result.url += ")";
        if (result.next) {
            return result;
        }
        tree.children.length -= 1;
    }
};