
import * as Sign from "pi_sys/load/sign";
import { Asset } from "../store/asset";
import { Depend } from "pi_sys/util/depend";
import { Store } from "../store/store";
import { FileLoadParam } from "pi_sys/load/fileload_base";
import { getCopyDepends } from "pi_sys/update/assets_to_data";
import { StoreType } from "pi_sys/store/store_base";
import { saveDbSign } from "pi_sys/load/down_load_hook";

declare var module;

export namespace Load {
    export function getLocalStore(storeName: string) {
        if (FileLoadParam.localStore) {
            return Promise.resolve(FileLoadParam.localStore);
        } else {
            return Store.createStore(storeName, "_db", StoreType.File).then((store) => {
                FileLoadParam.localStore = store as any;
                store.immediateWrite = false;
                return store;
            });
        }
    }


    /**
     * 初始化存储, 及默认参数。 limitLength 限制url的长度， sizeLimit 一次请求的byte限制
     */
    export function init(storeName: string) {
        return getLocalStore(storeName).then((store) => {
            return Promise.all([store.read(Sign.SIGN_KEY), Asset.get()]).then(([dbSigns, assetsDepends]) => {
                dbSigns = dbSigns || {};
                const assetsSign = {};
                for (let info of assetsDepends) {
                    let path = info[0];
                    let sign = info[2];
                    const file = Depend.getFile(path);
                    if (file && file.sign === sign) {
                        assetsSign[path] = sign;
                    }
                }
                FileLoadParam.assetsSign = assetsSign;
                const assetsCopySign = {};
                const assetsCopyDepends = getCopyDepends();
                for (let info of assetsCopyDepends) {
                    let path = info[0];
                    let sign = info[2];
                    const file = Depend.getFile(path);
                    if (file && file.sign === sign) {
                        assetsCopySign[path] = sign;
                    }
                }
                FileLoadParam.assetsCopySign = assetsCopySign;


                let save = false;
                for (let path in dbSigns) {
                    let sign = dbSigns[path];
                    const file = Depend.getFile(path);
                    if (!file || file.sign !== sign) {  // 文件删除 | 更新
                        delete dbSigns[path];
                        store.delete(path);
                        save = true;
                    }
                }
                FileLoadParam.dbSign = dbSigns;
                if (save) {
                    saveDbSign();
                }
            })
        });
    };

    /**
     * @description 获取本地存储
     * @example
     */
    export function getStore() {
        return FileLoadParam.localStore;
    }

    /**
     * 删除store
     */
    export function deleteStore() {
        if (FileLoadParam.localStore) {
            return Store.deleteStore(FileLoadParam.localStore.dbName);
        } else {
            return Promise.resolve();
        }
    }

}