import { IPiAudio } from "pi_sys/media/audio";
import { IPiInnerAudio, PiAudioBuffer } from "pi_sys/media/inner_audio";
import { IParamPiFont, PiFont } from "pi_sys/render/font";
import { PiImage } from "pi_sys/render/image";
import { IPiVideo } from "pi_sys/render/video";
import { IFileDependInfo } from "pi_sys/util/depend_base";

//#region object_load.ts

export type TloadImage = (filePath: string, args: any[]) => Promise<PiImage>;

export type TloadVideo = (filePath: string, args: any[]) => Promise<IPiVideo>;

export type TloadFont = (filePath: string, args: [IParamPiFont?]) => Promise<PiFont>;

export type TloadAudio = (filePath: string, args: any[]) => Promise<IPiAudio>;

export type TloadAudioBuffer = (filePath: string, args: any[]) => Promise<PiAudioBuffer>;

export type TloadFile = (filePath: string, args: any[], waitWrite?: boolean) => Promise<Uint8Array>;


//#endregion


//#region sign.ts


/**
 * @description 获得路径签名
 * @example
 */
export type TgetSign = (path: string) => string;

export type TcheckLocalSign = (newInfo: IFileDependInfo, path: string) => boolean;

/**
 * @description 检查文件是否会从本地加载, 返回 true | false | undefined
 * @example
 */
export type TisLocal = (filePath: string) => true | false | undefined;

/**
 * @description 设置文件的本地签名
 * @example
 */
export type TsetLocalSign = (files: IFileDependInfo[]) => Promise<any>;

/**
 * 判断是否为本地文件， "-"开头表示本地文件
 * @param sign 
 */
export type TisAsset = (path: string) => boolean;

/**
 * 判断是否是assets拷贝资源
 */
export type TisAssetCopy = (path: string) => boolean;


// 将签名 变成 本地签名
export type TchangeToAssetSign = (sign: string) => string;

// 签名是不是本地
export type TisAssetSign = (sign: string) => boolean;

//#endregion