import { FileLoad } from "./file_load";
import * as Sign from "./sign";
import { Asset } from "../store/asset";
import { FileLoadParam } from "./fileload_base";

import { Logger } from "pi_sys/util/logger";
import { getDecryptType, getEncryptKey, formatError } from "pi_sys/util/util";
import { readCopyFile } from "pi_sys/update/assets_to_data";
import { SimpleTaskPools } from "pi_sys/util/task_pools";
declare var module;
let logger = new Logger(module.name, "sys");

export class LocalLoad extends FileLoad {
    /**
     * @description 开始 
     * @example
     */
    public start() {
        let arr = [];
        let map: Map<string, Uint8Array> = new Map;
        const simpleTaskPools = new SimpleTaskPools();
        for (let info of this.files.values()) {
            let path = info.path;
            let size = info.size;
            const handleThen = (value: any) => {
                if (!(value instanceof Uint8Array)) {
                    value = new Uint8Array(value);
                }
                map.set(path, value);
                this.loaded += size;
                this.onProcess(path, "fileLocalLoad", this.total, this.loaded, value);
            }
            let p: Promise<any>;
            // 注意：只有Assets中的路径才加密,其他不用加密
            if (Sign.isAsset(path)) {
                const task = () => {
                    return Asset.read(path).then(handleThen);
                }
                simpleTaskPools.addTasks(task);
            } else if (Sign.isAssetCopy(path)) {
                const task = () => {
                    return readCopyFile(path).then(handleThen);
                }
                simpleTaskPools.addTasks(task);
            } else {
                // Store 的read本身就做了限流处理,且和读底层可以并发
                p = FileLoadParam.localStore.read(info.path, getDecryptType(info.encryptType), getEncryptKey(info.path), info.compressAlgorithm).then(handleThen);
                arr.push(p);
            }
        }
        arr.push(simpleTaskPools.start());
        return Promise.all(arr).then((_v) => {
            this.onResult(map);
            return map;
        }).catch((reason) => {
            logger.error("pi_sys, LoadLoad failed, reason = ", formatError(reason));
            this.onResult(null, reason);
            return map;
        });
    }
}