/**
 * 媒体：音频
 */
export * from "./audio_impl";
/**
 * @description 声音状态类型
 */
export enum StateType {
    INIT = 0, // 初始化
    READY, // 就绪
    PLAY, // 播放
    PAUSE // 暂停
}

export type StepType = 'linear' | 'quadIn' | 'quadOut' | 'quadInOut';

export interface IAudioPlayCfg {
    readonly context: AudioContext | wx.InnerAudioContext;
    volume: GainNode | number;
    playTime: number;
    pauseTime: number;
    onended: Function;
    delay: number;
    repeat: number;
    repeatDelay: number;
    startTime: number;
    loop: boolean;
    getState(): StateType;
    getDuration(): number;
    pause(): void;
    disconnect(): void;
    stop(): void;
    destroy(): void;
    play(volume?: number, delay?: number, offset?: number): void;
}

/**
 * 普通音效，外部只管发起播，不管后续,可能同时播多次同一音效
 * 背景音乐，外部控制 播放,暂停,停止，同时只能存在一个播放
 */
export interface IPiAudio {
    onended: Function;
    loop: boolean;
    arr: IAudioPlayCfg[];
    volume: number;
    play(delay?: number, repeat?: number, repeatDelay?: number): void;
    playAsBGM(volume?: number, loop?: boolean, stepTime?: number, stepType?: StepType): void;
    stopAsBGM(): void;
    pauseAsBGM(): void;
    destroy(): void;
}



