/**
 * 封装 Web-Audio 的 事件类
 */
export class AudioEventTarget {
    // key = event name; value = event cb
    private map: Map<string, Set<(...args: any[]) => void>> = new Map;

    public addListener(eventName: string, handler: (...args: any[]) => void) {
        let handlers = this.map.get(eventName);
        if (!handlers) {
            handlers = new Set();
            this.map.set(eventName, handlers);
        }
        if (!handlers.has(handler)) {
            handlers.add(handler);
        }
    }

    public removeListener(eventName: string, handler?: (...args: any[]) => void) {
        let handlers = this.map.get(eventName);
        if (handlers) {
            if (handler) {
                handlers.delete(handler);
            } else {
                handlers.clear();
            }
        }
    }

    /**
     * 派发事件
     * @param eventName 
     * @param args 
     */
    public dispatchEvent(eventName: string, ...args: any[]) {
        let handlers = this.map.get(eventName);
        if (handlers) {
            for (const h of handlers) {
                h && h(...args);
            }
        }
    }
}