import { ChannelInfo } from "./app_impl";

/**
 * App 相关接口
 */
export * from "./app_impl";

export type FlagAppLife = "onResume" | "onPause" | "onDestroy";

export type TexitApp = () => void;

export type TgetProcessId = () => Promise<string>;

export type TgetApiVersion = () => Promise<string>;

export type TgetPackageName = () => Promise<string>;

export type TgetFlavorName = () => Promise<string>;

export type TgetVersionName = () => Promise<string>;

export type TgetVersionCode = () => Promise<number>;

export type TgetRunconfigValue = (key: string) => Promise<string>;

export type TgetChannelInfo = () => Promise<ChannelInfo>;

export type TgetSdkInfo = () => Promise<ISdkInfo>;

export type TtriggerGC = () => void;

export type TonShow = (callback: () => void) => void;

export type TonHide = (callback: () => void) => void;

export type TonDestroy = (callback: () => void) => void;

export type TonBlur = (callback: () => void) => void;

export type ToffShow = (callback?: () => void) => void;

export type ToffHide = (callback?: () => void) => void;

export type ToffDestroy = (callback?: () => void) => void;

export type ToffBlur = (callback: () => void) => void;

export type ThideFullScreenPopupWindow = () => void;

export type TopenWebView = (url: string, params?: Object) => Promise<string>;

export type TcloseWebView = (tag: string) => Promise<void>;

export type TrequestPointerLock = () => void;
export type TexitPointerLock = () => void;
export type TonPointerLockError = (callback: () => void) => void;
export type ToffPointerLockError = (callback?: () => void) => void;
export type TonPointerLockChange = (callback: (locked: boolean) => void) => void;
export type ToffPointerLockChange = (callback?: (locked: boolean) => void) => void;

export type TinstallApk = (version: string) => Promise<void>;

export type TsetCursorVisible = (visible: boolean) => void;

// 打开渠道商店页
export type TopenAppstore = () => void;

// 获取CPU温度,单位摄氏度,保留小数点后一位
export type TgetTemperature = () => Promise<number>;


// 获取idfv
export type TgetIdfv = () => Promise<string>;

// 获取IpAdress
export type TgetIpAdress = () => Promise<string>;

// 获取Idfa
export type TgetIdfa = () => Promise<string>;


export type QuickChannel = "mi" | "huawei" | "vivo" | "oppo" | "yyb" | "uc9you";
/**
 * 获取quick渠道分发出来的渠道
 */
export type TgetChannelQuick = () => Promise<QuickChannel>;

/**
 * 通知底层隐私协议是否同意
 */
export type TprivacyAgreement = (isAccepted: boolean) => Promise<void>;

/**
 * 重启App
 */
export type TrestartApp = () => void;

/**
 * 清楚底层缓存数据
 */
export type TclearCache = () => Promise<void>;

/**
 * 唤起客服
 */
export type TshowWithEntranceId = () => void;

/**
 * 客服后台用户信息
 */
export type TupdateUserInfo = (str: string) => void;


/**
 * 唤起客服
 */
export type TshowWithEntranceIdLogin = () => void;

/**
 * 清空用户信息
 */
export type TresetUserInfo = () => void;

export interface ISdkKeyValue {
    key: string,
    value: string,
}

export interface ISdkFeature {
    name: string,
    config: Array<ISdkKeyValue>
}

export interface ISdkItem {
    name: string,
    init_config: Array<ISdkKeyValue>,
    features: Array<ISdkFeature>
}

export interface ISdkInfo {
    list: Array<ISdkItem>
}

// ============================ 下面接口 已经废弃

/**
 * @deprecated 已经废弃，请用 onShow, onHide, onDestroy 代替
 */
export type TregisterAppLifeListener = (callback: (flag: FlagAppLife) => void) => void;

/**
 * @deprecated 已经废弃，请用 offShow, offHide, offDestroy 代替
 */
export type TunregisterAppLifeListener = (callback?: (flag: FlagAppLife) => void) => void;

/**
 * @deprecated 已经废弃，请用 Keyboard.onKeyboardHeightChange 代替
 */
export type TregisterSoftInputChangedListener = (callback: (height: number) => void) => void;

/**
 * @deprecated 已经废弃，请用 KeyBoard.offKeyboardHeightChange 代替
 */
export type TunregisterSoftInputChangedListener = (callback?: (height: number) => void) => void;