/**
 * App 相关接口
 */

import { FlagAppLife, TexitApp, TgetFlavorName, TgetProcessId, TgetPackageName, TgetVersionCode, TgetVersionName, TgetSdkInfo, ISdkInfo, TtriggerGC, TonShow, TonHide, TonDestroy, ToffShow, ToffHide, ToffDestroy, ThideFullScreenPopupWindow, TregisterAppLifeListener, TunregisterAppLifeListener, TregisterSoftInputChangedListener, TunregisterSoftInputChangedListener, TgetApiVersion, TopenWebView, TcloseWebView, TonBlur, ToffBlur, TgetRunconfigValue, TgetChannelInfo, TrequestPointerLock, TexitPointerLock, ToffPointerLockChange, TonPointerLockChange, ToffPointerLockError, TonPointerLockError, TinstallApk, TsetCursorVisible, TopenAppstore, TgetTemperature, TgetChannelQuick, QuickChannel, TprivacyAgreement, TrestartApp, TclearCache, TshowWithEntranceId, TupdateUserInfo, TresetUserInfo, TshowWithEntranceIdLogin, TgetIdfv, TgetIpAdress, TgetIdfa } from "./app";

/**
 * 退出App
 * @note 微信小游戏必须有点击行为才能调用成功
 */
export const exitApp: TexitApp = () => { }

/**
 * 取进程ID
 */
export const getProcessId: TgetProcessId = () => {
    let now = new Date().getTime();
    return Promise.resolve(now.toString());
}

export const getApiVersion: TgetApiVersion = () => {
    return Promise.resolve("");
}

/**
 * 取包名
 */
export const getPackageName: TgetPackageName = () => {
    return Promise.resolve("");
}

/**
 * 取底层的APP 渠道名
 */
export const getFlavorName: TgetFlavorName = () => {
    return Promise.resolve("");
}

/**
 * 取底层的APP 安装版本名："主版本.次版本.补丁版本"
 */
export const getVersionName: TgetVersionName = () => {
    return Promise.resolve("");
}

/**
 * 取底层的APP 安装版本号
 */
export const getVersionCode: TgetVersionCode = () => {
    return Promise.resolve(0);
}

/**
 * 取 run_config.json 键对应的值
 */
export const getRunconfigValue: TgetRunconfigValue = (key: string): Promise<string> => {
    return Promise.resolve("");
}

export interface ChannelInfo {
    channel: string,
    param: {
        [x: string]: string
    }
};

/**
 * 渠道包对应的动态信息，用于 实现 安装参数化功能，场景：邀请码，广告统计 等
 */
export const getChannelInfo: TgetChannelInfo = (): Promise<ChannelInfo> => {
    let info = {
        channel: "",
        param: {}
    };
    return Promise.resolve(info);
}

/**
 * 取 App 安装的SDK的 JSON 配置
 */
export const getSdkInfo: TgetSdkInfo = (): Promise<ISdkInfo> => {
    return Promise.resolve({
        list: [],
    });
}

/**
 * 加快触发GC
 */
export const triggerGC: TtriggerGC = () => {
}

/**
 * 监听回到前台的事件
 * @note 和registerAppLifeListener在游戏生命周期内只能用其中一种方式
 */
export const onShow: TonShow = (callback: () => void) => { };

/**
 * 监听隐藏到后台事件。锁屏、按 HOME 键退到桌面、显示在聊天顶部等操作会触发此事件。
 * @note 和registerAppLifeListener在游戏生命周期内只能用其中一种方式
 */
export const onHide: TonHide = (callback: () => void) => { };

/**
 * 监听销毁事件
 * @note web和微信小游戏不支持
 * @note 和registerAppLifeListener在游戏生命周期内只能用其中一种方式
 */
export const onDestroy: TonDestroy = (callback: () => void) => { };

/**
 * 监听APP失焦事件
 * @note 仅web和exe支持
 */
export const onBlur: TonBlur = (callback: () => void) => { };

/**
 * 取消监听回到前台的事件
 * @note 和unregisterAppLifeListener在游戏生命周期内只能用其中一种方式,并且需要和注册时调用方式对应
 */
export const offShow: ToffShow = (callback?: () => void) => { };

/**
 * 取消监听隐藏到后台事件
 * @note 和unregisterAppLifeListener在游戏生命周期内只能用其中一种方式,并且需要和注册时调用方式对应
 */
export const offHide: ToffHide = (callback?: () => void) => { };

/**
 * 取消监听销毁事件
 * @note web和微信小游戏不支持
 * @note 和unregisterAppLifeListener在游戏生命周期内只能用其中一种方式,并且需要和注册时调用方式对应
 */
export const offDestroy: ToffDestroy = (callback?: () => void) => { };

/**
 * 取消 监听APP失焦事件
 * @note 仅web和exe支持
 */
export const offBlur: ToffBlur = (callback: () => void) => { };

/**
 * 隐藏NativeActivity的全屏开屏图PopupWindow
 */
export const hideFullScreenPopupWindow: ThideFullScreenPopupWindow = () => { }

/**
 * 打开WebView
 * @param url 打开的链接地址
 * @param params 额外的参数(方便以后扩展)
 * @returns 
 */
export const openWebView: TopenWebView = (url: string, params?: Object) => {
    return Promise.resolve("");
};

/**
 * 关闭webveiw
 * @param tag openWebView返回唯一标识
 * @returns 
 */
export const closeWebView: TcloseWebView = (tag: string) => {
    return Promise.resolve();
};

/**
 * 请求锁定鼠标
 */
export const requestPointerLock: TrequestPointerLock = () => {
};

/**
 * 退出鼠标锁定
 */
export const exitPointerLock: TexitPointerLock = () => {
};

/**
 * 监听鼠标锁定错误
 * @param callback 
 */
export const onPointerLockError: TonPointerLockError = (callback: () => void) => {
};

/**
 * 取消监听鼠标锁定错误
 * @param callback 
 */
export const offPointerLockError: ToffPointerLockError = (callback?: () => void) => {

};

/**
 * 监听鼠标锁定变化
 * @param callback 
 */
export const onPointerLockChange: TonPointerLockChange = (callback: (locked: boolean) => void) => {

};

/**
 * 取消监听鼠标锁定变化
 * @param callback 
 */
export const offPointerLockChange: ToffPointerLockChange = (callback?: (locked: boolean) => void) => {

};


export const installApk: TinstallApk = (version: string) => {
    return Promise.resolve(null)
}

export const setCursorVisible: TsetCursorVisible = (visible: boolean) => {
}


export const openAppstore: TopenAppstore = () => { };

// 获取CPU温度,单位摄氏度,保留小数点后一位
export const getTemperature: TgetTemperature = () => {
    return Promise.resolve(0);
};


// 获取idfv
export const getIdfv: TgetIdfv = () => {
    return Promise.reject("getIdfv not impl");
};

// 获取IpAdress
export const getIpAdress: TgetIpAdress = () => {
    return Promise.reject("getIpAdress not impl");
};

// 获取Idfa
export const getIdfa: TgetIdfa = () => {
    return Promise.reject("getIpAdress not impl");
};


/**
 * 获取quick渠道分发出来的渠道
 */
export const getChannelQuick: TgetChannelQuick = () => {
    return Promise.resolve(null);
};


export const privacyAgreement: TprivacyAgreement = (isAccepted: boolean) => {
    return Promise.resolve(null);
}


export const restartApp: TrestartApp = () => {
    return Promise.resolve(null);
};

export const clearCache: TclearCache = () => {
    return Promise.resolve(null);
};

/**
 * 唤起客服
 */
export const showWithEntranceId: TshowWithEntranceId = () => {
}

/**
 * 客服后台用户信息
 * @returns 
 */
export const updateUserInfo: TupdateUserInfo = (str: string) => {
}

/**
 * 清空用户信息
 */
export const resetUserInfo: TresetUserInfo = () => {
}

/**
 * 唤起客服
 */
export const showWithEntranceIdLogin: TshowWithEntranceIdLogin = () => {
}


// ============================ 下面接口 已经废弃

/**
 * 注册APP生命周期监听
 * @deprecated 已经废弃，请用 onShow, onHide, onDestroy 代替
 * @param callback 监听方法
 * @note 和onShow、onHide、onDestroy在游戏生命周期内只能用其中一种方式
 */
export const registerAppLifeListener: TregisterAppLifeListener = (callback: (flag: FlagAppLife) => void) => {
}

/**
 * 取消 注册APP生命周期监听,参数为空，则取消所有的事件监听
 * @deprecated 已经废弃，请用 onShow, onHide, onDestroy 代替
 * @param callback 监听方法
 * @note 和offShow、offHide、offDestroy在游戏生命周期内只能用其中一种方式,并且需要和注册时调用方式对应
 */
export const unregisterAppLifeListener: TunregisterAppLifeListener = (callback?: (flag: FlagAppLife) => void) => {
}

/**
 * 注册软件盘弹出监听
 * @deprecated 已经废弃，请用 Keyboard.onKeyboardHeightChange 代替
 */
export const registerSoftInputChangedListener: TregisterSoftInputChangedListener = (callback: (height: number) => void) => { };

/**
 * 取消软件盘弹出监听
 * @deprecated 已经废弃，请用 Keyboard.onKeyboardHeightChange 代替
 */
export const unregisterSoftInputChangedListener: TunregisterSoftInputChangedListener = (callback?: (height: number) => void) => { }; 