// 通用事件模块， 事件可以由该模块统一转发
// 该模块由pi_sys库内部调用
let events = {};

/**
 * 添加事件监听器
 * @param type 事件类型
 * @param handler 事件处理函数
 */
export const addListener = (type: string, handler: (event: any) => void) => {
	if (!events[type]) {
		events[type] = [];
	}
	events[type].push(handler);
}

/**
 * 移除事件监听器
 * @param type 事件类型
 * @param handler 事件处理函数
 */
export const removeListener = (type: string, handler: (event: any) => void) => {
	const listeners = events[type];

	if (listeners && listeners.length > 0) {
		for (let i = listeners.length; i--; i > 0) {
			if (listeners[i] === handler) {
				listeners.splice(i, 1)
				break
			}
		}
	}
}

/**
 * 派发事件
 * @param type 事件类型
 * @param event 事件
 */
export const dispatchEvent = (type: string, event: any) => {
	const listeners = events[type];
	if (listeners) {
		for (let i = 0; i < listeners.length; i++) {
			listeners[i](event)
		}
	}
}