/// 指针事件的添加、删除、派发接口
import {addListener, removeListener, dispatchEvent} from "./event";

export enum EventType {
	PointerDown = "pointerdown",
	PointerUp = "pointerup",
	PointerMove = "pointermove",
	PointerCancel = "pointercancel",
	Wheel = "wheel",
}

/**
 * 监听指针按下事件
 * @param handler 处理函数
 */
export const onPointerDown = (handler: (event: PointerEvent) => void) => {
	addListener(EventType.PointerDown, handler);
}

/**
 * 取消监听指针按下事件
 * @param handler 处理函数
 */
export const offPointerDown = (handler: (event: PointerEvent) => void) => {
	removeListener(EventType.PointerDown, handler);
}

/**
 * 监听指针抬起事件
 * @param handler 处理函数
 */
export const onPointerUp = (handler: (event: PointerEvent) => void) => {
	addListener(EventType.PointerUp, handler);
}

/**
 * 取消监听指针抬起事件
 * @param handler 处理函数
 */
export const offPointerUp = (handler: (event: PointerEvent) => void) => {
	removeListener(EventType.PointerUp, handler);
}

/**
 * 监听指针移动事件
 * @param handler 处理函数
 */
export const onPointerMove = (handler: (event: PointerEvent) => void) => {
	addListener(EventType.PointerMove, handler);
}

/**
 * 取消监听指针移动事件
 * @param handler 处理函数
 */
export const offPointerMove = (handler: (event: PointerEvent) => void) => {
	removeListener(EventType.PointerMove, handler);
}

/**
 * 监听指针取消事件
 * @param handler 处理函数
 */
export const onPointerCancel = (handler: (event: PointerEvent) => void) => {
	addListener(EventType.PointerCancel, handler);
}

/**
 * 取消监听指针取消事件
 * @param handler 处理函数
 */
export const offPointerCancele = (handler: (event: PointerEvent) => void) => {
	removeListener(EventType.PointerCancel, handler);
}

/**
 * 监听滚轮滚动事件
 * @param handler 处理函数
 */
export const onWheel = (handler: (event: WheelEvent) => void) => {
	addListener(EventType.Wheel, handler);
}

/**
 * 取消监听滚轮滚动事件
 * @param handler 处理函数
 */
export const offWheel = (handler: (event: PointerEvent) => void) => {
	removeListener(EventType.Wheel, handler);
}