/**
 * 系统信息
 */
export * from "./system_info_impl";

/**
 * 系统信息
 */
export interface SystemInfo {

    // 在竖屏正方向下的安全区域
    safeArea: {
        left: number;    // 安全区域 左上角 横坐标
        right: number;   // 安全区域 右下角 横坐标
        top: number;     // 安全区域 左上角 纵坐标
        bottom: number;  // 安全区域右下角纵坐标
        width: number;   // 安全区域的 宽度，单位 像素
        height: number;  // 安全区域的 高度，单位 像素
    };

    brand: string;              // 设备品牌
    model: string;              // 设备型号
    pixelRatio: number;         // 设备像素比;
    screenWidth: number;        // 屏幕宽度，单位px
    screenHeight: number;       // 屏幕高度，单位px
    statusBarHeight: number;    // 状态栏的高度，单位px
    language: string;           // 语言
    // @deprecated 已废弃  不可信
    cameraAuthorized: boolean;       // 允许使用摄像头的开关
    // @deprecated 已废弃  不可信
    locationAuthorized: boolean;     // 允许使用定位的开关
    // @deprecated 已废弃  不可信
    microphoneAuthorized: boolean;   // 允许使用麦克风的开关
    // @deprecated 已废弃  不可信
    notificationAuthorized: boolean; // 允许通知的开关
    // @deprecated 已废弃  不可信
    bluetoothEnabled: boolean;       // 蓝牙的系统开关
    // @deprecated 已废弃  不可信
    locationEnabled: boolean;        // 地理位置的系统开关
    // @deprecated 已废弃  不可信
    wifiEnabled: boolean;            // Wi-Fi 的系统开关

    colorDepth: number, // 返回目标设备或缓冲器上的调色板的比特深度。

    app: AppName,     // minigame、browser、native、webview
    platform: string,// ios、android、window、develops
    device: string,  // mobile、pc
    system: string;  // `platform version`
    benchLevel: BenchLevel,
};

/**
 * 平台名称
 */
export enum AppName {
    QQMinigame = "qq_minigame", // qq小游戏
    WXMinigame = "wx_minigame", // 华为小游戏
    TTMinigame = "tt_minigame", // 字节跳动小游戏
    HWMinigame = "hw_minigame", // 华为小游戏
    OPMinigame = "op_minigame", // oppo小游戏
    VVMinigame = "vv_minigame", // vivo小游戏
    KSMinigame = "ks_minigame", // 快手小游戏
    Browser = "browser", // 浏览器
    Webview = "webview", // Webview
    Native = "native", // 本地平台
    Unkown = "unkown", // 未知平台
};

/**
 * 性能指标
 */
export enum BenchLevel {
    CantRun = "CantRun", // 不能运行程序，缺少 WebGL 或 WebAssembly
    Unkown = "Unkown",  // 性能未知
    Low = "Low",     // 低端机
    Middle = "Middle",  // 中端机
    High = "High",    // 高端机
};

/**
 * 获取系统信息
 */
export type TgetSystemInfo = () => Promise<SystemInfo>;

/**
 * 取手机的设备ID
 */
export type TgetDeviceID = () => Promise<string>;

/**
 * 获取手机ANID
 */
export type TgetOAID = () => Promise<string>;

