/**
 * 系统信息
 */

import { Env } from "pi_sys/util/env";
import { AppName, BenchLevel, SystemInfo, TgetOAID, TgetDeviceID, TgetSystemInfo } from "./system_info";
import * as UAParser from "./ua_parser.min";

const version = (ua: string, rxp: RegExp) => {
    const arr = ua.match(rxp);
    if (!arr) {
        return;
    }

    return arr[1]
};

const getSystemInfoSys = (): SystemInfo => {
    let ua;
    try {
        ua = navigator && navigator.userAgent;
    } catch (_) {
        ua = "";
    }
    ua = ua.toLowerCase();

    let app: AppName;      // browser, webview, native
    let platform; // windows, android, ios;
    let device;   // pc or mobile;

    if (ua.indexOf("pi_webview") >= 0) {
        app = AppName.Webview;
    } else if (ua.indexOf("pi_native") >= 0) {
        app = AppName.Native;
    } else {
        app = AppName.Browser;
    }

    if (ua.indexOf("pi_android") >= 0) {
        device = "mobile";
        platform = "android";
    } else if (ua.indexOf("pi_ios") >= 0) {
        device = "mobile";
        platform = "ios";
    } else if (ua.indexOf("pi_windows") >= 0) {
        device = "pc";
        platform = "windows";
    } else {
        device = ua.indexOf("mobile") >= 0 ? "mobile" : "pc";

        if (ua.indexOf('windows nt') > -1) {
            platform = "windows";
        } else if (ua.indexOf('iphone os') > -1) {
            platform = "ios";
        } else if (ua.indexOf('android') > -1) {
            platform = "android";
        } else if (ua.indexOf('linux') > -1) {
            platform = "linux";
        }
    }

    let system = `${platform} ${version(ua, new RegExp(platform + "([\d.]+)"))}`;

    let benchLevel = BenchLevel.Unkown;

    let isSupportWasm = false;
    try {
        if (window["WebAssembly"] && window["WebAssembly"].instantiate) {
            isSupportWasm = true;
        }
    } catch (_e) {
        isSupportWasm = false;
    }
    if (!isSupportWasm) {
        benchLevel = BenchLevel.CantRun;
    }

    try {
        const parser = new UAParser(ua);
        const result = parser.getResult();
        if (result && result.os) {
            system = `${result.os.name} ${result.os.version}`;
        }
    } catch (e) {

    }


    return {
        safeArea: {
            left: 0,
            right: window.innerWidth,
            top: 0,
            bottom: window.innerHeight,
            width: window.innerWidth,
            height: window.innerHeight,
        },

        brand: "",
        model: "",
        pixelRatio: window.devicePixelRatio,
        screenWidth: window.innerWidth,
        screenHeight: window.innerHeight,
        statusBarHeight: 0,
        language: navigator.language,

        cameraAuthorized: false,
        locationAuthorized: false,
        microphoneAuthorized: false,
        notificationAuthorized: false,
        bluetoothEnabled: false,
        locationEnabled: false,
        wifiEnabled: false,

        colorDepth: screen.colorDepth,

        app,
        system,
        platform,
        device,
        benchLevel,
    };
}



export const getSystemInfo: TgetSystemInfo = async () => {
    let r = getSystemInfoSys();
    const rewriteSystemInfo = Env.get("rewriteSystemInfo") || window["winit"].env.rewriteSystemInfo;
    if (rewriteSystemInfo) {
        await rewriteSystemInfo(r, r);
    }
    return Promise.resolve(r);
}


export const getDeviceID: TgetDeviceID = () => {
    return Promise.resolve(generateDeviceId());
}


export const getOAID: TgetOAID = () => {
    return Promise.resolve("");
}


function generateDeviceId(): string {
    let old = window.localStorage.getItem("pi_sys_minigame_deviceId")
    if (old) { return old }
    const characters: string = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    let deviceId: string = '';
    for (let i = 0; i < 32; i++) {
        const randomIndex = Math.floor(Math.random() * characters.length);
        deviceId += characters.charAt(randomIndex);
    }
    window.localStorage.setItem("pi_sys_minigame_deviceId", deviceId)
    return deviceId;
}

