/**
 * 更新下载模块
 */
export * from "./update_impl";

export interface IDownloadBase {
    url: string;        // 远端下载url
    timeout: number;    // 网络请求超时时间
    size?: number;       // 远端文件大小，可不填 (如果不知道大小,不能填0)
    deltaTime: number;     // 进度回调间隔时间
    onsucceed: () => void;  // 下载成功回调
    onerror: (msg: any) => void;     // 下载失败回调
    onprogress: (total: number, loaded: number) => void;  // 下载进度回调
}


export interface IDownloadFileOption extends IDownloadBase {
    filePath: string;               // 文件保存路径
    fileName: string;             // 文件保存名称
}

export interface IUpdateOption extends IDownloadBase {
    version: string;                 // 此次更新的版本号，底层会做判断，如果发现本地已经有相同版本号的文件，会直接返回成功
}

/**
 * 下载文件到指定目录
 */
export type TdownloadFile = (option: IDownloadFileOption) => void;


/**
 * 更新apk
 */
export type TupdateApk = (option: IUpdateOption) => void;

/**
 * 更新so
 */
export type TupdateSo = (option: IUpdateOption) => void;

/**
 * 更新快照
 */
export type TupdateSnapshot = (option: IUpdateOption) => void;

/**
 * 更新pisys
 */
export type TupdateSys = (option: IUpdateOption) => void;

/**
 * 重启虚拟机
 * 注意:实现的时候 抛出beforeRestartApp事件
 */
export type TrestartApp = () => void;




