/**
 * 键盘
 */
export * from "./keyboard_impl";

export enum ConfirmType {
    GO = 'go',          // 前往
    DONE = 'done',      // 完成
    NEXT = 'next',      // 下一个
    SEND = 'send',      // 发送
    SEARCH = 'search'   // 搜索
}

export interface KeyboardConfig {
    left: number;                      // 输入框的位置
    top: number;                       // 输入框的位置
    defaultValue: string;              // 键盘输入框显示的默认值
    maxLength: number;                 // 键盘中文本的最大长度
    multiple: boolean;                 // 是否为多行输入
    confirmHold?: boolean;             // 当点击完成时键盘是否收起
    confirmType?: ConfirmType;         // 键盘右下角 confirm 按钮的类型，只影响按钮的文本内容
    inputBox?: boolean;                //  仅web支持,默认输入框， 默认为true
    attribute?: Object;                 //  仅web支持  key-value形式
    height?: number;                    // textarea需要设置此属性 影响换行
    width?: number;                      // textarea需要设置此属性 影响换行
    fontSize?: number;                   // textarea需要设置此属性 影响换行
    fontFamily?: string;                    // textarea需要设置此属性 影响换行
    textAlign?: string;                      // 右侧对齐
    success?: (res: any) => void;         // 键盘显示接口调用成功回调
    fail?: (res: any) => void;            // 键盘显示接口调用失败回调
    getCursorPosition?: () => [number, number]; // 取光标位置
}

export type Tshow = (config: KeyboardConfig) => void;

export type Thide = (options: { fail?: Function, success?: Function }) => void;

export type Tupdate = (options: { value: string, success?: Function, fail?: Function, complete?: (res: any) => void }) => void;

export type TonInput = (callback: ({ value: string }) => void) => void;

export type ToffInput = (callback?: ({ value: string }) => void) => void;

export type TonConfirm = (callback: ({ value: string }) => void) => void;

export type ToffConfirm = (callback?: ({ value: string }) => void) => void;

export type TonComplete = (callback: ({ value: string }) => void) => void;

export type ToffComplete = (callback?: ({ value: string }) => void) => void;

// TODO 监听 虚拟键盘 高度，仅限于 支持 虚拟键盘 的 平台 
export type TonKeyboardHeightChange = (callback: ({ height: number }) => void) => void;
// TODO 监听 虚拟键盘 高度，仅限于 支持 虚拟键盘 的 平台
export type ToffKeyboardHeightChange = (callback: ({ height: number }) => void) => void;

// TODO 微信小游戏，仅适用 PC 平台
export type TonKeyDown = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => void;
// TODO 微信小游戏，仅适用 PC 平台
export type ToffKeyDown = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => void;
// TODO 微信小游戏，仅适用 PC 平台
export type TonKeyUp = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => void;
// TODO 微信小游戏，仅适用 PC 平台
export type ToffKeyUp = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => void;

// -------------------- 小游戏没有直接对应的,我们自己添加的,仅在部分平台有效

export type TsetSelection = (index: number) => boolean;

export type TsupportSelection = () => boolean;

export type TonSelection = (callback: (value: number) => void) => void;

export type ToffSelection = (callback?: (value: number) => void) => void;


