/**
 * 底层设备：屏幕
 */

import { ScreenInfo, TgetBrightness, ToffUserCaptureScreen, TonUserCaptureScreen, TsetBrightness, TsetKeepOn } from "./screen";

/**
 * 设置屏幕亮度
 * @param value 屏幕亮度值，范围 0 ~ 1。0 最暗，1 最亮
 */
export const setBrightness: TsetBrightness = (value: number) => {
    return Promise.resolve();
}

/**
 * 设置是否保持常亮状态
 * @note 仅在当前程序生效，离开程序后设置失效
 * 
 * @param keepScreenOn 是否保持屏幕常亮
 */
export const setKeepOn: TsetKeepOn = (keepScreenOn: boolean) => {
    return Promise.resolve();
}

/**
 * 获取屏幕亮度
 * @note 若Android系统设置中开启了自动调节亮度功能，则屏幕亮度会根据光线自动调整
 * @note 该接口仅能获取自动调节亮度之前的值，而非实时的亮度值。
 * @note 该接口获取的值并不完全准确,有些机型在APK上获取亮度会超过1,此时底层会强制设为1
 */
export const getBrightness: TgetBrightness = (): Promise<ScreenInfo> => {
    return Promise.resolve(null);
}

/**
 * 监听用户主动截屏事件。
 * @note 用户使用系统截屏按键截屏时触发，只能注册一个监听
 * @param callback(path) 用户主动截屏事件的回调函数, path代表截屏后的在底层的路径
 * @note 回调path参数微信小游戏上返回空字符串
 */
export const onUserCaptureScreen: TonUserCaptureScreen = (callback: (path: string) => void) => {
}

/**
 * 取消监听用户主动截屏事件。
 */
export const offUserCaptureScreen: ToffUserCaptureScreen = () => {
}