import { AudioSource } from "../util/enum"
import { Format, SampleRate, TonError, TonFrameRecorded, TonInterruptionBegin, TonInterruptionEnd, TonPause, TonResume, TonStart, TonStop, Tpause, Tresume, Tstart, Tstop } from "./audio_recorder";


/**
 * 全局唯一的录音管理器
 */

/**
 * 开始录音
 * @param duration	        录音的时长，单位 ms，最大值 600000（10 分钟）	
 * @param sampleRate	    采样率	
 * @param numberOfChannels	录音通道数	
 * @param encodeBitRate	    编码码率，有效值见下表格	
 * @param format	        音频格式	
 * @param frameSize	        指定帧大小，单位 KB。传入 frameSize 后，每录制指定帧大小的内容后，会回调录制的文件内容，不指定则不会回调。暂仅支持 mp3 格式。	
 * @param audioSource	    指定录音的音频输入源，可通过 getAvailableAudioSources() 获取当前可用的音频源
 * @note 采样率与编码码率限制
 *      每种采样率有对应的编码码率范围有效值，设置不合法的采样率或编码码率会导致录音失败，具体对应关系如下表
 *          采样率	编码码率
 *          8000	16000 ~ 48000
 *          11025	16000 ~ 48000
 *          12000	24000 ~ 64000
 *          16000	24000 ~ 96000
 *          22050	32000 ~ 128000
 *          24000	32000 ~ 128000
 *          32000	48000 ~ 192000
 *          44100	64000 ~ 320000
 *          48000	64000 ~ 320000
 */
export const start: Tstart = (duration = 60000, sampleRate: SampleRate = SampleRate.N8000, numberOfChannels: 1 | 2 = 2, encodeBitRate = 48000, format: Format = Format.Aac, frameSize?: number, audioSource: AudioSource = AudioSource.Auto): Promise<void> => {
    return Promise.resolve();
}

/**
 * 暂停录音
 */
export const pause: Tpause = (): Promise<void> => {
    return Promise.resolve();
}

/**
 * 继续录音
 */
export const resume: Tresume = (): Promise<void> => {
    return Promise.resolve();
}

/**
 * 停止录音
 */
export const stop: Tstop = (): Promise<void> => {
    return Promise.resolve();
}

/**
 * 监听录音开始事件
 * @param cb 回调
 */
export const onStart: TonStart = (cb: () => void) => {
}

/**
 * 监听录音结束事件
 * @param cb 回调,参数:
 *      tempFilePath	录音文件的临时路径 (本地路径)
 *      duration	    录音总时长，单位：ms
 *      fileSize	    录音文件大小，单位：Byte
 */
export const onStop: TonStop = (cb: (tempFilePath: string, duration: number, fileSize: number) => void) => {
}

/**
 * 监听录音暂停事件
 * @param cb 回调
 */
export const onPause: TonPause = (cb: () => void) => {
}

/**
 * 监听录音继续事件
 * @param cb 回调
 */
export const onResume: TonResume = (cb: () => void) => {
}

/**
 * 监听录音错误事件
 * @param cb 回调,参数:
 *      errMsg	错误信息
 */
export const onError: TonError = (cb: (errMsg: string) => void) => {
}

/**
 * 监听已录制完指定帧大小的文件事件
 * @param cb 回调, 参数:
 *      frameBuffer	 录音分片数据
 *      isLastFrame	 当前帧是否正常录音结束前的最后一帧
 * @note 如果设置了 frameSize，则会回调此事件
 */
export const onFrameRecorded: TonFrameRecorded = (cb: (frameBuffer: ArrayBuffer, isLastFrame: boolean) => void) => {
}

/**
 * 监听录音因为受到系统占用而被中断开始事件
 * @param cb 回调
 * @note 以下场景会触发此事件：微信语音聊天、微信视频聊天。此事件触发后，录音会被暂停。pause 事件在此事件后触发
 */
export const onInterruptionBegin: TonInterruptionBegin = (cb: () => void) => {
}

/**
 * 监听录音中断结束事件
 * @param cb 回调
 * @note 在收到 interruptionBegin 事件之后，小程序内所有录音会暂停，收到此事件之后才可再次录音成功
 */
export const onInterruptionEnd: TonInterruptionEnd = (cb: () => void) => {
}