/**
 * 媒体：相机
 */
export * from "./camera_impl";

/**
 * 相册 文件 信息
 */
export interface AlbumFileInfo {
    type: "image" | "video" | "unknown",  // 类型：是 图像，视频，未知
    path: string,                         // 文件 的 绝对路径，通过 getBase64FromLocalFile 取 内容
    size: number,                         // 文件大小，单位：字节
    width: number,                        // 图片 或 视频 的 宽度，单位：px
    height: number,                       // 图片 或 视频 的 高度，单位：px
};


/**
 * 打开相册
 * @param mediaType "image" | "video" | "mix" ; 相册显示 图像、视频、所有
 * @param maxSelectNum 最大选择 数量：1-9
 * @param isCompress 是否 压缩，压缩质量 由 程序决定，参考 Luban Java 库
 * @returns 相册文件 数组
 */
export type TopenAlbum = (mediaType: "image" | "video" | "mix", maxSelectNum?: number, isCompress?: boolean) => Promise<AlbumFileInfo[]>;


/**
 * 打开相机 拍照 或 录像
 * @param mediaType "image" | "video" 拍照 或 录像
 * @param recordVideoMaxSecond 录像的 最长 时间，单位：秒
 * @returns 相册文件 数组,只有 一个元素
 */
export type TopenCamera = (mediaType: "image" | "video", recordVideoMaxSecond?: number) => Promise<AlbumFileInfo[]>;

/**
 * 扫码：拍照 or 相册选择
 * @returns 二维码内容
 */
export type TscanQRCode = () => Promise<string>;

/**
 * 保存 图片 到相册
 * @param path 文件的绝对路径
 * @returns 文件的绝对路径，通过 getBase64FromLocalFile 取 内容
 */
export type Tsave2Album = (path: string) => Promise<string>;

/**
 * 获取图片内容
 * @path  图片本地路径
 * @returns 图片内容base64编码字符串
 */
export type TgetBase64FromLocalFile = (path: string) => Promise<string>;