/**
 * 媒体：图片
 */
export * from "./image_impl";
/**
 * 图片的尺寸
 */
export enum SizeType {
    Original = "original",	    // 原图	
    Compressed = "compressed",	// 压缩图
}

/**
 * 图片的来源
 */
export enum SourceType {
    Album = "album",	 // 从相册选图	
    Camera = "camera",	 // 使用相机
}

/**
 * 选择的图片信息
 */
export interface ChooseImageInfo {
    path: string,   // 图片文件路径（本地临时文件）
    size: number;   // 图片文件大小，单位 B
}

/**
 * 保存图片到系统相册
 * @note 调用前需要 用户授权 scope.writePhotosAlbum
 * @param filePath	图片文件路径，可以是临时文件路径或永久文件路径 (本地路径) ，不支持网络路径
 * @deprecated 请使用Camera模块 save2Album 代替
 */
export type TsaveImageToPhotosAlbum = (filePath: string) => Promise<void>;

/**
 * 在新页面中全屏预览图片
 * @note 预览的过程中用户可以进行保存图片、发送给朋友等操作
 * 
 * @param urls	    需要预览的图片链接列表
 * @param current	当前显示图片的链接,默认值为urls 的第一张
 */
export type TpreviewImage = (urls: string[], current?: string) => Promise<void>;

/**
 * 从本地相册选择图片或使用相机拍照
 * @param count         最多可以选择的图片张数
 * @param sizeType	    所选的图片的尺寸
 * @param sourceType	选择图片的来源
 * @deprecated 请使用Camera模块 openCamera & openAlbum 代替
 */
export type TchooseImage = (count?: number, sizeType?: SizeType, sourceType?: SourceType) => Promise<ChooseImageInfo[]>;

