import { AudioSource } from "../util/enum"
import { ErrCode, IInnerAudioContext, TcreateInnerAudioContext, TgetAvailableAudioSources, TsetInnerAudioOption } from "./inner_audio";

/**
 * 媒体：音频
 */



/**
 * 设置 InnerAudioContext 的播放选项
 * @param mixWithOther	     是否与其他音频混播，设置为 true 之后，不会终止其他应用或微信内的音乐
 * @param obeyMuteSwitch	（仅在 iOS 生效）是否遵循静音开关，设置为 false 之后，即使是在静音模式下，也能播放声音
 * @note 设置之后对当前小程序全局生效
 */
export const setInnerAudioOption: TsetInnerAudioOption = (mixWithOther = true, obeyMuteSwitch = true): Promise<void> => {
    return Promise.resolve();
}

/**
 * 获取当前支持的音频输入源
 * @return Promise<支持的音频输入源列表>
 * @note 可在 RecorderManager.start() 接口中使用。
 *      返回值定义参考 https://developer.android.com/reference/kotlin/android/media/MediaRecorder.AudioSource
 */
export const getAvailableAudioSources: TgetAvailableAudioSources = (): Promise<AudioSource[]> => {
    return Promise.resolve([]);
}

/**
 * 创建内部 audio 上下文 InnerAudioContext 对象
 */
export const createInnerAudioContext: TcreateInnerAudioContext = (): Promise<IInnerAudioContext> => {
    return Promise.resolve(null);
}

/**
 * InnerAudioContext 实例
 * @note 可通过 createInnerAudioContext 接口获取实例
 * @note 支持格式
 *           格式	iOS	Android
 *           flac	x	√
 *           m4a	√	√
 *           ogg	x	√
 *           ape	x	√
 *           amr	x	√
 *           wma	x	√
 *           wav	√	√
 *           mp3	√	√
 *           mp4	x	√
 *           aac	√	√
 *           aiff	√	x
 *           caf	√	x
 */
export class InnerAudioContext implements IInnerAudioContext {
    /**
     * 销毁当前实例
     */
    destroy(): Promise<void> {
        return Promise.resolve();
    }

    /**
     * 暂停
     * @note 暂停后的音频再播放会从暂停处开始播放
     */
    pause(): Promise<void> {
        return Promise.resolve();
    }

    /**
     * 播放
     */
    play(): Promise<void> {
        return Promise.resolve();
    }

    /**
     * 跳转到指定位置
     * @param position  跳转的时间，单位 s。精确到小数点后 3 位，即支持 ms 级别精确度
     */
    seek(position: number): Promise<void> {
        return Promise.resolve();
    }

    /**
     * 停止
     * @note 停止后的音频再播放会从头开始播放
     */
    stop(): Promise<void> {
        return Promise.resolve();
    }

    /**
     * 取消监听音频进入可以播放状态的事件
     * @param cb 回调
     */
    offCanplay(cb: () => void) {
    }

    /**
     * 取消监听音频自然播放至结束的事件
     * @param cb 回调
     */
    offEnded(cb: () => void) {
    }

    /**
     * 取消监听音频播放错误事件
     * @param cb 回调
     */
    offError(cb: () => void) {
    }

    /**
     * 取消监听音频暂停事件
     * @param cb 回调
     */
    offPause(cb: () => void) {
    }

    /**
     * 取消监听音频播放事件
     * @param cb 回调
     */
    offPlay(cb: () => void) {
    }

    /**
     * 取消监听音频完成跳转操作的事件
     * @param cb 回调
     */
    offSeeked(cb: () => void) {
    }

    /**
     * 取消监听音频进行跳转操作的事件
     * @param cb 回调
     */
    offSeeking(cb: () => void) {
    }

    /**
     * 取消监听音频停止事件
     * @param cb 回调
     */
    offStop(cb: () => void) {
    }

    /**
     * 取消监听音频播放进度更新事件
     * @param cb 回调
     */
    offTimeUpdate(cb: () => void) {
    }

    /**
     * 取消监听音频加载中事件
     * @param cb 回调
     */
    offWaiting(cb: () => void) {
    }

    /**
     * 监听音频进入可以播放状态的事件。但不保证后面可以流畅播放
     * @param cb 回调
     */
    onCanplay(cb: () => void) {
    }

    /**
     * 监听音频自然播放至结束的事件
     * @param cb 回调
     */
    onEnded(cb: () => void) {
    }

    /**
     * 监听音频播放错误事件
     * @param cb 回调,参数:
     *      errMsg	string	
     *      errCode	number
     */
    onError(cb: (errMsg: string, errCode: ErrCode) => void) {
    }

    /**
     * 监听音频暂停事件
     * @param cb 回调
     */
    onPause(cb: () => void) {
    }

    /**
     * 监听音频播放事件
     * @param cb 回调
     */
    onPlay(cb: () => void) {
    }

    /**
     * 监听音频完成跳转操作的事件
     * @param cb 回调
     */
    onSeeked(cb: () => void) {
    }

    /**
     * 监听音频进行跳转操作的事件
     * @param cb 回调
     */
    onSeeking(cb: () => void) {
    }

    /**
     * 监听音频停止事件
     * @param cb 回调
     */
    onStop(cb: () => void) {
    }

    /**
     * 监听音频播放进度更新事件
     * @param cb 回调
     */
    onTimeUpdate(cb: () => void) {
    }

    /**
     * 监听音频加载中事件
     * @param cb 回调
     * @note 当音频因为数据不足，需要停下来加载时会触发
     */
    onWaiting(cb: () => void) {
    }
}

