/**
 * 登录 SDK
 * ========================== 注：不要直接使用这里的接口，请使用 pi_common/sdk 的 函数
 */

import { Tlogin, Tlogout, TregisterAccountSwitchCb, TregisterLoginCb, TregisterLogoutCb, TunRegisterAccountSwitchCb, TunRegisterLoginCb, TunRegisterLogoutCb } from "./login";

export const login: Tlogin = (sdkName: string, _param?: any) => {
    let cb = loginCbMap.get(sdkName);
    if (cb) {
        cb.success && cb.success({
            sdkName,
        });
    }
}

export const logout: Tlogout = (sdkName: string, _param?: any) => {
    let cb = logoutCbMap.get(sdkName);
    if (cb) {
        cb.success && cb.success({
            sdkName,
        });
    }
}

export const registerLoginCb: TregisterLoginCb = (
    sdkName: string,
    success?: (info: any) => void,
    fail?: (reason: any) => void
) => {
    loginCbMap.set(sdkName, {
        success,
        fail,
    });
}

export const unRegisterLoginCb: TunRegisterLoginCb = (
    sdkName: string,
    _success?: (info: any) => void,
    _fail?: (reason: any) => void
) => {
    loginCbMap.delete(sdkName);
}

export const registerLogoutCb: TregisterLogoutCb = (
    sdkName: string,
    success?: (info: any) => void,
    fail?: (reason: any) => void
) => {
    logoutCbMap.set(sdkName, {
        success,
        fail,
    });
}

export const unRegisterLogoutCb: TunRegisterLogoutCb = (
    sdkName: string,
    _success?: (info: any) => void,
    _fail?: (reason: any) => void
) => {
    logoutCbMap.delete(sdkName);
}


export const registerAccountSwitchCb: TregisterAccountSwitchCb = (
    sdkName: string,
    success?: (info: any) => void,
    fail?: (reason: any) => void
) => {
    checkLoginCbMap.set(sdkName, {
        success,
        fail,
    });
}

export const unRegisterAccountSwitchCb: TunRegisterAccountSwitchCb = (
    sdkName: string,
    _success?: (info: any) => void,
    _fail?: (reason: any) => void
) => {
    checkLoginCbMap.delete(sdkName);
}

// =========================== 实现

// 模拟 login 回调
// 任何 SDK 只能有一个 回调，后面的调用会清掉上一次调用
// 正确性由 pi_common/sdk/index.ts 保证
let loginCbMap = new Map<string, {
    success?: (info: any) => void,
    fail?: (reason: any) => void
}>();

// 模拟 logout 回调
// 任何 SDK 只能有一个 回调，后面的调用会清掉上一次调用
// 正确性由 pi_common/sdk/index.ts 保证
let logoutCbMap = new Map<string, {
    success?: (info: any) => void,
    fail?: (reason: any) => void
}>();


// 模拟 切换账号 回调
// 任何 SDK 只能有一个 回调，后面的调用会清掉上一次调用
// 正确性由 pi_common/sdk/index.ts 保证
let checkLoginCbMap = new Map<string, {
    success?: (info: any) => void,
    fail?: (reason: any) => void
}>();
