export * from "./specialcall_impl";

// 看对应的 sdk 的文档
export interface ChannelTypeInfo {
    channelType: number,
}

export interface VerifyRealNameInfo {
    uid: string,         // 用户id
    age: number,         // 年龄, 如果渠道没返回默认为-1
    realName: boolean,   // 是否已实名： true表示已实名， false表示未实名；如果渠道没返回默认为 false
    resumeGame: boolean, // 渠道实名认证失败之后是否可以继续游戏 ：true表示可以， false表示不可以；如果渠道没返回默认为 true
    other: string,       // 预留字段，如果渠道没返回默认为 ""
}

/**
 * 取渠道类型
 * 
 * 如果相关 SDK 不支持该函数，则 返回 Promise.reject
 */
export type TgetChannelType = (_sdkName: string) => Promise<ChannelTypeInfo>;

/**
 * 显示悬浮框：某些渠道 有 悬浮按扭、进入用户中心、进入论坛等功能
 * 
 * 如果相关 SDK 不支持该函数，则 返回 Promise.reject
 */
export type TshowToolbar = (_sdkName: string) => Promise<void>;

/**
 * 隐藏悬浮框：某些渠道 有 悬浮按扭、进入用户中心、进入论坛等功能
 * 
 * 如果相关 SDK 不支持该函数，则 返回 Promise.reject
 */
export type ThideToolbar = (_sdkName: string) => Promise<void>;

/**
 * 实名认证，登录成功后 进行调用
 * 
 * 如果相关 SDK 不支持该函数，则 返回 Promise.reject
 */
export type TverifyRealName = (_sdkName: string) => Promise<VerifyRealNameInfo>;

/**
 * 通用 特殊调用 接口，其他调用
 */
export type TSpecialCall = (_sdkName: string, _methodName: string, _paramJson?: any[]) => Promise<any>;

/**
 * 通用 监听初始化完成
 */
export type TSetInitListener = (_sdkName: string) => Promise<any>;

// ========================= 下面接口 全部 废弃

/**
 * @deprecated
 * 
 * 阿里云  绑定标签
 * @link https://help.aliyun.com/document_detail/30066.html?spm=a2c4g.11186623.6.570.11ca1cd3umWHxr
 * @param target 目标类型，1：本设备； 2：本设备绑定账号； 3：别名
 * target(V2.3.5及以上版本) 目标类型，CloudPushService.DEVICE_TARGET：本设备； CloudPushService.ACCOUNT_TARGET：本账号； CloudPushService.ALIAS_TARGET：别名
 * @param tags 标签（数组输入）
 * @param alias (仅当target = 3时生效）
 */
export type Taliyun_bindTag = (target: number, tags: string[], alias: string) => Promise<any>;

/**
 * 阿里云  解绑定标签
 * @deprecated
 * 
 * @link https://help.aliyun.com/document_detail/30066.html?spm=a2c4g.11186623.6.570.11ca1cd3umWHxr
 * @param target 目标类型，1：本设备； 2：本设备绑定账号； 3：别名
 * target(V2.3.5及以上版本) 目标类型，CloudPushService.DEVICE_TARGET：本设备； CloudPushService.ACCOUNT_TARGET：本账号； CloudPushService.ALIAS_TARGET：别名
 * @param tags 标签（数组输入）
 * @param alias (仅当target = 3时生效）
 */
export type Taliyun_unbindTag = (target: number, tags: string[], alias: string) => Promise<any>;

/**
 * @deprecated
 *
 */
export type Taliyun_bindAccount = (account: string) => Promise<any>;

/**
 * @deprecated
 *
 */
export type Taliyun_unbindAccount = () => Promise<any>;

/**
 * @deprecated
 * 煜辰 - 实名认证
 * @returns 
 */
export type Tyc_idVerification = () => Promise<any>;

/**
 * @deprecated
 * 煜辰 - 初始化成功监听
 * 
 * @returns 
 */
export type Tyc_setInitListener = (cd: () => void) => void;
//#endregion

/**
 * @deprecated
 * 萌友 - 初始化成功监听
 * @returns 
 */
export type Tmy_setInitListener = (cd: () => void) => void;


//#region  动玩
/**
 * @deprecated
 * 初始化成功监听
 * @param cd 
 */
export type Tdw_setInitListener = (cd: () => void) => void;

/**
 * @deprecated
 * 获取用户防沉迷相关信息接口
 *  接口说明：*此方法一定要在登录游戏成功后调用
 * 在用户登录游戏成功后调用获取用户防沉迷相关信息接口，会有如下三种情况需要处理。
 * a）渠道sdk有自己的实名接口，游戏方只需要在needAntiIndulge(String uid, int age)回调方法中做防沉迷相关处理
 * b）渠道sdk没有自己的实名接口，游戏方需要在needRealNameAndAntiIndulge(String uid)回调方法中先进行实名认证再作防沉迷相关处理
 * c）渠道sdk有自己的实名和防沉迷功能，游戏方这不需要作处理，在normalGame(String uid)回调方法正常进行游戏，不需要做实名认证和防沉迷处理
 */
export type Tdw_getSqRealNameInfo = () => Promise<any>;

/**
 * @deprecated
 * 退出接口
 * 接口说明：
 *  在游戏退出前调用退出接口，会有如下不同处理：
 * a）	渠道sdk存在退出界面，游戏方只需在onChannelExit()回调中进行退出游戏逻辑即可，无需再弹退出界面。
 * b）	渠道sdk不存在退出界面，游戏方需在onGameExit()回调中弹出自己的游戏退出确认界面，然后再进行退出逻辑。
 * @returns 
 */
export type Tdw_exit = () => Promise<any>;

/**
 * @deprecated
 * 资源回收接口
 * 在游戏退出前，退出接口中调用
 * @returns 
 */
export type Tdw_applicationDestroy = () => Promise<any>;

/**
 * @deprecated
 * 获取包ID接口
 * 此接口用来获取当前包的唯一标识，可用于区分不同的包；
 * @returns 
 */
export type Tdw_getPackageId = () => Promise<any>;
//#endregion

/**
 * @deprecated
 */
export type Tjrtt_log_getHumeSDKChannel = () => Promise<any>;

/**
 * @deprecated
 */
export type Tjrtt_log_getHumeSDKVersion = () => Promise<any>;