/**
 * 界面：交互
 */
export * from "./dialog_impl"

/**
 * 图标
 */
export enum Icon {
    Success = "success",  // 显示成功图标，此时 title 文本最多显示 7 个汉字长度
    Loading = "loading",  // 显示加载图标，此时 title 文本最多显示 7 个汉字长度		
    None = "none",        // 不显示图标，此时 title 文本最多可显示两行
}

/**
 * 用户点击的按钮信息
 */
export interface ModalInfo {
    confirm: boolean;	//为 true 时，表示用户点击了确定按钮	
    cancel: boolean;    //为 true 时，表示用户点击了取消（用于 Android 系统区分点击蒙层关闭还是点击取消按钮关闭）
}

/**
 * 用户点击的按钮序号
 */
export interface ActionSheetInfo {
    tapIndex: number;	//用户点击的按钮序号，从上到下的顺序，从0开始
}



/**
 * 显示消息提示框
 * @param title		提示的内容
 * @param icon		图标
 * @param image		自定义图标的本地路径，image 的优先级高于 icon
 * @param duration	提示的延迟时间
 * @param mask		是否显示透明蒙层，防止触摸穿透
 * @note showLoading 和 showToast 同时只能显示一个
 * @note showToast 应与 hideToast 配对使用
 */
export type TshowToast = (title: string, icon?: Icon, image?: string, duration?: number, mask?: boolean) => Promise<void>


/**
 * 隐藏消息提示框
 */
export type ThideToast = () => Promise<void>;


/**
 * 显示 loading 提示框
 * @note 需主动调用 hideLoading 才能关闭提示框
 * @note showLoading 和 showToast 同时只能显示一个
 * @note showLoading 应与 hideLoading 配对使用
 * 
 * @param title	 提示的内容
 * @param mask	 是否显示透明蒙层，防止触摸穿透
 */
export type TshowLoading = (title: string, mask: boolean) => Promise<void>;



/**
 * 隐藏 loading 提示框
 */
export type ThideLoading = () => Promise<void>;



/**
 * 显示模态对话框
 * @param title		    提示的标题
 * @param content	    提示的内容
 * @param showCancel    是否显示取消按钮
 * @param cancelText    取消按钮的文字，最多 4 个字符
 * @param cancelColor	取消按钮的文字颜色，必须是 16 进制格式的颜色字符串
 * @param confirmText	确认按钮的文字，最多 4 个字符
 * @param confirmColor	确认按钮的文字颜色，必须是 16 进制格式的颜色字符串
 * @note Android 6.7.2 以下版本，点击取消或蒙层时，回调 fail, errMsg 为 "fail cancel"
 * @note Android 6.7.2 及以上版本 和 iOS 点击蒙层不会关闭模态弹窗，所以尽量避免使用「取消」分支中实现业务逻辑
 */
export type TshowModel = (title?: string, content?: string, showCancel?: boolean, cancelText?: string, cancelColor?: string, confirmText?: string, confirmColor?: string) => Promise<ModalInfo>;


/**
 * 显示操作菜单
 * @note Android 6.7.2 以下版本，点击取消或蒙层时，回调 fail, errMsg 为 "fail cancel"
 * @note Android 6.7.2 及以上版本 和 iOS 点击蒙层不会关闭模态弹窗，所以尽量避免使用「取消」分支中实现业务逻辑
 * 
 * @param itemList   按钮的文字数组,数组长度最大为 6
 * @param itemColor	 按钮的文字颜色
 */
export type TshowActionSheet = (itemList: string[], itemColor: string) => Promise<ActionSheetInfo>;

