
declare var HttpDownload;
declare var _$pi;
declare var FileSys;
declare var getMainWindow;
declare var VConsole;
declare var sysVersion;        // 构建工具会把pi_sys package.json中的版本注入到sysVersion字段
declare var wx;

/**
 * 项目配置
 */
interface IProject {
    name: string;                           // !!每个项目必须定义自己的名称，该名称决定了访问二级域名和本地存储名
    debug: boolean;                         // 是否开启debug  默认值:true
    catch: boolean;                         // 全局错误捕捉，记录次数后发送到服务器上 默认值:false
    biUpload: boolean;                      // 是否进行bi上报 默认值:true
    biConsole: boolean;                      // bi上报的时候是否打印,默认值:false
    giUpload: boolean;                      // 是否进行gi上报 默认值:true
    giConsole: boolean;                      // gi上报的时候是否打印,默认值:false
    vConsole: boolean;                      // 是否开启vConsole 默认值:true 仅在debug=true是有效,debug=false始终不开启
    imm_load_suffixs: string[];             // 需要定义时候立即加载的 模块后缀  默认值:["struct"]
    version_path: string;                   // 版本文件的路径  默认值:"game.version"
    version: JSON;                          // 程序自动填写 版本文件的JSON内容
    user_agent: any;                        // 程序自动填写
    h2: boolean;                             // 是否启用http2.0 默认false
    sysVersion: string;                      // pi_sys的版本号,由构建工具自动填写
    platform: {
        minigame?: IMinigamePackages;         // 小游戏环境需要配置
    },
    supportedPlatforms: string[];             // 支持的平台
    collect: boolean,    // 收集模式,此模式会按请求资源顺序将资源收集起来,以满足打包的时候因包体限制而导致只能打包部分资源,会根据收集的顺序优先打包优先级高的资源
    collect_ignore_suffixs: string[]  // 收集忽略后缀,一般指配置的后缀,因为我们预计会将配置全部打入底层包。如果不配置,默认使用cfg_suffixs
    batchLoadSizeLimit?: number;    // 一次批量下载大小限制
    updateMfa: string[],            // 更新方法,默认写死
    waitVideoDownloaded: boolean;  // 视频播放模式,是否等待video下载完成,默认true，等待下载完成,如果设置为false,边下边播,但是流量可能消耗两倍甚至更多
    useGameEnvServer?: boolean;   // 是否使用game_env.server中的配置,false表示根据运行环境自动拼接对应的domains和remotes
    minifyJs?: boolean;             // 是否是js压缩后的版本,判断是否是dst_release,由构建工具处理
    rewriteSystemInfo?: (r: any, res: WechatMinigame.SystemInfo) => void;    // 外部重写systemInfo
    netOption: NetOption;            // http请求超时时间
}


export interface NetOption {
    // 响应超时：在请求发出后,如果在该超时时间内没有任何数据响应,且连续noResponseCount次,则认为响应超时,默认2秒
    responseTimeout: number;
    // 响应超时最大值：响应超时的最大值,超过该值后,统一使用该值作为响应超时时间,默认6秒
    responseTimeoutMax: number;
    // 连续未响应次数：连续触发响应超时的次数,到达该值,则认为响应超时,默认2次
    noResponseCount: number;
    // 每秒下载字节数最低值,默认100K;3G网络理论下载速率大概在每秒300K
    bpsMin: number;
    // 连续下载数据不足次数： 每秒下载数据不足连续次数,等于该次数则认为该次请求大概率会请求超时,中断请求,默认3次
    insufficientBufferCount: number;
    // 超时增长因子：在请求重试时,会以该数值*上次requestTimeout作为当次的requestTimeout,默认1.6
    timeoutFactor: number;
    // 重试次数：请求失败重试次数,每次重试会增加超时时间,默认3次
    retryCount: number;
}


/**
 * 小游戏分包相关配置
 */
interface IMinigamePackages {
    MaxMainAllSize: number;                // 最大主存配置
    MaxMainCfgSize: number;                // 配置文件总内容大小 ， 需要小于 MaxMainAllSize ，单位(byte)
    subpackages: { name: string; root: string }[];       // 分包配置，与 game.json 内对应
}

/**
 * gui相关配置
 */
interface IGui {
    img_zoom: number;                                                 // ???
    img_zoom_options: number[];                                       // ???
    antialias: boolean;                                               // 是否抗锯齿    默认值:false
    design_size: { width: number; height: number };                   // 设计尺寸,通常设置为美术制作的最高分辨率 
    /**
     * gpu性能评估（有效值0~1, 通常都为1，除非发现一些设备渲染性能低，并愿意牺牲画质来提升性能时，该值小于1），
     * 如果是数组，头三项分别代表 低，中，高 三种配置的评估系数，如果对应的选项没有填，默认就是1  
     */
    gpu_performance: number | [number, number, number];                // 默认值:1
    /**
     * 当设备尺寸与设计尺寸宽高比存在差距时,为了使显示区域尽量填充屏幕，
     * 会在某一个方向扩展尺寸（这会导致显示尺寸和设计尺寸比例不等），
     * max_extend则表明最大可扩展的尺寸与该方向当前的尺寸的比率。
     * 如，当max_extend为0时，表示不扩展，此时显示宽高比等于设计宽高比。
     */
    max_extend: number; // 最大扩展尺寸比   默认值:0
    show_rect: { top: number; bottom: number; right: number; left: number };  // 显示区域,undefined表示设备的安全区域 默认值:undefined
    rotate_fit: boolean;     // 当设备尺寸与设计尺寸宽高比差距很大时，是否允许旋转   默认值:false
    /**
     * 设备支持的压缩纹理，如果是null，将默认检测设备的压缩纹理和构建出的纹理，在交集中选择一个合适的压缩纹理格式，
     * 否则会将设备支持的纹理和compressed_tex_formats取交集，在交集中选择一个合适的压缩纹理格式
     */
    compressed_tex_formats: string[];                                  // 默认值:null
    /**
     * 项目指定使用哪个等级的压缩纹理
     * 如果是null,将根据设备配置及构建出的纹理,选择一个合适的压缩纹理格式
     */
    compressed_tex_grade: string;                           // 默认值:null

    //=============gui-debug相关===============
    control_panel?: boolean;      // 移动端控制面板
    force_close_control_panel?: boolean;    // 强制关闭control_panel。debug模式下默认会打开控制面板,但是仙书项目用的还是老的机制,新版控制面板无法打开,这里额外为他们增加一个字段强制关闭
    gui_check_nan?: boolean;     // 检查gui样式是否存在nan
    gui_record?: boolean;        // gui录制
}

/**
 * Logger 日志
 */
interface ILogCfg {

    appenders: {
        name: string,   // 供下面过滤器使用的 名字
        kind: string,   // 底层通过注册的 Appender实现类，目前只有: "ConsoleKind", "HttpKind"
        args?: { // 目前：对 ConsoleKind来说，无参；对HttpKind 来说，参数类型定义看声明
            url: string,
            authorization: string,

            refresh_interval?: number, // 几秒 刷新一次
            refresh_max_size?: number, // 达到 多少字节 刷新一次
        }
    }[],

    meta: {
        platform: "client" | "server", // 客户端 还是 服务器端
        project: string,    // 项目名
        channel: string,    // 渠道名
        process_id?: string,   // 进程ID
    },

    root: {     // 默认 target
        level: number,        // 最大过滤等级, 5 verbose, 4 debug, 3 info, 2 warn, 1 error, 0 none
        appenders: string[],  // 输出的地方，可以填 "console", "http"
    },

    loggers: {
        // wasm 里面叫做："wasm_engine"
        target: string,       // 目标名，相当于其他日志库的 tag 或者 category
        level: number,        // 最大过滤等级, 5 verbose, 4 debug, 3 info, 2 warn, 1 error, 0 none
        appenders: string[],  // 输出的地方，可以填 "console", "http"
    }[]
}

/**
 * 加载资源相关配置
 */
interface ILoadRes {
    root_path: string;           // 文件根路径 默认值:"/"
    batch_path: string;         // 指定批处理协议的名字 默认值:"/batch?"
    code_suffixs: string[];     // 代码文件后缀  debug会默认设置为["js"]
    cfg_suffixs: string[];      // 配置文件后缀
    /**
     * 区分平台的配置文件后缀,在depend初始化的时候过滤掉不是本平台的信息
     * - 目前主要是为了实现css文件区分平台,以适应在不同平台使用不同的全局CSS,所以其默认值为["kcss","dcss"]
     */
    cfg_platform_suffixs: string[];

    /**
     * 以下语言相关字段为了实现中英文切换功能
     */
    lang_cfg_suffixs?: string[];     // 语言影响的配置文件后缀,默认值为["kcss","dcss"]
    current_lang?: string;          // 初始语言
    langs?: string[];            // 支持的语言


    obj_suffixs: string[];      // 资源对象后缀
    res_suffixs: string[];      // 资源后缀
    /**
     * 某类型资源未使用的缓存时间不超过该值 将用于 在所有类型资源的管理触发释放时 
     * 对该类型的资源释放的选择中对具体资源排序；
     * 设置为一个数字表示所有类型资源缓存时间相同,另可为各类型配置不同缓存时间;
     * 具体类型没有配置则使用默认值;
     * 资源类型为 RES_IMAGE、RES_FONT等 resTab.load 接口的第一个参数
     * (也是regist接口注册某种资源的加载时传递的第一个参数,项目自定义类型可以在调用regist注册时手动传入)
     * example:{ RES_IMAGE: 60 * 1000 } (RES_IMAGE类型的资源缓存时间)
     */
    res_timeout: number | JSON;  //  资源缓存超时时间；默认值:60  单位:秒
    /**
     * 某类型资源的最小容量，资源未使用的缓存大小不超过该值 不会触发释放(资源一直被缓存); 
     * 在所有类型资源的管理触发释放时 可能会触发该类型的释放；
     * 设置为一个数字表示所有类型资源缓存大小相同,另可为各类型配置不同缓存大小;
     * 具体类型没有配置则使用默认值;资源类型为 RES_IMAGE、RES_FONT、TEXTURE_RES等 resTab.load 接口的第一个参数
     * (也是regist接口注册某种资源的加载时传递的第一个参数,项目自定义类型可以在调用regist注册时手动传入)
     * example:{ RES_IMAGE: 60 * 1024 * 1024 } (RES_IMAGE类型的资源缓存大小)
     */
    res_cache_size: number | JSON;  // 资源缓存大小； 默认值:10 * 1024 * 1024 单位:M
    /**
     * 某类型资源的最大容量，未使用的缓存大小超过该值 在所有类型资源的管理触发释放时 一定会触发该类型的释放；
     * 设置为一个数字表示所有类型资源缓存大小相同,另可为各类型配置不同缓存大小;
     * 具体类型没有配置则使用默认值;
     * 资源类型为 RES_IMAGE、RES_FONT、TEXTURE_RES等 resTab.load 接口的第一个参数
     * (也是regist接口注册某种资源的加载时传递的第一个参数,项目自定义类型可以在调用regist注册时手动传入)
     * example:{ RES_IMAGE: 60 * 1024 * 1024 } (RES_IMAGE类型的资源缓存最大值)
     */
    res_cache_max_size: number | JSON; // 资源缓存最大值；默认值：10 * 1024 * 1024 单位:M
    res_cache_total_size: number;     // 所有类型资源 未使用的资源可缓存的总大小限制 - 超过该值将在各类型中进行比较选择性进行释放 默认值:60 * 1024 * 1024 单位:M
}

/**
 * 加载流程相关配置
 */
interface ILoadProcess {
    load: string[];            // 一般用于加载底层库，比如：pi_utils/pi_gui  默认值:["pi_util", "pi_gui"]
    exec: any[][];             // load部分加载完成后，执行的函数；每个库的初始化部分
    next_load: string[];        // 一般用于加载项目登录界面相关的目录，比如：app_a / pi_common
    next_exec: any[][];         // 项目启动的地方，一般是app_a的启动入口
    last_load: string[];        // 供点击登录后要加载的模块，一般是 pi_babylon 和 app_i / app_b / app_c
}


interface IProcessBar {
    image: HTMLImageElement;
    text_color: string;      // 进度条的文字颜色  仅web&webview适用
    bar_bgcolor: string;     // 进度条的背景颜色  仅web&webview适用
    bar_color: string;       // 进度条的颜色      仅web&webview适用
    canvas: string;          // ???                 仅小游戏适用
    bgurl: string;          // 加载界面背景图的项目路径  仅小游戏适用
    bgColor: [number, number, number];    // 进度条的背景颜色 仅小游戏适用 不配默认值 [0.2,0.2,0.2]
    pgColor: [number, number, number];    // 进度条的前景颜色 仅小游戏适用 不配默认值 [0,0.8,0]
}



/**
 * 加载进度条相关配置
 */
interface ILoadProcessBar {
    load_bar: IProcessBar;         // 第一次加载的进度条 默认值:简单样式
    load_text: string;                  // 第一次加载时候的提示文字  默认值:""
    next_load_bar: IProcessBar;     // 第二次次加载的进度条 默认值:简单样式
    next_load_text: string;             // 第一次加载时候的提示文字  默认值:""
}

/**
 * 服务器相关配置
 */
interface IServer {
    domains: string[];                 // 资源服务器地址;第一个域名一般为CDN域名，第二个域名一般为主域名。 一般下载尝试3次， 2次cdn， 1次主域名
    main: string;                       // ???
    remote: string;                     // ???
    remoteBase: string;                // 远端逻辑服地址(webSocket地址) --- 过时
    remotePort: string;                // websocket端口  --- 过时
    remotes: string | string[]          // ws全链接(带协议端口),如果是数组会使用链接速度最快的那个
    biUrl: string;                      // bi上报地址
    http_counts: number;                 // 单条http请求总次数,大于1请求失败会尝试重试,此字段用于pi_common中的http模块
    http_interval: number;               // http重试请求间隔,此字段用于pi_common中的http模块
    ptconfig?: {                        // 从ptconfig.json中提取出来的配置,由构建工具处理
        ip: string;
        mqttPort?: number;
        dstPort?: number;
        dstReleasePort?: number;
        dstPublishPort?: number;
        dstReleasePublishPort?: number;
    },
    multiVersionProduction?: boolean;        // 多版本production构建,由构建工具处理
}

/**
 * OS平台相关配置
 */
interface IOs {
    mobile_asset_path: string;    // 放到assets的资源目录是啥  默认值:"dst"
    exe: boolean;                  // 是否是window平台的exe 程序自动填写   ???
    platform_type: string;        // 程序自动填写 值是下面之一：web, minigame, webview, app_webview, android, ios, windows
    windows_asset_path: string;   // ???
}


interface IEnv extends IProject, IGui, ILoadRes, ILoadProcess, ILoadProcessBar, IServer, IOs {
    logger: ILogCfg,
    currentVersion?: number,  // 当前版本号
    resDirname?: string;     // 版本号下目录名
    encryptKey?: string;    // 加密密钥
    updateBg?: string;       // 大版本更新时的背景
    assetsToData: boolean;      // 是否把assets拷贝到data目录,默认应该拷贝,有时为了调试暴露出去
    [key: string]: any
}

class Winit {
    private readyCb: Function;
    private env: IEnv;
    private win: Object;
    private runBenchPromise: Promise<any>;
    private depend_data: any;
    private forceDebug: boolean;   // 是否强制debug模式  仅浏览器有效  url含debug关键字既强制开启debug模式
    private loggerMod: any;
    private mainMod: any;
    private constantMod: any;

    constructor(debug?: boolean) {
        this.forceDebug = debug;
        this.env = {
            //#region IProject
            name: undefined,
            app_id: undefined,
            debug: true,
            vConsole: true,
            catch: false,
            biUpload: true,
            biConsole: false,
            giUpload: true,
            giConsole: false,
            imm_load_suffixs: ["stuct"],
            version_path: "game.version",
            version: undefined,
            user_agent: navigator.userAgent,
            h2: false,
            sysVersion: typeof (<any>window).sysVersion === "undefined" ? "" : (<any>window).sysVersion,  // 如果变量未定义
            platform: {},
            supportedPlatforms: ["web", "android", "ios", "minigame", "win", "webview", "app_webview"],
            collect: false,
            collect_ignore_suffixs: null,
            batchLoadSizeLimit: undefined,
            updateMfa: ["pi_update/update_mgr", "init"],
            waitVideoDownloaded: true,
            use_old_engine: true,
            netOption: undefined,
            //#endregion

            //#region IGui
            img_zoom: undefined,
            img_zoom_options: undefined,
            antialias: false,
            design_size: undefined,
            gpu_performance: 1,
            max_extend: 0,
            show_rect: undefined,
            rotate_fit: false,
            compressed_tex_formats: null,
            compressed_tex_grade: null,
            //#endregion

            //#region ILoadRes
            root_path: "/",
            batch_path: "/batch?",
            code_suffixs: undefined,
            cfg_suffixs: undefined,
            cfg_platform_suffixs: ["kcss", "dcss"],
            lang_cfg_suffixs: ["kcss", "dcss"],
            obj_suffixs: undefined,
            res_suffixs: undefined,
            res_timeout: 60,
            res_cache_size: 10 * 1024 * 1024,
            res_cache_max_size: 10 * 1024 * 1024,
            res_cache_total_size: 60 * 1024 * 1024,
            //#endregion

            //#region ILoadProcess
            load: ["pi_util", "pi_gui"],
            exec: undefined,
            next_load: undefined,
            next_exec: undefined,
            last_load: undefined,
            //#endregion

            //#region ILoadProcessBar
            load_bar: {
                text_color: undefined,
                bar_bgcolor: undefined,
                bar_color: undefined,
                canvas: undefined,
                bgurl: undefined,
                image: undefined,
                bgColor: [0.2, 0.2, 0.2],
                pgColor: [0, 0.8, 0]
            },
            load_text: "",
            next_load_bar: {
                text_color: undefined,
                bar_bgcolor: undefined,
                bar_color: undefined,
                canvas: undefined,
                bgurl: undefined,
                image: undefined,
                bgColor: [0.2, 0.2, 0.2],
                pgColor: [0, 0.8, 0]
            },
            next_load_text: "",
            //#endregion

            //#region IServer
            domains: undefined,
            main: undefined,
            remote: undefined,
            remoteBase: undefined,
            remotePort: undefined,
            remotes: undefined,
            biUrl: undefined,
            http_counts: 1,
            http_interval: 0,
            //#endregion

            //#region IOs
            mobile_asset_path: "dst",
            exe: (<any>window).piAssetsPath !== undefined,
            platform_type: undefined,
            windows_asset_path: `${(<any>window).piAssetsPath}/dst`,
            //#endregion

            //#region ILogCfg
            logger: {
                appenders: [{
                    name: "console",
                    kind: "ConsoleKind",
                }],

                meta: {
                    platform: "client",
                    project: "",
                    channel: "",
                },

                root: {
                    level: 4, // debug
                    appenders: ["console"]
                },

                loggers: [],
            },
            //#endregion
            assetsToData: true
        }

        this.setPlatformType();
        if (this.env.platform_type === "minigame") {
            this.minigameInit();
        } else if (this.env.platform_type === "windows") {
            this.winInit();
        } else if (this.env.platform_type === "android") {
            this.winInit();
        } else if (this.env.platform_type === "app_webview") {
            this.appWebviewInit();
        } else {
            this.webInit();
        }
    }


    // 扩展项目配置
    setCfg(cfg: Object) {
        Object.assign(this.env, cfg);
        return this;
    }

    // 获取项目配置
    getCfg(key: string) {
        return this.env[key]
    }

    /**
     * 设置项目配置
     * @param cfg IProject
     * @returns 
     */
    setProject(cfg: IProject) {
        Object.assign(this.env, cfg);
        return this;
    }

    setLogger(cfg: ILogCfg) {
        Object.assign(this.env.logger, cfg);
    }

    /**
     * 设置gui相关配置
     * @param cfg IGui
     * @returns 
     */
    setGui(cfg: IGui) {
        Object.assign(this.env, cfg);
        return this;
    }

    /**
     * 设置加载资源相关配置
     * @param cfg ILoadRes
     * @returns 
     */
    setLoadRes(cfg: ILoadRes) {
        Object.assign(this.env, cfg);
        return this;
    }

    /**
     * 设置加载流程相关配置
     * @param cfg ILoadProcess
     * @returns 
     */
    setLoadProcess(cfg: ILoadProcess) {
        Object.assign(this.env, cfg);
        return this;
    }

    /**
     * 设置加载进度条相关配置
     * @param cfg ILoadProcessBar
     * @returns 
     */
    setLoadProcessBar(cfg: ILoadProcessBar) {
        Object.assign(this.env, cfg);
        return this;
    }

    /**
     * 设置服务器相关配置
     * @param cfg IServer
     * @returns 
     */
    setServer(cfg: IServer) {
        Object.assign(this.env, cfg);
        return this;
    }

    /**
     * 设置OS平台相关配置
     * @param cfg IOs
     * @returns 
     */
    setOs(cfg: IOs) {
        Object.assign(this.env, cfg);
        return this;
    }

    /**
     * 设置小游戏分包相关配置
     * @param cfg IMinigamePackages
     */
    setMinigamePackages(cfg: IMinigamePackages) {
        if (!this.env.platform.minigame) {
            this.env.platform.minigame = {} as any;
        }
        Object.assign(this.env.platform.minigame, cfg);
        return this;
    }



    /**
     * 设置回调,同时做一些初始化操作
     */
    set onReady(func) {
        this.readyCb = func;

        setTimeout(() => {
            this.readyCb && this.readyCb();
        }, 0);
    }

    getMod(modPath: string): Promise<any> {
        return new Promise(resolve => {
            _$pi.require(modPath, (mod) => {
                resolve(mod);
            });
        })
    }

    setCfgInfo(cfg: any) {
        this.env.cfg = cfg;
        // 下载资源的时候会拼接上这两个目录 currentVersion存在的时候resDirname一定存在
        this.env.currentVersion = cfg.currentVersion;
        this.env.originResDirname = cfg.originResDirname;      // 原始版本的目录名
        this.env.encryptResDirname = cfg.encryptResDirname;     // 加密版本的目录名
        this.env.resDirname = cfg.resDirname;                   // 当前使用的目录名
        this.env.encryptKey = cfg.encryptKey;
        this.env.projectCfg = cfg.projectCfg;
        if (this.env.platform_type === "webview") {     // ios webview始终使用未加密下载
            this.env.resDirname = cfg.originResDirname;
        }
        if (this.loggerMod) {
            this.loggerMod.addMeta("project_version", cfg.currentVersion);
        }
        if (cfg.server) {
            console.log("======= user new server ", cfg.server);
            this.setServer(cfg.server);
        }
        this.setServerConf();
    }

    webInit() {

        // 防止广告过滤
        this.win = {
            _$pi: 1
        };
        let k;
        for (k in window) {
            if (window.hasOwnProperty(k)) this.win[k] = 1;
        }

    }


    minigameInit() {

        // 因为小游戏的内存不够及时释放，所以需要加个这个
        // setInterval(() => {
        //     wx.triggerGC();
        // }, 1500);

        let adapter = "pi_sys/code_adapter/index";
        // pi_sys已经成功加载，适配模块仅依赖pi_sys自身的模块，因此这里可以使用同步方式require适配模块
        _$pi.require(adapter);

        this.commonInit();

    }



    winInit() {
        // 这里需要显示引入  否则window.canvase = undefined
        _$pi.require("pi_sys/code_adapter/index_init");

        globalThis.iDB = {};
        // 调用一次，注册窗口事件
        getMainWindow();

        // ZHUYI TODO
        (<any>globalThis).localStorage = {
            map: new Map(),

            setItem(key, val) {
                if (val === null) {
                    val = "null";
                } else if (val === undefined) {
                    val = "undefined";
                } else {
                    val = val.toString();
                }
                this.map.set(key, val);
            },
            getItem(key) {
                let v = this.map.get(key) || null;
                return v;
            },

            removeItem(key) {
                if (this.map.has(key)) {
                    this.map.delete(key);
                }
            }
        }

        this.commonInit();


    }


    commonInit() {
        const {
            HttpDownload
        } = _$pi.require("pi_sys/net/http");
        // TODO: 和 白鹏 讨论 原因，可替代方案
        (<any>window).HttpDownload = HttpDownload;
    }

    /**
     * 处理debug默认行为
     */
    debugDefault() {
        this.env.debug = this.forceDebug || this.env.debug;

        // debug模式  默认开启
        if (this.env.debug) {
            this.env.control_panel = true;  // 打开控制面板
            this.env.gui_check_nan = true;  // 打开gui nan检查
            if (this.env.force_close_control_panel) {
                this.env.control_panel = false;   // 仙书项目特有
            }
            // 日志配置检查
            if (this.env.logger) {
                if (this.env.logger.root) {
                    this.debugLoggerCheck(this.env.logger.root);
                }
                this.env.logger.loggers.forEach(logger => this.debugLoggerCheck(logger));
            }

        }

    }

    // debug模式下日志检查
    debugLoggerCheck(logger: { target?: string, level: number, appenders: string[] }) {
        if (logger.level > 4) {
            console.error(`Logger CFG Warn:Target ${logger.target || "root"} level is below 4!`);
        }
        if (!logger.appenders.includes("console")) {
            console.error(`Logger CFG Warn:Target ${logger.target || "root"} appenders not includes console`);
        }
    }

    /**
     * 开始游戏
     */
    start() {
        this.setNetOption();
        this.debugDefault();
        this.setServerConf();
        this.env.start_time = Date.now();
        if (this.env.platform_type === "web" || this.env.platform_type === "webview") {
            this.webStart();  // 此时还没有pi-sys
        } else {
            this.initSomeMod().then(() => {
                this.loggerMod.logStep(this.loggerMod.LogStepId.WINIT_START);
                this.loggerMod.flushLog();
                if (this.env.platform_type === "minigame") {
                    this.minigameStart();
                } else if (this.env.platform_type === "windows") {
                    this.winStart();
                } else if (this.env.platform_type === "android") {
                    this.winStart();
                    // TODO: this.androidStart();
                } else if (this.env.platform_type === "app_webview") {
                    this.appWebviewStart();
                }
            });
        }

    }

    setNetOption() {
        let netOption: NetOption = this.env.netOption || {} as any;
        if (netOption.responseTimeout === undefined) {
            netOption.responseTimeout = 2 * 1000;
        }

        if (netOption.responseTimeoutMax === undefined) {
            netOption.responseTimeoutMax = 6 * 1000;
        }

        if (netOption.noResponseCount === undefined) {
            netOption.noResponseCount = 2;
        }

        if (netOption.bpsMin === undefined) {
            netOption.bpsMin = 100 * 1024;
        }

        if (netOption.insufficientBufferCount === undefined) {
            netOption.insufficientBufferCount = 3;
        }

        if (netOption.timeoutFactor === undefined) {
            netOption.timeoutFactor = 1.6;
        }

        if (netOption.retryCount === undefined) {
            netOption.retryCount = 3;
        }
        this.env.netOption = netOption;
    }

    /**
     * 设置服务相关配置
     */
    setServerConf() {
        // 明确声明使用game_env.server配置
        if (this.env.useGameEnvServer) return;
        // 如果是多版本的正式环境,使用game_env.server的配置
        if (this.env.multiVersionProduction) return;
        let ptconfig = this.env.ptconfig;
        if (ptconfig) {
            let httpPort;
            if (this.env.currentVersion) {
                // 多版本
                httpPort = this.env.minifyJs ? ptconfig.dstReleasePublishPort : ptconfig.dstPublishPort;
            } else {
                // 单版本,只有web和小游戏可以跑单版本
                httpPort = this.env.minifyJs ? ptconfig.dstReleasePort : ptconfig.dstPort;
            }
            let ip = ptconfig.ip;
            let protocol = "http:";
            if (this.env.platform_type === "web") {
                protocol = window.location.protocol;
                ip = window.location.hostname;
                httpPort = window.location.port ? window.location.port : (protocol === "http:" ? 80 : 443);
            }
            if (httpPort) {
                // 覆盖domains
                let domain;
                if (protocol === "http:") {
                    domain = httpPort === 80 ? `http://${ip}` : `http://${ip}:${httpPort}`;
                } else {
                    domain = httpPort === 443 ? `https://${ip}` : `https://${ip}:${httpPort}`;
                }
                this.env.domains = [domain];
            }
            if (ptconfig.mqttPort) {
                // 在telegram小游戏里,为了方便本地调试,使用ws协议,telegram小游戏不遵守https的规则,即https协议下可以使用ws协议
                let inTelegram = (<any>window).Telegram || (<any>window).TelegramWebviewProxy;
                if (protocol === "http:" || inTelegram) {
                    this.env.remotes = [`ws://${ip}:${ptconfig.mqttPort}`];
                }
                else {
                    this.env.remotes = [`wss://${ip}:${ptconfig.mqttPort}`];
                }
            }
        }
    }

    initSomeMod() {
        return Promise.all([this.getMod("pi_sys/util/logger"), this.getMod("pi_sys/setup/main"), this.getMod("pi_sys/util/constant")]).then(async ([loggerMod, mainMod, constantMod]) => {
            this.loggerMod = loggerMod;
            this.mainMod = mainMod;
            this.constantMod = constantMod;
            loggerMod.initLogger(this.env.logger);
            let info = await mainMod.initLoggerMeta(this.env);
            this.env.info = info;
        })
    }



    webStart() {
        if ((<any>window).__cfg) {  // 多版本启动
            this.env.netCfg = (<any>window).__cfg;
            this.setCfgInfo((<any>window).__cfg);
        }
        this.runBenchPromise = this.testBenchLevel().catch((e) => {
            return e;
        });
        if (this.env.debug) {
            this.env.code_suffixs.push('js');
            if (this.env.vConsole) {
                let currentVersion = this.env.netCfg && this.env.netCfg.currentVersion;
                let resDirname = this.env.netCfg && this.env.netCfg.resDirname;
                let path = "";
                if (currentVersion) {
                    path = this.env.root_path + currentVersion + "/" + resDirname + "/" + "/pi_sys/setup/vConsole.js";
                } else {
                    path = this.env.root_path + "pi_sys/setup/vConsole.js";
                }
                this.loadJS(this.env.domains, path, () => {
                    (<any>window).vConsole = new VConsole();
                    const vConsoleDom = document.querySelector("#__vconsole");
                    if (vConsoleDom) {
                        // 阻止冒泡,否则会穿透到页面上
                        vConsoleDom.addEventListener("pointerdown", (e) => {
                            e.stopPropagation();
                        })
                    }
                }, this.loadFail);
            }
        } else {
            this.env.cfg_suffixs.push('js');
            // console.log = function () { };
        }

        /**
         * 作者: 子午线
         * 
         * 处理 刚打开页面，就将APP切到后台，然后回到前台的时候，没办法执行下去；
         * 比如:同一个链接同时被打开多次,如果和服务端连接依赖本地localstorage存储信息的话,会导致连接断开
         * 
         * document.hidden 的意思是 页面不可见；
         */
        if (document.hidden === true) {
            setTimeout(() => {
                var div = document.createElement('div');
                div.onclick = () => {
                    div.remove();
                    this.webStartGame();
                };
                div.setAttribute('style', 'position:absolute;width: 100%;height: 100%;');
                div.innerHTML =
                    '<div style="position:absolute;width: 100%;top:50%;text-align:center;color:white;">Page paused, press to continue</div>';
                document.body.appendChild(div);
            }, 1);
        } else {
            this.webStartGame();
        }

    }

    minigameStart() {
        if (this.env.debug) {
            wx.setEnableDebug({
                enableDebug: true,
            });
        } else {
            // console.log = function () { };
            // console.warn = function () { };
            // wx.setPreferredFramesPerSecond(30); // 华为快游戏30帧会闪屏
        }
        this.minigameStartGame();
    }

    winStart() {
        if (this.env.debug) {
            if (this.env.vConsole) {
                try {
                    // globalThis.require("pi_sys/setup/vConsole.js");
                    (<any>window).vConsole = new VConsole();
                } catch (e) {

                }
            }
        }
        // this.env.code_suffixs.push('js');
        this.env.cfg_suffixs.push('js');
        this.winStartGame();
    }

    appWebviewStart() {
        if (this.env.debug) {
            this.env.code_suffixs.push('js');
            if (this.env.vConsole) {
                this.loadJS(this.env.domains, this.env.root_path + "pi_sys/setup/vConsole.js", () => {
                    (<any>window).vConsole = new VConsole();
                }, this.loadFail);
            }
        } else {
            this.env.cfg_suffixs.push('js');
            // console.log = function () { };
        }
        this.appWebviewStartGame();
    }

    /**
     * 
     * 获取游戏入口
     * @returns 
     * @note 这段代码从pi_init.js中拷贝过来的,新增了app_webview平台,pi_init.js中可以不用改,认为是webview平台即可
     */
    setPlatformType() {
        let type = "web";
        let agent = "";
        try {
            agent = navigator.userAgent || "";
        } catch (e) {
            agent = "";
        }
        if ((<any>window).wx) {
            agent = agent + ";PI_WX_GAME";
        } else if ((<any>window).qg) {
            agent = agent + ";PI_WX_GAME";    // qg是华为快游戏全局对象
        } else if ((<any>window).qq) {
            agent = agent + ";PI_WX_GAME";    // qq是qq小游戏全局对象
        }
        console.log("========== agent", agent);
        agent = agent.toLowerCase();
        if (agent === "") {
            // 一定是小游戏平台
            type = "minigame";
        } else if (agent.indexOf("pi_appcenter") >= 0) {
            type = "app_webview";    // agent = "...PI_WEBVIEW;PI_APPCENTER;PI_ANDROID|PI_WIDNOW|PI_IOS..."
            if (agent.indexOf("pi_android") >= 0) {
                this.env.platform_type2 = "android";   // 提供给app_webview 的file_system进一步判断android还是exe
            }
            if (agent.indexOf("pi_windows") >= 0) {
                this.env.platform_type2 = "windows";
            }
        } else if (agent.indexOf("pi_webview") >= 0) {
            type = "webview";        // agent = "...PI_WEBVIEW;PI_ANDROID|PI_WIDNOW|PI_IOS..."
        } else if (agent.indexOf("pi_android") >= 0) {
            type = "android";        // agent = "...PI_ANDROID;PI_NATVIE;..."
        } else if (agent.indexOf("pi_ios") >= 0) {
            type = "ios";            // agent = "...PI_IOS;PI_NATVIE;..."
        } else if (agent.indexOf("pi_windows") >= 0) {
            type = "windows";        // agent = "...PI_WINDOWS;PI_NATVIE;..."
        } else if (agent.indexOf("wechatdevtools") >= 0) {
            type = "minigame";       // game.js -- agent = "...PI_WX_GAME;..."
        } else if (agent.indexOf("qqdevtools") >= 0) {
            type = "minigame";       // game.js -- agent = "...PI_WX_GAME;..."
        } else if (agent.indexOf("pi_wx_game") >= 0) {
            type = "minigame";       // game.js -- agent = "...PI_WX_GAME;..."
        }
        this.env.platform_type = type;
        console.log("========= platform_type ", type);
        return type;
    }


    // 非小游戏平台会用这个接口
    // 在 html 中会调用
    testBenchLevel() {
        return new Promise((resolve, reject) => {
            let path = this.env.currentVersion ? `/${this.env.currentVersion}/${this.env.resDirname}/boot/worker.js` : "/boot/worker.js";
            let worker = new Worker(path);
            if (!worker) {
                reject({
                    cpu: {
                        count: -1,
                        time: -1,
                    },
                    memory: {
                        count: -1,
                        time: -1,
                    }
                });
                return;
            }

            worker.postMessage("run");

            worker.onmessage = function (e) {
                worker.terminate();
                resolve(e.data);
            }

            worker.onerror = function () {
                reject({
                    cpu: {
                        count: -1,
                        time: -1,
                    },
                    memory: {
                        count: -1,
                        time: -1,
                    }
                });
            }
        });
    }

    // web平台使用，目的：
    // TODO：目前不要调用，因为调用会清掉 全局变量，比如 pi_sys的 APK-Native 回调 函数；
    clear() {
        //清除window上新增的对象
        let k;
        for (k in window) {
            if (window.hasOwnProperty(k) && !this.win[k]) window[k] = null;
        }
        //清除body里面的非pi元素（自己添加的元素都有pi属性）
        let i,
            arr = document.body.children;
        for (i = arr.length - 1; i >= 0; i--) {
            k = arr[i];
            if (!k.getAttribute('pi')) document.body.removeChild(k);
        }
        window["winit"] = undefined;
    }

    loadJS(roots: string[], url: string, callback: Function, errorCallback: Function, errText?: string, i?: number) {
        let u, n, head;
        if (i >= roots.length) {
            return errorCallback && errorCallback(roots[0] + url, errText);
        }
        head = document.head || document.getElementsByTagName('head')[0] || document.documentElement;
        n = document.createElement('script');
        n.charset = 'utf8';
        n.onerror = () => {
            n.onload = n.onerror = undefined;
            head.removeChild(n);
            this.loadJS(roots, url, callback, errorCallback, errText, i === undefined ? 0 : i + 1);
        };
        n.onload = () => {
            n.onload = n.onerror = undefined;
            head.removeChild(n);
            callback && callback(roots[i || 0] + url);
        };
        n.async = true;
        // n.crossOrigin = 'anonymous';
        n.src = roots[i || 0] + url;
        head.appendChild(n);
    }

    loadJson(roots: string[], url: string, timeout: number, callback: Function, errorCallback: Function, errText?: string, i?: number) {
        if (i >= roots.length) {
            return errorCallback && errorCallback(roots[0] + url, errText);
        }
        let xhr = new XMLHttpRequest();
        xhr.withCredentials = false;
        xhr.onabort = function () {
            if (!xhr) return;
            timeout && clearTimeout((<any>xhr).timerRef);
            errorCallback && errorCallback(url, 'abort');
        };
        (<any>xhr).activeTime = Date.now();
        let timer = () => {
            let t = (<any>xhr).activeTime + timeout - Date.now();
            if (t > 0) {
                (<any>xhr).timerRef = setTimeout(timer, t);
            } else {
                // 如处理过就不用管了，否则就设置为处理过了。
                // 必须设置，否则，就会调用两次errorCallback
                if (!xhr) return;
                xhr = undefined;
                timeout = undefined;
                this.loadJson(roots, url, timeout, callback, errorCallback, 'timeout', i ===
                    undefined ? 0 : i + 1);
            }
        };

        (<any>xhr).timerRef = setTimeout(timer, timeout);
        xhr.onerror = (ev) => {
            console.log("loadJson error ", roots[i || 0] + url);
            // 避免定时器超时后重复调用
            if (!xhr) return;
            timeout && clearTimeout((<any>xhr).timerRef);
            this.loadJson(roots, url, timeout, callback, errorCallback, 'error status: ' + xhr
                .status + ' ' + xhr.statusText, i === undefined ? 0 : i + 1);
        };
        xhr.onload = (ev) => {
            // 避免定时器超时后重复调用
            if (!xhr) return;
            timeout && clearTimeout((<any>xhr).timerRef);
            if (xhr.status === 300 || xhr.status === 301 || xhr.status === 302 || xhr.status ===
                303) {
                this.loadJson(roots, xhr.getResponseHeader('Location'), timeout, callback,
                    errorCallback);
            } else if (xhr.status !== 0 && xhr.status !== 200 && xhr.status !== 304) {
                // iOS的file协议，成功的状态码是0
                this.loadJson(roots, url, timeout, callback, errorCallback, 'error status: ' + xhr
                    .status + ' ' + xhr.statusText, i === undefined ? 0 : i + 1);
            } else {
                try {
                    callback(JSON.parse(xhr.responseText));
                } catch (e) {
                    this.loadJson(roots, url, timeout, callback, errorCallback, 'error json: ' +
                        xhr.responseText, i === undefined ? 0 : i + 1);
                }
            }
        };
        console.log("loadJson url = ", roots[i || 0] + url);
        xhr.open('GET', roots[i || 0] + url, true);
        xhr.send();
    }

    simpleLoadJson(url, timeout, callback, errorCallback, sendEtag) {
        let xhr = new XMLHttpRequest();
        xhr.withCredentials = false;
        let cfgContent;
        if (sendEtag) {
            cfgContent = localStorage.getItem("cfgContent");
        }


        xhr.onabort = function () {
            if (!xhr) return;
            timeout && clearTimeout((<any>xhr).timerRef);
            errorCallback && errorCallback(url, 'abort');
        };
        (<any>xhr).activeTime = Date.now();
        let timer = () => {
            let t = (<any>xhr).activeTime + timeout - Date.now();
            if (t > 0) {
                (<any>xhr).timerRef = setTimeout(timer, t);
            } else {
                // 如处理过就不用管了，否则就设置为处理过了。
                // 必须设置，否则，就会调用两次errorCallback
                if (!xhr) return;
                xhr = undefined;
                timeout = undefined;
                errorCallback && errorCallback(url, 'timeout');
            }
        };

        (<any>xhr).timerRef = setTimeout(timer, timeout);
        xhr.onerror = (ev) => {
            // 避免定时器超时后重复调用
            if (!xhr) return;
            timeout && clearTimeout((<any>xhr).timerRef);
            errorCallback && errorCallback(url, ev, xhr.status);
        };
        xhr.onload = (ev) => {
            // 避免定时器超时后重复调用
            if (!xhr) return;
            timeout && clearTimeout((<any>xhr).timerRef);
            if (xhr.status === 300 || xhr.status === 301 || xhr.status === 302 || xhr.status ===
                303) {
                this.simpleLoadJson(xhr.getResponseHeader('Location'), timeout, callback, errorCallback, sendEtag);
            } else if (xhr.status !== 0 && xhr.status !== 200 && xhr.status !== 304) {
                // iOS的file协议，成功的状态码是0
                errorCallback && errorCallback(url, "error stauts " + xhr.status);
            } else {
                try {
                    // webview和app_webview记录etag,主要为了获取cfg的时候使用etag进行协商缓存
                    // 如果是V8,底层做了类似处理
                    if (sendEtag && (this.env.platform_type === "webview" || this.env.platform_type === "app_webview")) {
                        let etag = xhr.getResponseHeader('ETag');
                        if (etag) {
                            localStorage.setItem("cfgEtag", etag);
                        }
                    }
                    let responseText;
                    if (xhr.status === 304) {
                        responseText = cfgContent;
                    } else {
                        responseText = xhr.responseText;
                        if (sendEtag) {
                            localStorage.setItem("cfgContent", responseText);
                        }
                    }
                    callback(JSON.parse(responseText));
                } catch (e) {
                    errorCallback && errorCallback(url, e);
                }
            }
        };

        xhr.open('GET', url, true);
        if (xhr.setRequestHeader) {
            // 设置请求头信息，禁用强制缓存,启用协商缓存
            xhr.setRequestHeader('Cache-Control', 'no-cache');
            let etag = localStorage.getItem("cfgEtag");
            if (sendEtag && etag && cfgContent) {
                xhr.setRequestHeader('If-None-Match', etag);
            }
        }

        xhr.send();
    }

    minigameLoadJson(roots: string[], url: string, timeout: number, callback: Function, errorCallback: Function, errText?: string, i?: number) {
        if (i >= roots.length) {
            return errorCallback && errorCallback(roots[0] + url, errText);
        }
        const down = new HttpDownload(roots, url, timeout);
        down.start()
            .then((data) => {
                const {
                    utf8Decode
                } = _$pi.require("pi_sys/util/utf8");
                const str = utf8Decode(new Uint8Array(data));
                callback(JSON.parse(str));
            }).catch(e => {
                return errorCallback && errorCallback(roots[0] + url, errText);
            });
    }

    loadFail(url: string, err: string) {
        console.error(err + ', ' + url);
    }


    webStartGame() {
        let times = [Date.now(), 0, 0];
        // 加载版本json成功的处理方法
        let versionHandle = (version) => {
            // 表示加载版本成功，可以做成渐变过去的效果
            document.body.style.backgroundColor = "#2F2F2F";
            this.env.version = version;
            // 同时加载sys和.depend
            let sys, depend;
            let sys_path = version.sys[this.env.platform_type];

            // 注意：这里一定要先清除运营商代码，才能开始加载底层代码，否则，有可能会 清除掉 底层注入的全局回调函数；
            if (!this.env.debug) {
                // TODO：会引起 native注入的全局变量消失；
                // winit.clear();
            }

            if (this.env.cfg) { // 多版本,pisys和depend都已经准备好了
                depend = this.env.cfg[version.depend.split("?")[0]];
                this.webNext(depend, times);
            } else {
                // 下载pisys
                this.loadJS(this.env.domains, this.env.root_path + sys_path, () => {
                    sys = true;
                    times[1] = Date.now() - times[0];
                    depend && this.webNext(depend, times);
                }, (url: string, err: string) => {
                    this.loadFail(url, err);
                });

                // 下载game.depend
                this.loadJson(this.env.domains, this.env.root_path + version.depend, 10 * 1000,
                    (json) => {
                        depend = json;
                        times[2] = Date.now() - times[0];
                        sys && this.webNext(depend, times);
                    }, (url: string, err: string) => {
                        this.loadFail(url, err);
                    });
            }

        }

        if (this.env.cfg) {  // 多版本,game.version已经在cfg中
            versionHandle(this.env.cfg[this.env.version_path]);
        } else {
            // 从主站上加载 game.version
            let path = `${this.env.root_path}${this.env.version_path}?${Math.random()}`;

            this.loadJson(this.env.domains, path, 8 * 1000, versionHandle, (url: string, err: string) => {
                this.loadFail(url, err);
            });
        }
    }

    webNext(depend: any, times: number[]) {
        Promise.all([this.runBenchPromise, this.initSomeMod()]).then(([testResult, _]) => {
            this.env["benchLevelTest"] = testResult;

            times[0] -= this.env.start_time;
            console.log("ver, sys, depend times: ", times);
            this.loggerMod.addMeta("project_version", this.env.currentVersion);
            // 整个加载流程入口，包括：屏幕适配，加载初始文件，等工作；
            this.mainMod.main(this.env, depend);
        })
    }



    minigameStartGame() {
        var times = [Date.now(), 0, 0];
        let startTime = Date.now();
        this.loggerMod.logStep(this.loggerMod.LogStepId.CFG_DOWNLOAD);
        this.loggerMod.flushLog();
        if ((<any>wx).__cfgUrls) {  // 多版本,cfg中已经有depend相关信息
            const loadCfg = () => {
                let cfgUrl = (<any>wx).__cfgUrls.shift();
                if (!cfgUrl) {
                    this.mainMod.showRestartModel(this.constantMod.ErrorCode.ALL_CFG_FAILED, "All cfg url try!But no success!");
                    return;
                }
                this.minigameLoadJson([cfgUrl], "", 10 * 1000, (cfg) => {
                    if (cfg.deleted) {
                        return loadCfg();
                    }
                    this.env.netCfg = cfg;
                    this.cfgHandle(cfg, times);
                }, (url, error) => {
                    return loadCfg();
                });
            }
            loadCfg();
        } else { // 开发版本,走旧流程
            /**
             * 永远从网上加载 最新 的 game.version
             */
            this.minigameLoadJson(this.env.domains, this.env.root_path + this.env.version_path + "?" + Math.random(), 8 * 1000, (version) => {
                this.env.version = version;
                // 如果 depend 的版本 不同，才会去网上下 game.depend
                this.minigameLoadJson(this.env.domains, this.env.root_path + version.depend + "?" + Math.random(), 10 * 1000, (json) => {
                    let depend = json;
                    times[2] = Date.now() - times[0];

                    this.minigameNext(depend, times);

                }, (url: string, err: string) => {
                    const message = `minigameLoadJson  GAME_DEPEND_FAILED failed,url = ${url},err = ${err}`;
                    this.mainMod.showRestartModel(this.constantMod.ErrorCode.GAME_DEPEND_FAILED, message);
                });

            }, (url: string, err: string) => {
                const message = `minigameLoadJson GAME_VERSION_FAILED failed,url = ${url},err = ${err}`;
                this.mainMod.showRestartModel(this.constantMod.ErrorCode.GAME_VERSION_FAILED, message);
            });
        }

        Promise.all([this.getMod("pi_sys/load/package_load"), this.getMod("pi_sys/util/env")]).then(([r1, r2]) => {
            const PackageLoad = r1.PackageLoad;
            const packageLoad = new PackageLoad;
            const Env = r2.Env;
            Env.set("platform", this.env.platform); // PackageLoad需要,提交配置
            const load = this.env.load;
            for (let s of load) {
                if (s.charAt(s.length - 1) === '/') {
                    packageLoad.addPackage(s);
                }
            }

            let p = packageLoad.start().then(r => {
                const loadResult = Env.get("load_result");
                console.log("============ loadResult ", Date.now() - startTime);
                if (!loadResult) {  // 此时其他下载还未完成,下载next_load
                    const nextLoad = this.env.next_load;
                    const nextPackageLoad = new PackageLoad;
                    for (let s of nextLoad) {
                        if (s.charAt(s.length - 1) === '/') {
                            nextPackageLoad.addPackage(s);
                        }
                    }
                    if (nextPackageLoad.packages.size > 0) {
                        let startTime = Date.now()
                        let p = nextPackageLoad.start().then(r => {
                            console.log("=========== loadNext ", Date.now() - startTime);
                            return r;
                        });
                        Env.set("next_load_packageLoad", nextPackageLoad);
                        Env.set("next_load_packageLoad_promise", p);
                    }
                }
                return r;
            });
            Env.set("load_packageLoad", packageLoad);
            Env.set("load_packageLoad_promise", p);
        })


    }

    minigameNext(depend: any, times: number[]) {
        times[0] -= this.env.start_time;
        console.log("ver, sys, depend times: ", times);
        this.depend_data = depend;
        this.sysSetupMain();
    };


    winStartGame() {
        this.v8StartGame();
    }

    // v8通用启动  app_webview android win
    v8StartGame() {
        this.getMod("pi_sys/update/version").then(versionMod => {
            this.loggerMod.logStep(this.loggerMod.LogStepId.CFG_DOWNLOAD);
            this.loggerMod.flushLog();
            var times = [Date.now(), 0, 0];
            let assetsCfg = (<any>window).__assetsCfg;
            let dataCfg = (<any>window).__dataCfg;
            this.env.assetsCfg = assetsCfg;
            this.env.dataCfg = dataCfg;
            let execFunc = false;
            const l = versionMod.compareVersion(dataCfg && dataCfg.apk && dataCfg.apk.version, assetsCfg && assetsCfg.apk && assetsCfg.apk.version);
            let cfg = assetsCfg;
            if (dataCfg && l !== -1) {
                cfg = dataCfg;
            }

            let func = (netCfg?: Object, channel?: string) => {
                if (execFunc) return;
                channel = channel || "RUST CFG SUCCESS CALLBACK";
                (<any>window).__netCfg = netCfg;
                execFunc = true;
                let logger = new this.loggerMod.Logger("winit", "sys");
                logger.info("handle cfg with ", channel);
                // 如果netCfg不存在,则认为是无网进入
                if (netCfg) {
                    this.env.netCfg = netCfg;
                    cfg = netCfg;
                }
                (<any>window).__netCfgCallback = null;
                this.cfgHandle(cfg, times);
            }
            if ((<any>window).__netCfg) { // 底层已经注入远端cfg
                func((<any>window).__netCfg, "RUST CFG SUCCESS");
            } else {
                (<any>window).__netCfgCallback = func;
                const cfgUrls = [cfg.app_url, cfg.base_url + "?" + Math.random()];
                const loadCfg = (sendEtag: boolean) => {
                    let cfgUrl = cfgUrls.shift();
                    if (!cfgUrl) {
                        if ((<any>window).__netCfgCallback) {
                            (<any>window).__netCfgCallback(null, "JS CFG FAIL");
                        }
                        return;
                    }
                    this.simpleLoadJson(cfgUrl, 10 * 1000, (cfg) => {
                        if (cfg.deleted) {
                            return loadCfg(false);
                        }
                        func(cfg[0], "JS CFG SUCCESS " + cfgUrl);
                    }, (url, error) => {
                        return loadCfg(false);
                    }, sendEtag);
                }
                loadCfg(true);
            }


        })
    }

    cfgHandle(cfg, times) {
        let now = Date.now();
        let cost = now - times[0];
        this.setCfgInfo(cfg);
        this.loggerMod.logStep(this.loggerMod.LogStepId.CFG_DOWNLOAD_SUCCESS, cost);
        this.loggerMod.flushLog();

        let gameVersion = cfg[this.env.version_path];
        if (gameVersion) {
            this.env.version = gameVersion;
            let path = gameVersion.depend;
            // 路径得值是："path?sign"
            let len = path.indexOf("?");
            if (len >= 0) {
                path = path.slice(0, len);
            }
            let depend = cfg[path];
            if (depend) {
                this.minigameNext(depend, times);
            } else {
                const message = `game depend is null,path = ${path}`;
                this.mainMod.showRestartModel(this.constantMod.ErrorCode.GAME_DEPEND_NULL, message);
            }
        } else {
            const message = `game version is null,path = ${this.env.version_path}`;
            this.mainMod.showRestartModel(this.constantMod.ErrorCode.GAME_VERSION_NULL, message);
        }
    }

    sysSetupMain() {
        if (this.depend_data) {
            // 启动 pi-sys 加载流程
            this.mainMod.main(this.env, this.depend_data);
        }
    }

    appWebviewInit() {

    }


    appWebviewStartGame() {
        this.v8StartGame();
    }

}

// TODO: 和 小燕 协商，看下能不能放到 gui-config 去。 
window["__isNew"] = true;

(function () {
    let debug;
    if (window.location && window.location.search && window.location.search.includes("debug")) {
        debug = true;
    }
    if (debug) {
        Object.freeze(console);
    }
    window["winit"] = new Winit(debug);
})();



