/**
 * 此模块维护一个内部使用的store,并为外部提供一些服务
 */

import { BenchLevel } from "pi_sys/native/base/system_info";
import { Store } from "./store"
import { IPiStoreBase } from "./store_base"

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");


let innerStore: IPiStoreBase;
const userConfigKey = "__userConfig";
/**
 * 初始化内部store
 * @param storeName 
 * @returns 
 */
export const initDb = (storeName: string) => {
    return Store.createStore(`${storeName}_inner`, "_db").then(store => innerStore = store);
}

/**
 * 用户配置
 */
export interface IUserConfig {
    engine?: {                             // 引擎配置
        benchLevel: BenchLevel;             // 性能指标 
    },
    project?: {                  // 项目配置
        [key: string]: any
    }
}

/**
 * 设置用户配置(包括引擎需要的配置和项目自己的)
 * pi_sys初始化的时候会取得引擎配置覆盖默认配置
 * 并将所有配置设置到Env中供项目使用
 * @param cfg 
 */
export const setUserConfig = (cfg: IUserConfig) => {
    innerStore.write(userConfigKey, cfg);
}

/**
 * 获取用户配置
 * @returns 
 */
export const getUserConfig = (): Promise<IUserConfig> => {
    return innerStore.read(userConfigKey).catch(err => {
        logger.warn("getUserConfig error ", err);
    });
}