export * from "./file_system_impl"

export type Taccess = (path: string) => Promise<boolean>;

export type TappendFile = (path: string, data: string | ArrayBuffer) => Promise<null>;

export type TcopyFile = (srcPath: string, destPath: string) => Promise<null>;

export type Tmkdir = (path: string) => Promise<null>;

export type TreadFile = (path: string, encoding?: "utf8" | "binary", offset?: number, size?: number) => Promise<string | ArrayBuffer>;

export type TreadAssets = (path: string) => Promise<ArrayBuffer>;

export type Treaddir = (path: string) => Promise<string[]>;

export type Trename = (oldPath: string, newPath: string) => Promise<null>;

export type Trmdir = (path: string) => Promise<null>;

export interface IStats {
    size: number;     // 文件大小，单位：B
    lastAccessedTime: number;   // 文件最近一次被存取或被执行的时间
    lastModifiedTime: number;   // 文件最后一次被修改的时间
    isDirectory: () => boolean;   // 判断当前文件是否一个目录
    isFile: () => boolean;           // 判断当前文件是否一个普通文件
}

export type Tstat = (path: string) => Promise<IStats>;

export type Tunlink = (path: string) => Promise<null>;

export type TwriteFile = (path: string, data: string | ArrayBuffer) => Promise<null>;


/**
 * 对文件内容进行截断操作
 * 截断位置，默认0
 */
export type Tftruncate = (path: string, length: number) => Promise<null>;

