import { CompressAlgorithm, CompressLevel, EncryptType } from "pi_sys/util/constant";

export interface IPiStoreBase {
    readonly dbName: string;
    readonly tabName: string;
    tab?: any;
    map?: Map<string, any>;
    immediateWrite: boolean;  // 是否立即写入
    /**
     * 读取数据
     * 解密功能只在exe & android平台上生效,并且是File模式,Data模式不生效
     */
    read: (key: string, encryptType?: EncryptType, encryptKey?: string, compressAlgorithm?: CompressAlgorithm) => Promise<any>;
    /**
     * 删除数据
     */
    delete: (key: string) => Promise<any>;
    /**
     * 写入数据，如果键名存在则替换
     * 加密功能只在exe & android平台上生效,并且是File模式,Data模式不生效
     */
    write: (key: string, data: any, encryptType?: EncryptType, encryptKey?: string, compressAlgorithm?: CompressAlgorithm, compressLevel?: CompressLevel, immediate?: boolean) => Promise<any>;
    /**
     * 清除存储
     */
    clear: () => Promise<any>;
    /**
     * @description 迭代, callback返回false表示停止迭代
     * @note 迭代本身不做解密操作,需要高层自己处理
     * @example
     */
    iterate: (callback: (result: { key: any, value: any }) => boolean, errorCallback: (err: any) => void) => void;
}

export enum StoreType {
    File = 1,
    Data = 0,
}


export interface IPiStore extends IPiStoreBase {
    new(): IPiStoreBase;
    /**
     * 删除指定的数据库
     */
    deleteStore: (dbName: string) => Promise<any>;
    /**
     * 创建指定名称的储存
     * storeType = StoreType.File要求存储数据必须是Uint8Array
     */
    createStore: (dbName: string, tabName: string | "_db", storeType: StoreType) => Promise<IPiStore>;
}

/* class decorator */
export function staticPiStoreImplements<T>() {
    return <U extends T>(constructor: U) => {
        return { constructor };
    };
}