export namespace BaseTools {
    export function fileDot(file: string) {
        for (let i = file.length - 1; i >= 0; i--) {
            let c = file.charCodeAt(i);
            if (c === 47) {
                return -1;
            }
            if (c === 46) {
                return i;
            }
        }
        return -1;
    }
    /**
     * 文件后缀
     * @param file 
     */
    export function fileSuffix(file: string) {
        let dot = fileDot(file);
        return (dot >= 0) ? file.slice(dot + 1) : '';
    }

    /**
     * 文件伪后缀
     * @param file 
     */
    export function filePseudoSuffix(file: string): string {
        let dot = file.indexOf(".");
        return (dot >= 0) ? file.slice(dot + 1) : "";
    }



    /**
     * 文件名 - 带后缀
     * @param file 
     */
    export function fileName(file: string) {
        let i = file.lastIndexOf("/");
        return (i < 0) ? file : file.slice(i + 1);
    }
    /**
     * 获取文件的去掉第一个后缀的没有路径的文件名。
     * @param file 
     */
    export function fileBasename(file: string) {
        let i = file.lastIndexOf('/'),
            j = file.lastIndexOf('.');
        return j > i ? file.slice(i + 1, j) : file.slice(i + 1);
    }

    /**
     * 文件上级目录
     * @param fille 
     */
    export function fileDirname(file: string) {
        let i = file.lastIndexOf('/');
        return i >= 0 ? file.slice(0, i) : "";
    }

    export function relativePath(filePath: string, dir: string) {
        if (filePath.charCodeAt(0) !== 46) {
            return filePath;
        }
        let i = 0;
        let len = filePath.length;
        let j = dir.length - 1;
        if (dir.charCodeAt(j) !== 47) {
            j = dir.lastIndexOf("/");
        }
        while (i < len) {
            if (filePath.charCodeAt(i) !== 46) {
                break;
            }
            if (filePath.charCodeAt(i + 1) === 47) { // ./的情况
                i += 2;
                break;
            }
            if (filePath.charCodeAt(i + 1) !== 46 || filePath.charCodeAt(i + 2) !== 47) {
                break;
            }
            // ../的情况
            i += 3;
            j = dir.lastIndexOf("/", j - 1);
        }
        if (i > 0) {
            filePath = filePath.slice(i);
        }
        if (j < 0) {
            return filePath;
        }
        if (j < dir.length - 1) {
            dir = dir.slice(0, j + 1);
        }
        return dir + filePath;
    }

    /// from ./feature/http

    /**
     * @description URL增加参数
     * @exampleexport const
     */
    export const seedUrl = (url: string, seed?: string) => {
        if (!seed) {
            seed = "" + Math.random();
        }
        return (url.indexOf("?") > 0) ? url + "&" + seed : url + "?" + seed;
    };

    /**
     * @description 参数化对象数据
     * @return {string} 被encodeURIComponent编码过的&=分隔的字符串， "Content-Type":"application/x-www-form-urlencoded"
     * @example
     */
    export const param = (data: any): string => {
        let arr = [];
        for (let k in data) {
            if (!data.hasOwnProperty(k)) {
                continue;
            }
            let v = data[k];
            if (v === undefined) {
                v = "undefined";
            } else if (v === null) {
                v = "null";
            } else if (typeof v === typeof "") {
                v = encodeURIComponent(v);
            } else if (typeof v === typeof 0) { } else {
                v = encodeURIComponent(JSON.stringify(v));
            }
            arr.push(encodeURIComponent(k), "=", v, "&");
        }
        if (arr.length > 0) {
            arr.length--;
        }
        return arr.join("");
    };

    // /**
    //  * 上行消息结构： (后缀1(文件名1:文件名2)后缀2(文件名1:文件名2):目录1()目录2())
    //  * @param tree
    //  * @param limit
    //  * @param result
    //  */
    // export const stringify = (tree: IDirDependInfo, limit: number, result: Result) => {
    //     // 先写入本目录下的文件，按后缀归类
    //     if (tree.suffixMap) {
    //         for (let [k, arr] of tree.suffixMap) {
    //             let i = arr.length - 1;
    //             let f = arr[i--];
    //             let rk = replace(k);
    //             let rs = replace(fileBasename(f.path));
    //             if (result.url.length + rk.length + rs.length > limit || result.size + f.size > FileLoadParam.sizeLimit && result.files.length) {
    //                 result.next = true;
    //                 result.size = 0;
    //                 return;
    //             }
    //             result.files.push(f);
    //             result.size += f.size;
    //             result.url += rk + "(" + rs;
    //             for (; i >= 0; i--) {
    //                 f = arr[i];
    //                 rs = replace(fileBasename(f.path));
    //                 if (result.url.length + rs.length > limit || result.size + f.size > FileLoadParam.sizeLimit && result.files.length) {
    //                     arr.length = i + 1;
    //                     result.next = true;
    //                     result.url += ")";
    //                     result.size = 0;
    //                     return;
    //                 }
    //                 result.url += ":" + rs;
    //                 result.files.push(f);
    //                 result.size += f.size;
    //             }
    //             result.url += ")";
    //             tree.suffixMap.delete(k);
    //         }
    //     }
    //     result.url += ":";
    //     if (!tree.children) {
    //         return;
    //     }
    //     let dir: IDirDependInfo;
    //     while (dir = tree.children[tree.children.length - 1]) {
    //         let i = dir.path.lastIndexOf("/", dir.path.length - 2);
    //         let rk = replace(dir.path.slice(i + 1, dir.path.length - 1));
    //         result.url += rk + "(";
    //         stringify(dir, limit, result);
    //         result.url += ")";
    //         if (result.next) {
    //             return result;
    //         }
    //         tree.children.length -= 1;
    //     }
    // };
    /**
     * 3个分隔符 (): $作为转义, $转$$ (转$1 )转$2
     * @param s
     */
    export const replace = (s: string) => {
        return s.replace(/\(/g, "$1").replace(/\)/g, "$2").replace(/\$/g, "$$");
    };
    /**
     * 计算字符串的hash
     * @param s
     * @param h
     */
    export const strHash = (s: string, h: number) => {
        if (!s)
            return h;
        let c: number, i = 0, len = s.length;
        for (; i < len; i++) {
            c = s.charCodeAt(i);
            h = (((h << 5) - h) + c) | 0; //乘31，然后使用了 “|” 运算符，转换参数为 32bit，确保结果是个32位整数
        }
        return h;
    };

    /**
     * 返回：path是否满足within，但不满足without
     */
    export const filter = (path: string, within: RegExp[], without: RegExp[]) => {
        if (within.length > 0) {
            for (let r of within) {
                r.lastIndex = 0;
                if (r.test(path)) {
                    return isWithout(path, without);
                }
            }
            return false;
        }
        return isWithout(path, without);
    }

    /**
     * 模式匹配。
     * 给定目录的pattern，生成对应的正则表达式
     * pattern的模式：*, **
     * 星号*，匹配除 斜杠 的东西；
     * 星星号*，匹配 0层 或 多层 斜杠 /
     */
    export const pattern = (p: string): RegExp => {
        let str = '';
        const arr = p.split('/');
        for (let i = 0; i < arr.length; ++i) {
            let p = arr[i];
            if (p === '**') {
                str += '(?:(.*)\\\/)?';
            } else {
                // 将所有的*替换掉
                p = p.replace(/\./g, '\\\.');
                p = p.replace(/\*/g, '([^/]*)');

                str += p;
                if (i + 1 < arr.length) {
                    str += '\\\/';
                }
            }
        }
        return new RegExp('^' + str + '$', 'g');
    };

    // =========================== 本地

    // 判断文件是否不符合
    const isWithout = (path: string, without: RegExp[]) => {
        for (let r of without) {
            r.lastIndex = 0;
            if (r.test(path))
                return false;
        }
        return true
    }
}