/**
 * Bi日志上报模块
 * 实现pi_sys/util/logger.ts 定义的ILogAppender和ILogEncoder
 */

import { post } from "pi_sys/net/http";
import { Env } from "./env";
import { addAppender, addLogFilter, ILogAppender, ILogEncoder, Logger, LogLevel, registerAppenderKind } from "./logger";
import { formatStringImpl } from "./util";
declare var module;
let logger = new Logger(module.name, "sys");

class BiEncoder implements ILogEncoder {
    encode(level: LogLevel, logger: Logger, format: string, args: any[], callLocation = "") {
        let [str, _currentArg] = formatStringImpl(format, args);

        return str;
    }
    getMessage(format: string, args: any[], callLocation?: string) {
        return ""
    }
}

class BiAppender implements ILogAppender {

    url: string;
    encoder: ILogEncoder;

    constructor(url: string) {
        this.url = url;
        this.encoder = new BiEncoder();
    }

    flush() {
        return Promise.resolve();
    }

    setEncoder(encoder: ILogEncoder) {
        this.encoder = encoder;
    }

    write(level: LogLevel, logger: Logger, format: string, args: any[], callLocation = "") {
        if (!this.url || !this.encoder) {
            return;
        }

        let info = this.encoder.encode(level, logger, format, args, callLocation);
        const header = {
            "x-log-apiversion": "0.6.0",
            "x-log-bodyrawsize": "0",
        };
        return post(this.url, header, info, "application/json", "text", 3 * 1000);
    }

}



/**
 * bi上报
 */
export const BiAppenderKind = "BiKind";


/**
 * 通用模块,本质上就是bi模块,和bi的区别在于上报url不同
 */
export const CommonAppenderKind = "CommonKind";

/**
 * 初始化Bi 日志上报
 * @returns 
 */
export const initBiLogger = () => {
    const appenderNames = [];
    const biUpload = Env.get("biUpload");
    if (biUpload) {
        const url = Env.get("biUrl");
        if (!url) {
            logger.warn(`initBiLogger biUrl not set!`);
            return;
        }

        addAppender("bi", BiAppenderKind, url);
        appenderNames.push("bi"); // target = "bi" 在pi_common/bi/bi_sdk.ts中写死的  ["bi"]代表appender name
    }

    if (Env.get("biConsole")) {
        appenderNames.push("console");
    }
    if (appenderNames.length) {
        addLogFilter("bi", LogLevel.Info, appenderNames);
    }

}

export const initGiLogger = () => {
    const appenderNames = [];
    const giUpload = Env.get("giUpload");
    if (giUpload) {
        const url = Env.get("biUrl");
        if (!url) {
            logger.warn(`initGiLogger biUrl not set!`);
            return;
        }
        addAppender("gi", BiAppenderKind, url);
        appenderNames.push("gi"); // target = "gi" 在pi_common/bi/bi_sdk.ts中写死的  ["gi"]代表appender name
    }

    if (Env.get("giConsole")) {
        appenderNames.push("console");
    }
    if (appenderNames.length) {
        addLogFilter("gi", LogLevel.Info, appenderNames);
    }
}


// 注册两个Appender
(function () {
    registerAppenderKind(BiAppenderKind, BiAppender);
    registerAppenderKind(CommonAppenderKind, BiAppender);
})();