/**
 * 收集资源文字等
 */

import { IFileDependInfo } from "./depend_base";
import { Env } from "./env";


export enum CollectType {
    RES = 1,    // 资源 包括具体的资源或者图片名字
    TEXT = 2    // SDF文字
}

/**
 * 文字资源慢下载收集
 * @param info 有可能是资源depend信息,有可能是图片名字,有可能是文字
 * @param key 1 资源 2 文字
 * @param font  文字字体
 */
export const doCollect = (key: IFileDependInfo | string, type: CollectType, font?: string) => {
    const collect = Env.get("collect");
    if (collect) {  // 慢下载收集
        let now = Date.now();
        let __collect__ = (<any>window).__collect__ as Map<string, [number, number]>;
        let startTime = (<any>window).__collectStartTime__;
        if (!__collect__) {
            (<any>window).__collect__ = __collect__ = new Map;
            (<any>window).__collectStartTime__ = startTime = now;
            (<any>__collect__).minify = () => {  // 最小化收集的内容,剔除psd的图片,有名字即可
                let arr = [];
                for (const [k, v] of __collect__) {
                    if (!k.startsWith("psd/")) {
                        arr.push([k, v])
                    }
                }
                return arr;
            }
        }
        const collectIgnoreSuffixs = Env.get("collect_ignore_suffixs") || Env.get("cfg_suffixs");
        if (!collectIgnoreSuffixs.includes("js")) {
            collectIgnoreSuffixs.push("js");
        }
        const timeInLine = Math.ceil((now - startTime) / 1000);  // 在时间线上的时刻
        const value: [number, number] = [timeInLine, type];
        if (typeof key === "string") { // 图片名字或者文字
            if (font) {
                key = `${key}|${font}`;
            }
            if (!__collect__.has(key)) {
                __collect__.set(key, value);
            }
        } else {
            if (!collectIgnoreSuffixs.includes(key.suffix) && !__collect__.has(key.path)) {
                __collect__.set(key.path, value)
            }
        }

    }
}


type callback = (name: string | number, type: CollectType, fontPath?: string) => void;
// 资源&文字使用通知

const resUsedCallbacks: Set<callback> = new Set;

export const registerResUsed = (cb: callback) => {
    if (resUsedCallbacks.has(cb)) return;
    resUsedCallbacks.add(cb);
}

export const removeResUsed = (cb?: callback) => {
    if (cb) {
        resUsedCallbacks.delete(cb)
    } else {
        resUsedCallbacks.clear();
    }
}

export const notifyResUsed = (name: string | number, type: CollectType = CollectType.RES, fontPath?: string) => {
    if (resUsedCallbacks.size === 0) return;
    for (const cb of resUsedCallbacks) {
        cb && cb(name, type, fontPath);
    }
}