import { CompressAlgorithm, CompressLevel, EncryptType } from "./constant";

export type TDependFileName = string;
export type TDependFileSize = number;
export type TDependFileSign = string;
export type TDependFileDepend = TDependFileName[];
export type TDependFileEncryptType = EncryptType;
export type TDependFileEncryptPath = string;
export type TDependFileEncryptSize = number;
export type TDependFileCompressAlgorithm = CompressAlgorithm;
export type TDependFileCompressLevel = CompressLevel;
export type TDependJson = [TDependFileName, TDependFileSize, TDependFileSign, TDependFileEncryptType?, TDependFileEncryptPath?, TDependFileEncryptSize?, TDependFileCompressAlgorithm?, TDependFileCompressLevel?];

export interface IFileDependInfo {
    /**
     * 文件的项目路径
     */
    path: string;
    /**
     * 文件签名
     */
    sign: string;
    /**
     * 文件大小
     */
    size: number;
    /**
     * 文件后缀
     */
    suffix: string;
    /**
     * 文件依赖
     */
    depend: string[] | undefined;
    /**
     * 加密类型
     */
    encryptType?: EncryptType;
    /**
    * 文件的加密后的路径
    */
    encryptPath?: string;
    /**
     * 文件加密后的大小
     */
    encryptSize?: number;
    /**
     * 压缩算法
     */
    compressAlgorithm?: CompressAlgorithm;
    /**
     * 压缩等级
     */
    compressLevel?: CompressLevel;
}
export interface IDirDependInfo {
    /**
     * 父目录
     */
    parent: IDirDependInfo | undefined;
    /**
     * 文件夹的项目路径
     */
    path: string;
    /**
     * 文件夹层级
     */
    level: number;
    /**
     * 文件夹数据大小
     */
    size: number;
    /**
     * 子文件夹
     */
    children: IDirDependInfo[];
    /**
     * 目录下文件
     */
    files: IFileDependInfo[];
    /**
     * 目录下文件后缀表
     * 好像只需要数量和大小，后面需要计算所有签名的异或值
     */
    suffixMap: Map<string, IFileDependInfo[]>;
    add(info: IFileDependInfo, isSuffix: boolean): void;
}