
/**
 * 环境配置模块
 * 
 * + 提供设置接口
 * + 提供查询接口
 * + 提供键名遍历接口
 */

import { CommonKey } from "./env_base";

/**
 * @preferred
 * KV内存对象，用于存储项目和引擎的初始配置信息
 */
export namespace Env {
    export const KeyWords = CommonKey;

    /**
     * 初始化环境变量数据
     * @param json winit.env
     */
    export const Init = (json: any) => {
        for (let k in json) {
            if (json.hasOwnProperty(k)) {
                if (json[k] !== undefined && json[k] !== null) {
                    map.set(k, json[k]);
                }
            }
        }
    }

    /**
     * 获取指定名称的环境变量属性值
     * @param key 键
     * @returns key对应的值
     */
    export const get = (key: string): any => {
        return map.get(key);
    }

    /**
     * 设置指定名称的环境变量属性
     * @param key 键
     * @param val 值
     */
    export const set = (key: string, val: any): void => {
        map.set(key, val);
    }

    export const has = (key: string) => {
        return map.has(key);
    }

    /**
     * 取 环境变量 所有的键
     */
    export const keys = (): string[] => {
        return [...map.keys()];
    }

    /**
     * 删除 环境变量
     * @param key 
     * @returns 
     */
    export const remove = (key: string) => {
        return map.delete(key);
    }

    /**
     * @hidden
     * 配置属性表
     */
    const map: Map<string, any> = new Map();
}