declare var _$pi;

/**
 * 模块，对_$pi的封装
 */

/**
 * 设置模块调度函数，一般和任务管理器绑定；
 */
export const setModuleScheduler = (func: Function) => {
    _$pi.exec = func;
}

/**
 * require方法，异步加载
 */
export const requireAsync = (name: string): Promise<any> => {
    return requireImpl(name);
}

/**
 * require方法，同步加载
 */
export const requireSync = (name: string) => {
    return _$pi.require(name);
}

/**
 * 获取模块的导出对象
 */
export const getModule = (name: string) => {
    name = replaceWhenWasm(name);
    return _$pi._modules.get(name);
}

/**
 * 获取模块的导出对象
 */
export const getModuleExport = (name: string) => {
    let m = getModule(name)
    return m && m.exports;
}

/**
 * 设置那些模块的后缀需要立即加载
 * 这里设置的后缀，全部都会立即加载
 */
export const setImmSuffixs = (suffixNames: string[]) => {
    _$pi._immLoadSuffixs.push(...suffixNames);
}

/**
 * 当前平台是否可以用 wasm 模块
 */
export const canUseWasm = () => {
    return _$pi.canUseWasm();
}

// =========================== 本地

const requireImpl = (name) => {
    return new Promise((resolve, reject) => {
        _$pi.require(name, (exp) => {
            resolve(exp);
        });
    });
}

// 当name是wasm模块，而且不支持wasm的时候，要退化到asm模块
const replaceWhenWasm = (name) => {
    if (name.endsWith(".wasm") && !canUseWasm()) {
        name = name.slice(0, name.length - ".wasm".length) + ".asm";
    }
    return name;
}
