


export type Task = () => Promise<any>
/**
 * 简单异步任务池
 * 如果有一个任务失败,其他任务将终止执行
 */
export class SimpleTaskPools {
    tasks: Set<Task> = new Set;  // 任务队列
    maxCount: number;
    currentCount: number = 0;
    total: number = 0;            // 任务总数
    private onsuccess: Function;
    private onfail: Function;
    private abort: boolean = false;  // 如果有任务失败,置true,终止后续任务执行

    constructor(maxCount: number = 6) {
        this.maxCount = maxCount;
    }

    addTasks(tasks: Task | Task[]) {
        if (Array.isArray(tasks)) {
            tasks.forEach(task => this.tasks.add(task));
            this.total += tasks.length;
        } else {
            this.tasks.add(tasks);
            this.total++;
        }
    }

    /**
     * 开始执行任务
     */
    start() {
        return new Promise((resolve, reject) => {
            this.onsuccess = resolve;
            this.onfail = reject;

            for (let i = 0; i < this.maxCount; i++) {
                this.run();
            }
        })
    }

    run() {
        if (this.abort) return;
        if (this.tasks.size === 0 && this.currentCount === 0) {
            this.onsuccess();
            return;
        }

        if (this.currentCount >= this.maxCount) return;
        let iter = this.tasks.values();
        let { done, value } = iter.next();
        if (value) {
            let task = value as Task;
            this.tasks.delete(task);
            this.currentCount++;
            task().then(() => {
                this.currentCount--;
                this.run();
            }).catch(e => {
                this.abort = true;
                this.onfail(e);
            })
        }

    }
}