/**
 * 代码加载器
 *  
 */

import { FileLoad } from "pi_sys/load/file_load";
import { createURL } from "pi_sys/net/bloburl";
import { IFileDependInfo } from "pi_sys/util/depend_base";
import { Env } from "pi_sys/util/env";

// ============================== 导入


// ============================== 导出

export class CodeLoad extends FileLoad {

    /**
     * @description 开始
     * @example
     */
    public start() {
        let arr = [];
        for (let info of this.files.values()) {

            // TODO: 这里要和minigame脱钩
            if (info.path.indexOf('/game.js') >= 0) {
                continue;
            }

            arr.push(new Promise((resolve, reject) => {
                loadJS(this, info, resolve, reject);
            }));
        }
        return Promise.all(arr);
    }
}


export const loadJSFromAB = (file: string, data: Uint8Array): Promise<void> => {
    return new Promise((resolve, reject) => {
        const url = createURL(data, 'application/javascript');

        let head = document.head || document.getElementsByTagName('head')[0] || document.documentElement;
        let n = document.createElement('script');

        n.onerror = (err) => {
            n.onload = n.onerror = undefined;
            head.removeChild(n);
            reject(err);
        };

        n.onload = () => {
            n.onload = n.onerror = undefined;
            head.removeChild(n);
            resolve();
        };

        n.charset = 'utf8';
        n.async = true;
        n.crossOrigin = "anonymous";
        n.src = url;

        head.appendChild(n);
    });
}


// ============================== 本地

const loadJS = (load: CodeLoad, file: IFileDependInfo, callback: (e: Element) => void, errorCallback: (err: string) => void, errText?: string, i?: number) => {
    const urls = Env.get('domains');
    const downPath = Env.get('root_path');
    const currentVersion = Env.get("currentVersion");  // 版本
    const resDirname = Env.get("resDirname"); // 版本目录下的二级目录

    if (i >= urls.length) {
        return errorCallback && errorCallback(urls[0] + file.path + ", " + errText);
    }
    let head = document.head || document.getElementsByTagName('head')[0] || document.documentElement;
    let n = document.createElement('script');
    n.charset = 'utf8';
    n.onerror = () => {
        n.onload = n.onerror = undefined;
        head.removeChild(n);
        loadJS(load, file, callback, errorCallback, errText, i === undefined ? 0 : i + 1);
    };
    n.onload = () => {
        n.onload = n.onerror = undefined;
        head.removeChild(n);
        load.loaded += file.size;
        load.onProcess(file.path, "codeLoad", load.total, load.loaded);
        callback && callback(n);
    };
    n.async = true;
    n.crossOrigin = "anonymous";
    n.src = urls[i || 0] + downPath + (currentVersion ? `${currentVersion}/${resDirname}/` : "") + file.path + "?" + file.sign;
    head.appendChild(n);
};