/**
 * 本地进度条
 */

export class ProgressBar {
    image: HTMLImageElement;
    text: HTMLDivElement;
    div: HTMLDivElement;
    divProcess: HTMLDivElement;
    timeRef: number;
    // 加载的总文件长度
    public total: number = 0;
    // 加载的当前进度
    public loaded: number = 0;
    old: number = 0;
    last: number = 0;
    opacity: number = 0;


    public constructor(cfg: any) {
        if (cfg.image) {
            this.image = cfg.image;
            this.image.setAttribute("style", "position:absolute;width:100%;height:100%;top:0;left:0;")
        }
        this.text = document.createElement('div');
        this.text.setAttribute("pi", "1");
        this.text.setAttribute("style", "position:absolute;bottom:30px;text-align:center;width: 100%;height: 32px;font-size:20px;");
        this.text.style.color = cfg.text_color || "#fff";
        this.div = document.createElement('div');
        this.div.setAttribute("pi", "1");
        this.div.setAttribute("style", "position:absolute;bottom:16px;left: 3%;width: 94%;height: 10px;padding: 1px;border-radius: 20px;border-top: 1px solid #000;border-bottom: 1px solid #7992a8;");
        this.div.style.backgroundColor = cfg.bar_bgcolor || "#262626";
        this.divProcess = document.createElement('div');
        this.divProcess.setAttribute("style", "width: 0%;height: 100%;border-radius: 20px;");
        this.divProcess.style.backgroundColor = cfg.bar_color || "rgb(162, 131, 39)";
        this.div.appendChild(this.divProcess);

    }
    // 显示文字和进度条， 文字有呼吸灯的效果
    public show(text: string, total: number, loaded: number) {
        this.text.innerHTML = text;
        this.total = total;
        this.old = loaded;
        if (this.timeRef)
            return;
        if (this.image) {
            document.body.appendChild(this.image);
        }

        document.body.appendChild(this.text);
        document.body.appendChild(this.div);
        this.timeRef = <any>setTimeout(this.calc, 100);
    }

    public onProcess(url: string, type: string, total: number, loaded: number, data?: Uint8Array) {
        if (total) {
            this.total = total;
        }

        this.loaded = loaded;
    }

    calc = () => {
        this.flush();
        this.timeRef = setTimeout(this.calc, 100);
    }

    flush() {
        this.opacity += 3;
        this.text.style.opacity = "" + (Math.abs(this.opacity % 80 - 40) + 10) / 50;;

        if (this.last === this.loaded) {
            return;
        }

        this.last = this.loaded;

        let value = 0;
        if (this.total !== this.old) {
            value = (this.last - this.old) * 100 / (this.total - this.old)
        }
        this.divProcess.style.width = value + "%";
    }

    public clear() {
        this.total = 0;
        this.loaded = 0;
        this.old = 0;
        if (this.timeRef)
            clearTimeout(this.timeRef);
        this.timeRef = 0;
        if (this.image) {
            document.body.removeChild(this.image);
        }
        document.body.removeChild(this.text);
        document.body.removeChild(this.div);
    }
}