/**
 * App 相关接口
 */

import { FlagAppLife, TexitApp, TgetFlavorName, TgetPackageName, TgetVersionCode, TgetVersionName, TgetSdkInfo, ISdkInfo, TtriggerGC, TonShow, TonHide, TonDestroy, ToffShow, ToffHide, ToffDestroy, ThideFullScreenPopupWindow, TgetProcessId, TgetApiVersion, TopenWebView, TcloseWebView, TonBlur, ToffBlur, TgetRunconfigValue, TregisterAppLifeListener, TunregisterAppLifeListener, TregisterSoftInputChangedListener, TunregisterSoftInputChangedListener, TgetChannelInfo, TrequestPointerLock, ChannelInfo, TexitPointerLock, TonPointerLockError, TonPointerLockChange, ToffPointerLockError, ToffPointerLockChange, TsetCursorVisible, TopenAppstore, TgetTemperature, TgetChannelQuick, TprivacyAgreement, TclearCache, TshowWithEntranceId, TupdateUserInfo, TresetUserInfo, TshowWithEntranceIdLogin, TgetIdfv, TgetIpAdress, TgetIdfa } from "pi_sys/native/base/app";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

export const exitApp: TexitApp = () => { }

export const getProcessId: TgetProcessId = () => {
    let now = new Date().getTime();
    return Promise.resolve(now.toString());
}

export const getApiVersion: TgetApiVersion = () => {
    return Promise.resolve("");
}

export const getPackageName: TgetPackageName = () => {
    return Promise.resolve("");
}

export const getFlavorName: TgetFlavorName = () => {
    return Promise.resolve("");
}

export const getVersionName: TgetVersionName = () => {
    return Promise.resolve("");
}

export const getVersionCode: TgetVersionCode = () => {
    return Promise.resolve(0);
}

export const getRunconfigValue: TgetRunconfigValue = (key: string): Promise<string> => {
    return Promise.resolve("");
}

export const getChannelInfo: TgetChannelInfo = (): Promise<ChannelInfo> => {
    let info = {
        channel: "",
        param: {}
    };
    return Promise.resolve(info);
}

export const getSdkInfo: TgetSdkInfo = (): Promise<ISdkInfo> => {
    return Promise.resolve({
        list: [],
    });
}

export const triggerGC: TtriggerGC = () => {
}

export const openWebView: TopenWebView = (url: string, params?: Object) => {
    return Promise.resolve("");
};


export const closeWebView: TcloseWebView = (tag: string) => {
    return Promise.resolve();
};



export const requestPointerLock: TrequestPointerLock = () => {
    if (window["canvas"] && window["canvas"].requestPointerLock) {
        window["canvas"].requestPointerLock();
    }
};

export const exitPointerLock: TexitPointerLock = () => {
    if (window.document.exitPointerLock) {
        window.document.exitPointerLock();
    }
};

export const openAppstore: TopenAppstore = () => { };

// 获取CPU温度,单位摄氏度,保留小数点后一位
export const getTemperature: TgetTemperature = () => {
    return Promise.resolve(0);
};


// 获取idfv
export const getIdfv: TgetIdfv = () => {
    return Promise.reject("getIdfv not impl");
};

// 获取IpAdress
export const getIpAdress: TgetIpAdress = () => {
    return Promise.reject("getIpAdress not impl");
};

// 获取Idfa
export const getIdfa: TgetIdfa = () => {
    return Promise.reject("getIpAdress not impl");
};


/**
 * 获取quick渠道分发出来的渠道
 */
export const getChannelQuick: TgetChannelQuick = () => {
    return Promise.resolve(null);
};


let lockErrorFuncs: Set<Function>;
let lockError: Function;

export const onPointerLockError: TonPointerLockError = (callback: () => void) => {
    if (!lockErrorFuncs) {
        lockErrorFuncs = new Set;
    }
    lockErrorFuncs.add(callback);
    if (!lockError) {
        lockError = () => {
            lockErrorFuncs && lockErrorFuncs.forEach(cb => cb());
        }
        document.addEventListener("pointerlockerror", lockError as any);
    }

};



export const offPointerLockError: ToffPointerLockError = (callback?: () => void) => {
    if (callback) {
        if (lockErrorFuncs) {
            lockErrorFuncs.delete(callback);
        }
    } else {
        lockErrorFuncs.clear();
    }
    if (!lockErrorFuncs.size) {
        document.removeEventListener("pointerlockerror", lockError as any);
        lockError = null;
    }

};


export const privacyAgreement: TprivacyAgreement = (isAccepted: boolean) => {
    return Promise.resolve(null);
}


export const clearCache: TclearCache = () => {
    return Promise.resolve(null);
};

let lockChangeFuncs: Set<Function>;
let lockChange: Function;


export const onPointerLockChange: TonPointerLockChange = (callback: (locked: boolean) => void) => {
    if (!lockChangeFuncs) {
        lockChangeFuncs = new Set;
    }
    lockChangeFuncs.add(callback);
    if (!lockChange) {
        lockChange = (e) => {
            lockChangeFuncs && lockChangeFuncs.forEach(cb => cb(!!document.pointerLockElement));
        }
        document.addEventListener("pointerlockchange", lockChange as any);
    }
};


export const offPointerLockChange: ToffPointerLockChange = (callback?: (locked: boolean) => void) => {
    if (callback) {
        if (lockChangeFuncs) {
            lockChangeFuncs.delete(callback);
        }
    } else {
        lockChangeFuncs.clear();
    }
    if (!lockChangeFuncs.size) {
        document.removeEventListener("pointerlockchange", lockChange as any);
        lockChange = null;
    }
};


export const setCursorVisible: TsetCursorVisible = (visible: boolean) => {
    if (visible) {
        document.body.style.cursor = "default";
    } else {
        document.body.style.cursor = "none";
    }
}



let funcs: Set<Function>;         // registerAppLifeListener监听回调
let showFuncs: Set<Function>;     // onShow监听回调
let hideFuncs: Set<Function>;     // onHide监听回调
let destroyFuncs: Set<Function>;  // onDestroy监听回调
let runFunc: Function;            // 实际注册回调函数

export const registerAppLifeListener: TregisterAppLifeListener = (callback: (flag: FlagAppLife) => void) => {
    if (!funcs) {
        funcs = new Set;
    }
    funcs.add(callback);
    __register();
}

export const unregisterAppLifeListener: TunregisterAppLifeListener = (callback?: (flag: FlagAppLife) => void) => {
    if (!funcs) return;
    if (callback) {
        funcs.delete(callback);
    } else {
        funcs.clear();
    }
    if (!funcs.size) {
        funcs = undefined;
        __unRegister();
    }

}


export const onShow: TonShow = (callback: () => void) => {
    if (!showFuncs) {
        showFuncs = new Set;
    }
    showFuncs.add(callback);
    __register();
};

export const onHide: TonHide = (callback: () => void) => {
    if (!hideFuncs) {
        hideFuncs = new Set;
    }
    hideFuncs.add(callback);
    __register();
};

export const onDestroy: TonDestroy = (callback: () => void) => {
    logger.warn("Web platform does not support");
};

export const offShow: ToffShow = (callback?: () => void) => {
    if (!showFuncs) return;
    if (callback) {
        showFuncs.delete(callback);
    } else {
        showFuncs.clear();
    }
    if (!showFuncs.size) {
        showFuncs = undefined;
        __unRegister();
    }
};

export const offHide: ToffHide = (callback?: () => void) => {
    if (!hideFuncs) return;
    if (callback) {
        hideFuncs.delete(callback);
    } else {
        hideFuncs.clear();
    }
    if (!hideFuncs.size) {
        hideFuncs = undefined;
        __unRegister();
    }
};

export const offDestroy: ToffDestroy = (callback?: () => void) => {
    logger.warn("Web platform does not support");
};


const __register = () => {
    if (runFunc) return;
    runFunc = () => {
        let flag;
        if (document.hidden) {
            flag = "onPause"
            hideFuncs && hideFuncs.forEach(func => func && func());
        } else {
            flag = "onResume";
            showFuncs && showFuncs.forEach(func => func && func());
        }
        funcs && funcs.forEach(func => func && func(flag));
    }
    document.addEventListener("visibilitychange", runFunc as any, false);
}

const __unRegister = () => {
    if (funcs || showFuncs || hideFuncs || destroyFuncs) return;
    document.removeEventListener("visibilitychange", runFunc as any, false);
    runFunc = undefined;
}



/**
 * 监听APP失焦事件
 * @note 仅web和exe支持
 */
export const onBlur: TonBlur = (callback: () => void) => {
    window.addEventListener("blur", callback);
};


/**
* 取消 监听APP失焦事件
* @note 仅web和exe支持
*/
export const offBlur: ToffBlur = (callback: () => void) => {
    window.removeEventListener("blur", callback);
};

/**
 * 注册软件盘弹出监听
 * @param callback 
 */
export const registerSoftInputChangedListener: TregisterSoftInputChangedListener = (callback: (height: number) => void) => { };

/**
 * 取消软件盘弹出监听
 * @param callback 
 */
export const unregisterSoftInputChangedListener: TunregisterSoftInputChangedListener = (callback?: (height: number) => void) => { };

/**
 * 隐藏NativeActivity的全屏开屏图PopupWindow
 */
export const hideFullScreenPopupWindow: ThideFullScreenPopupWindow = () => { }



/**
 * 唤起客服
 */
export const showWithEntranceId: TshowWithEntranceId = () => {
}

/**
 * 客服后台用户信息
 * @returns 
 */
export const updateUserInfo: TupdateUserInfo = (str: string) => {
}

/**
 * 清空用户信息
 */
export const resetUserInfo: TresetUserInfo = () => {
}

/**
 * 唤起客服
 */
export const showWithEntranceIdLogin: TshowWithEntranceIdLogin = () => {
}
