import { TgetData, TsetData } from "pi_sys/native/device/clipboard";

let __data: string;
/**
 * 获取剪贴板的数据，仅文本.
 */
export const getData: TgetData = (): Promise<string> => {
    if (navigator.clipboard) {
        return navigator.clipboard.readText();
    } else {
        return new Promise((resolve, reject) => {
            resolve(__data)
        });
    }
}

/**
 * 设置剪贴板内容. 调用成功后，会弹出 toast 提示"内容已复制"，持续 1.5s.
 * @param data 
 */
export const setData: TsetData = (data: string): Promise<void> => {
    if (navigator.clipboard) {
        // Chrome 浏览器规定，只有 HTTPS 协议的页面才能使用这个 API（localhost,127.0.0.1除外）
        // http协议的页面navigator.clipboard = undefined
        return navigator.clipboard.writeText(data).then(() => {
            __data = data;
        });
    } else {
        return new Promise((resolve, reject) => {
            var textarea = document.createElement('textarea');
            document.body.appendChild(textarea);
            // 隐藏此输入框
            textarea.style.position = 'fixed';
            textarea.style.clip = 'rect(0 0 0 0)';
            textarea.style.top = '10px';
            // 赋值
            textarea.value = data;
            // 选中
            textarea.select();
            // 复制
            const flag = document.execCommand('copy', true);
            // 移除输入框
            document.body.removeChild(textarea);
            if (flag) {
                __data = data;
                resolve()
            } else {
                reject();
            }
        });
    }
}