import { ActionSheetInfo, Icon, ModalInfo, ThideLoading, ThideToast, TshowActionSheet, TshowLoading, TshowModel, TshowToast } from "pi_sys/native/ui/dialog";

/**
 * 显示消息提示框
 * @param title		提示的内容
 * @param icon		图标
 * @param image		自定义图标的本地路径，image 的优先级高于 icon
 * @param duration	提示的延迟时间
 * @param mask		是否显示透明蒙层，防止触摸穿透
 * @note showLoading 和 showToast 同时只能显示一个
 * @note showToast 应与 hideToast 配对使用
 */
export const showToast: TshowToast = (title: string, icon: Icon = Icon.Success, image?: string, duration: number = 1500, mask: boolean = false): Promise<void> => {
    return Promise.resolve();
}


/**
 * 隐藏消息提示框
 */
export const hideToast: ThideToast = (): Promise<void> => {
    return Promise.resolve();
}

/**
 * 显示 loading 提示框
 * @note 需主动调用 hideLoading 才能关闭提示框
 * @note showLoading 和 showToast 同时只能显示一个
 * @note showLoading 应与 hideLoading 配对使用
 * 
 * @param title	 提示的内容
 * @param mask	 是否显示透明蒙层，防止触摸穿透
 */
export const showLoading: TshowLoading = (title: string, mask: boolean = false): Promise<void> => {
    return Promise.resolve();
}


/**
 * 隐藏 loading 提示框
 */
export const hideLoading: ThideLoading = (): Promise<void> => {
    return Promise.resolve();
}

/**
 * 显示模态对话框
 * @param title		    提示的标题
 * @param content	    提示的内容
 * @param showCancel    是否显示取消按钮
 * @param cancelText    取消按钮的文字，最多 4 个字符
 * @param cancelColor	取消按钮的文字颜色，必须是 16 进制格式的颜色字符串
 * @param confirmText	确认按钮的文字，最多 4 个字符
 * @param confirmColor	确认按钮的文字颜色，必须是 16 进制格式的颜色字符串
 * @note Android 6.7.2 以下版本，点击取消或蒙层时，回调 fail, errMsg 为 "fail cancel"
 * @note Android 6.7.2 及以上版本 和 iOS 点击蒙层不会关闭模态弹窗，所以尽量避免使用「取消」分支中实现业务逻辑
 */
export const showModel: TshowModel = (title?: string, content?: string, showCancel: boolean = true, cancelText: string = "cancel", cancelColor: string = "#000000", confirmText: string = "confirm", confirmColor: string = "#576B95"): Promise<ModalInfo> => {
    const message = title || content;
    let flag = true;
    if (showCancel) {
        flag = confirm(message);
    } else {
        alert(message);
    }
    return Promise.resolve({
        confirm: flag,
        cancel: !flag
    });
}

/**
 * 显示操作菜单
 * @note Android 6.7.2 以下版本，点击取消或蒙层时，回调 fail, errMsg 为 "fail cancel"
 * @note Android 6.7.2 及以上版本 和 iOS 点击蒙层不会关闭模态弹窗，所以尽量避免使用「取消」分支中实现业务逻辑
 * 
 * @param itemList   按钮的文字数组,数组长度最大为 6
 * @param itemColor	 按钮的文字颜色
 */
export const showActionSheet: TshowActionSheet = (itemList: string[], itemColor: string = "#000000"): Promise<ActionSheetInfo> => {
    return Promise.resolve(null);
}