import { KeyboardConfig, Thide, ToffComplete, ToffConfirm, ToffInput, ToffKeyboardHeightChange, ToffKeyDown, ToffKeyUp, ToffSelection, TonComplete, TonConfirm, TonInput, TonKeyboardHeightChange, TonKeyDown, TonKeyUp, TonSelection, TsetSelection, Tshow, TsupportSelection, Tupdate } from "pi_sys/native/device/keyboard";
import { Env } from "pi_sys/util/env";


/**
 * web键盘输入
 */

const DEFAULT_CFG: any = {
    multiple: false,
    maxLength: 100,
    confirmHold: false,
    inputBox: true,

};

let INPUT_ELEMENTS: { [index: string]: HTMLInputElement | HTMLTextAreaElement } = {};
let inputElement: HTMLInputElement | HTMLTextAreaElement;
let inputType: string;
let selectionStart: number;
let lastValue: string;
let clickCb: Function;


// 唤起键盘
export const show: Tshow = (config: KeyboardConfig) => {
    let elementName = config.multiple ? "textarea" : "input";
    let ty = `${elementName}-${config.attribute["type"]}`;
    inputType = config.attribute["type"] || "text";
    if (!INPUT_ELEMENTS[ty]) {
        INPUT_ELEMENTS[ty] = document.createElement(elementName) as HTMLInputElement | HTMLTextAreaElement;
        inputElement = INPUT_ELEMENTS[ty];
        document.body.appendChild(inputElement);
        inputElement.style.position = "absolute";
        inputElement.style.width = "100%";
        inputElement.style.boxSizing = "border-box"
        if (inputElement instanceof HTMLTextAreaElement) {
            const style = document.createElement('style');
            style.innerHTML = ".___textarea::-webkit-scrollbar { width: 0  }";
            document.querySelector("head").appendChild(style);
            inputElement.setAttribute("class", "___textarea");

            inputElement.style.border = "0";
            inputElement.style.padding = "0";


        }
        inputElement["inputListenerList"] = [];
        inputElement["blurListenerList"] = [];
        inputElement["selectionListenerList"] = [];

        let blur = () => {
            if (window.visualViewport && platform_type === "web") {
                window.visualViewport.removeEventListener('resize', keyboardResize);
                document.body.style.position = "";
            }
            let el = INPUT_ELEMENTS[ty];
            if (!el) {
                return;
            }
            for (let bi of el["blurListenerList"]) {
                bi({ value: el.value });
            }
            hidden(el);

            if (INPUT_ELEMENTS[ty] === inputElement) {
                inputElement = null;
            }
        };

        let input = () => {
            if (!inputElement) {
                return;
            }
            let value = inputElement.value;
            let flag = false;
            let charIndex = inputElement.selectionStart - 1;
            if (inputType === "number" && !selectionStart) {
                // 如果是number类型,并且没有开启中文输入法,此时input是一个字符一个字符响应
                const char = value[charIndex];
                if (char === ".") { // 处理小数点
                    if (lastValue.includes(".")) { // 只能有一个小数点
                        flag = true;
                    }
                } else if (char === "-") {
                    if (lastValue.includes("-") || inputElement.selectionStart !== 1) {  // 减号只能有一个并且只能在首位
                        flag = true;
                    }
                } else if (isNaN(Number(char))) {
                    flag = true;  // 不是数字
                }
            }
            if (flag) {
                value = value.substring(0, inputElement.selectionStart - 1) + value.substring(inputElement.selectionStart);
                inputElement.value = value;
                setSelection(charIndex);
            }
            lastValue = value;
            for (let bi of inputElement["inputListenerList"]) {
                bi({ value });
            }
            // tex.innerText = "input";
        };
        const click = () => {
            if (clickCb) {
                clickCb();
            }
        }
        inputElement.onblur = blur;
        inputElement.oninput = input;
        inputElement.onclick = click;

        if (inputType === "number") {
            inputElement.addEventListener('compositionstart', (event) => {
                selectionStart = inputElement.selectionStart;  // 记录当中文输入法开始的时候光标的位置
            });

            // 如果是number类型,需要监听compositionend事件(中文输入法),在end事件中处理无效的字符
            inputElement.addEventListener('compositionend', (event) => {
                let data = (<any>event).data;  // 按空格之后的输入中文文本内容,此文本不是中文就算英文字符,都属于无效字符
                let value = inputElement.value;
                value = value.substring(0, selectionStart) + value.substring(selectionStart + data.length);
                console.log("========== end value ", value);
                inputElement.value = value;
                setSelection(selectionStart);
                selectionStart = undefined;
                setTimeout(() => {
                    for (let bi of inputElement["inputListenerList"]) {
                        bi({ value: inputElement.value });
                    }
                    for (let f of inputElement["selectionListenerList"]) {
                        f(inputElement.selectionStart);
                    }
                }, 16);


            });
        }

    }
    config = {
        ...DEFAULT_CFG,
        ...config,
    };
    inputElement = INPUT_ELEMENTS[ty];
    for (let key in config.attribute) {
        if (key === "type") continue;  // textarea type属性只读
        inputElement[key] = config.attribute[key];
    }
    // if (inputElement instanceof HTMLTextAreaElement) {
    if (config.width) {
        inputElement.style.width = config.width + "px";
    }
    if (config.height) {
        inputElement.style.height = config.height + "px";
    }
    if (config.fontSize) {
        inputElement.style.fontSize = config.fontSize + "px";
    }
    if (config.fontFamily) {
        inputElement.style.fontFamily = config.fontFamily;
    }
    // }
    if (config.textAlign) {
        inputElement.style.textAlign = config.textAlign;
    }
    inputElement["attributeList"] = config.attribute;
    inputElement.style.visibility = "visible";
    lastValue = "";
    if (config.defaultValue !== undefined) {
        inputElement.value = config.defaultValue;
        lastValue = config.defaultValue;
    }
    const platform_type = Env.get("platform_type")
    if (window.visualViewport && platform_type === "web") {
        window.visualViewport.addEventListener('resize', keyboardResize);
    }
    const isApple = navigator.userAgent.match(/iPhone|iPad|iPod/i) ? true : false;
    const systemInfo = Env.get('system_info');
    const isMobile = systemInfo?.device?.indexOf("mobile") > -1;
    if (isApple && isMobile) {
        inputElement.focus();
    } else {
        if (window.parent !== window) {
            // iframe嵌套的时候,不知道为啥调用focus后input会马上失焦
            // 这里延迟一会就不会。具体原因未知
            setTimeout(() => {
                inputElement.focus();
            }, 200);
        } else {
            inputElement.focus();
        }
    }
    // 根据配置和当前平台决定是否显示input输入框
    let showInputBox = Env.get("showInputBox");
    let device = Env.get("device");
    if (device && device.type === "pc") {
        showInputBox = false;
    } else if (showInputBox === undefined) {
        showInputBox = false; // or true?
    }
    document.body.style.height = `${window.innerHeight}px`;
    if (showInputBox) {
        inputElement.style.top = `0px`;
        inputElement.style.opacity = "1";
    } else {
        inputElement.style.left = config.left + "px";
        inputElement.style.top = config.top + "px";
        // inputElement.style.left = "0px";
        // inputElement.style.top = "0px";
        inputElement.style.width = config.width + "px";
        inputElement.style.height = config.height + "px";
        inputElement.style.opacity = "0";
        // inputElement.style.top = "-1000px";
        // inputElement.style.top= "0px";
    }


}

// 隐藏键盘
export const hide: Thide = (options: { fail?: Function, success?: Function }) => {
    if (!inputElement) {
        options.fail && options.fail();
        return;
    }
    hidden(inputElement);
    inputElement = null;
    options.success && options.success()

    // tex.innerText = "to input";
}
// wx.hideKeyboard;


export const update: Tupdate = (options: { value: string, success?: Function, fail?: Function, complete?: (res: any) => void }) => {
    if (!inputElement) {
        return;
    }
    inputElement.value = options.value;
    options.success && options.success(options.value);
    options.complete && options.complete(options.value);
    setTimeout(() => {
        for (let f of inputElement["selectionListenerList"]) {
            f(inputElement.selectionStart);
        }
    }, 16);
}

/**
 * 设置光标位置（html支持，其他平台可能不支持，不支持的平台，请返回false）
 * @param index 
 */
export const setSelection: TsetSelection = (index: number): boolean => {
    if (!inputElement) {
        return false;
    }
    // 如果type=number,则不能调用此方法,会抛异常,所以需要自己处理number类型
    inputElement.setSelectionRange(index, index);
    clickCb = () => {
        // 有可能input的点击时间后于gui事件,导致设置光标后会被点击事件重置成标准input的光标位置,与GUI的光标位置不一致
        inputElement.setSelectionRange(index, index);
        clickCb = null;
    }
    setTimeout(() => {
        for (let f of inputElement["selectionListenerList"]) {
            f(index);
        }
    }, 30);
    return true;
}




export const supportSelection: TsupportSelection = () => {
    return true;
}

/**
 * 监听光标改变
 * @param callback 
 * @param 
 * @returns 
 */
export const onSelection: TonSelection = (callback: (value: number) => void) => {
    inputElement["selectionListenerList"].push(callback);
}

export const offSelection: ToffSelection = (callback?: (value: number) => void) => {
    if (!inputElement) return;
    if (callback) {
        const list: Function[] = inputElement["selectionListenerList"];
        const index = list.indexOf(callback);
        if (index >= 0) {
            list.splice(index, 1);
        }
    } else {
        inputElement["selectionListenerList"] = [];
    }
}

export const onInput: TonInput = (callback: ({ value: string }) => void) => {
    inputElement["inputListenerList"].push(callback);
}

export const offInput: ToffInput = (callback?: ({ value: string }) => void) => {
    if (!inputElement) return;
    if (callback) {
        const list: Function[] = inputElement["inputListenerList"];
        const index = list.indexOf(callback);
        if (index >= 0) {
            list.splice(index, 1);
        }
    } else {
        inputElement["inputListenerList"] = [];
    }

}


export const onConfirm: TonConfirm = (callback: ({ value: string }) => void) => {
    // inputElement.addEventListener("change", () => {
    // 	callback({value: inputElement.value});
    // });
}


export const offConfirm: ToffConfirm = (callback?: ({ value: string }) => void) => {
    // TODO
}

export const onComplete: TonComplete = (callback: ({ value: string }) => void) => {
    inputElement["blurListenerList"].push(callback);
}

export const offComplete: ToffComplete = (callback?: ({ value: string }) => void) => {
    if (callback) {
        const list: Function[] = inputElement["blurListenerList"];
        const index = list.indexOf(callback);
        if (index >= 0) {
            list.splice(index, 1);
        }
    } else {
        inputElement["blurListenerList"] = [];
    }
}

let keyboardHeightChangeFuncs: Set<Function>;         // onKeyboardHeightChange监听回调


/**
 * 监听 虚拟键盘 高度，仅限于 支持 虚拟键盘 的 平台 
 */
export const onKeyboardHeightChange: TonKeyboardHeightChange = (callback: ({ height: number }) => void) => {
    if (!keyboardHeightChangeFuncs) {
        keyboardHeightChangeFuncs = new Set;

    }
    keyboardHeightChangeFuncs.add(callback);
}

/**
 * 取消监听 虚拟键盘 高度，仅限于 支持 虚拟键盘 的 平台.参数为空，则取消所有的事件监听。
 */
export const offKeyboardHeightChange: ToffKeyboardHeightChange = (callback?: ({ height: number }) => void) => {
    if (!keyboardHeightChangeFuncs) return;
    if (!callback) {
        return keyboardHeightChangeFuncs.clear();
    }
    keyboardHeightChangeFuncs.delete(callback);
}


/**
 * 监听键盘按键按下事件，仅适用于 PC 平台
 * @param callback 
 */
export const onKeyDown: TonKeyDown = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    window.addEventListener("keydown", callback);
}

/**
 *  取消监听键盘按键按下事件，仅适用于 PC 平台
 * @param callback 
 */
export const offKeyDown: ToffKeyDown = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    window.removeEventListener("keydown", callback);
};


/**
 * 监听键盘按键弹起事件，仅适用于 PC 平台
 * @param callback 
 */
export const onKeyUp: TonKeyUp = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    window.addEventListener("keyup", callback);
}

/**
 *  取消监听键盘按键弹起事件，仅适用于 PC 平台
 * @param callback 
 */
export const offKeyUp: ToffKeyUp = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    window.removeEventListener("keyup", callback);
};




const hidden = (inputElement: HTMLInputElement | HTMLTextAreaElement) => {
    if (!inputElement) {
        return;
    }

    // inputElement.onblur = null;
    // inputElement.onchange = null;
    // inputElement.oninput = null;
    inputElement["blurListenerList"].length = 0;
    inputElement["inputListenerList"].length = 0;
    inputElement["selectionListenerList"] = [];
    inputElement.style.visibility = "hidden";
    inputElement.style.top = "-1000px";
    // tex.innerText = "hidden input";

    for (let key in inputElement["attributeList"]) {
        if (key === "type") continue
        inputElement[key] = inputElement["attributeList"][key];
    }
    inputElement["attributeList"] = undefined;
    // alert("hidden input");
}

window.addEventListener("keydown", (e) => {
    if (inputElement &&
        inputElement["selectionListenerList"].length > 0 &&
        (e.code === "ArrowUp" ||
            e.code === "ArrowRight" ||
            e.code === "ArrowDown" ||
            e.code === "ArrowLeft" || e.code === "Home" || e.code === "End" || e.code === "Delete")
    ) {
        let funs = inputElement["selectionListenerList"];
        // 异步通知，因为此时selectionStart还未改变
        setTimeout(() => {
            if (!inputElement) {
                return;
            }

            // 监听列表已经改变，不在是异步前的input， 则不通知
            if (inputElement["selectionListenerList"] !== funs) {
                return;
            }

            for (let f of inputElement["selectionListenerList"]) {
                f(inputElement.selectionStart);
            }
        }, 0);
    }
});


const initialViewportHeight = window.visualViewport?.height;
const keyboardResize = () => {
    let timer;
    // ios浏览器如果input输入框被输入框遮挡,会自动上移页面让input在可视区域内,这里固定不让页面上移
    document.body.style.position = "fixed";
    if (timer) {
        clearTimeout(timer);
    }
    timer = setTimeout(() => {
        if (keyboardHeightChangeFuncs) {
            const newHeight = window.visualViewport.height;
            const changeHeight = initialViewportHeight - newHeight;
            keyboardHeightChangeFuncs.forEach(cb => {
                cb && cb({ height: changeHeight });
            })
        }
    }, 150);

}