import { IParamPiFont, IPiFont } from "pi_sys/render/font";

declare var FontFace;

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

/**
 * 字体资源
 */
export class PiFont implements IPiFont {
    /**
     * 加载参数
     */
    public arg: IParamPiFont;
    /**
     * @hidden
     */
    private _family: string;
    /**
     * 字体名
     * @access
     */
    public get family() {
        return this._family;
    }
    /**
     * @hidden
     */
    private _src: string;
    /**
     * 字体文件路径
     * @access
     */
    public get src() {
        return this._src;
    }
    /**
     * 字体文件路径
     * 
     */
    public set src(value: string) {
        this._src = value;

        if (FontFace) {
            logger.warn(`#Version update# Font loading parameters adjusted to IParamPiFont: {family:"xx"}`);

            const font = new FontFace(this.arg.family, `url(${this._src})`);

            this._family = this.arg.family;
            // 添加到全局的 FontFaceSet 中
            (document as any).fonts.add(font);
            Promise.race([font.load(), timeout(5000)]).then((r) => {
                if (r === TIMEOUT_RESOLVE) {
                    // 超时都没有加载，视为错误
                    // 为了解决手机兼容性的一个Bug：OPPO PBDM00 使用浏览器FontFace加载字体的时候有极小情况下promise没有任何响应,既不走then,也不走reject；
                    this.onerror && this.onerror("loadFont failed: wait long time");
                } else {
                    this.onload && this.onload();
                }
            }).catch((err) => {
                (document as any).fonts.delete(font);
                this.onerror && this.onerror(err);
            })
        }
        else {
            if (this.onerror) {
                this.onerror(`Operating environment does not support FontFace!`);
            }
        }
    }
    public onload: Function;
    public onerror: Function;
};

const TIMEOUT_RESOLVE = "timeout";

const timeout = (ms: number): Promise<string> => {
    return new Promise((resolve) => {
        setTimeout(() => {
            resolve(TIMEOUT_RESOLVE);
        }, ms);
    });
}