import { ICfg } from "pi_sys/update/update_mgr";
import { getUpdateLevel, UpdateLevel } from "pi_sys/update/version";
import { Env } from "pi_sys/util/env";
import { Store } from "pi_sys/store/store";


/**
 * 这里只处理大版本更新,中小版本更新交由项目处理
 */
export namespace UpdateMgr {
    let cfg: ICfg;
    let oldAppVersion: string;

    export const init = (_cfg: ICfg) => {
        cfg = _cfg;
        if (window.localStorage && window.localStorage.getItem) { // 鸿蒙webview没有localStorage
            oldAppVersion = window.localStorage.getItem("appVersion");
        }

    }

    /**
     * 是否是大版本更新
     * @returns 
     */
    export const isMajorUpdate = () => {
        if (!oldAppVersion) {
            if (window.localStorage && window.localStorage.getItem && cfg) { // 鸿蒙webview没有localStorage
                window.localStorage.setItem("appVersion", cfg.appVersion);
            }
            return false;
        }
        if (!cfg) {
            return false;
        } else {
            return getUpdateLevel(oldAppVersion, cfg.appVersion) === UpdateLevel.Major;
        }
    }

    /**
     * 清除本地数据
     */
    export const clearLocalData = () => {
        const dbName = Env.get("name");
        return Store.deleteStore(dbName);
    }

    /**
     * 开始更新
     */
    export const startUpdate = () => {
        if (window.localStorage && window.localStorage.getItem) { // 鸿蒙webview没有localStorage
            window.localStorage.setItem("appVersion", cfg.appVersion);
        }
        return Promise.resolve();
    }


}
