/**
 * App 相关接口
 */

import { ChannelInfo, FlagAppLife, ISdkInfo, QuickChannel, TclearCache, TcloseWebView, TexitApp, TexitPointerLock, TgetApiVersion, TgetChannelInfo, TgetChannelQuick, TgetFlavorName, TgetIdfa, TgetIdfv, TgetIpAdress, TgetPackageName, TgetProcessId, TgetRunconfigValue, TgetSdkInfo, TgetTemperature, TgetVersionCode, TgetVersionName, ThideFullScreenPopupWindow, TinstallApk, ToffBlur, ToffDestroy, ToffHide, ToffPointerLockChange, ToffPointerLockError, ToffShow, TonBlur, TonDestroy, TonHide, TonPointerLockChange, TonPointerLockError, TonShow, TopenAppstore, TopenWebView, TprivacyAgreement, TregisterAppLifeListener, TregisterSoftInputChangedListener, TrequestPointerLock, TresetUserInfo, TrestartApp, TshowWithEntranceId, TshowWithEntranceIdLogin, TtriggerGC, TunregisterAppLifeListener, TunregisterSoftInputChangedListener, TupdateUserInfo } from "pi_sys/native/base/app";
import { Native } from "../core/index";
import { getAssetFile } from "../../util/js_intercept";

import { Logger } from "pi_sys/util/logger";
import { CompressAlgorithm, EncryptType } from "pi_sys/util/constant";
declare var module;
let logger = new Logger(module.name, "sys");

export const exitApp: TexitApp = () => {
    Native.App.init().then(() => {
        Native.App.exitApp();
    });
}

export const getProcessId: TgetProcessId = (): Promise<string> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getProcessId(
                function (id: number) {
                    resolve(id.toString());
                },
                reject
            );
        }).catch(reject);
    });
}

export const getApiVersion: TgetApiVersion = (): Promise<string> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getApiVersion(
                function (version: string) {
                    resolve(version);
                },
                reject
            );
        }).catch(reject);
    });
}

export const getPackageName: TgetPackageName = (): Promise<string> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getPackageName(
                function (pName: string) {
                    resolve(pName);
                },
                reject
            );
        }).catch(reject);
    });
}

export const getFlavorName: TgetFlavorName = (): Promise<string> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getFlavorName(
                function (pName: string) {
                    resolve(pName);
                },
                reject
            );
        }).catch(reject);

    });
}

export const getVersionName: TgetVersionName = (): Promise<string> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getVersionName(
                function (pName) {
                    resolve(pName);
                },
                reject
            );
        }).catch(reject);

    });
}

export const getVersionCode: TgetVersionCode = (): Promise<number> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getVersionCode(
                function (pName: string) {
                    resolve((<any>pName) - 0);
                },
                reject
            );
        }).catch(reject);

    });
}

export const getRunconfigValue: TgetRunconfigValue = (key: string): Promise<string> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getRunconfigValue(
                key,
                function (pName: string) {
                    resolve(pName);
                },
                reject
            );
        }).catch(reject);

    });
}

export const getChannelInfo: TgetChannelInfo = (): Promise<ChannelInfo> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getChannelInfo(
                function (pName: string) {
                    let info = {
                        channel: "",
                        param: {}
                    };
                    try {
                        if (typeof pName === "object") {
                            info = pName;    // IOS返回对象
                        } else if (pName !== "") {
                            info = JSON.parse(pName);
                        }
                    } catch (_) {
                    }
                    resolve(info);
                },
                reject
            );
        }).catch(reject);

    });
}

/**
 * 取 App 安装的SDK的 JSON 配置
 * 配置的格式：
 */
export const getSdkInfo: TgetSdkInfo = (): Promise<ISdkInfo> => {
    return new Promise((resolve) => {
        getAssetFile("sdk_config.json", EncryptType.None, "", CompressAlgorithm.NONE, (content) => {
            if (content) {
                let c = content as string;
                let info = JSON.parse(c);
                resolve(info);
            } else {
                logger.warn("getSdkInfo failed, can't find sdk_config.json in assets/");
                resolve({
                    list: [],
                });
            }
        }, true);
    });
}
export const triggerGC: TtriggerGC = () => { }


/**
 * 打开webView
 * @param url 
 * @param params 
 * @returns 
 */
export const openWebView: TopenWebView = (url: string, params?: Object) => {
    const obj = { url };
    if (params) {
        Object.assign(obj, params);
    }
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.openWebView(JSON.stringify(obj),
                function (tag: string) {
                    resolve(tag);
                },
                reject
            );
        }).catch(reject);

    });
};

export const closeWebView: TcloseWebView = (tag: string) => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.closeWebView(tag,
                resolve,
                reject
            );
        }).catch(reject);

    });
};


export const openAppstore: TopenAppstore = () => {
    Native.App.init().then(() => {
        Native.App.openAppstore();
    })
};


// 获取CPU温度,单位摄氏度,保留小数点后一位
export const getTemperature: TgetTemperature = () => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getTemperature(
                function (str: string) {
                    resolve(str ? Number(str) : 0);
                },
                reject
            );
        }).catch(reject);

    });
};


// 获取idfv
export const getIdfv: TgetIdfv = () => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getIdfv(
                function (str: string) {
                    resolve(str);
                },
                reject
            );
        }).catch(reject);

    });
};

// 获取IpAdress
export const getIpAdress: TgetIpAdress = () => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getIpAdress(
                function (str: string) {
                    resolve(str);
                },
                reject
            );
        }).catch(reject);

    });
};

// 获取Idfa
export const getIdfa: TgetIdfa = () => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getIdfa(
                function (str: string) {
                    resolve(str);
                },
                reject
            );
        }).catch(reject);

    });
};


/**
 * 获取quick渠道分发出来的渠道
 */
export const getChannelQuick: TgetChannelQuick = () => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getChannelQuick(
                function (str: string) {
                    resolve(str as QuickChannel);
                },
                reject
            );
        }).catch(reject);

    });
};

export const privacyAgreement: TprivacyAgreement = (isAccepted: boolean) => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.privacyAgreement(isAccepted,
                function () {
                    resolve();
                },
                reject
            );
        }).catch(reject);

    });
}

export const restartApp: TrestartApp = () => {
    Native.App.init().then(() => {
        Native.App.restartApp();
    });
};


export const clearCache: TclearCache = () => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.clearCache(resolve, reject);
        }).catch(reject);

    });
};

export const requestPointerLock: TrequestPointerLock = () => {
};

export const exitPointerLock: TexitPointerLock = () => {
};

export const onPointerLockError: TonPointerLockError = (callback: () => void) => {
};

export const offPointerLockError: ToffPointerLockError = (callback?: () => void) => {

};

export const onPointerLockChange: TonPointerLockChange = (callback: (locked: boolean) => void) => {

};

export const offPointerLockChange: ToffPointerLockChange = (callback?: (locked: boolean) => void) => {

};

export const installApk: TinstallApk = (version: string) => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.installApk(version,
                resolve,
                reject
            );
        }).catch(reject);

    });
}


let funcs: Set<Function>;         // registerAppLifeListener监听回调
let showFuncs: Set<Function>;     // onShow监听回调
let hideFuncs: Set<Function>;     // onHide监听回调
let destroyFuncs: Set<Function>;  // onDestroy监听回调
let runFunc: Function;            // 实际注册回调函数
let cbID: number;           // NativeHook监听回调ID

export const registerAppLifeListener: TregisterAppLifeListener = (callback: (flag: FlagAppLife) => void) => {
    if (!funcs) {
        funcs = new Set;
    }
    funcs.add(callback);
    __register();
}


export const unregisterAppLifeListener: TunregisterAppLifeListener = (callback?: (flag: FlagAppLife) => void) => {
    if (!funcs) return;
    if (callback) {
        funcs.delete(callback);
    } else {
        funcs.clear();
    }
    if (!funcs.size) {
        funcs = undefined;
        __unRegister();
    }
}


export const onShow: TonShow = (callback: () => void) => {
    if (!showFuncs) {
        showFuncs = new Set;
    }
    showFuncs.add(callback);
    __register();
};

export const onHide: TonHide = (callback: () => void) => {
    if (!hideFuncs) {
        hideFuncs = new Set;
    }
    hideFuncs.add(callback);
    __register();
};

export const onDestroy: TonDestroy = (callback: () => void) => {
    if (!destroyFuncs) {
        destroyFuncs = new Set;
    }
    destroyFuncs.add(callback);
    __register();
};

export const offShow: ToffShow = (callback?: () => void) => {
    if (!showFuncs) return;
    if (callback) {
        showFuncs.delete(callback);
    } else {
        showFuncs.clear();
    }
    if (!showFuncs.size) {
        showFuncs = undefined;
        __unRegister();
    }
};

export const offHide: ToffHide = (callback?: () => void) => {
    if (!hideFuncs) return;
    if (callback) {
        hideFuncs.delete(callback);
    } else {
        hideFuncs.clear();
    }
    if (!hideFuncs.size) {
        hideFuncs = undefined;
        __unRegister();
    }
};

export const offDestroy: ToffDestroy = (callback?: () => void) => {
    if (!destroyFuncs) return;
    if (callback) {
        destroyFuncs.delete(callback);
    } else {
        destroyFuncs.clear();
    }
    if (!destroyFuncs.size) {
        destroyFuncs = undefined;
        __unRegister();
    }
};


const __register = () => {
    if (runFunc) return;
    runFunc = (flag: FlagAppLife) => {
        if (flag === "onResume") {
            showFuncs && showFuncs.forEach(func => func && func());
        } else if (flag === "onPause") {
            hideFuncs && hideFuncs.forEach(func => func && func());
        } else { // "onDestroy"
            destroyFuncs && destroyFuncs.forEach(func => func && func());
        }
        funcs && funcs.forEach(func => func && func(flag));
    }
    Native.App.init().then(() => {
        const id = Native.App.registerAppLifeListener(runFunc as any, undefined);
        cbID = id;
    });
}

const __unRegister = () => {
    if (funcs || showFuncs || hideFuncs || destroyFuncs) return;
    Native.App.init().then(() => {
        if (cbID) {
            Native.App.unRegisterAppLifeListener(cbID);
            cbID = undefined;
            console.log("free unRegisterAppLifeListener");
        }
    });
    runFunc = undefined;
}



let inputFuncs: Set<Function>;         // registerSoftInputChangedListener监听回调
let inputRunFunc: Function;            // registerSoftInputChangedListener实际注册回调函数
let inputCbID: number;           // NativeHook监听回调ID


/**
 * 注册软件盘弹出监听
 * @param callback 
 */
export const registerSoftInputChangedListener: TregisterSoftInputChangedListener = (callback: (height: number) => void) => {
    if (!inputFuncs) {
        inputFuncs = new Set;
    }
    inputFuncs.add(callback);
    if (inputRunFunc) return;
    inputRunFunc = (height: number) => {
        inputFuncs && inputFuncs.forEach(func => func && func(height));
    }
    Native.App.init().then(() => {
        const id = Native.App.registerSoftInputChangedListener(inputRunFunc as any, undefined);
        inputCbID = id;
    });
};

/**
 * 取消软件盘弹出监听
 * @param callback 
 */
export const unregisterSoftInputChangedListener: TunregisterSoftInputChangedListener = (callback?: (height: number) => void) => {
    if (!inputFuncs) return;
    if (callback) {
        inputFuncs.delete(callback);
    } else {
        inputFuncs.clear();
    }
    if (!inputFuncs.size) {
        inputFuncs = undefined;
        Native.App.init().then(() => {
            if (inputCbID) {
                Native.App.unregisterSoftInputChangedListener(inputCbID);
                inputCbID = undefined;
            }
        });
        inputRunFunc = undefined;
    }

};

let hasHideFullScreen = false;
export const hideFullScreenPopupWindow: ThideFullScreenPopupWindow = () => {
    if (hasHideFullScreen) return;
    hasHideFullScreen = true;
    Native.App.init().then(() => {
        Native.App.hideFullScreenPopupWindow();
    });
}


/**
 * 监听APP失焦事件
 * @note 仅web和exe支持
 */
export const onBlur: TonBlur = (callback: () => void) => { };


/**
 * 取消 监听APP失焦事件
 * @note 仅web和exe支持
 */
export const offBlur: ToffBlur = (callback: () => void) => { };


/**
 * 唤起客服
 */
export const showWithEntranceId: TshowWithEntranceId = () => {
    Native.App.init().then(() => {
        Native.App.showWithEntranceId();
    });
}

/**
 * 客服后台用户信息
 * @returns 
 */
export const updateUserInfo: TupdateUserInfo = (str: string) => {
    Native.App.init().then(() => {
        Native.App.updateUserInfo(str);
    });
}

/**
 * 清空用户信息
 */
export const resetUserInfo: TresetUserInfo = () => {
    Native.App.init().then(() => {
        Native.App.resetUserInfo();
    });
}


/**
 * 唤起客服
 */
export const showWithEntranceIdLogin: TshowWithEntranceIdLogin = () => {
    Native.App.init().then(() => {
        Native.App.showWithEntranceIdLogin();
    });
}