import { LogLevel, TnotifySendLog, TsetConsoleLogLevel, TsetLocalLogLevel, TsetLogLocal } from "pi_sys/native/base/log";
import { Native } from "../core/index";

/**
 * 读取单个落地日志文件的内容,并上传到服务器.
 * @param indexOfLogFile 日志文件索引(0表示最新日志,1表示上一天日志文件..)
 * @param userName  标识符(后台用来区分是那个用户的日志,建议填写用户名)
 */
export const notifySendLog: TnotifySendLog = (indexOfLogFile: number, userName: string) => {
    return new Promise((resolve, reject) => {
        Native.Log.init().then(() => {
            Native.Log.notifySendLog(
                indexOfLogFile,
                userName,
                function () {
                    resolve(null);
                },
                reject
            );
            return undefined;
        })
            .catch((e) => {
                reject(e);
            });
    });
}

/**
 * 设置日志落地的最低等级,包含该等级.
 * @param level 
 */
export const setLocalLogLevel: TsetLocalLogLevel = (level: LogLevel) => {
    return Native.Log.init().then(() => {
        Native.Log.setLocalLogLevel(level);
    })
}

/**
 * 设置AndroidStudio控制台日志显示的最低等级,包含该等级.
 * @param level 
 * @returns 
 */
export const setConsoleLogLevel: TsetConsoleLogLevel = (level: LogLevel) => {
    return Native.Log.init().then(() => {
        Native.Log.setConsoleLogLevel(level);
    })
}

/**
 * 是否开启日志落地
 * @param isOpen 
 * @returns 
 */
export const setLogLocal: TsetLogLocal = (isOpen: boolean) => {
    return Native.Log.init().then(() => {
        Native.Log.setLogLocal(isOpen);
    })
}