import { IDownloadFileOption, IUpdateOption, TdownloadFile, TupdateApk, TupdateSnapshot, TupdateSo, TupdateSys, TrestartApp } from "pi_sys/native/base/update";
import { Native } from "../core/index";


/**
 * 下载文件到指定目录
 */
export const downloadFile: TdownloadFile = (option: IDownloadFileOption) => {
    Native.Download.init().then(() => {
        let successCbID, errorCbID, progressCbID;
        const unregister = () => {
            successCbID && Native.Download.unregisterCb(successCbID);
            errorCbID && Native.Download.unregisterCb(errorCbID);
            progressCbID && Native.Download.unregisterCb(progressCbID);
        }
        successCbID = Native.Download.registerCb(() => {
            unregister();
            option.onsucceed && option.onsucceed();
        });
        errorCbID = Native.Download.registerCb((e) => {
            unregister();
            option.onerror && option.onerror(e);
        });
        progressCbID = Native.Download.registerCb((total: number, progress: number) => {
            option.onprogress && option.onprogress(total, progress);
        });
        Native.Download.downloadFile(successCbID, errorCbID, progressCbID, option);
    });
}

/**
 * 更新apk
 */
export const updateApk: TupdateApk = (option: IUpdateOption) => {
    Native.Download.init().then(() => {
        let successCbID, errorCbID, progressCbID;
        const unregister = () => {
            successCbID && Native.Download.unregisterCb(successCbID);
            errorCbID && Native.Download.unregisterCb(errorCbID);
            progressCbID && Native.Download.unregisterCb(progressCbID);
        }
        successCbID = Native.Download.registerCb(() => {
            unregister();
            option.onsucceed && option.onsucceed();
        });
        errorCbID = Native.Download.registerCb((e) => {
            unregister();
            option.onerror && option.onerror(e);
        });
        progressCbID = Native.Download.registerCb((total: number, progress: number) => {
            option.onprogress && option.onprogress(total, progress);
        });
        Native.Download.updateApk(successCbID, errorCbID, progressCbID, option);
    });
}

/**
 * 更新so
 */
export const updateSo: TupdateSo = (option: IUpdateOption) => {
    Native.Download.init().then(() => {
        let successCbID, errorCbID, progressCbID;
        const unregister = () => {
            successCbID && Native.Download.unregisterCb(successCbID);
            errorCbID && Native.Download.unregisterCb(errorCbID);
            progressCbID && Native.Download.unregisterCb(progressCbID);
        }
        successCbID = Native.Download.registerCb(() => {
            unregister();
            option.onsucceed && option.onsucceed();
        });
        errorCbID = Native.Download.registerCb((e) => {
            unregister();
            option.onerror && option.onerror(e);
        });
        progressCbID = Native.Download.registerCb((total: number, progress: number) => {
            option.onprogress && option.onprogress(total, progress);
        });
        Native.Download.updateSo(successCbID, errorCbID, progressCbID, option);
    });
}


/**
 * 更新快照
 */
export const updateSnapshot: TupdateSnapshot = (option: IUpdateOption) => {
    Native.Download.init().then(() => {
        let successCbID, errorCbID, progressCbID;
        const unregister = () => {
            successCbID && Native.Download.unregisterCb(successCbID);
            errorCbID && Native.Download.unregisterCb(errorCbID);
            progressCbID && Native.Download.unregisterCb(progressCbID);
        }
        successCbID = Native.Download.registerCb(() => {
            unregister();
            option.onsucceed && option.onsucceed();
        });
        errorCbID = Native.Download.registerCb((e) => {
            unregister();
            option.onerror && option.onerror(e);
        });
        progressCbID = Native.Download.registerCb((total: number, progress: number) => {
            option.onprogress && option.onprogress(total, progress);
        });
        Native.Download.updateSnapshot(successCbID, errorCbID, progressCbID, option);
    });
}


/**
 * 更新pisys
 */
export const updateSys: TupdateSys = (option: IUpdateOption) => {
    Native.Download.init().then(() => {
        let successCbID, errorCbID, progressCbID;
        const unregister = () => {
            successCbID && Native.Download.unregisterCb(successCbID);
            errorCbID && Native.Download.unregisterCb(errorCbID);
            progressCbID && Native.Download.unregisterCb(progressCbID);
        }
        successCbID = Native.Download.registerCb(() => {
            unregister();
            option.onsucceed && option.onsucceed();
        });
        errorCbID = Native.Download.registerCb((e) => {
            unregister();
            option.onerror && option.onerror(e);
        });
        progressCbID = Native.Download.registerCb((total: number, progress: number) => {
            option.onprogress && option.onprogress(total, progress);
        });
        Native.Download.updateSys(successCbID, errorCbID, progressCbID, option);
    });

}


/**
 * 重启虚拟机
 */
export const restartApp: TrestartApp = () => {
    if ((<any>window).onBeforeRestartApp) {
        (<any>window).onBeforeRestartApp();
    }
    Native.Download.init().then(() => {
        Native.Download.restartApp();
    });
}