
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit, nativeHookCallNoRes } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace Accelerometer {
    const className = 'Accelerometer';

    const paramCfg: NativeHook.FuntionParameCfg = {
        register: [3],
        unregister: []
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 注册监听
     * interval - 频率，如果值无效，返回错误，有效值如下： "normal": 200 ms/次(默认) "ui": 60 ms/次 "game": 20 ms/次
     */
    export function register(interval: string, cbID: number, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'register';

        const paramList: [string] = [interval];

        if (uniqueID === undefined) {

            logger.warn('Accelerometer not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID, cbID);
        }
    }

    /**
     * 取消注册过的监听器.
     */
    export function unregister(cbID: number) {

        const funcName = 'unregister';

        const paramList: [] = [];

        if (uniqueID === undefined) {

            logger.warn('Accelerometer not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID, cbID)
        }
    }

}
