
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookInit, nativeHookCallNoRes } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export class Audio {
    public readonly className = 'Audio';

    public promise: Promise<void> | undefined;
    public uniqueID: number | undefined;
    public isInited: boolean = false;
    public isClosed: boolean = false;

    private paramCfg: NativeHook.FuntionParameCfg = {
        initAudio: [3],
        play: [],
        pause: [],
        seek: [1],
        stop: [],
        destroy: [],
        registerCanplay: [],
        unregisterCanplay: [],
        registerEnded: [],
        unregisterEnded: [],
        registerError: [],
        unregisterError: [],
        registerPlay: [],
        unregisterPlay: [],
        registerPause: [],
        unregisterPause: [],
        registerStop: [],
        unregisterStop: [],
        registerSeeking: [],
        unregisterSeeking: [],
        registerSeeked: [],
        unregisterSeeked: [],
        registerTimeUpdate: [],
        unregisterTimeUpdate: [],
        registerBufferUpdate: [],
        unregisterBufferUpdate: [],
        registerWaiting: [],
        unregisterWaiting: []
    };

    public error(msg: string) {
        throw new Error(msg);
    }

    public fail(msg: string) {
        logger.warn(msg);
    }

    constructor() {
        this.init();
    }

    public dispose() {
        this.close();
    }

    private init() {
        let funcName = 'init';
        if (this.promise === undefined) {
            this.promise = nativeHookInit(this.className, funcName, []).then(r => {
                this.uniqueID = r;
            }).catch((msg) => {
                this.promise = undefined;
                logger.warn(msg);
            })
        }

        return this.promise;
    }

    private close() {
        let funcName = 'close';
        if (this.uniqueID !== undefined) {
            this.isClosed = true;
            this.uniqueID = undefined;

            NativeHook.call(
                this.className,
                funcName,
                this.uniqueID,
                {
                    success: (arg: string) => {
                    },
                    fail: (msg: string) => {
                        logger.warn(msg);
                    },
                    error: this.error
                },
                []
            );
        }

        this.promise = Promise.reject("Has Closed.");
    }

    public BufferUpdateListenList: Function[] | undefined;
    public _onBufferUpdate(info: any) {
        this.BufferUpdateListenList && this.BufferUpdateListenList.forEach((f) => { f && f(info); });
    }
    /**
     * onBufferUpdate
     */
    public onBufferUpdate(cb: () => void) {
        if (!this.BufferUpdateListenList) {
            this.BufferUpdateListenList = [];
            this.registerBufferUpdate(this._onBufferUpdate, this.fail);
        }

        if (!this.BufferUpdateListenList.includes(cb)) {
            this.BufferUpdateListenList.push(cb);
        }
    }
    /**
     * offBufferUpdate
     */
    public offBufferUpdate(cb: () => void) {
        if (!this.BufferUpdateListenList) {
            return;
        }
        this.BufferUpdateListenList.splice(this.BufferUpdateListenList.indexOf(cb));
    }

    public CanplayListenList: Function[] | undefined;
    public _onCanplay(info: any) {
        this.CanplayListenList && this.CanplayListenList.forEach((f) => { f && f(info); });
    }
    /**
     * onCanplay
     */
    public onCanplay(cb: () => void) {
        if (!this.CanplayListenList) {
            this.CanplayListenList = [];
            this.registerCanplay(this._onCanplay, this.fail);
        }

        if (!this.CanplayListenList.includes(cb)) {
            this.CanplayListenList.push(cb);
        }
    }
    /**
     * offCanplay
     */
    public offCanplay(cb: () => void) {
        if (!this.CanplayListenList) {
            return;
        }
        this.CanplayListenList.splice(this.CanplayListenList.indexOf(cb));
    }

    public EndedListenList: Function[] | undefined;
    public _onEnded(info: any) {
        this.EndedListenList && this.EndedListenList.forEach((f) => { f && f(info); });
    }
    /**
     * onEnded
     */
    public onEnded(cb: () => void) {
        if (!this.EndedListenList) {
            this.EndedListenList = [];
            this.registerEnded(this._onEnded, this.fail);
        }

        if (!this.EndedListenList.includes(cb)) {
            this.EndedListenList.push(cb);
        }
    }
    /**
     * offEnded
     */
    public offEnded(cb: () => void) {
        if (!this.EndedListenList) {
            return;
        }
        this.EndedListenList.splice(this.EndedListenList.indexOf(cb));
    }

    public ErrorListenList: Function[] | undefined;
    public _onError(info: any) {

    }
    /**
     * onError
     */
    public onError(cb: (errMsg: any, errCode: any) => void) {
        if (!this.ErrorListenList) {
            this.ErrorListenList = [];
            this.registerError(this._onError, this.fail);
        }

        if (!this.ErrorListenList.includes(cb)) {
            this.ErrorListenList.push(cb);
        }
    }
    /**
     * offError
     */
    public offError(cb: (errMsg: any, errCode: any) => void) {
        if (!this.ErrorListenList) {
            return;
        }
        this.ErrorListenList.splice(this.ErrorListenList.indexOf(cb));
    }

    public PauseListenList: Function[] | undefined;
    public _onPause(info: any) {
        this.PauseListenList && this.PauseListenList.forEach((f) => { f && f(info); });
    }
    /**
     * onPause
     */
    public onPause(cb: () => void) {
        if (!this.PauseListenList) {
            this.PauseListenList = [];
            this.registerPause(this._onPause, this.fail);
        }

        if (!this.PauseListenList.includes(cb)) {
            this.PauseListenList.push(cb);
        }
    }
    /**
     * offPause
     */
    public offPause(cb: () => void) {
        if (!this.PauseListenList) {
            return;
        }
        this.PauseListenList.splice(this.PauseListenList.indexOf(cb));
    }

    public PlayListenList: Function[] | undefined;
    public _onPlay(info: any) {
        this.PlayListenList && this.PlayListenList.forEach((f) => { f && f(info); });
    }
    /**
     * onPlay
     */
    public onPlay(cb: () => void) {
        if (!this.PlayListenList) {
            this.PlayListenList = [];
            this.registerPlay(this._onPlay, this.fail);
        }

        if (!this.PlayListenList.includes(cb)) {
            this.PlayListenList.push(cb);
        }
    }
    /**
     * offPlay
     */
    public offPlay(cb: () => void) {
        if (!this.PlayListenList) {
            return;
        }
        this.PlayListenList.splice(this.PlayListenList.indexOf(cb));
    }

    public SeekedListenList: Function[] | undefined;
    public _onSeeked(info: any) {
        this.SeekedListenList && this.SeekedListenList.forEach((f) => { f && f(info); });
    }
    /**
     * onSeeked
     */
    public onSeeked(cb: () => void) {
        if (!this.SeekedListenList) {
            this.SeekedListenList = [];
            this.registerSeeked(this._onSeeked, this.fail);
        }

        if (!this.SeekedListenList.includes(cb)) {
            this.SeekedListenList.push(cb);
        }
    }
    /**
     * offSeeked
     */
    public offSeeked(cb: () => void) {
        if (!this.SeekedListenList) {
            return;
        }
        this.SeekedListenList.splice(this.SeekedListenList.indexOf(cb));
    }

    public SeekingListenList: Function[] | undefined;
    public _onSeeking(info: any) {
        this.SeekingListenList && this.SeekingListenList.forEach((f) => { f && f(info); });
    }
    /**
     * onSeeking
     */
    public onSeeking(cb: () => void) {
        if (!this.SeekingListenList) {
            this.SeekingListenList = [];
            this.registerSeeking(this._onSeeking, this.fail);
        }

        if (!this.SeekingListenList.includes(cb)) {
            this.SeekingListenList.push(cb);
        }
    }
    /**
     * offSeeking
     */
    public offSeeking(cb: () => void) {
        if (!this.SeekingListenList) {
            return;
        }
        this.SeekingListenList.splice(this.SeekingListenList.indexOf(cb));
    }

    public StopListenList: Function[] | undefined;
    public _onStop(info: any) {
        this.StopListenList && this.StopListenList.forEach((f) => { f && f(info); });
    }
    /**
     * onStop
     */
    public onStop(cb: () => void) {
        if (!this.StopListenList) {
            this.StopListenList = [];
            this.registerStop(this._onStop, this.fail);
        }

        if (!this.StopListenList.includes(cb)) {
            this.StopListenList.push(cb);
        }
    }
    /**
     * offStop
     */
    public offStop(cb: () => void) {
        if (!this.StopListenList) {
            return;
        }
        this.StopListenList.splice(this.StopListenList.indexOf(cb));
    }

    public TimeUpdateListenList: Function[] | undefined;
    public _onTimeUpdate(info: any) {
        this.TimeUpdateListenList && this.TimeUpdateListenList.forEach((f) => { f && f(info); });
    }
    /**
     * onTimeUpdate
     */
    public onTimeUpdate(cb: () => void) {
        if (!this.TimeUpdateListenList) {
            this.TimeUpdateListenList = [];
            this.registerTimeUpdate(this._onTimeUpdate, this.fail);
        }

        if (!this.TimeUpdateListenList.includes(cb)) {
            this.TimeUpdateListenList.push(cb);
        }
    }
    /**
     * offTimeUpdate
     */
    public offTimeUpdate(cb: () => void) {
        if (!this.TimeUpdateListenList) {
            return;
        }
        this.TimeUpdateListenList.splice(this.TimeUpdateListenList.indexOf(cb));
    }

    public WaitingListenList: Function[] | undefined;
    public _onWaiting(info: any) {
        this.WaitingListenList && this.WaitingListenList.forEach((f) => { f && f(info); });
    }
    /**
     * onWaiting
     */
    public onWaiting(cb: () => void) {
        if (!this.WaitingListenList) {
            this.WaitingListenList = [];
            this.registerWaiting(this._onWaiting, this.fail);
        }

        if (!this.WaitingListenList.includes(cb)) {
            this.WaitingListenList.push(cb);
        }
    }
    /**
     * offWaiting
     */
    public offWaiting(cb: () => void) {
        if (!this.WaitingListenList) {
            return;
        }
        this.WaitingListenList.splice(this.WaitingListenList.indexOf(cb));
    }



    /**
     * 初始化MediaPlayer并且播放.(调用该方法是任何回调要生效的前提).
     * @param src - 要播放的文件路径
     */
    public initAudio(src: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'initAudio';

        const paramList: [string] = [src];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 播放.
     */
    public play(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'play';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 暂停。暂停后的音频再播放会从暂停处开始播放.
     */
    public pause(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'pause';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 跳转的时间，单位 s。 精确到小数点后 3 位，即支持 ms 级别精确度
     */
    public seek(position: number, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'seek';

        const paramList: [number] = [position];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 停止。停止后的音频再播放会从头开始播放。
     */
    public stop(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'stop';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 销毁当前实例
     */
    public destroy() {

        const funcName = 'destroy';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频进入可以开始播放状态的事件。 不保证后面可以流畅播放
     */
    public registerCanplay(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerCanplay';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频进入可以播放状态的事件。
     */
    public unregisterCanplay() {

        const funcName = 'unregisterCanplay';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频自然播放至结束的事件
     */
    public registerEnded(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerEnded';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频自然播放至结束的事件
     */
    public unregisterEnded() {

        const funcName = 'unregisterEnded';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频播放错误事件
     */
    public registerError(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerError';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频播放错误事件
     */
    public unregisterError() {

        const funcName = 'unregisterError';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频播放事件
     */
    public registerPlay(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerPlay';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频播放事件
     */
    public unregisterPlay() {

        const funcName = 'unregisterPlay';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频播放暂停事件
     */
    public registerPause(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerPause';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频播放事件
     */
    public unregisterPause() {

        const funcName = 'unregisterPause';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频停止事件
     */
    public registerStop(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerStop';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频停止事件
     */
    public unregisterStop() {

        const funcName = 'unregisterStop';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频进行跳转操作的事
     */
    public registerSeeking(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerSeeking';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频进行跳转操作的事
     */
    public unregisterSeeking() {

        const funcName = 'unregisterSeeking';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频完成跳转事件
     */
    public registerSeeked(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerSeeked';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频完成跳转事件
     */
    public unregisterSeeked() {

        const funcName = 'unregisterSeeked';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频播放进度更新事件(回调在子线程)
     */
    public registerTimeUpdate(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerTimeUpdate';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频播放进度更新事件
     */
    public unregisterTimeUpdate() {

        const funcName = 'unregisterTimeUpdate';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频播放进度更新事件
     */
    public registerBufferUpdate(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerBufferUpdate';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频播放进度更新事件
     */
    public unregisterBufferUpdate() {

        const funcName = 'unregisterBufferUpdate';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

    /**
     * 监听音频加载中事件
     */
    public registerWaiting(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerWaiting';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 取消监听音频加载中事件
     */
    public unregisterWaiting() {

        const funcName = 'unregisterWaiting';

        const paramList: [] = [];

        if (this.uniqueID === undefined) {

            logger.warn('Audio not init');
        }
        else {
            nativeHookCallNoRes(paramList, this.paramCfg, this.className, funcName, this.uniqueID)
        }
    }

}
