
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace AutoTest {
    const className = 'AutoTest';

    const paramCfg: NativeHook.FuntionParameCfg = {
        isEnable: [],
        registerRunScript: [3, 3],
        getAttr: [3, 3, 1]
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = new Promise((resolve, reject) => {
                NativeHook.call(
                    className,
                    funcName,
                    0,
                    {
                        success: (arg: string) => {
                            const params: [number] = JSON.parse(arg);
                            uniqueID = params[0];

                            resolve();
                        },
                        fail: (msg: string) => {
                            promise = undefined;
                            logger.warn(msg);
                            reject(msg);
                        }
                    },
                    paramList
                );
            });
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    export function isEnable(succCall: (enable: boolean) => void, failCall: (msg: string) => void) {

        const funcName = 'isEnable';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('AutoTest not init');
        } else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    export function registerRunScript(
        sdkName: string,
        scriptName: string,
        queryFunc: (info: any) => void
    ) {

        const funcName = 'registerRunScript';

        const paramList = [sdkName, scriptName];

        if (uniqueID === undefined) {
            logger.warn('AutoTest not init');
        }
        else {
            nativeHookCall(queryFunc, (msg: string) => {
                logger.debug("============= autotest, runScript error, msg = ", msg)
            }, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    export function getAttr(
        sdkName: string,
        result: string,
        id: number
    ) {
        const funcName = 'getAttr';

        const paramList = [sdkName, result, id];

        if (uniqueID === undefined) {
            logger.warn('AutoTest not init');
        }
        else {
            nativeHookCall((arg: string) => {
                logger.debug("============= autotest getAttr callback ok");
            }, (msg: string) => {
                logger.debug("============= autotest getAttr callback err");
            }, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }
}