import { NativeHook } from "./nativehook";

/**
 * 初始化底层接口
 * @param className 
 * @param funcName 
 * @param paramList 
 * @returns 
 */
export const nativeHookInit = (className: string, funcName: string, paramList: any[]): Promise<number> => {
    return new Promise((resolve, reject) => {
        NativeHook.call(
            className,
            funcName,
            0,
            {
                success: (arg: string) => {
                    const params: [number] = JSON.parse(arg);
                    let uniqueID = params[0];

                    resolve(uniqueID);
                },
                fail: (msg: string) => {
                    reject(msg);
                }
            },
            paramList
        );
    });
}

/**
 * 调用底层方法
 * @param succCall 
 * @param failCall 
 * @param error 
 * @param paramList 
 * @param paramCfg 
 * @param className 
 * @param funcName 
 * @param uniqueID 
 * @param cbID 
 * @returns 
 */
export const nativeHookCall = (succCall: Function, failCall: Function, error: Function, paramList: any[], paramCfg: NativeHook.FuntionParameCfg, className: string, funcName: string, uniqueID: number, cbID?: number) => {
    const cbListen = {
        success: (arg: string) => {
            const params: [any] = JSON.parse(arg);

            succCall && succCall(...params);
        },
        fail: (msg: string) => {
            failCall && failCall(msg);
        },
        error
    };
    if (NativeHook.checkParams(paramList, paramCfg[funcName])) {
        if (cbID) NativeHook.removeNativeListen(cbID);
        return NativeHook.call(
            className,
            funcName,
            uniqueID,
            cbListen,
            paramList
        );
    }
}



export const nativeHookFakeCall = (succCall: Function, failCall: Function, error: Function, cbID?: number) => {
    const cbListen = {
        success: (arg: string) => {
            const params: [any] = JSON.parse(arg);

            succCall && succCall(...params);
        },
        fail: (msg: string) => {
            failCall && failCall(msg);
        },
        error
    };
    if (cbID) NativeHook.removeNativeListen(cbID);
    return NativeHook.fakeCall(cbListen);
}


export const removeNativeHookFakeCall = (cbID: number) => {
    NativeHook.removeNativeListen(cbID);
}


/**
 * 不关心结果的调用
 * @param succCall 
 * @param failCall 
 * @param paramList 
 * @param className 
 * @param funcName 
 * @param uniqueID 
 */
export const nativeHookCallNoRes = (paramList: any[], paramCfg: NativeHook.FuntionParameCfg, className: string, funcName: string, uniqueID: number, cbID?: number) => {
    const cbListen = undefined;
    if (NativeHook.checkParams(paramList, paramCfg[funcName])) {
        if (cbID) {
            NativeHook.removeNativeListen(cbID);
        }
        NativeHook.call(
            className,
            funcName,
            uniqueID,
            cbListen,
            paramList
        );
    }
}

/**
 * 关闭底层接口
 * @param succCall 
 * @param failCall 
 * @param paramList 
 * @param className 
 * @param funcName 
 * @param uniqueID 
 */
export const nativeHookClose = (succCall: Function, failCall: Function, paramList: any[], className: string, funcName: string, uniqueID: number) => {
    NativeHook.call(
        className,
        funcName,
        uniqueID,
        {
            success: () => {
                succCall && succCall();
            },
            fail: (msg: string) => {
                failCall && failCall(msg);
            }
        },
        paramList
    );
}
