
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit, nativeHookCallNoRes } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace Battery {
    const className = 'Battery';

    const paramCfg: NativeHook.FuntionParameCfg = {
        getLevel: [],
        isCharging: [],
        registerBatteryLevel: [],
        unRegisterBatteryLevel: [],
        registerBatteryCharge: [],
        unRegisterBatteryCharge: []
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 获取设备当前电量.
     */
    export function getLevel(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'getLevel';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Battery not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 设备是否正在充电中.
     */
    export function isCharging(succCall: (isIng: boolean) => void, failCall: (msg: string) => void) {

        const funcName = 'isCharging';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Battery not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 监听电量变化.电量每变化1%回调一次.
     */
    export function registerBatteryLevel(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerBatteryLevel';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Battery not init');
        }
        else {
            return nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 取消监听电量变化.
     */
    export function unRegisterBatteryLevel(cbID: number) {

        const funcName = 'unRegisterBatteryLevel';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Battery not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID, cbID)
        }
    }

    /**
     * 监听充放电变化.
     */
    export function registerBatteryCharge(succCall: (isIng: boolean) => void, failCall: (msg: string) => void) {

        const funcName = 'registerBatteryCharge';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Battery not init');
        }
        else {
            return nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 取消监听充放电.
     */
    export function unRegisterBatteryCharge(cbID: number) {

        const funcName = 'unRegisterBatteryCharge';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Battery not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID, cbID)
        }
    }

}
