
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export class Camera {
    public readonly className = 'Camera';

    public promise: Promise<void> | undefined;
    public uniqueID: number | undefined;
    public isInited: boolean = false;
    public isClosed: boolean = false;

    private paramCfg: NativeHook.FuntionParameCfg = {
        openPhoto: [2],
        openRecord: [2]
    };

    public error(msg: string) {
        throw new Error(msg);
    }

    public fail(msg: string) {
        logger.warn(msg);
    }

    constructor() {
        this.init();
    }

    public dispose() {
        this.close();
    }

    private init() {
        let funcName = 'init';
        if (this.promise === undefined) {
            this.promise = new Promise((resolve, reject) => {
                NativeHook.call(
                    this.className,
                    funcName,
                    0,
                    {
                        success: (arg: string) => {
                            const params: [number] = JSON.parse(arg);
                            this.uniqueID = params[0];

                            resolve();
                        },
                        fail: (msg: string) => {
                            this.promise = undefined;
                            logger.warn(msg);
                            reject(msg);
                        },
                        error: this.error
                    },
                    []
                );
            });
        }

        return this.promise;
    }

    private close() {
        let funcName = 'close';
        if (this.uniqueID !== undefined) {
            this.isClosed = true;
            this.uniqueID = undefined;

            NativeHook.call(
                this.className,
                funcName,
                this.uniqueID,
                {
                    success: (arg: string) => {
                    },
                    fail: (msg: string) => {
                        logger.warn(msg);
                    },
                    error: this.error
                },
                []
            );
        }

        this.promise = Promise.reject("Has Closed.");
    }



    /**
     * 打开拍照相机.
     */
    public openPhoto(isCompress: boolean, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'openPhoto';

        const paramList: [boolean] = [isCompress];

        if (this.uniqueID === undefined) {
            logger.warn('Camera not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

    /**
     * 打开录像机.
     */
    public openRecord(isCompress: boolean, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'openRecord';

        const paramList: [boolean] = [isCompress];

        if (this.uniqueID === undefined) {
            logger.warn('Camera not init');
        }
        else {
            nativeHookCall(succCall, failCall, this.error, paramList, this.paramCfg, this.className, funcName, this.uniqueID);
        }
    }

}
