
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace Clipboard {
    const className = 'Clipboard';

    const paramCfg: NativeHook.FuntionParameCfg = {
        set: [3],
        get: []
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 设置剪贴板内容. 调用成功后，会弹出 toast 提示"内容已复制"，持续 1.5s.
     */
    export function set(data: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'set';

        const paramList: [string] = [data];

        if (uniqueID === undefined) {
            logger.warn('Clipboard not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 获取剪贴板的数据，仅文本.
     */
    export function get(succCall: (text: string) => void, failCall: (msg: string) => void) {

        const funcName = 'get';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Clipboard not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

}
