
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit, nativeHookCallNoRes } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace Compass {
    const className = 'Compass';

    const paramCfg: NativeHook.FuntionParameCfg = {
        register: [],
        unregister: []
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 监听罗盘数据变化事件。频率：5 次/秒，
     * 接口调用后会自动开始监听，可使用 wx.stopCompass 停止监听。 
     * 到微信实验，不调用start，直接调用onChange会如何。(也会触发监听) 
     * 到微信实验，调用了stop后，再调用start，会不会继续触发原有的监听器。(不会触发监听)
     */
    export function register(succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'register';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Compass not init');
        }
        else {
            return nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 取消注册过的监听器
     */
    export function unregister(cbID: number) {

        const funcName = 'unregister';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Compass not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID, cbID)
        }
    }

}
