
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit, nativeHookCallNoRes } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace DeviceOrientation {
    const className = 'DeviceOrientation';

    const paramCfg: NativeHook.FuntionParameCfg = {
        register: [],
        unregister: []
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 注册监听器.
     * + portrait 竖屏 + landscape 横屏正方向，以 HOME 键在屏幕右侧为正方向 + landscapeReverse 横屏反方向，以 HOME 键在屏幕左侧为反方向
     */
    export function register(succCall: (flag: string) => void, failCall: (msg: string) => void) {

        const funcName = 'register';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('DeviceOrientation not init');
        }
        else {
            return nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 取消注册过的监听器.
     */
    export function unregister(cbID: number) {

        const funcName = 'unregister';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('DeviceOrientation not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID, cbID)
        }
    }

}
