
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit, nativeHookCallNoRes, nativeHookFakeCall, removeNativeHookFakeCall } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace Download {
    const className = 'Download';

    const paramCfg: NativeHook.FuntionParameCfg = {
        downloadFile: [1, 1, 1, 4],
        updateApk: [1, 1, 1, 4],
        updateSo: [1, 1, 1, 4],
        updateSnapshot: [1, 1, 1, 4],
        updateSys: [1, 1, 1, 4],
        restartApp: []
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 下载文件.
     */
    export function downloadFile(successCbID: number, errorCbID: number, progressCbId: number, option: any) {

        const funcName = 'downloadFile';

        const paramList: [number, number, number, any] = [successCbID, errorCbID, progressCbId, option];

        if (uniqueID === undefined) {

            logger.warn('Download not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID)
        }
    }

    /**
     * 更新apk
     * @param successCbID 
     * @param errorCbID 
     * @param progressCbId 
     * @param option 
     */
    export function updateApk(successCbID: number, errorCbID: number, progressCbId: number, option: any) {

        const funcName = 'updateApk';

        const paramList: [number, number, number, any] = [successCbID, errorCbID, progressCbId, option];

        if (uniqueID === undefined) {

            logger.warn('Download not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID)
        }
    }

    /**
     * 更新so
     * @param successCbID 
     * @param errorCbID 
     * @param progressCbId 
     * @param option 
     */
    export function updateSo(successCbID: number, errorCbID: number, progressCbId: number, option: any) {

        const funcName = 'updateSo';

        const paramList: [number, number, number, any] = [successCbID, errorCbID, progressCbId, option];

        if (uniqueID === undefined) {

            logger.warn('Download not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID)
        }
    }

    /**
     * 更新快照
     * @param successCbID 
     * @param errorCbID 
     * @param progressCbId 
     * @param option 
     */
    export function updateSnapshot(successCbID: number, errorCbID: number, progressCbId: number, option: any) {

        const funcName = 'updateSnapshot';

        const paramList: [number, number, number, any] = [successCbID, errorCbID, progressCbId, option];

        if (uniqueID === undefined) {

            logger.warn('Download not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID)
        }
    }


    /**
     * 更新sys
     * @param successCbID 
     * @param errorCbID 
     * @param progressCbId 
     * @param option 
     */
    export function updateSys(successCbID: number, errorCbID: number, progressCbId: number, option: any) {

        const funcName = 'updateSys';

        const paramList: [number, number, number, any] = [successCbID, errorCbID, progressCbId, option];

        if (uniqueID === undefined) {

            logger.warn('Download not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID)
        }
    }

    export function restartApp() {

        const funcName = 'restartApp';

        const paramList: [] = [];

        if (uniqueID === undefined) {

            logger.warn('Download not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID)
        }
    }


    export function registerCb(succCall: Function) {
        return nativeHookFakeCall(succCall, undefined, error);
    }

    export function unregisterCb(cbID: number) {
        return removeNativeHookFakeCall(cbID);
    }

}
