
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace Image {
    const className = 'Image';

    const paramCfg: NativeHook.FuntionParameCfg = {
        choose: [1, 2, 2],
        preview: [1, 3],
        saveToPhotosAlbum: [3]
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 从本地相册选择图片或使用相机拍照。
     * @param maxSelectNum - 最多可以选择的图片张数
     * @param isCompress - 是否压缩
     * @param isGallery - 是否从相册选(true相册;false拍照)
     */
    export function choose(maxSelectNum: number, isCompress: boolean, isGallery: boolean, succCall: (list: any) => void, failCall: (msg: string) => void) {

        const funcName = 'choose';

        const paramList: [number, boolean, boolean] = [maxSelectNum, isCompress, isGallery];

        if (uniqueID === undefined) {
            logger.warn('Image not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 在新页面中全屏预览图片。预览的过程中用户可以进行保存图片。//发送给朋友等操作(暂未实现)
     * @param position - 当前显示图片的索引
     * @param jsonUrlArr - 需要预览的图片链接数组对应的Json(android8必须https)
     */
    export function preview(position: number, jsonUrlArr: string, succCall: (flag: boolean) => void, failCall: (msg: string) => void) {

        const funcName = 'preview';

        const paramList: [number, string] = [position, jsonUrlArr];

        if (uniqueID === undefined) {
            logger.warn('Image not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 保存图片到系统相册.
     * @param path - 图片文件路径，可以是临时文件路径或永久文件路径 (本地路径) ，不支持网络路径
     */
    export function saveToPhotosAlbum(path: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'saveToPhotosAlbum';

        const paramList: [string] = [path];

        if (uniqueID === undefined) {
            logger.warn('Image not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

}
