
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace Login {
    const className = 'Login';

    const paramCfg: NativeHook.FuntionParameCfg = {
        login: [3],
        logout: [3],
        registerLoginCb: [3],
        unRegisterLoginCb: [3],
        registerLogoutCb: [3],
        unRegisterLogoutCb: [3],
        registerAccountSwitchCb: [3],
        unRegisterAccountSwitchCb: [3]
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;

            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 启动三方登录.
     * @param sdkName 三方平台名称 - "qq","wx","weibo"
     */
    export function login(sdkName: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'login';

        const paramList: [string] = [sdkName];

        if (uniqueID === undefined) {
            logger.warn('Login not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);;
        }
    }

    /**
     * 退出三方登录.
     * @param sdkName 三方平台名称 - "qq","wx","weibo"
     */
    export function logout(sdkName: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'logout';

        const paramList: [string] = [sdkName];

        if (uniqueID === undefined) {
            logger.warn('Login not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);;
        }
    }

    /**
      * 注册三方登录的监听
      * @param sdkName 三方平台名称
      */
    export function registerLoginCb(sdkName: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerLoginCb';

        const paramList: [string] = [sdkName];

        if (uniqueID === undefined) {
            logger.warn('Login not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);;
        }
    }

    /**
     * 注销三方登录的监听
     * @param sdkName 三方平台名称
     */
    export function unRegisterLoginCb(sdkName: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'unRegisterLoginCb';

        const paramList: [string] = [sdkName];

        if (uniqueID === undefined) {
            logger.warn('Login not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);;
        }
    }

    /**
     * 注册三方登录的监听
     * @param sdkName 三方平台名称
     */
    export function registerLogoutCb(sdkName: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerLogoutCb';

        const paramList: [string] = [sdkName];

        if (uniqueID === undefined) {
            logger.warn('Login not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);;
        }
    }

    /**
     * 注销 三方登出 的监听
     * @param sdkName 三方平台名称
     */
    export function unRegisterLogoutCb(sdkName: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'unRegisterLogoutCb';

        const paramList: [string] = [sdkName];

        if (uniqueID === undefined) {
            logger.warn('Login not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);;
        }
    }


    /**
      * 注册三方登录的监听
      * @param sdkName 三方平台名称
      */
    export function registerAccountSwitchCb(sdkName: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerAccountSwitchCb';

        const paramList: [string] = [sdkName];

        if (uniqueID === undefined) {
            logger.warn('Login not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);;
        }
    }

    /**
     * 注销三方登录的监听
     * @param sdkName 三方平台名称
     */
    export function unRegisterAccountSwitchCb(sdkName: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'unRegisterAccountSwitchCb';

        const paramList: [string] = [sdkName];

        if (uniqueID === undefined) {
            logger.warn('Login not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);;
        }
    }
}


