
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit, nativeHookCallNoRes } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace MemoryWarning {
    const className = 'MemoryWarning';

    const paramCfg: NativeHook.FuntionParameCfg = {
        register: []
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 发出内存警告时，触发该事件。触发该事件不意味小程序被杀，大部分情况下仅仅是告警 开发者可在收到通知后回收一些不必要资源避免进一步加剧内存紧张。
     * TRIM_MEMORY_RUNNING_CRITICAL(15)紧急 
     * TRIM_MEMORY_RUNNING_LOW(10)低的 
     * TRIM_MEMORY_RUNNING_MODERATE(5)适度
     */
    export function register(succCall: (level: number) => void, failCall: (msg: string) => void) {

        const funcName = 'register';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('MemoryWarning not init');
        }
        else {
            return nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }


    export function unRegister(cbID: number) {

        const funcName = 'unRegister';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('MemoryWarning not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID, cbID)
        }
    }

}
