
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit, nativeHookCallNoRes } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace Network {
    const className = 'Network';

    const paramCfg: NativeHook.FuntionParameCfg = {
        getType: [],
        registerNetworkState: [],
        unRegisterNetworkState: []
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 获取网络类型.
     * 回调泛型String.内容为网络类型,如下： "wifi", "2g", "3g", "4g", , "5g", "none", "unknown"
     */
    export function getType(succCall: (nType: string) => void, failCall: (msg: string) => void) {

        const funcName = 'getType';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Network not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 监听网络类型变化.
     */
    export function registerNetworkState(succCall: (nType: any) => void, failCall: (msg: string) => void) {

        const funcName = 'registerNetworkState';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Network not init');
        }
        else {
            return nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 取消监听网络类型变化.
     */
    export function unRegisterNetworkState(cbID: number) {

        const funcName = 'unRegisterNetworkState';

        const paramList: [] = [];

        if (uniqueID === undefined) {
            logger.warn('Network not init');
        }
        else {
            nativeHookCallNoRes(paramList, paramCfg, className, funcName, uniqueID, cbID)
        }
    }

}
