
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace Push {
    const className = 'Push';

    const paramCfg: NativeHook.FuntionParameCfg = {
        onPush: [3]
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 开始监听推送信息.
     * @param sdkName 三方平台名称 "aliyunpush"
     */
    export function onPush(sdkName: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'onPush';

        const paramList: [string] = [sdkName];

        if (uniqueID === undefined) {
            logger.warn('Push not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

}
