
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace Share {
    const className = 'Share';

    const paramCfg: NativeHook.FuntionParameCfg = {
        shareText: [3, 3],
        shareImage: [3, 3]
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 三方文字分享.
     * @param sdkName 三方平台名称 "qq","wx","weibo"
     * @param jsonConfig Json格式的请求参数
     * @example - qq - { text: , url: }
     * @example - wx - { text: }
     * @example - weibo - { text: }
     */
    export function shareText(sdkName: string, jsonConfig: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'shareText';

        const paramList: [string, string] = [sdkName, jsonConfig];

        if (uniqueID === undefined) {
            logger.warn('Share not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 三方图片分享.
     * @param sdkName 三方平台名称 "qq","wx","weibo"
     * @param jsonConfig Json格式的请求参数
     * @example - qq      - { path: 分享图片的路径 }
     * @example - wx      - { path: 分享图片的路径 }
     * @example - weibo   - { path: 分享图片的路径 }
     */
    export function shareImage(sdkName: string, jsonConfig: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'shareImage';

        const paramList: [string, string] = [sdkName, jsonConfig];

        if (uniqueID === undefined) {
            logger.warn('Share not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

}
