
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace ShareNative {
    const className = 'ShareNative';

    const paramCfg: NativeHook.FuntionParameCfg = {
        shareText: [3],
        shareImage: [3]
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 安卓源生文字分享.
     * @param text - 要分享的文字
     */
    export function shareText(text: string, succCall: (msg: string) => void, failCall: (msg: string) => void) {

        const funcName = 'shareText';

        const paramList: [string] = [text];

        if (uniqueID === undefined) {
            logger.warn('ShareNative not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

    /**
     * 安卓源生图片分享.
     * @param picPath - 要分享的图片路径
     */
    export function shareImage(picPath: string, succCall: (msg: string) => void, failCall: (msg: string) => void) {

        const funcName = 'shareImage';

        const paramList: [string] = [picPath];

        if (uniqueID === undefined) {
            logger.warn('ShareNative not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

}
