
import { NativeHook } from "./nativehook";

import { Logger } from "pi_sys/util/logger";
import { nativeHookCall, nativeHookClose, nativeHookInit } from "./base";
declare var module;
let logger = new Logger(module.name, "sys");

export namespace UpLoad {
    const className = 'UpLoad';

    const paramCfg: NativeHook.FuntionParameCfg = {
        upload: [3, 3]
    };

    let promise: Promise<void> | undefined;
    let uniqueID: number | undefined;
    let isInited: boolean = false;

    function error(msg: string) {
        throw new Error(msg);
    }

    /**
     * init
     */
    export function init() {

        const funcName = 'init';

        const paramList: [] = [];

        if (promise === undefined && uniqueID === undefined) {

            isInited = true;
            promise = nativeHookInit(className, funcName, paramList).then(r => {
                uniqueID = r;
            }).catch((msg) => {
                promise = undefined;
                logger.warn(msg);
            })
        }

        return promise;
    }

    /**
     * close
     */
    export function close() {

        const funcName = 'close';

        const paramList: [] = [];

        if (uniqueID !== undefined && isInited) {

            nativeHookClose(() => {
                uniqueID = undefined;
                isInited = false;
            }, (msg: string) => {
                logger.warn(msg);
            }, paramList, className, funcName, uniqueID);

            uniqueID = undefined;
        }
    }

    /**
     * 消息上报.
     * @param sdkName 三方平台名称
     * @param jsonConfig Json格式的请求参数
     * @example - tapdb - { ... } - 格式请参考：http://open.quicksdk.net/help/detail/aid/190
     * @tapdb - upload_type   - 上报类型(必传,可选值:user,name,level,server,charge,event)
     * @tapdb - user          - 用户ID(upload_type=user时必传)
     * @tapdb - name          - 用户名称(upload_type=name时必传
     * @tapdb - level         - 用户等级(upload_type=level时必传)
     * @tapdb - server        - 用户所在服务器(upload_type=server时必传)
     * @tapdb - amount        - 充值金额。单位分，即无论什么币种，都需要乘以100(upload_type=chare时必传)
     * @tapdb - orderId       - 订单ID(upload_type=chare时可传)
     * @tapdb - product       - 商品名称(upload_type=chare时可传)
     * @tapdb - currencyType  - 货币类型(upload_type=chare时可传)
     * @tapdb - payment       - 充值渠道(upload_type=chare时可传)
     * @tapdb - eventCode     - 在控制台中配置得到的事件编码(upload_type=event时必传)
     * @tapdb - properties    - 事件属性.需要和控制台的配置匹配(upload_type=event时可传)
     * @example - quicksdk - { ... }
     * @quicksdk - is_create_role     (必传,true创建角色,false进入游戏及角色升级)
     * @quicksdk - server_id          服务器ID(必传,数字字符串，不能含有中文字符).
     * @quicksdk - server_name        服务器名称(必传).
     * @quicksdk - game_role_name     游戏角色名称(必传).
     * @quicksdk - game_role_id       游戏角色ID(必传).
     * @quicksdk - game_balance       游戏角色用户余额(必传).
     * @quicksdk - vip_level          设置当前用户vip等级，必须为数字整型字符串,请勿传"vip1"等类似字符串(必传).
     * @quicksdk - game_user_level    充值渠道(upload_type=chare时可传)
     * @quicksdk - payment            设置游戏用户等级(必传)
     * @quicksdk - party_name         设置帮派名称(必传).
     * @quicksdk - role_create_time   UC，当乐与1881，TT渠道必传，值为10位数时间戳(必传).
     * @quicksdk - party_id           360渠道参数，设置帮派id，必须为整型字符串(必传)
     * @quicksdk - game_role_gender   360渠道参数(必传)
     * @quicksdk - game_role_power    360渠道参数(必传)
     * @quicksdk - party_role_id      360渠道参数(必传)
     * @quicksdk - party_role_name    360渠道参数，设置角色在帮派中的名称(必传)
     * @quicksdk - profession_id      360渠道参数，设置角色职业id，必须为整型字符串(必传)
     * @quicksdk - profession         360渠道参数，设置角色职业名称(必传)
     * @quicksdk - friend_list        360渠道参数，设置好友关系列表(必传)
     */
    export function upload(sdkName: string, jsonConfig: string, succCall: (info: any) => void, failCall: (msg: string) => void) {

        const funcName = 'upload';

        const paramList: [string, string] = [sdkName, jsonConfig];

        if (uniqueID === undefined) {
            logger.warn('UpLoad not init');
        }
        else {
            nativeHookCall(succCall, failCall, error, paramList, paramCfg, className, funcName, uniqueID);
        }
    }

}
