import { AccelerometerData, ToffChange, TonChange, Tstart, Tstop } from "pi_sys/native/device/accelerometer";
import { Native } from "../core/index";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");


let funcs: Set<Function>;      // 监听回调
let cbID: number;              // NativeHook监听回调ID
let oldInterval: string;       // 频率  

export const start: Tstart = (interval: string = 'normal') => {
    if (oldInterval === interval) return Promise.resolve();
    oldInterval = interval;
    return Native.Accelerometer.init().then(() => {
        const id = Native.Accelerometer.register(
            interval,
            cbID,
            (info: AccelerometerData) => {
                if (!funcs || !funcs.size) {
                    logger.warn(`Accelerometer started,But no callback Function ,Please call "stop" Function to improve performance!!!`);
                } else {
                    funcs.forEach(func => func && func(info))
                }
            },
            undefined
        );
        cbID = id;
    });

}


export const stop: Tstop = () => {
    if (!oldInterval) return Promise.resolve();
    oldInterval = undefined;
    return Native.Accelerometer.init().then(() => {
        Native.Accelerometer.unregister(cbID);
        cbID = undefined;
    });
}

export const onChange: TonChange = (callback: (info: AccelerometerData) => void) => {
    const _onChange = () => {
        if (!funcs) {
            funcs = new Set;
        }
        funcs.add(callback);
    }
    if (!oldInterval) {
        start().then(_onChange);
    } else {
        _onChange();
    }
}

export const offChange: ToffChange = (callback?: (info: AccelerometerData) => void) => {
    if (funcs) {
        if (callback) {
            funcs.delete(callback);
        } else {
            funcs.clear();
        }
        if (!funcs.size) funcs = undefined;
    }
}