import { BatteryLevel, TgetInfo, ToffChargingChange, ToffLevelChange, TonChargingChange, TonLevelChange } from "pi_sys/native/device/battery";
import { Native } from "../core/index";


export const getInfo: TgetInfo = () => {
    return Promise.all([getLevel(), isCharging()]).then(([info, isCharging]) => ({ level: info.percent, isCharging }));
}

let levelFuncs: Set<Function>;   // 监听回调
let levelCbID: number;           // NativeHook监听回调ID


export const onLevelChange: TonLevelChange = (callback: (level: BatteryLevel) => void) => {
    if (!levelFuncs) {
        levelFuncs = new Set;
        Native.Battery.init().then(() => {
            const id = Native.Battery.registerBatteryLevel(
                (info: any) => {
                    levelFuncs.forEach(func => func && func(info));
                },
                undefined
            );
            levelCbID = id;
        });
    }
    levelFuncs.add(callback);
}


export const offLevelChange: ToffLevelChange = (callback?: (level: BatteryLevel) => void) => {
    if (!levelFuncs) return;
    if (callback) {
        levelFuncs.delete(callback);
    } else {
        levelFuncs.clear();
    }
    if (!levelFuncs.size) {
        Native.Battery.init().then(() => {
            if (levelCbID) {
                Native.Battery.unRegisterBatteryLevel(levelCbID);
                levelCbID = undefined;
            }
        });
        levelFuncs = undefined;
    }

}


let chargingFuncs: Set<Function>;   // 监听回调
let chargingCbID: number;        // NativeHook监听回调ID

export const onChargingChange: TonChargingChange = (callback: (isCharging: boolean) => void) => {
    if (!chargingFuncs) {
        chargingFuncs = new Set;
        Native.Battery.init().then(() => {
            const id = Native.Battery.registerBatteryCharge(
                (info: any) => {
                    chargingFuncs.forEach(func => func && func(info));
                },
                undefined
            );
            chargingCbID = id;
        });
    }
    chargingFuncs.add(callback);
}


export const offChargingChange: ToffChargingChange = (callback?: (isCharging: boolean) => void) => {
    if (!chargingFuncs) return;
    if (callback) {
        chargingFuncs.delete(callback);
    } else {
        chargingFuncs.clear();
    }
    if (!chargingFuncs.size) {
        Native.Battery.init().then(() => {
            if (chargingCbID) {
                Native.Battery.unRegisterBatteryCharge(chargingCbID);
                chargingCbID = undefined;
            }
        });
        chargingFuncs = undefined;
    }
}



/**
 * 获取设备电量
 */
const getLevel = (): Promise<BatteryLevel> => {
    return new Promise((resolve, reject) => {
        Native.Battery.init().then(() => {
            Native.Battery.getLevel(
                (info) => {
                    resolve(info);
                },
                reject
            )
        });
    });
}

/**
 * 是否正在充电
 */
const isCharging = (): Promise<boolean> => {
    return new Promise((resolve, reject) => {
        Native.Battery.init().then(() => {
            Native.Battery.isCharging(
                (flag) => {
                    resolve(flag);
                },
                reject
            )
        });
    });
}