import { CompassData, ToffChange, TonChange, Tstart, Tstop } from "pi_sys/native/device/compass";
import { Native } from "../core/index";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

let funcs: Set<Function>;   // 监听回调
let cbID: number;        // NativeHook监听回调ID
let isStart: boolean = false;     // 记录是否已经start

export const start: Tstart = () => {
    if (isStart) return Promise.resolve();
    isStart = true;
    return Native.Compass.init().then(() => {
        const id = Native.Compass.register(
            (info: any) => {
                if (!funcs || !funcs.size) {
                    logger.warn(`Compass started,But no callback Function ,Please call "stop" Function to improve performance!!!`);
                } else {
                    funcs.forEach(func => func && func(info))
                }
            },
            undefined
        );
        cbID = id;
    });
}


export const stop: Tstop = () => {
    if (!isStart) return Promise.resolve();
    isStart = false;
    return Native.Compass.init().then(() => {
        Native.Compass.unregister(cbID);
        cbID = undefined;
    });
}


export const onChange: TonChange = (callback: (data: CompassData) => void) => {
    const _onChange = () => {
        if (!funcs) {
            funcs = new Set;
        }
        funcs.add(callback);
    }

    if (!isStart) {
        start().then(_onChange);
    } else {
        _onChange();
    }
}

export const offChange: ToffChange = (callback?: (info: CompassData) => void) => {
    if (funcs) {
        if (callback) {
            funcs.delete(callback);
        } else {
            funcs.clear();
        }
        if (!funcs.size) funcs = undefined;
    }
}