import { GyroscopeData, ToffChange, TonChange, Tstart, Tstop } from "pi_sys/native/device/gyroscope";
import { Native } from "../core/index";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

let funcs: Set<Function>;      // 监听回调
let cbID: number;              // NativeHook监听回调ID
let oldInterval: string;       // 频率  


export const start: Tstart = (interval: string = 'normal') => {
    if (oldInterval === interval) return Promise.resolve();
    oldInterval = interval;
    return Native.Gyroscope.init().then(() => {
        const id = Native.Gyroscope.register(
            interval,
            cbID,
            (info: GyroscopeData) => {
                if (!funcs || !funcs.size) {
                    logger.warn(`Gyroscope started,But no callback Function ,Please call "stop" Function to improve performance!!!`);
                } else {
                    funcs.forEach(func => func && func(info))
                }
            },
            undefined
        );
        cbID = id;
    });

}


export const stop: Tstop = () => {
    if (!oldInterval) return Promise.resolve();
    oldInterval = undefined;
    return Native.Gyroscope.init().then(() => {
        Native.Gyroscope.unregister(cbID);
        cbID = undefined;
    });
}

export const onChange: TonChange = (callback: (info: GyroscopeData) => void) => {
    if (!funcs) {
        funcs = new Set;
    }
    funcs.add(callback);
}

export const offChange: ToffChange = (callback?: (info: GyroscopeData) => void) => {
    if (funcs) {
        if (callback) {
            funcs.delete(callback);
        } else {
            funcs.clear();
        }
        if (!funcs.size) funcs = undefined;
    }
}