import { Env } from "pi_sys/util/env";
import { Native } from "../core/index";
import { ToffKeyboardHeightChange, TonKeyboardHeightChange } from "pi_sys/native/device/keyboard";
export * from "../../../web/native_impl/device/keyboard"


let inputFuncs: Set<Function>;         // registerSoftInputChangedListener监听回调
let inputRunFunc: Function;            // registerSoftInputChangedListener实际注册回调函数
let inputCbID: number;           // NativeHook监听回调ID


/**
 * 监听 虚拟键盘 高度，仅限于 支持 虚拟键盘 的 平台 
 * @param callback 
 * @note 等效于app模块中的registerSoftInputChangedListener
 */
export const onKeyboardHeightChange: TonKeyboardHeightChange = (callback: ({ height: number }) => void) => {
    if (!inputFuncs) {
        inputFuncs = new Set;
    }
    inputFuncs.add(callback);
    if (inputRunFunc) return;
    inputRunFunc = (height: number) => {
        const systemInfo = Env.get("system_info");
        const pixelRatio = systemInfo.pixelRatio;
        inputFuncs && inputFuncs.forEach(func => func && func({ height: height / pixelRatio }));
    }
    Native.App.init().then(() => {
        const id = Native.App.registerSoftInputChangedListener(inputRunFunc as any, undefined);
        inputCbID = id;
    });
};


/**
 * 取消监听 虚拟键盘 高度，仅限于 支持 虚拟键盘 的 平台.参数为空，则取消所有的事件监听
 * @param callback 
 * @note 等效于app模块中的unregisterSoftInputChangedListener
 */
export const offKeyboardHeightChange: ToffKeyboardHeightChange = (callback: ({ height: number }) => void) => {
    if (!inputFuncs) return;
    if (callback) {
        inputFuncs.delete(callback);
    } else {
        inputFuncs.clear();
    }
    if (!inputFuncs.size) {
        inputFuncs = undefined;
        Native.App.init().then(() => {
            if (inputCbID) {
                Native.App.unregisterSoftInputChangedListener(inputCbID);
                inputCbID = undefined;
            }
        });
        inputRunFunc = undefined;
    }

};

