import { LocationInfo, TgetLocation, ToffChange, TonChange, Tstart, Tstop } from "pi_sys/native/device/location";
import { Native } from "../core/index";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

export const getLocation: TgetLocation = (type: string, altitude: string, isHighAccuracy: boolean, highAccuracyExpireTime: number): Promise<LocationInfo> => {
    return new Promise((resolve, reject) => {
        Native.Location.init().then(() => {
            Native.Location.get(
                type,
                altitude,
                isHighAccuracy,
                highAccuracyExpireTime,
                (info: LocationInfo) => {
                    resolve(info);
                },
                reject
            );
        })
            .catch((e) => {
                reject(e);
            });
    });
}



let funcs: Set<Function>;   // 监听回调
let cbID: number;        // NativeHook监听回调ID
let isStart: boolean = false;     // 记录是否已经start

export const start: Tstart = () => {
    if (isStart) return Promise.resolve();
    isStart = true;
    return Native.Location.init().then(() => {
        // 因为微信小游戏的接口无法设置这些参数,所以为了统一,webview这些参数内部设置默认值
        const isHighAccuracy = false; // 需要高精度吗
        const interval = 1000;       // - 时间间隔，单位：ms
        const distance = 1;          // - 触发距离，单位：米
        const id = Native.Location.register(
            isHighAccuracy,
            interval,
            distance,
            (info: any) => {
                if (!funcs || !funcs.size) {
                    logger.warn(`Location started,But no callback Function ,Please call "stop" Function to improve performance!!!`);
                } else {
                    funcs.forEach(func => func && func(info))
                }
            },
            undefined
        );
        cbID = id;
    });
}


export const stop: Tstop = () => {
    if (!isStart) return Promise.resolve();
    isStart = false;
    return Native.Location.init().then(() => {
        Native.Location.unregister(cbID);
        cbID = undefined;
    });
}


export const onChange: TonChange = (callback: (data: LocationInfo) => void) => {
    if (!funcs) {
        funcs = new Set;
    }
    funcs.add(callback);
}

export const offChange: ToffChange = (callback?: (info: LocationInfo) => void) => {
    if (funcs) {
        if (callback) {
            funcs.delete(callback);
        } else {
            funcs.clear();
        }
        if (!funcs.size) funcs = undefined;
    }
}
