import { ToffWarning, TonWarning, TRIM_MEMORY_RUNNING_LEVEL } from "pi_sys/native/device/memorywarning";
import { Native } from "../core/index";


let funcs: Set<Function>;   // 监听回调
let cbID: number;        // NativeHook监听回调ID

export const onWarning: TonWarning = (callback: (data: TRIM_MEMORY_RUNNING_LEVEL) => void) => {
    if (!funcs) {
        funcs = new Set;
        Native.MemoryWarning.init().then(() => {
            const id = Native.MemoryWarning.register(
                (info: TRIM_MEMORY_RUNNING_LEVEL) => {
                    funcs.forEach(func => func && func(info));
                },
                undefined
            );
            cbID = id;
        });
    }
    funcs.add(callback);
}


export const offWarning: ToffWarning = (callback?: (data: TRIM_MEMORY_RUNNING_LEVEL) => void) => {
    if (!funcs) return;
    if (callback) {
        funcs.delete(callback);
    } else {
        funcs.clear();
    }
    if (!funcs.size) {
        Native.MemoryWarning.init().then(() => {
            if (cbID) {
                Native.MemoryWarning.unRegister(cbID);
                cbID = undefined;
            }
        });
        funcs = undefined;
    }
}
