
import { NetWrokType, TgetType, ToffStatusChange, TonStatusChange } from "pi_sys/native/device/network";
import { Native } from "../core/index";

export const getType: TgetType = (): Promise<NetWrokType> => {
    return new Promise((resolve, reject) => {
        Native.Network.init().then(() => {
            Native.Network.getType(
                (nType) => {
                    resolve(nType as any);
                },
                reject
            )
        })
            .catch((e) => {
                reject(e);
            });
    });
}


let funcs: Set<Function>;   // 监听回调
let cbID: number;        // NativeHook监听回调ID

export const onStatusChange: TonStatusChange = (callback: ({ isConnected: boolean, networkType: NetWrokType }) => void) => {
    if (!funcs) {
        funcs = new Set;
        Native.Network.init().then(() => {
            const id = Native.Network.registerNetworkState(
                (info: string) => {
                    funcs.forEach(func => func && func(info));
                },
                undefined
            );
            cbID = id;
        });
    }
    funcs.add(callback);
}


export const offStatusChange: ToffStatusChange = (callback?: ({ isConnected: boolean, networkType: NetWrokType }) => void) => {
    if (!funcs) return;
    if (callback) {
        funcs.delete(callback);
    } else {
        funcs.clear();
    }
    if (!funcs.size) {
        Native.Network.init().then(() => {
            if (cbID) {
                Native.Network.unRegisterNetworkState(cbID);
                cbID = undefined;
            }
        });
        funcs = undefined;
    }
}


