import { ToffChange, TonChange } from "pi_sys/native/device/orientation";
import { Native } from "../core/index";

let funcs: Set<Function>;   // 监听回调
let cbID: number;        // NativeHook监听回调ID

export const onChange: TonChange = (callback: (data: string) => void) => {
    if (!funcs) {
        funcs = new Set;
        Native.DeviceOrientation.init().then(() => {
            const id = Native.DeviceOrientation.register(
                (info: string) => {
                    funcs.forEach(func => func && func(info));
                },
                undefined
            );
            cbID = id;
        });
    }
    funcs.add(callback);
}


export const offChange: ToffChange = (callback?: (data: string) => void) => {
    if (!funcs) return;
    if (callback) {
        funcs.delete(callback);
    } else {
        funcs.clear();
    }
    if (!funcs.size) {
        Native.DeviceOrientation.init().then(() => {
            if (cbID) {
                Native.DeviceOrientation.unregister(cbID);
                cbID = undefined;
            }
        });
        funcs = undefined;
    }
}
