/**
 * 底层设备：屏幕
 */

import { ScreenInfo, TgetBrightness, ToffUserCaptureScreen, TonUserCaptureScreen, TsetBrightness, TsetKeepOn } from "pi_sys/native/device/screen";
import { Native } from "../core/index";

export const setBrightness: TsetBrightness = (value: number): Promise<undefined> => {
    return Native.Screen.init().then(() => {
        Native.Screen.setBrightness(value, undefined, undefined);
        return undefined;
    });
}

export const setKeepOn: TsetKeepOn = (keepScreenOn: boolean): Promise<undefined> => {
    return Native.Screen.init().then(() => {
        Native.Screen.setKeepOn(keepScreenOn, undefined, undefined);
        return undefined;
    });
}

export const getBrightness: TgetBrightness = (): Promise<ScreenInfo> => {
    return new Promise((resolve, reject) => {
        Native.Screen.init().then(() => {
            Native.Screen.getBrightness(
                (value) => {
                    resolve({ value })
                },
                reject
            );
        })
            .catch((e) => {
                reject(e);
            });
    });
}

let func: Function;
let cbID: number;        // NativeHook监听回调ID
export const onUserCaptureScreen: TonUserCaptureScreen = (callback: (path: string) => void) => {
    if (!func) {
        func = callback;
        Native.Screen.init().then(() => {
            const id = Native.Screen.registerCaptureScreen(cbID, callback, undefined);
            cbID = id;
        });
    }

}

export const offUserCaptureScreen: ToffUserCaptureScreen = () => {
    if (!func) return;
    Native.Screen.init().then(() => {
        if (cbID) {
            Native.Screen.unRegisterCaptureScreen(cbID);
            cbID = undefined;
        }
    });
    func = undefined;
}