
import { ChangeInfo, Texit, Tjoin, TjoinVideoRoom, Tlogin, Tlogout, TregisterMembersChanged, TregisterSpeakersChanged, TunregisterMembersChanged, TunregisterSpeakersChanged, TupdateMuteConfig } from "pi_sys/native/device/voipchat";
import { Native } from "../core/index";


let loginFlag: boolean = false;

export const login: Tlogin = (sdkAppId: number, userId: string, userSig: string): Promise<any> => {
    return new Promise((resolve, reject) => {
        if (!loginFlag) {
            Native.VoIPChat.init().then(() => {
                Native.VoIPChat.login(
                    sdkAppId,
                    userId,
                    userSig,
                    (info: any) => {
                        loginFlag = true;
                        resolve(info);
                    },
                    reject
                );
            })
                .catch((e) => {
                    reject(e);
                });
        } else {
            reject('Logged in');
        }
    });
}

export const logout: Tlogout = (): Promise<any> => {
    return new Promise((resolve, reject) => {
        if (loginFlag) {
            Native.VoIPChat.init().then(() => {
                Native.VoIPChat.logout(
                    (info: any) => {
                        loginFlag = false;
                        resolve(info);
                    },
                    reject
                );
            })
                .catch((e) => {
                    reject(e);
                });
        } else {
            reject('Logged out')
        }
    });
}

export const joinVideoRoom: TjoinVideoRoom = (roomId: number, userName: string, userAvatar: string, openCamera: boolean, openAudio: boolean, audioQuality: number) => {
    return new Promise((resolve, reject) => {
        if (loginFlag) {
            Native.VoIPChat.init().then(() => {
                Native.VoIPChat.joinVideoRoom(
                    roomId,
                    userName,
                    userAvatar,
                    openCamera,
                    openAudio,
                    audioQuality,
                    () => {
                        resolve(undefined);
                    },
                    reject
                );
            })
                .catch((e) => {
                    reject(e);
                });
        } else {
            reject('Not logged in');
        }
    });
}

export const join: Tjoin = (roomId: string, muteMicrophone: boolean, muteEarphone: boolean) => {
    return new Promise((resolve, reject) => {

        if (loginFlag) {
            Native.VoIPChat.init().then(() => {
                Native.VoIPChat.join(
                    roomId,
                    muteMicrophone,
                    muteEarphone,
                    () => {
                        resolve(undefined);
                    },
                    reject
                );
            })
                .catch((e) => {
                    reject(e);
                });
        } else {
            reject('Not logged in');
        }
    });
}

export const exit: Texit = () => {
    return new Promise((resolve, reject) => {
        if (loginFlag) {
            Native.VoIPChat.init().then(() => {
                Native.VoIPChat.exit(
                    () => {
                        resolve(undefined);
                    },
                    reject
                );
            })
                .catch((e) => {
                    reject(e);
                });
        } else {
            reject('Not logged in');
        }
    });
}

export const updateMuteConfig: TupdateMuteConfig = (muteMicrophone: boolean, muteEarphone: boolean) => {
    return new Promise((resolve, reject) => {
        if (loginFlag) {
            Native.VoIPChat.init().then(() => {
                Native.VoIPChat.updateMuteConfig(
                    muteMicrophone,
                    muteEarphone,
                    () => {
                        resolve(undefined);
                    },
                    reject
                );
            })
                .catch((e) => {
                    reject(e);
                });
        } else {
            reject('Not logged in');
        }
    });
}

const listenMap: Map<string, ((data: ChangeInfo) => void)> = new Map();
const listenCallsMap: Map<string, ((data: ChangeInfo) => void)[]> = new Map();

export const registerMembersChanged: TregisterMembersChanged = (callback: (flag: ChangeInfo) => void) => {

    const listenName = 'registerMembersChanged';

    if (!listenMap.get(listenName)) {
        listenCallsMap.set(listenName, []);
        listenMap.set(listenName, function (level: ChangeInfo) {
            listenCallsMap.get(listenName).forEach(f => {
                f && f(level);
            });
        });

        if (loginFlag) {
            Native.VoIPChat.init().then(() => {
                Native.VoIPChat.registerMembersChanged(
                    (info: ChangeInfo) => {
                        listenMap.get(listenName)(info);
                    },
                    undefined
                );
            });
        }
    }

    const arr = listenCallsMap.get(listenName);
    if (!arr.includes(callback)) {
        arr.push(callback);
    }
}

export const unregisterMembersChanged: TunregisterMembersChanged = (callback: (flag: ChangeInfo) => void) => {
    const listenName = 'registerMembersChanged';
    const arr = listenCallsMap.get(listenName);
    arr.splice(arr.indexOf(callback));
}

export const registerSpeakersChanged: TregisterSpeakersChanged = (callback: (flag: ChangeInfo) => void) => {

    const listenName = 'registerSpeakersChanged';

    if (!listenMap.get(listenName)) {
        listenCallsMap.set(listenName, []);
        listenMap.set(listenName, function (level: ChangeInfo) {
            listenCallsMap.get(listenName).forEach(f => {
                f && f(level);
            });
        });

        Native.VoIPChat.init().then(() => {
            Native.VoIPChat.registerSpeakersChanged(
                (info: ChangeInfo) => {
                    listenMap.get(listenName)(info);
                },
                undefined
            );
        });
    }

    const arr = listenCallsMap.get(listenName);
    if (!arr.includes(callback)) {
        arr.push(callback);
    }
}

export const unregisterSpeakersChanged: TunregisterSpeakersChanged = (callback: (flag: ChangeInfo) => void) => {
    const listenName = 'registerSpeakersChanged';
    const arr = listenCallsMap.get(listenName);
    arr.splice(arr.indexOf(callback));
}