/**
 * 媒体：相机
 */
import { TopenAlbum, TopenCamera, TscanQRCode, Tsave2Album, TgetBase64FromLocalFile } from "pi_sys/native/media/camera";
import { specialCall } from "../platform/specialcall";
import { Native } from "../core/index";
import { Logger } from "pi_sys/util/logger";
import { formatError } from "pi_sys/util/util";
declare var module;
let logger = new Logger(module.name, "sys");

const SDK_NAME = "api_camera";

/**
 * 打开相册
 * @param mediaType "image" | "video" | "mix" ; 相册显示 图像、视频、所有
 * @param maxSelectNum 最大选择 数量：1-9
 * @param isCompress 否 压缩，压缩质量 由 程序决定，参考 Luban Java 库
 * @returns 相册文件 数组
 */
export const openAlbum: TopenAlbum = (mediaType: "image" | "video" | "mix", maxSelectNum: number = 9, isCompress: boolean = false) => {
    let param = {
        mediaType,
        maxSelectNum,
        isCompress
    }
    return specialCall(SDK_NAME, "openAlbum", [JSON.stringify(param)]);
}


/**
 * 打开相机 拍照 或 录像
 * @param mediaType "image" | "video" 拍照 或 录像
 * @param recordVideoMaxSecond 录像的 最长 时间，单位：秒
 * @returns 相册文件 数组,只有 一个元素
 */
export const openCamera: TopenCamera = (mediaType: "image" | "video", recordVideoMaxSecond: number = 60) => {
    let param = {
        mediaType,
        recordVideoMaxSecond
    };
    return specialCall(SDK_NAME, "openCamera", [JSON.stringify(param)]);
}

/**
 * 扫码：拍照 or 相册选择
 * @returns 文件的绝对路径，通过 getBase64FromLocalFile 取 内容
 */
export const scanQRCode: TscanQRCode = () => {
    return specialCall(SDK_NAME, "scanQRCode");
}

/**
 * 保存 图片 到相册
 * @param path 文件的绝对路径
 * @returns 文件的绝对路径，通过 getBase64FromLocalFile 取 内容
 */
export const save2Album: Tsave2Album = (path: string) => {
    return specialCall(SDK_NAME, "save2Album", [path]);
}


/**
 * 获取图片内容
 * @localPath  图片本地路径
 * @returns 图片内容base64编码字符串
 */
export const getBase64FromLocalFile: TgetBase64FromLocalFile = (localPath: string) => {
    return new Promise((resolve, reject) => {
        Native.File.init()
            .then(() => {
                Native.File.getBase64FromLocalFile(
                    localPath,
                    0,
                    function (info) {
                        resolve(info);
                    },
                    (e: any) => {
                        logger.error(`File - ${getBase64FromLocalFile} call fail. `, formatError(e));
                        reject(e);
                    }
                );
            })
            .catch((e: any) => {
                logger.error(`File init fail. `, formatError(e));
                reject(e);
            });
    })
}