/**
 * 媒体：图片
 */

import { ChooseImageInfo, SizeType, SourceType, TchooseImage, TpreviewImage, TsaveImageToPhotosAlbum } from "pi_sys/native/media/image";
import { Native } from "../core/index";





/**
 * 保存图片到系统相册
 * @note 调用前需要 用户授权 scope.writePhotosAlbum
 * @param filePath	图片文件路径，可以是临时文件路径或永久文件路径 (本地路径) ，不支持网络路径
 */
export const saveImageToPhotosAlbum: TsaveImageToPhotosAlbum = (filePath: string): Promise<void> => {
    return Native.Image.init().then(() => {
        Native.Image.saveToPhotosAlbum(filePath, undefined, undefined);
        return undefined;
    });
}

/**
 * 在新页面中全屏预览图片
 * @note 预览的过程中用户可以进行保存图片、发送给朋友等操作
 * 
 * @param urls	    需要预览的图片链接列表
 * @param current	当前显示图片的链接,默认值为urls 的第一张
 */
export const previewImage: TpreviewImage = (urls: string[], current?: string): Promise<void> => {
    return Native.Image.init().then(() => {
        let index = urls.indexOf(current);
        index = index >= 0 ? index : 0;
        Native.Image.preview(index, JSON.stringify(urls), undefined, undefined);
        return undefined;
    });
}

/**
 * 从本地相册选择图片或使用相机拍照
 * @param count         最多可以选择的图片张数
 * @param sizeType	    所选的图片的尺寸
 * @param sourceType	选择图片的来源
 */
export const chooseImage: TchooseImage = (count = 9, sizeType = SizeType.Compressed, sourceType = SourceType.Album): Promise<ChooseImageInfo[]> => {
    return new Promise((resolve, reject) => {
        Native.Image.init().then(() => {
            Native.Image.choose(
                count,
                sizeType === SizeType.Compressed,
                sourceType === SourceType.Camera,
                function (list: any) {
                    resolve(list);
                },
                reject
            );
        })
            .catch((e) => {
                reject(e);
            });
    });
}