/**
 * 广告 SDK
 */

import { Tdestroy, Tload, ToffClose, TonClose, Tshow } from "pi_sys/native/platform/ad";
import { Native } from "../core/index";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");


// /**
//  * 初始化广告顺序,对广告进行排序(load前调用有效).
//  * @param josn - Json格式的广告优先级
//  * @example - .eg: [ {"adType" : "splash", "ads" : ["csj", "gdt"] } ]
//  */
// export const initAd(json: IADJson[]) {
//     return Native.init().then(() => {
//         Native.initAd(JSON.stringify(json));
//     });
// }

/**
 * 加载广告(调用一次即可).
 * @param adType - 广告类型 - "splash","interstitial","reward","fullscreen"
 */
export const load: Tload = (adType: string) => {
    return new Promise((resolve, reject) => {
        Native.Ad.init().then(() => {
            Native.Ad.load(
                adType,
                (info: any) => {
                    resolve(info);
                },
                reject
            );
        })
            .catch((e) => {
                reject(e);
            });
    });
}

/**
 * 展示广告.
 * @param adType - 广告类型 - "splash","interstitial","reward","fullscreen"
 */
export const show: Tshow = (adType: string) => {
    return new Promise((resolve, reject) => {
        Native.Ad.init().then(() => {
            Native.Ad.show(
                adType,
                (info: any) => {
                    resolve(info);
                },
                reject
            );
        })
            .catch((e) => {
                reject(e);
            });
    });
}

/**
 * 销毁广告实例.
 * @param adType - 广告类型 - "splash","interstitial","reward","fullscreen"
 */
export const destroy: Tdestroy = (adType: string) => {
    return Native.Ad.init().then(() => {
        Native.Ad.destroy(adType);
    });
}

const closeMap: Map<string, Function[]> = new Map();
export const _close = (adType: string, flag: boolean) => {
    logger.debug(`AD - OnClose`);
    const arr = closeMap.get(adType);
    if (arr) {
        arr.forEach((f) => {
            f && f(flag);
        });
        logger.debug(`Listen Length ${arr.length}`);
    }
}

/**
 * 监听广告关闭事件(load后,调用才有效)..
 * @param adType - 广告类型 - "splash","interstitial","reward","fullscreen"
 */
export const onClose: TonClose = (adType: string, cb: (flag: boolean) => void) => {
    let arr = closeMap.get(adType);

    if (!arr) {
        arr = [];
        closeMap.set(adType, arr);

        Native.Ad.init().then(() => {
            Native.Ad.onClose(
                adType,
                (flag: boolean) => {
                    _close(adType, flag);
                },
                undefined
            );
        });
    }

    arr.push(cb);
}
/**
 * 取消广告关闭事件的监听.
 * @param adType - 广告类型 - "splash","interstitial","reward","fullscreen"
 */
export const offClose: ToffClose = (adType: string) => {
    const arr = closeMap.get(adType);
    if (arr) {
        arr.length = 0;
    }
}