
/**
 * 自动化测试
 */

import { TgetAttr, TisEnable, TregisterRunScript } from "pi_sys/native/platform/auto_test";
import { Native } from "../core/index";

/**
 * 底层是否支持自动化测试
 */
export const isEnable: TisEnable = (): Promise<boolean> => {
    return Native.AutoTest.init().then(() => {
        return new Promise((resolve) => {
            Native.AutoTest.isEnable(resolve, null);
        });
    });
}

/**
 * 运行自动化测试脚本
 * @param name 脚本名
 * @param queryFunc 底层向高层查询属性的回调
 */
export const registerRunScript: TregisterRunScript = (
    sdkName: string,
    scriptName: string,
    queryFunc: (params: JSON, attrNamesStr: string, id: number) => void): Promise<void> => {
    return Native.AutoTest.init().then(() => {
        Native.AutoTest.registerRunScript(sdkName, scriptName, ({ query, attrNamesStr, id }) => {
            queryFunc(query, attrNamesStr, id);
        });
    });
}

/**
 * 当底层调用了高层的queryFunc，高层查到对应的信息之后，需要返回给底层；
 * @param result Json字符串
 * @param id 和queryFunc的id参数对应；
 */
export const getAttr: TgetAttr = (sdkName: string, result: string, id: number): Promise<void> => {
    return Native.AutoTest.init().then(() => {
        Native.AutoTest.getAttr(sdkName, result, id);
    });
}