
/**
 * 登录 SDK
 * 
 * ========================== 注：不要直接使用这里的接口，请使用 pi_common/sdk 的 函数
 */

import { Tlogin, Tlogout } from "pi_sys/native/platform/login";
import { Native } from "../core/index";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

/**
  * 通用 登录
  */
export const login: Tlogin = (sdkName: string, _param?: any) => {
    Native.Login.init().then(() => {
        Native.Login.login(sdkName, () => {

        }, (reason) => {
            logger.error("PISYS.Native.Login.login failed, reason = " + reason);
        });
    }).catch(reason => {
        logger.error("login: PISYS.Native.Login.init failed, reason = " + reason);
    });
}

/**
 * 通用 登出
 */
export const logout: Tlogout = (sdkName: string, _param?: any) => {
    Native.Login.init().then(() => {
        Native.Login.logout(sdkName, () => {

        }, (reason) => {
            logger.error("PISYS.Native.Login.logout failed, reason = " + reason);
        });
    }).catch(reason => {
        logger.error("logout: PISYS.Native.Login.init failed, reason = " + reason);
    });
}

/**
 * 通用 注册 登录 回调
 */
export function registerLoginCb(
    sdkName: string,
    success?: (info: any) => void,
    fail?: (reason: any) => void
) {
    Native.Login.init().then(() => {
        Native.Login.registerLoginCb(sdkName, success, fail);
    }).catch((e) => {
        fail && fail(e);
    });
}

/**
 * 通用 取消注册 登录 回调
 */
export function unRegisterLoginCb(
    sdkName: string,
    success?: (info: any) => void,
    fail?: (reason: any) => void
) {
    Native.Login.init().then(() => {
        Native.Login.unRegisterLoginCb(sdkName, success, fail);
    }).catch((e) => {
        fail && fail(e);
    });
}

/**
 * 通用 注册 登录 回调
 */
export function registerLogoutCb(
    sdkName: string,
    success?: (info: any) => void,
    fail?: (reason: any) => void
) {
    Native.Login.init().then(() => {
        Native.Login.registerLogoutCb(sdkName, success, fail);
    }).catch((e) => {
        fail && fail(e);
    });
}

/**
 * 通用 取消注册 登出 回调
 */
export function unRegisterLogoutCb(
    sdkName: string,
    success?: (info: any) => void,
    fail?: (reason: any) => void
) {
    Native.Login.init().then(() => {
        Native.Login.unRegisterLogoutCb(sdkName, success, fail);
    }).catch((e) => {
        fail && fail(e);
    });
}


/**
 * 通用 注册 切换账号 回调
 */
export function registerAccountSwitchCb(
    sdkName: string,
    success?: (info: any) => void,
    fail?: (reason: any) => void
) {
    Native.Login.init().then(() => {
        Native.Login.registerAccountSwitchCb(sdkName, success, fail);
    }).catch((e) => {
        fail && fail(e);
    });
}

/**
 * 通用 取消注册 切换账号 回调
 */
export function unRegisterAccountSwitchCb(
    sdkName: string,
    success?: (info: any) => void,
    fail?: (reason: any) => void
) {
    Native.Login.init().then(() => {
        Native.Login.unRegisterAccountSwitchCb(sdkName, success, fail);
    }).catch((e) => {
        fail && fail(e);
    });
}
