import { ChannelTypeInfo, Taliyun_bindAccount, Taliyun_bindTag, Taliyun_unbindAccount, Taliyun_unbindTag, Tdw_applicationDestroy, Tdw_exit, Tdw_getPackageId, Tdw_getSqRealNameInfo, Tdw_setInitListener, TgetChannelType, ThideToolbar, Tjrtt_log_getHumeSDKChannel, Tjrtt_log_getHumeSDKVersion, Tmy_setInitListener, TSetInitListener, TshowToolbar, TSpecialCall, TverifyRealName, Tyc_idVerification, Tyc_setInitListener, VerifyRealNameInfo } from "pi_sys/native/platform/specialcall";
import { Native } from "../core/index";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

export const getChannelType: TgetChannelType = (sdkName: string): Promise<ChannelTypeInfo> => {
    return new Promise((resolve, reject) => {
        Native.SpecialCall.init()
            .then(() => {
                Native.SpecialCall.getChannelType(
                    sdkName,
                    function (info) {
                        resolve(info);
                    },
                    (e: any) => {
                        logger.error(`SpecialCall ${sdkName} getChannelType fail. `, e);
                        reject(e);
                    }
                );
            })
            .catch((e: any) => {
                logger.error(`SpecialCall ${sdkName} init fail. `, e);
                reject(e);
            });
    })
}

export const showToolbar: TshowToolbar = (sdkName: string): Promise<void> => {
    return new Promise((resolve, reject) => {
        Native.SpecialCall.init()
            .then(() => {
                Native.SpecialCall.showToolbar(
                    sdkName,
                    function (info) {
                        resolve(info);
                    },
                    (e: any) => {
                        logger.error(`SpecialCall ${sdkName} - showToolbar call fail. `, e);
                        reject(e);
                    }
                );
            })
            .catch((e: any) => {
                logger.error(`SpecialCall ${sdkName} init fail. `, e);
                reject(e);
            });
    })
}

export const hideToolbar: ThideToolbar = (sdkName: string): Promise<void> => {
    return new Promise((resolve, reject) => {
        Native.SpecialCall.init()
            .then(() => {
                Native.SpecialCall.hideToolbar(
                    sdkName,
                    function (info) {
                        resolve(info);
                    },
                    (e: any) => {
                        logger.error(`SpecialCall ${sdkName} -hideToolbar call fail. `, e);
                        reject(e);
                    }
                );
            })
            .catch((e: any) => {
                logger.error(`SpecialCall ${sdkName} init fail. `, e);
                reject(e);
            });
    })
}

export const verifyRealName: TverifyRealName = (sdkName: string): Promise<VerifyRealNameInfo> => {
    return new Promise((resolve, reject) => {
        Native.SpecialCall.init()
            .then(() => {
                Native.SpecialCall.verifyRealName(
                    sdkName,
                    function (info) {
                        resolve(info);
                    },
                    (e: any) => {
                        logger.error(`SpecialCall ${sdkName} - verifyRealName call fail. `, e);
                        reject(e);
                    }
                );
            })
            .catch((e: any) => {
                logger.error(`SpecialCall ${sdkName} init fail. `, e);
                reject(e);
            });
    })
}

/**
 * 通用
 * @param sdkName 三方平台名称
 */
export const setInitListener: TSetInitListener = (sdkName: string) => {
    return call(sdkName, 'setInitListener', []);
}

/**
 * 通用特殊调用
 * @param sdkName 三方平台名称
 * @param methodName 想要调用的方法名
 * @param paramJson 被调用的方法的参数(JsonArray格式)
 */
export const specialCall: TSpecialCall = (sdkName: string, methodName: string, paramJson: any[] = []) => {
    return call(sdkName, methodName, paramJson);
}

/**
 * 特殊调用.
 * @param sdkName 三方平台名称
 * @param methodName 想要调用的方法名
 * @param paramsJson - 被调用的方法的参数(JsonArray格式)
 * @example - [0, 0.1, true, "Hi", "{"name":"好嗨"}"]
 */
export function call(sdkName: string, methodName: string, paramJson: any[]): Promise<any> {
    return new Promise((resolve, reject) => {
        Native.SpecialCall.init()
            .then(() => {
                const str = JSON.stringify(paramJson);
                Native.SpecialCall.call(
                    sdkName,
                    methodName,
                    str,
                    function (info) {
                        resolve(info);
                    },
                    (e: any) => {
                        logger.error(`SpecialCall ${sdkName} - ${methodName} call fail. `, e);
                        reject(e);
                    }
                );
            })
            .catch((e: any) => {
                logger.error(`SpecialCall ${sdkName} init fail. `, e);
                reject(e);
            });
    })
}

// ============ 下面接口全部作废

/**
 * @deprecated
 * 阿里云  绑定标签
 * @link https://help.aliyun.com/document_detail/30066.html?spm=a2c4g.11186623.6.570.11ca1cd3umWHxr
 * @param target 目标类型，1：本设备； 2：本设备绑定账号； 3：别名
 * target(V2.3.5及以上版本) 目标类型，CloudPushService.DEVICE_TARGET：本设备； CloudPushService.ACCOUNT_TARGET：本账号； CloudPushService.ALIAS_TARGET：别名
 * @param tags 标签（数组输入）
 * @param alias (仅当target = 3时生效）
 */
export const aliyun_bindTag: Taliyun_bindTag = (target: number, tags: string[], alias: string = ""): Promise<any> => {
    return call('aliyunpush', 'bindTag', [target, tags, alias]);
}

/**
 * @deprecated
 * 阿里云  解绑定标签
 * @link https://help.aliyun.com/document_detail/30066.html?spm=a2c4g.11186623.6.570.11ca1cd3umWHxr
 * @param target 目标类型，1：本设备； 2：本设备绑定账号； 3：别名
 * target(V2.3.5及以上版本) 目标类型，CloudPushService.DEVICE_TARGET：本设备； CloudPushService.ACCOUNT_TARGET：本账号； CloudPushService.ALIAS_TARGET：别名
 * @param tags 标签（数组输入）
 * @param alias (仅当target = 3时生效）
 */
export const aliyun_unbindTag: Taliyun_unbindTag = (target: number, tags: string[], alias: string = ""): Promise<any> => {
    return call('aliyunpush', 'unbindTag', [target, tags, alias]);
}

/**
 * @deprecated
 */
export const aliyun_bindAccount: Taliyun_bindAccount = (account: string): Promise<any> => {
    return call('aliyunpush', 'bindAccount', [account]);
}

/**
 * @deprecated
 */
export const aliyun_unbindAccount: Taliyun_unbindAccount = (): Promise<any> => {
    return call('aliyunpush', 'unbindAccount', []);
}


//#region 煜辰
/**
 * @deprecated
 * 煜辰 - 实名认证
 * @returns 
 */
export const yc_idVerification: Tyc_idVerification = (): Promise<any> => {
    return call('yc', 'idVerification', []);
}

/**
 * @deprecated
 * 煜辰 - 初始化成功监听
 * @returns 
 */
export const yc_setInitListener: Tyc_setInitListener = (cd: () => void) => {
    setInitListener('yc').then(() => {
        cd && cd();
    });
}
//#endregion

//#region 萌友

/**
 * @deprecated
 * 萌友 - 初始化成功监听
 * @returns 
 */
export const my_setInitListener: Tmy_setInitListener = (cd: () => void) => {
    setInitListener('my').then(() => {
        cd && cd();
    });
}
//#endregion

//#region  动玩
/**
 * @deprecated
 * 初始化成功监听
 * @param cd 
 */
export const dw_setInitListener: Tdw_setInitListener = (cd: () => void) => {
    setInitListener('dw').then(() => {
        cd && cd();
    });
}

/**
 * @deprecated
 * 获取用户防沉迷相关信息接口
 *  接口说明：*此方法一定要在登录游戏成功后调用
 * 在用户登录游戏成功后调用获取用户防沉迷相关信息接口，会有如下三种情况需要处理。
 * a）渠道sdk有自己的实名接口，游戏方只需要在needAntiIndulge(String uid, int age)回调方法中做防沉迷相关处理
 * b）渠道sdk没有自己的实名接口，游戏方需要在needRealNameAndAntiIndulge(String uid)回调方法中先进行实名认证再作防沉迷相关处理
 * c）渠道sdk有自己的实名和防沉迷功能，游戏方这不需要作处理，在normalGame(String uid)回调方法正常进行游戏，不需要做实名认证和防沉迷处理
 */
export const dw_getSqRealNameInfo: Tdw_getSqRealNameInfo = () => {
    return call('dw', 'getSqRealNameInfo', []);
}

/**
 * @deprecated
 * 退出接口
 * 接口说明：
 *  在游戏退出前调用退出接口，会有如下不同处理：
 * a）	渠道sdk存在退出界面，游戏方只需在onChannelExit()回调中进行退出游戏逻辑即可，无需再弹退出界面。
 * b）	渠道sdk不存在退出界面，游戏方需在onGameExit()回调中弹出自己的游戏退出确认界面，然后再进行退出逻辑。
 * @returns 
 */
export const dw_exit: Tdw_exit = () => {
    return call('dw', 'exit', []);
}

/**
 * @deprecated
 * 资源回收接口
 * 在游戏退出前，退出接口中调用
 * @returns 
 */
export const dw_applicationDestroy: Tdw_applicationDestroy = () => {
    return call('dw', 'applicationDestroy', []);
}

/**
 * @deprecated
 * 获取包ID接口
 * 此接口用来获取当前包的唯一标识，可用于区分不同的包；
 * @returns 
 */
export const dw_getPackageId: Tdw_getPackageId = () => {
    return call('dw', 'getPackageId', []);
}
//#endregion

/**
 * @deprecated
 */
export const jrtt_log_getHumeSDKChannel: Tjrtt_log_getHumeSDKChannel = () => {
    return call('jrtt_log', 'getHumeSDKChannel', []);
}

/**
 * @deprecated
 */
export const jrtt_log_getHumeSDKVersion: Tjrtt_log_getHumeSDKVersion = () => {
    return call('jrtt_log', 'getHumeSDKVersion', []);
}