import { IStats, Taccess, TappendFile, TcopyFile, Tmkdir, TreadAssets, Treaddir, TreadFile, Trename, Trmdir, Tstat, Tunlink, TwriteFile } from "pi_sys/store/file_system";
import { get, post } from "pi_sys/net/http"
import { RESP_TYPE } from "pi_sys/net/http_base"
import { Env } from "pi_sys/util/env";
import { SystemInfo } from "pi_sys/native/base/system_info";
import { Native } from "../../webview/native_impl/core/index";
import { fromByteArray } from "pi_sys/util/base64js";
/**
 * 底层通过拦截请求来实现文件操作
 */

const DOMAIN = "http://file.system.com/"

const getUrl = (method: string, args: [string, string | number][]) => {
    let argsStr = "";
    args.forEach(arg => {
        const [key, value] = arg;
        argsStr += `${key}=${value}&`
    });
    argsStr = argsStr.slice(0, argsStr.length - 1); // 删除末尾 &
    return `${DOMAIN}${method}?${argsStr}`;
}

const TIMEOUT = 60 * 1000;

export const access: Taccess = (path: string) => {
    return get(getUrl("access", [["path", path]]), null, null, RESP_TYPE.TEXT, TIMEOUT).then(r => r === "true") as Promise<boolean>;
}

export const appendFile: TappendFile = (path: string, data: string | ArrayBuffer) => {
    return post(getUrl("appendFile", [["path", path]]), null, data, RESP_TYPE.BIN, RESP_TYPE.TEXT, TIMEOUT) as Promise<null>;
}

export const copyFile: TcopyFile = (srcPath: string, destPath: string) => {
    return get(getUrl("copyFile", [["from", srcPath], ["to", destPath]]), null, null, RESP_TYPE.TEXT, TIMEOUT) as Promise<null>;
}

export const mkdir: Tmkdir = (path: string) => {
    return get(getUrl("mkdir", [["path", path]]), null, null, RESP_TYPE.TEXT, TIMEOUT) as Promise<null>;
}

const __readFile = (method: string, path: string, encoding: "utf8" | "binary" = "utf8", offset?: number, size?: number) => {
    const args: [string, string | number][] = [["path", path]];
    if (offset) {
        args.push(["offset", offset]);
    }
    if (size) {
        args.push(["size", size]);
    }
    let respType = RESP_TYPE.TEXT;
    if (encoding === "binary") {
        respType = RESP_TYPE.BIN;
    }
    return get(getUrl(method, args), null, null, respType, TIMEOUT) as Promise<null>;
}
export const readFile: TreadFile = (path: string, encoding: "utf8" | "binary" = "utf8", offset?: number, size?: number) => {
    return __readFile("readFile", path, encoding, offset, size);
}

export const readAssets: TreadAssets = (path: string) => {
    return get(getUrl("asset/read", [["path", path]]), null, null, RESP_TYPE.BIN, TIMEOUT) as Promise<ArrayBuffer>;
}

export const readdir: Treaddir = (path: string) => {
    return get(getUrl("readdir", [["path", path]]), null, null, RESP_TYPE.TEXT, TIMEOUT) as Promise<string[]>;
}

export const rename: Trename = (oldPath: string, newPath: string) => {
    return get(getUrl("rename", [["oldPath", oldPath], ["newPath", newPath]]), null, null, RESP_TYPE.TEXT, TIMEOUT) as Promise<null>;
}

export const rmdir: Trmdir = (path: string) => {
    return get(getUrl("rmdir", [["path", path]]), null, null, RESP_TYPE.TEXT, TIMEOUT) as Promise<null>;
}


export const stat: Tstat = (path: string) => {
    return get(getUrl("stat", [["path", path]]), null, null, RESP_TYPE.TEXT, TIMEOUT) as Promise<IStats>;
}

export const unlink: Tunlink = (path: string) => {
    return get(getUrl("unlink", [["path", path]]), null, null, RESP_TYPE.TEXT, TIMEOUT) as Promise<null>;
}

export const writeFile: TwriteFile = (path: string, data: string | ArrayBuffer) => {
    const platform_type = Env.get("platform_type2");
    if (platform_type === "android") {  // 安卓底层无法通过拦截拿到body中的数据,所以这里通过postMessage方式
        return new Promise((resolve, reject) => {
            Native.File.init().then(() => {
                if (typeof data === "string") {
                    Native.File.writeFileString(path, data, () => {
                        resolve(null);
                    }, (e) => {
                        reject(e);
                    });
                } else {
                    data = fromByteArray(new Uint8Array(data));
                    Native.File.writeFileByte(path, data, () => {
                        resolve(null);
                    }, (e) => {
                        reject(e);
                    });
                }
            }).catch(reject);

        });

    } else {
        return post(getUrl("writeFile", [["path", path]]), null, data, RESP_TYPE.BIN, RESP_TYPE.TEXT, TIMEOUT) as Promise<null>;
    }

}
