import { PlatformType, getPlatformType } from "./platform"
import { toByteArray } from "../../pi_sys/util/base64js";
import { Asset } from "../native_impl/core/asset";

import { Logger } from "pi_sys/util/logger";
import { CompressAlgorithm, EncryptType } from "pi_sys/util/constant";

declare var module;
let logger = new Logger(module.name, "sys");

let initPromise = undefined;

/**
 * 读本地文件
 * cb：如果错误，返回null，否则返回 String | Uint8Array
 */
export const getAssetFile = (path: string, decryptType: EncryptType, encryptKey: string, compressAlgorithm: CompressAlgorithm, cb?: (content: Uint8Array | String) => void, isString = false) => {
    let platformType = getType();

    if (platformType === PlatformType.Web) {

        setTimeout(() => {
            cb && cb(null);
        }, 0);
        return;
    }

    if (!initPromise) {
        initPromise = Asset.init();
    }

    initPromise.then(() => {
        if (platformType === PlatformType.IOS) {

            // IOS使用加密解密接口
            if (isString) {
                Asset.getStringFromAssetEcrypt(path, decryptType, encryptKey, compressAlgorithm, (content) => {
                    cb(JSON.stringify(content))
                }, (msg) => {
                    logger.warn(`Asset.getStringFromAssetEcrypt failed, path = ${path}, message = `, msg);
                    cb(null);
                });
            } else {
                Asset.getBase64FromAssetEcrypt(path, decryptType, encryptKey, compressAlgorithm, (base64Str) => {
                    let u8Array = toByteArray(base64Str) as Uint8Array;
                    cb(u8Array);
                }, (msg) => {
                    logger.warn(`Asset.getBase64FromAssetEcrypt failed, path = ${path}, message = `, msg);
                    cb(null);
                });
            }
        } else {
            if (isString) {
                Asset.getStringFromAsset(path, (content) => {
                    cb(content);
                }, (msg) => {
                    logger.warn(`Asset.getStringFromAsset failed, path = ${path}, message = `, msg);
                    cb(null);
                });
            } else {
                Asset.getBase64FromAsset(path, (base64Str) => {
                    let u8Array = toByteArray(base64Str) as Uint8Array;
                    cb(u8Array);
                }, (msg) => {
                    logger.warn(`Asset.getBase64FromAsset failed, path = ${path}, message = `, msg);
                    cb(null);
                });
            }
        }
    });
}

// ========================== 本地


let platform: PlatformType = undefined;

const getType = () => {
    if (platform !== undefined) {
        return platform;
    }
    platform = getPlatformType();
    return platform;
}