declare var Canvas;

// 将微信的touch事件转换为pointer事件,并派发到pi_sys的事件系统

import { dispatchEvent } from "pi_sys/native/base/event";
import "./platform"; // 屏蔽各小游戏平台差异
// 将微信的touch事件转换为pointer事件,并派发到pi_sys的事件系统
// import { dispatchEvent } from "pi_sys/native/base/event";
import { AppName } from "pi_sys/native/base/system_info";

if (!globalThis["canvas"]) {
	globalThis["canvas"] = new Canvas();
}

const CLONE_PROPS = [

	// MouseEvent
	'bubbles',
	'cancelable',
	'view',
	'detail',
	'screenX',
	'screenY',
	'clientX',
	'clientY',
	'ctrlKey',
	'altKey',
	'shiftKey',
	'metaKey',
	'button',
	'relatedTarget',

	// PointerEvent
	'pointerId',
	'width',
	'height',
	'pressure',
	'tiltX',
	'tiltY',
	'pointerType',
	'hwTimestamp',
	'isPrimary',

	// event instance
	'pageX',
	'pageY',
	'timeStamp'
];

const CLONE_DEFAULTS = [

	// MouseEvent
	false,
	false,
	null,
	null,
	0,
	0,
	0,
	0,
	false,
	false,
	false,
	false,
	0,
	null,

	// DOM Level 3
	0,

	// PointerEvent
	0,
	0,
	0,
	0,
	0,
	0,
	'',
	0,
	false,

	// event instance
	0,
	0,
	0
];

const POINTER_TYPE = 'touch';

export const noop = () => { }

let lastTouch: Touch;
const downList: any[] = [];
const lastPositionMap: Map<number, { x: number, y: number }> = new Map;

const touchToPointer = (type: string, touch: Touch) => {
	const id = (<any>touch).id || 0;
	const e = {
		pointerId: id
	} as any;

	const index = downList.findIndex(evt => evt.pointerId === id);
	if (type === "pointerdown") {
		if (index >= 0) {
			return;
		}
		if (downList.length >= 2) {       // 三指事件在手机上一般是截屏操作,不会触发pointerup事件,所以这里我们只支持两指事件
			// let [evt1, evt2] = downList;
			// evt1.type = "pointerup";
			// evt2.type = "pointerup";
			// setTimeout(() => {
			// 	dispatchEvent(evt1.type, evt1);
			// }, 0);
			// setTimeout(() => {
			// 	dispatchEvent(evt2.type, evt2);
			// }, 0);
			// downList.length = 0;
			return;
		}
		downList.push(e);
	} else if (type === "pointerup") {
		if (index >= 0) {
			downList.splice(index, 1);
		} else {
			return;
		}
	}


	for (let i = 0; i < CLONE_PROPS.length; i++) {
		const p = CLONE_PROPS[i];
		if (p === "pointerId") continue;
		e[p] = touch[p] || CLONE_DEFAULTS[i];
	}

	e.type = type;
	e.target = window["canvas"];
	e.currentTarget = window["canvas"];
	e.timeStamp = Date.now();
	e.buttons = typeToButtons(type);
	e.which = e.buttons;

	e.bubbles = true;
	e.cancelable = true;
	// e.detail = this.clickCount;
	// e.button = 0;

	e.width = 5;
	e.height = 5;
	e.pressure = touch.force || 0.5;
	// e.isPrimary = isPrimaryPointer(touch) || true;
	if (type === "pointerdown") {
		e.isPrimary = downList.length === 1;
	} else if (type === "pointerup") {
		e.isPrimary = index === 0;
	} else {
		e.isPrimary = true;
	}
	e.pointerType = POINTER_TYPE;

	e.preventDefault = noop;
	e.stopPropagation = noop;
	// 底层没有movementX movementY,这里模拟出这两个字段,如果鼠标锁定,始终为0
	if (touch.movementX !== undefined) {
		e.movementX = touch.movementX;
		e.movementY = touch.movementY;
	} else if (type === "pointerup") {
		lastPositionMap.delete(id);
	} else {
		const lastPosition = lastPositionMap.get(id);
		if (lastPosition) {
			e.movementX = e.clientX - lastPosition.x;
			e.movementY = e.clientY - lastPosition.y;
		} else {
			e.movementX = 0;
			e.movementY = 0;
		}
		lastPositionMap.set(id, { x: e.clientX, y: e.clientY })
	}

	lastTouch = touch;
	return e;
}

function typeToButtons(type) {
	let ret = 0;
	if (type === 'touchstart' || type === 'touchmove' || type === 'pointerdown' || type === 'pointermove') {
		ret = 1;
	}
	return ret;
}

// 如果监听了触摸事件， 需要该方法转换为
export const touchEventHandlerFactory = (type) => {
	return (rawEvent) => {
		const changedTouches = rawEvent.changedTouches;
		for (let i = 0; i < changedTouches.length; i++) {
			const touch = changedTouches[i];

			const event = touchToPointer(type, touch);
			if (event) {
				dispatchEvent(event.type, event);
			}

		}
	}
}


// 如果监听了触摸事件， 需要该方法转换为
export const winEventHandlerFactory = (type) => {
	return (rawEvent) => {
		const event = touchToPointer(type, rawEvent);
		// console.log("----------------------  rawEvent   ", event.movementX, event.movementY);
		// console.log("---------------------- event.type" + event.type + "; event" + JSON.stringify(event));
		if (event) {
			dispatchEvent(event.type, event);
		}

	}
}

const dispatch = (rawEvent: any) => {
	dispatchEvent(rawEvent.type as any, rawEvent);
}


globalThis.onTouchStart(winEventHandlerFactory('pointerdown'));
globalThis.onTouchMove(winEventHandlerFactory('pointermove'));
globalThis.onTouchEnd(winEventHandlerFactory('pointerup'));
globalThis.onTouchCancel(winEventHandlerFactory('pointercancel'));
globalThis.onMouseWheel(dispatch);
globalThis.onMouseUp(winEventHandlerFactory('pointerup'));
globalThis.onMouseMove(winEventHandlerFactory('pointermove'));
globalThis.onMouseDown(winEventHandlerFactory('pointerdown'));


