/**
 * 代码加载器
 *  
 */

import { FileLoadParam } from "pi_sys/load/fileload_base";
import { FileLoad } from "pi_sys/load/file_load";
import { Load } from "pi_sys/load/init";
import { HttpDownload } from "pi_sys/net/http"
import { Env } from "pi_sys/util/env";
import { utf8Decode } from "pi_sys/util/utf8";
import { formatError } from "pi_sys/util/util";
import { showRestartModel } from "pi_sys/setup/main";
import { ErrorCode } from "pi_sys/util/constant";




// ============================== 导入


// ============================== 导出

export class CodeLoad extends FileLoad {

    /**
     * @description 开始
     * @example
     */
    public start() {
        let arr = [];
        for (let info of this.files.values()) {
            // console.log("=================== Codeload start, path = " + info.path);
            // console.log(`--------------- info.sign = ${info.sign}, 
            // ------------------- assetsSign = ${FileLoadParam.assetsSign[info.path]}, 
            // ------------------- dbSign = ${FileLoadParam.dbSign[info.path]}`);
            try {
                if (info.sign === FileLoadParam.assetsSign[info.path]) {
                    globalThis.require(info.path);
                } else if (info.sign === FileLoadParam.dbSign[info.path]) {
                    let store = Load.getStore();
                    let tempPromise = store.read(info.path).then((data) => {
                        globalThis.loadScript(info.path, data);
                    });
                    arr.push(tempPromise);

                } else {
                    const down = new HttpDownload(Env.get("domains"), Env.get("root_path") + info.path, info.encryptSize || info.size);
                    let tempPromise = down.start()
                        .then((data) => {
                            const str = utf8Decode(new Uint8Array(data as ArrayBuffer));
                            globalThis.loadScript(info.path, str);
                        });

                    arr.push(tempPromise);
                }

            } catch (e) {
                console.error("=================== Codeload error, path = " + info.path + ", e = ", e);
                return Promise.reject(e);
            }
        }
        console.log("===================== CodeLoad load OK ");
        return Promise.all(arr);
    }
}

export const loadJSFromAB = (file: string, data: Uint8Array | ArrayBuffer): Promise<void> => {
    // let begin = performance.now();
    return new Promise((resolve, reject) => {
        try {
            // console.log(`loadScriptFromBuffer image time1 = ${performance.now() - begin}`);
            if (data instanceof ArrayBuffer) {
                let r = globalThis.loadScriptFromBuffer(file, data);
                if (!!r) {
                    throw r;
                }
            } else if (data instanceof Uint8Array) {
                let r = globalThis.loadScriptFromBuffer(file, data.slice().buffer);
                if (!!r) {
                    throw r;
                }
            } else {
                const message = `loadJSFromAB error!!! data format invalid!!! file = ${file}`;
                showRestartModel(ErrorCode.LOAD_JS_FROM_AB_ERROR, message);
                reject(message);
            }

            resolve();
        }
        catch (err) {
            const message = `file = ${file} loadJSFromAB error, error = ${formatError(err)}`
            showRestartModel(ErrorCode.LOAD_JS_FROM_AB_ERROR, message);
            reject(message);
        }
    });
}