import { ChannelInfo, TexitPointerLock, TgetChannelInfo, TgetChannelQuick, TgetIdfa, TgetIdfv, TgetIpAdress, TgetTemperature, ToffPointerLockChange, ToffPointerLockError, TonPointerLockChange, TonPointerLockError, TopenAppstore, TprivacyAgreement, TrequestPointerLock, TresetUserInfo, TsetCursorVisible, TshowWithEntranceId, TshowWithEntranceIdLogin, TupdateUserInfo } from 'pi_sys/native/base/app';

export { exitApp, getProcessId, getApiVersion, getPackageName, getFlavorName, getVersionName, getVersionCode, getRunconfigValue, getSdkInfo, triggerGC, openWebView, closeWebView, registerAppLifeListener, unregisterAppLifeListener, onShow, onHide, onDestroy, offShow, offHide, offDestroy, registerSoftInputChangedListener, unregisterSoftInputChangedListener, hideFullScreenPopupWindow, onBlur, offBlur, clearCache, restartApp } from '../../../webview/native_impl/base/app';

export const getChannelInfo: TgetChannelInfo = (): Promise<ChannelInfo> => {
    let info = {
        channel: "",
        param: {}
    };
    return Promise.resolve(info);
}


// 获取CPU温度,单位摄氏度,保留小数点后一位
export const getTemperature: TgetTemperature = () => {
    return Promise.resolve(0);
};


// 获取idfv
export const getIdfv: TgetIdfv = () => {
    return Promise.reject("getIdfv not impl");
};

// 获取IpAdress
export const getIpAdress: TgetIpAdress = () => {
    return Promise.reject("getIpAdress not impl");
};

// 获取Idfa
export const getIdfa: TgetIdfa = () => {
    return Promise.reject("getIpAdress not impl");
};


/**
 * 获取quick渠道分发出来的渠道
 */
export const getChannelQuick: TgetChannelQuick = () => {
    return Promise.resolve(null);
};



let locked: boolean = false;

export const requestPointerLock: TrequestPointerLock = () => {
    if (locked) return;
    try {
        (<any>globalThis.Window).requestPointerLock();
        locked = true;
        setTimeout(() => {
            lockChange && lockChange();
        }, 0);
    } catch (e) {
        setTimeout(() => {
            lockError && lockError(e);
        }, 0);
    }

};

export const exitPointerLock: TexitPointerLock = () => {
    if (!locked) return;
    try {
        (<any>globalThis.Window).exitPointerLock();
        locked = false;
        setTimeout(() => {
            lockChange && lockChange();
        }, 0);
    } catch (e) {
        setTimeout(() => {
            lockError && lockError(e);
        }, 0);
    }
};



let lockErrorFuncs: Set<Function>;
let lockError: Function;

export const onPointerLockError: TonPointerLockError = (callback: () => void) => {
    if (!lockErrorFuncs) {
        lockErrorFuncs = new Set;
    }
    lockErrorFuncs.add(callback);
    if (!lockError) {
        lockError = () => {
            lockErrorFuncs && lockErrorFuncs.forEach(cb => cb());
        }
    }
};

export const offPointerLockError: ToffPointerLockError = (callback?: () => void) => {
    if (callback) {
        if (lockErrorFuncs) {
            lockErrorFuncs.delete(callback);
        }
    } else {
        lockErrorFuncs.clear();
    }
    if (!lockErrorFuncs.size) {
        lockError = null;
    }
};


let lockChangeFuncs: Set<Function>;
let lockChange: Function;


export const onPointerLockChange: TonPointerLockChange = (callback: (locked: boolean) => void) => {
    if (!lockChangeFuncs) {
        lockChangeFuncs = new Set;
    }
    lockChangeFuncs.add(callback);
    if (!lockChange) {
        lockChange = (e) => {
            lockChangeFuncs && lockChangeFuncs.forEach(cb => cb(locked));
        }
    }

};

export const offPointerLockChange: ToffPointerLockChange = (callback?: (locked: boolean) => void) => {
    if (callback) {
        if (lockChangeFuncs) {
            lockChangeFuncs.delete(callback);
        }
    } else {
        lockChangeFuncs.clear();
    }
    if (!lockChangeFuncs.size) {
        lockChange = null;
    }
};


export const setCursorVisible: TsetCursorVisible = (visible: boolean) => {
    (<any>globalThis.Window).setCursorVisible(visible);
}

export const openAppstore: TopenAppstore = () => { };


export const privacyAgreement: TprivacyAgreement = (isAccepted: boolean) => {
    return Promise.resolve(null);
}


/**
 * 唤起客服
 */
export const showWithEntranceId: TshowWithEntranceId = () => {
}

/**
 * 客服后台用户信息
 * @returns 
 */
export const updateUserInfo: TupdateUserInfo = (str: string) => {
}

/**
 * 清空用户信息
 */
export const resetUserInfo: TresetUserInfo = () => {
}


/**
 * 唤起客服
 */
export const showWithEntranceIdLogin: TshowWithEntranceIdLogin = () => {
}
