/**
 * 系统信息
 */

import { AppName, BenchLevel, SystemInfo, TgetOAID, TgetDeviceID, TgetSystemInfo } from "pi_sys/native/base/system_info";
import { Native } from "../../../webview/native_impl/core/index";
import { Env } from "pi_sys/util/env";

const getSystemInfoSys = (): SystemInfo => {
    let ua;
    try {
        ua = navigator && navigator.userAgent;
    } catch (_) {
        ua = "";
    }
    ua = ua.toLowerCase();

    let app: AppName;      // browser, webview, native
    let platform; // windows, android, ios;
    let device;   // pc or mobile;

    if (ua.indexOf("pi_webview") >= 0) {
        app = AppName.Webview;
    } else if (ua.indexOf("pi_native") >= 0) {
        app = AppName.Native;
    } else {
        app = AppName.Browser;
    }

    if (ua.indexOf("pi_android") >= 0) {
        device = "mobile";
        platform = "android";
    } else if (ua.indexOf("pi_ios") >= 0) {
        device = "mobile";
        platform = "ios";
    } else if (ua.indexOf("pi_windows") >= 0) {
        device = "pc";
        platform = "windows";
    } else {
        device = ua.indexOf("mobile") >= 0 ? "mobile" : "pc";

        if (ua.indexOf('windows nt') > -1) {
            platform = "windows";
        } else if (ua.indexOf('iphone os') > -1) {
            platform = "ios";
        } else if (ua.indexOf('android') > -1) {
            platform = "android";
        } else if (ua.indexOf('linux') > -1) {
            platform = "linux";
        }
    }

    let system = `${platform} ${version(ua, new RegExp(platform + "([\d.]+)"))}`;

    let benchLevel = BenchLevel.Unkown;

    let isSupportWasm = false;
    try {
        if (window["WebAssembly"] && window["WebAssembly"].instantiate) {
            isSupportWasm = true;
        }
    } catch (_e) {
        isSupportWasm = false;
    }
    if (!isSupportWasm) {
        benchLevel = BenchLevel.CantRun;
    }

    return {
        safeArea: {
            left: 0,
            right: window.innerWidth,
            top: 0,
            bottom: window.innerHeight,
            width: window.innerWidth,
            height: window.innerHeight,
        },

        brand: "",
        model: "",
        pixelRatio: 1.0,
        screenWidth: window.innerWidth,
        screenHeight: window.innerHeight,
        statusBarHeight: 0,
        language: navigator.language,

        cameraAuthorized: false,
        locationAuthorized: false,
        microphoneAuthorized: false,
        notificationAuthorized: false,
        bluetoothEnabled: false,
        locationEnabled: false,
        wifiEnabled: false,

        colorDepth: 24,

        app,
        system,
        platform,
        device,
        benchLevel,
    };
}

/**
 * 获取系统信息
 */
export const getSystemInfo: TgetSystemInfo = async (): Promise<SystemInfo> => {
    let r = getSystemInfoSys();
    const rewriteSystemInfo = Env.get("rewriteSystemInfo") || window["winit"].env.rewriteSystemInfo;
    if (rewriteSystemInfo) {
        await rewriteSystemInfo(r, r);
    }
    return Promise.resolve(r);
}



/**
 * 取手机的设备ID
 */
export const getDeviceID: TgetDeviceID = (): Promise<string> => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getDeviceID(
                function (pName) {
                    resolve(pName);
                },
                reject
            );
        })
            .catch((e) => {
                reject(e);
            });
    });
}

export const getOAID: TgetOAID = () => {
    return new Promise((resolve, reject) => {
        Native.App.init().then(() => {
            Native.App.getOAID(
                function (pName) {
                    resolve(pName);
                },
                reject
            );
        })
            .catch((e) => {
                reject(e);
            });
    });
}

const version = (ua, rxp) => {
    const arr = ua.match(rxp);
    if (!arr) {
        return;
    }

    return arr[1]
};