
/**
 * 小游戏键盘输入
 */

import { KeyboardConfig, Thide, ToffKeyboardHeightChange, ToffKeyDown, ToffKeyUp, TonComplete, TonInput, TonKeyboardHeightChange, TonKeyDown, TonKeyUp, TonSelection, TsetSelection, Tshow, Tupdate, ToffInput, ToffSelection, TsupportSelection } from "pi_sys/native/device/keyboard";

import { Logger } from "pi_sys/util/logger";
declare var module;
let logger = new Logger(module.name, "sys");

// // 通知键盘监听函数
// const keyboardNotify = (notifier: Function, params: string) => {
//     if (notifier && notifier instanceof Function) {
//         notifier(params);
//     }
// };

// 维护么个输入字符串的光标位置
let CURSOR_INDEX_LIST: { [index: string]: object } = {};
let CURSOR_INDEX = {};
let keyboardValue = "";
let ctrlKeyDown = false;         // Control是否被按下,如果按下不响应其他按钮。因为有Ctrl+V等快捷键需求

let getCursorPosition;
let inputType: string;

let imeEnabled = false;


const DEFAULT_CFG: any = {
    multiple: false,
    maxLength: 0,
    confirmHold: false
};


export type ListenFunction = (value: string) => void;

// 唤起键盘
export const show: Tshow = (config: KeyboardConfig) => {
    // console.log("==========Keybord show");
    keyboardValue = config.defaultValue;
    inputType = config.attribute["type"] || "text";
    getCursorPosition = config.getCursorPosition;
    (<any>globalThis.Window).showSoftInput(keyboardValue, inputType);

    CURSOR_INDEX_LIST[config.attribute["type"]] = { selectionStart: config.defaultValue.length, selectionEnd: config.defaultValue.length };
    CURSOR_INDEX = CURSOR_INDEX_LIST[config.attribute["type"]];
    if (!isShow) {
        config = {
            ...DEFAULT_CFG,
            ...config,
        };
        delete config.attribute;
        // wx.showKeyboard(config);
        bindListener();
    }
}

// 隐藏键盘
export const hide: Thide = (p: any) => {
    // console.log("==========Keybord hide");
    if (imeEnabled) {
        imeInputListener({ type: 3, posX: 0, posY: 0, str: "" });     // 主动触发Disable事件
    }
    (<any>globalThis.Window).hideSoftInput(true);
    keyboardValue = "";
    unbindListener();
};


// 更新键盘
export const update: Tupdate = (options: { value: string, success?: Function, fail?: Function, complete?: (res: any) => void }) => {
    // console.log("==========Keybord update", options.value);
    if (typeof options.value !== typeof "") {
        options.value = options.value.toString();
    }
    if (CURSOR_INDEX['selectionEnd'] > options.value.length) {
        CURSOR_INDEX['selectionStart'] = CURSOR_INDEX['selectionEnd'] = options.value.length;
    }
    if (!keyboardValue) {
        CURSOR_INDEX['selectionStart'] = CURSOR_INDEX['selectionEnd'] = options.value.length;
    }
    keyboardValue = options.value;
    options.success && options.success(options.value);
    options.complete && options.complete(options.value);
};

export const onInput: TonInput = (callback: ({ value: string }) => void) => {
    inputListenerList.push(callback);
}

export const offInput: ToffInput = (callback?: ({ value: string }) => void) => {
    if (callback) {
        const index = inputListenerList.indexOf(callback);
        if (index >= 0) {
            inputListenerList.splice(index, 1);
        }
    } else {
        inputListenerList = []
    }

}

export const onComplete: TonComplete = (callback: (obj: { value: string }) => void) => {
    completeListenerList.push(callback);
}

export const supportSelection: TsupportSelection = () => {
    return true;
}

/**
 * 设置光标位置
 * @param index
 */
export const setSelection: TsetSelection = (index: number, x?: number, y?: number): boolean => {
    // console.log("==========Keybord setSelection", index);
    if (typeof index != typeof 0) {
        return true;
    }
    CURSOR_INDEX['selectionStart'] = index;
    CURSOR_INDEX['selectionEnd'] = index;
    for (let li of selectionListenerList) {
        (li instanceof Function) && li(CURSOR_INDEX['selectionStart']);
    }
    return true;
}

/**
 * 监听光标改变
 * @param callback 
 * @param 
 * @returns 
 */
export const onSelection: TonSelection = (callback: (value: number) => void) => {
    selectionListenerList.push(callback);
}


export const offSelection: ToffSelection = (callback?: (value: number) => void) => {
    if (callback) {
        const index = selectionListenerList.indexOf(callback);
        if (index >= 0) {
            selectionListenerList.splice(index, 1);
        }
    } else {
        selectionListenerList = []
    }
}



/**
 * 监听 虚拟键盘 高度，仅限于 支持 虚拟键盘 的 平台 
 */
export const onKeyboardHeightChange: TonKeyboardHeightChange = (callback: ({ height: number }) => void) => {
    console.warn("windows platform not supported!")
}

/**
 * 取消监听 虚拟键盘 高度，仅限于 支持 虚拟键盘 的 平台.参数为空，则取消所有的事件监听。
 */
export const offKeyboardHeightChange: ToffKeyboardHeightChange = (callback?: ({ height: number }) => void) => {
    console.warn("windows platform not supported!")
}

/**
 * 监听键盘按键按下事件，仅适用于 PC 平台
 * @param callback 
 */
export const onKeyDown: TonKeyDown = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    globalThis.onKeyDown(callback);
}

/**
 *  取消监听键盘按键按下事件，仅适用于 PC 平台
 * @param callback 
 */
export const offKeyDown: ToffKeyDown = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    globalThis.offKeyDown(callback);
};


/**
 * 监听键盘按键弹起事件，仅适用于 PC 平台
 * @param callback 
 */
export const onKeyUp: TonKeyUp = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    globalThis.onKeyUp(callback);
}

/**
 *  取消监听键盘按键弹起事件，仅适用于 PC 平台
 * @param callback 
 */
export const offKeyUp: ToffKeyUp = (callback: (param: { key: string, code: string, timeStamp: number }) => void) => {
    globalThis.offKeyUp(callback);
};


let commit = false;
let imeSelectionStart = 0;
let imeValue = "";

const imeInputListener = ({ type, posX, posY, str }) => {
    // https://docs.rs/winit/0.28.6/winit/event/enum.Ime.html#variant.Enabled
    // console.log("==========Keybord imeInputListener", "type: " + type + "; posX: " + posX + "; posY: " + posY + "; str: " + str);
    let changed = false;
    /**
     * Space空格会依次触发1 2 3事件
     * 此时1事件posX = -1,posY = -1,str = ""
     */
    switch (type) {
        // Enabled(启用) 事件  【获取此事件后，您可能会收到预编辑和提交事件】  开启中文输入法  设置输入位置
        case 0:
            imeEnabled = true;
            if (!!getCursorPosition) {
                let cursor_pos = getCursorPosition();
                // console.log("cursor_pos: " + cursor_pos[0] + " " + cursor_pos[1]);
                (<any>globalThis.Window).setImePosition(cursor_pos[0], cursor_pos[1] + 8); // 返回的y为文字底部坐标，加8像素
            }

        //  Preedit(预编辑) 事件     输入事件和左右键移动光标事件都会触发此事件，Space空格也会触发,但此时posX为-1
        case 1:
            if (posX >= 0) {
                imeValue = str;
                imeSelectionStart = posX;
                changed = true;
            }
            break;
        // Commit(提交) 事件 Space空格会触发
        case 2:
            // 此时posX为0
            if (inputType !== "number") {
                changed = true;
                imeValue = str;
                imeSelectionStart = imeValue.length;
            }
            commit = true;
            break;
        // Disabled(禁用) 事件  Space空格会触发 如果鼠标直接点其他地方,并不会触发此事件
        // 但是会触发hide(隐藏键盘)函数,这里可以通过hide函数主动触发模拟Disabled(禁用) 事件
        case 3:
            imeEnabled = false;
            if (!commit) {  // 通过模拟触发的,这时候需要消除'
                imeValue = imeValue.replace(/'/g, "");
                imeSelectionStart = imeValue.length;
                changed = true;
            }
            keyboardValue = getFinalValue();
            CURSOR_INDEX['selectionStart'] = CURSOR_INDEX['selectionEnd'] = getSelection();
            imeSelectionStart = 0;
            imeValue = "";
            commit = false;
            break;
    }

    if (changed) {
        dispatchValueChange();
        dispatchSelection();
    }

}


const bindListener = () => {
    // 在中文输入法打开的时候onKeyboardInput不会响应
    globalThis.onKeyboardInput(KeyboardInputListener);
    globalThis.onKeyDown(decodeOnKeyDown);
    globalThis.onKeyUp(decodeOnKeyUp);
    isShow = true;
    globalThis.onImeInput(imeInputListener);  // 监听中文输入
}
const unbindListener = () => {
    inputListenerList.length = 0;
    completeListenerList.length = 0;
    globalThis.offKeyboardInput(KeyboardInputListener);
    globalThis.offKeyUp(decodeOnKeyUp);
    isShow = false;

    globalThis.offImeInput(imeInputListener);
}



// 普通字符串输入
const KeyboardInputListener = (obj) => {
    if (ctrlKeyDown) return;
    let changed = false;
    switch (obj.value) {
        case '\u{7f}': // delete键
        case '\u{8}':  // backspace 键
            if (CURSOR_INDEX['selectionStart'] === 0) return;
            keyboardValue = keyboardValue.slice(0, CURSOR_INDEX['selectionStart'] - 1) + keyboardValue.slice(CURSOR_INDEX['selectionEnd']); // 删除一个字符
            CURSOR_INDEX['selectionStart'] = CURSOR_INDEX['selectionEnd'] = Math.min(keyboardValue.length, Math.max(CURSOR_INDEX['selectionStart'] - 1, 0));
            changed = true;
            break;
        case '\t': // Tab键
        case '\r': // 回车键
            break;
        default:
            if (inputType === "number") {
                // 如果是number类型,并且没有开启中文输入法,此时input是一个字符一个字符响应
                const char = obj.value;
                if (char === ".") { // 处理小数点
                    if (keyboardValue.includes(".")) { // 只能有一个小数点
                        return;
                    }
                } else if (char === "-") {
                    if (keyboardValue.includes("-") || CURSOR_INDEX['selectionStart'] !== 0) {  // 减号只能有一个并且只能在首位
                        return;
                    }
                } else if (isNaN(Number(char))) {
                    return;  // 不是数字
                }
            }
            keyboardValue = keyboardValue.slice(0, CURSOR_INDEX['selectionStart']) + obj.value + keyboardValue.slice(CURSOR_INDEX['selectionEnd']);
            CURSOR_INDEX['selectionEnd'] = CURSOR_INDEX['selectionStart'] = Math.min(keyboardValue.length, Math.max(CURSOR_INDEX['selectionStart'] + 1, 0));
            changed = true;
            break;
    }
    if (changed) {
        dispatchValueChange();
        dispatchSelection();
    }
}



const decodeOnKeyDown = (obj) => {
    switch (obj.key) {
        // 左方向(<-)键 
        case "ArrowLeft":
            CURSOR_INDEX['selectionStart'] = CURSOR_INDEX['selectionEnd'] = Math.min(keyboardValue.length, Math.max(CURSOR_INDEX['selectionStart'] - 1, 0))
            dispatchSelection();
            break;
        // 右方向(->)键 
        case "ArrowRight":
            CURSOR_INDEX['selectionStart'] = CURSOR_INDEX['selectionEnd'] = Math.min(keyboardValue.length, Math.max(CURSOR_INDEX['selectionStart'] + 1, 0))
            dispatchSelection();
            break;
        case "Control":
            ctrlKeyDown = true;
            break;
    }
}

const decodeOnKeyUp = (obj) => {
    switch (obj.key) {
        case "Control":
            ctrlKeyDown = false;
            break;
    }
}

// 获取最终的value值,合并ime和普通字符
const getFinalValue = () => {
    if (!imeValue) return keyboardValue;
    let selectionStart = CURSOR_INDEX['selectionStart'];
    let preValue = keyboardValue.substring(0, selectionStart);
    let nextValue = keyboardValue.substring(selectionStart);
    return `${preValue}${imeValue}${nextValue}`;
}

const getSelection = () => {
    if (!imeValue) return CURSOR_INDEX['selectionStart'];
    let selectionStart = CURSOR_INDEX['selectionStart'];
    return selectionStart + imeSelectionStart;
}

// 触发值更新
const dispatchValueChange = () => {
    let r = { value: getFinalValue() };
    for (let li of inputListenerList) {
        (li instanceof Function) && li(r);
    }
}

// 触发光标更新
const dispatchSelection = () => {
    for (let li of selectionListenerList) {
        (li instanceof Function) && li(getSelection());
    }
}
let inputListenerList = [];
const completeListenerList = [];
let selectionListenerList = [];
let isShow = false;



