declare var NativeObject;
declare var _$cid;

export class WebGLRenderingContextBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): WebGLRenderingContextBase {
        return new WebGLRenderingContextBase(obj);
    }

    public viewport(x: number, y: number, width: number, height: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(57, this.self, x, y, width, height);
    }

    public clear_color(r: number, g: number, b: number, a: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(58, this.self, r, g, b, a);
    }

    public clear(mask: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(59, this.self, mask);
    }

    public create_buffer(): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(60, this.self) as number;
        return result;
    }

    public bind_buffer(target: number, buffer: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(61, this.self, target, buffer);
    }

    public buffer_data(target: number, size: number, data: ArrayBuffer, usage: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(62, this.self, target, size, data, usage);
    }

    public create_shader(shader_type: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(63, this.self, shader_type) as number;
        return result;
    }

    public shader_source(shader: number, shader_code: string): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(64, this.self, shader, shader_code);
    }

    public compile_shader(shader: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(65, this.self, shader);
    }

    public attach_shader(program: number, shader: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(66, this.self, program, shader);
    }

    public create_program(): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(67, this.self) as number;
        return result;
    }

    public link_program(program: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(68, this.self, program);
    }

    public use_program(program: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(69, this.self, program);
    }

    public get_attrib_location(program: number, key: string): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(70, this.self, program, key) as number;
        return result;
    }

    public enable_vertex_attrib_array(pos: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(71, this.self, pos);
    }

    public vertex_attrib_pointer(index: number, size: number, gl_type: number, normalized: number, stride: number, pointer: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(72, this.self, index, size, gl_type, normalized, stride, pointer);
    }

    public enable(state: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(73, this.self, state);
    }

    public draw_arrays(mode: number, first: number, count: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(74, this.self, mode, first, count);
    }

    public draw_elements(mode: number, count: number, buffer_type: number, offset: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(75, this.self, mode, count, buffer_type, offset);
        if (result instanceof Error) {
            console.error(result);
            throw result;
        }
    }

    /**
     * 每渲染完一帧，调用一次此函数
     * webgl 没有此函数，不应该将此函数暴露给高层用户
     */
    public swap_buffer(): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(76, this.self);
    }

    public active_texture(texture: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(77, this.self, texture);
    }

    public bind_attrib_location(program: number, index: number, name: string): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(78, this.self, program, index, name);
    }

    public bind_framebuffer(target: number, framebuffer: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(79, this.self, target, framebuffer);
    }

    public bind_renderbuffer(target: number, renderbuffer: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(80, this.self, target, renderbuffer);
    }

    public bind_texture(target: number, texture: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(81, this.self, target, texture);
    }

    public blend_color(r: number, g: number, b: number, a: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(82, this.self, r, g, b, a);
    }

    public blend_equation(mode: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(83, this.self, mode);
    }

    public blend_equation_separate(mode_rgb: number, mode_alpha: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(84, this.self, mode_rgb, mode_alpha);
    }

    public blend_func(sfactor: number, dfactor: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(85, this.self, sfactor, dfactor);
    }

    public blend_func_separate(src_rgb: number, dst_rgb: number, src_alpha: number, dst_alpha: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(86, this.self, src_rgb, dst_rgb, src_alpha, dst_alpha);
    }

    public buffer_sub_data(target: number, offset: number, src_data: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(87, this.self, target, offset, src_data);
    }

    public check_framebuffer_status(target: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(88, this.self, target);
    }

    public clear_depthf(depth: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(89, this.self, depth);
    }

    public clear_stencil(s: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(90, this.self, s);
    }

    public color_mask(r: boolean, g: boolean, b: boolean, a: boolean): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(91, this.self, r, g, b, a);
    }

    public copy_tex_image_2d(target: number, level: number, internalformat: number, x: number, y: number, width: number, height: number, border: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(92, this.self, target, level, internalformat, x, y, width, height, border);
    }

    public copy_tex_sub_image_2d(target: number, level: number, xoffset: number, yoffset: number, x: number, y: number, width: number, height: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(93, this.self, target, level, xoffset, yoffset, x, y, width, height);
    }

    public cull_face(mode: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(94, this.self, mode);
    }

    public delete_buffers(buffers: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(95, this.self, buffers);
    }

    public delete_framebuffers(framebuffers: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(96, this.self, framebuffers);
    }

    public delete_program(program: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(97, this.self, program);
    }

    public delete_renderbuffers(renderbuffer: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(98, this.self, renderbuffer);
    }

    public delete_shader(shader: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(99, this.self, shader);
    }

    public delete_textures(textures: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(100, this.self, textures);
    }

    public depth_func(func: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(101, this.self, func);
    }

    public depth_mask(flag: boolean): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(102, this.self, flag);
    }

    public depth_range(z_near: number, z_far: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(103, this.self, z_near, z_far);
    }

    public detach_shader(program: number, shader: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(104, this.self, program, shader);
    }

    public disable(cap: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(105, this.self, cap);
    }

    public disable_vertex_attrib_array(index: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(106, this.self, index);
    }

    public finish(): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(107, this.self);
    }

    public flush(): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(108, this.self);
    }

    public framebuffer_renderbuffer(target: number, attachment: number, renderbuffertarget: number, renderbuffer: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(109, this.self, target, attachment, renderbuffertarget, renderbuffer);
    }

    public framebuffer_texture_2d(target: number, attachment: number, textarget: number, texture: number, level: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(110, this.self, target, attachment, textarget, texture, level);
    }

    public front_face(mode: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(111, this.self, mode);
    }

    public create_framebuffer(): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(112, this.self) as number;
        return result;
    }

    public create_render_buffer(): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(113, this.self) as number;
        return result;
    }

    public create_texture(): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(114, this.self) as number;
        return result;
    }

    public generate_mipmap(target: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(115, this.self, target);
    }

    public get_program_parameter(program: number, index: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(116, this.self, program, index) as number;
        return result;
    }

    public get_active_attrib(program: number, index: number): object {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(117, this.self, program, index) as object;
        return result;
    }

    public get_active_uniform(program: number, index: number): object {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(118, this.self, program, index) as object;
        return result;
    }

    public get_attached_shaders(program: number): number[] {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(119, this.self, program) as number[];
        return result;
    }

    public get_buffer_parameteriv(target: number, pname: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(120, this.self, target, pname) as number;
        return result;
    }

    public get_error(): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(121, this.self) as number;
        return result;
    }

    public get_framebuffer_attachment_parameteriv(target: number, attachment: number, pname: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(122, this.self, target, attachment, pname) as number;
        return result;
    }

    public get_program_info_log(program: number): string {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(123, this.self, program) as string;
        return result;
    }

    public get_renderbuffer_parameteriv(target: number, pname: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(124, this.self, target, pname) as number;
        return result;
    }

    public get_shader_info_log(shader: number): string {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(125, this.self, shader) as string;
        return result;
    }

    public get_shader_precision_format(shader_type: number, precision_type: number): object {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(126, this.self, shader_type, precision_type) as object;
        return result;
    }

    public get_shader_source(shader: number): string {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(127, this.self, shader) as string;
        return result;
    }

    public get_shader_parameter(shader: number, pname: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(128, this.self, shader, pname) as number;
        return result;
    }

    public get_tex_parameter(target: number, pname: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(129, this.self, target, pname) as number;
        return result;
    }

    public get_uniform(program: number, location: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(130, this.self, program, location) as number;
        return result;
    }

    public get_uniform_location(program: number, name: string): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(131, this.self, program, name) as number;
        return result;
    }

    public get_vertex_attrib(index: number, pname: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(132, this.self, index, pname) as number;
        return result;
    }

    public get_vertex_attrib_pointerv(index: number, pname: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(133, this.self, index, pname) as number;
        return result;
    }

    public hint(target: number, mode: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(134, this.self, target, mode);
    }

    public is_buffer(buffer: number): boolean {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(135, this.self, buffer) as boolean;
        return result;
    }

    public is_enabled(cap: number): boolean {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(136, this.self, cap) as boolean;
        return result;
    }

    public is_framebuffer(framebuffer: number): boolean {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(137, this.self, framebuffer) as boolean;
        return result;
    }

    public is_program(program: number): boolean {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(138, this.self, program) as boolean;
        return result;
    }

    public is_renderbuffer(renderbuffer: number): boolean {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(139, this.self, renderbuffer) as boolean;
        return result;
    }

    public is_shader(shader: number): boolean {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(140, this.self, shader) as boolean;
        return result;
    }

    public is_texture(texture: number): boolean {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(141, this.self, texture) as boolean;
        return result;
    }

    public line_width(width: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(142, this.self, width);
    }

    public pixel_storei(pname: number, param: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(143, this.self, pname, param);
    }

    public polygon_offset(factor: number, units: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(144, this.self, factor, units);
    }

    public read_pixels(x: number, y: number, width: number, height: number, format: number, gl_type: number): ArrayBuffer {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(145, this.self, x, y, width, height, format, gl_type) as ArrayBuffer;
        return result;
    }

    public renderbuffer_storage(target: number, internal_format: number, width: number, height: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(146, this.self, target, internal_format, width, height);
    }

    public sample_coverage(value: number, invert: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(147, this.self, value, invert);
    }

    public scissor(x: number, y: number, width: number, height: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(148, this.self, x, y, width, height);
    }

    public stencil_func(func: number, gl_ref: number, mask: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(149, this.self, func, gl_ref, mask);
    }

    public stencil_func_separate(face: number, func: number, gl_ref: number, mask: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(150, this.self, face, func, gl_ref, mask);
    }

    public stencil_mask(mask: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(151, this.self, mask);
    }

    public stencil_mask_separate(face: number, mask: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(152, this.self, face, mask);
    }

    public stencil_op(fail: number, zfail: number, zpass: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(153, this.self, fail, zfail, zpass);
    }

    public stencil_op_separate(face: number, fail: number, zfail: number, zpass: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(154, this.self, face, fail, zfail, zpass);
    }

    public tex_image_2d(target: number, level: number, internalformat: number, width: number, height: number, border: number, format: number, gl_type: number, pixels: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(155, this.self, target, level, internalformat, width, height, border, format, gl_type, pixels);
    }

    public tex_image_2d_of_ptr(target: number, level: number, internalformat: number, width: number, height: number, border: number, format: number, gl_type: number, pixels_ptr: object): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(156, this.self, target, level, internalformat, width, height, border, format, gl_type, pixels_ptr);
    }

    public tex_parameter_f(target: number, pname: number, param: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(157, this.self, target, pname, param);
    }

    public tex_parameter_i(target: number, pname: number, param: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(158, this.self, target, pname, param);
    }

    public tex_sub_image_2d(target: number, level: number, xoffset: number, yoffset: number, width: number, height: number, format: number, gl_type: number, pixels: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(159, this.self, target, level, xoffset, yoffset, width, height, format, gl_type, pixels);
    }

    public uniform1i(location: number, v0: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(160, this.self, location, v0);
    }

    public uniform1f(location: number, v0: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(161, this.self, location, v0);
    }

    public uniform2i(location: number, v0: number, v1: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(162, this.self, location, v0, v1);
    }

    public uniform2f(location: number, v0: number, v1: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(163, this.self, location, v0, v1);
    }

    public uniform3i(location: number, v0: number, v1: number, v2: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(164, this.self, location, v0, v1, v2);
    }

    public uniform3f(location: number, v0: number, v1: number, v2: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(165, this.self, location, v0, v1, v2);
    }

    public uniform4i(location: number, v0: number, v1: number, v2: number, v3: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(166, this.self, location, v0, v1, v2, v3);
    }

    public uniform4f(location: number, v0: number, v1: number, v2: number, v3: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(167, this.self, location, v0, v1, v2, v3);
    }

    public uniform1iv(location: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(168, this.self, location, value);
    }

    public uniform1fv(location: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(169, this.self, location, value);
    }

    public uniform2iv(location: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(170, this.self, location, value);
    }

    public uniform2fv(location: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(171, this.self, location, value);
    }

    public uniform3iv(location: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(172, this.self, location, value);
    }

    public uniform3fv(location: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(173, this.self, location, value);
    }

    public uniform4iv(location: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(174, this.self, location, value);
    }

    public uniform4fv(location: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(175, this.self, location, value);
    }

    public uniform_matrix2fv(location: number, transpose: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(176, this.self, location, transpose, value);
    }

    public uniform_matrix3fv(location: number, transpose: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(177, this.self, location, transpose, value);
    }

    public uniform_matrix4fv(location: number, transpose: boolean, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(178, this.self, location, transpose, value);
    }

    public validate_program(program: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(179, this.self, program);
    }

    public vertex_attrib1f(index: number, v0: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(180, this.self, index, v0);
    }

    public vertex_attrib2f(index: number, v0: number, v1: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(181, this.self, index, v0, v1);
    }

    public vertex_attrib3f(index: number, v0: number, v1: number, v2: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(182, this.self, index, v0, v1, v2);
    }

    public vertex_attrib4f(index: number, v0: number, v1: number, v2: number, v3: number): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(183, this.self, index, v0, v1, v2, v3);
    }

    public vertex_attrib1fv(index: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(184, this.self, index, value);
    }

    public vertex_attrib2fv(index: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(185, this.self, index, value);
    }

    public vertex_attrib3fv(index: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(186, this.self, index, value);
    }

    public vertex_attrib4fv(index: number, value: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(187, this.self, index, value);
    }

    public compressed_tex_image_2d(target: number, level: number, internalformat: number, width: number, height: number, border: number, data: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(188, this.self, target, level, internalformat, width, height, border, data);
    }

    public compressed_tex_sub_image_2d(target: number, level: number, xoffset: number, yoffset: number, width: number, height: number, format: number, data: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(189, this.self, target, level, xoffset, yoffset, width, height, format, data);
    }

    /**
     * 反回一个所有的支持WebGL 扩展的列表
     */
    public get_supported_extensions(): string {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(190, this.self) as string;
        return result;
    }

    /**
     * 启用WebGL扩展。
     */
    public get_compressed_texture_extension(_name: string): object {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(191, this.self, _name) as object;
        return result;
    }

    public get_instanced_arrays_extension(_name: string): object | undefined {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(192, this.self, _name) as object;
        return result;
    }

    public get_vertex_array_object_extension(_name: string): object | undefined {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(193, this.self, _name) as object;
        return result;
    }

    public get_integerv(pname: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(194, this.self, pname) as number;
        return result;
    }

    public get_floatv(pname: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(195, this.self, pname) as number;
        return result;
    }

    public get_booleanv(pname: number): number {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(196, this.self, pname) as number;
        return result;
    }

    public get_string(pname: number): string {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(197, this.self, pname) as string;
        return result;
    }

    public unbind_surface(version: string): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(198, this.self, version);
    }

    public bind_surface(): string | Error {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        let result = NativeObject.call(199, this.self) as string;
        return result;
    }

    public destory_context(): void {
        if (this.self == undefined) {
            throw new Error("WebGLRenderingContextBase object already destroy");
        }

        NativeObject.call(200, this.self);
    }

}









/**
* opengl es 调试用，着色器加载后激活的attribule，uniform等变量的信息
*/
class WebGLActiveInfoBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("WebGLActiveInfoBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): WebGLActiveInfoBase {
        return new WebGLActiveInfoBase(obj);
    }

    public get_name(): string {
        if (this.self == undefined) {
            throw new Error("WebGLActiveInfoBase object already destroy");
        }

        let result = NativeObject.call(51, this.self) as string;
        return result;
    }

    public get_size(): number {
        if (this.self == undefined) {
            throw new Error("WebGLActiveInfoBase object already destroy");
        }

        let result = NativeObject.call(52, this.self) as number;
        return result;
    }

    public get_type(): number {
        if (this.self == undefined) {
            throw new Error("WebGLActiveInfoBase object already destroy");
        }

        let result = NativeObject.call(53, this.self) as number;
        return result;
    }

}

class WebGLShaderPrecisionFormatBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("WebGLShaderPrecisionFormatBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): WebGLShaderPrecisionFormatBase {
        return new WebGLShaderPrecisionFormatBase(obj);
    }

    public get_range_max(): number {
        if (this.self == undefined) {
            throw new Error("WebGLShaderPrecisionFormatBase object already destroy");
        }

        let result = NativeObject.call(54, this.self) as number;
        return result;
    }

    public get_rangr_min(): number {
        if (this.self == undefined) {
            throw new Error("WebGLShaderPrecisionFormatBase object already destroy");
        }

        let result = NativeObject.call(55, this.self) as number;
        return result;
    }

    public get_precision(): number {
        if (this.self == undefined) {
            throw new Error("WebGLShaderPrecisionFormatBase object already destroy");
        }

        let result = NativeObject.call(56, this.self) as number;
        return result;
    }

}








/**
* 纹理压缩
*/
class WebGLCompressedTextureS3TCBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("WebGLCompressedTextureS3TCBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): WebGLCompressedTextureS3TCBase {
        return new WebGLCompressedTextureS3TCBase(obj);
    }

    public get_rgba_dxt1(): boolean {
        if (this.self == undefined) {
            throw new Error("WebGLCompressedTextureS3TCBase object already destroy");
        }

        let result = NativeObject.call(201, this.self) as boolean;
        return result;
    }

    public get_rgba_dxt3(): boolean {
        if (this.self == undefined) {
            throw new Error("WebGLCompressedTextureS3TCBase object already destroy");
        }

        let result = NativeObject.call(202, this.self) as boolean;
        return result;
    }

    public get_rgba_dxt5(): boolean {
        if (this.self == undefined) {
            throw new Error("WebGLCompressedTextureS3TCBase object already destroy");
        }

        let result = NativeObject.call(203, this.self) as boolean;
        return result;
    }

}

/**
 * VAO 扩展
 */
class OESVertexArrayObjectBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("OESVertexArrayObjectBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): OESVertexArrayObjectBase {
        return new OESVertexArrayObjectBase(obj);
    }

    public bind_vertex_array_oes(vao: number): void {
        if (this.self == undefined) {
            throw new Error("OESVertexArrayObjectBase object already destroy");
        }

        NativeObject.call(204, this.self, vao);
    }

    public create_vertex_array_oes(): number {
        if (this.self == undefined) {
            throw new Error("OESVertexArrayObjectBase object already destroy");
        }

        let result = NativeObject.call(205, this.self) as number;
        return result;
    }

    public delete_vertex_array_oes(vao: number): void {
        if (this.self == undefined) {
            throw new Error("OESVertexArrayObjectBase object already destroy");
        }

        NativeObject.call(206, this.self, vao);
    }

    public is_vertex_array_oes(vao: number): boolean {
        if (this.self == undefined) {
            throw new Error("OESVertexArrayObjectBase object already destroy");
        }

        let result = NativeObject.call(207, this.self, vao) as boolean;
        return result;
    }

}

/**
 * 几何实例化
 */
class ANGLEInstancedArraysBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("ANGLEInstancedArraysBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): ANGLEInstancedArraysBase {
        return new ANGLEInstancedArraysBase(obj);
    }

    public draw_arrays_instanced_angle(mode: number, first: number, count: number, primcount: number): void {
        if (this.self == undefined) {
            throw new Error("ANGLEInstancedArraysBase object already destroy");
        }

        NativeObject.call(208, this.self, mode, first, count, primcount);
    }

    public draw_elements_instanced_angle(mode: number, count: number, gl_type: number, offset: number, primcount: number): void {
        if (this.self == undefined) {
            throw new Error("ANGLEInstancedArraysBase object already destroy");
        }

        let result = NativeObject.call(209, this.self, mode, count, gl_type, offset, primcount);
        if (result instanceof Error) {
            console.error(result);
            throw result;
        }
    }

    public vertex_attrib_divisor_angle(index: number, divisor: number): void {
        if (this.self == undefined) {
            throw new Error("ANGLEInstancedArraysBase object already destroy");
        }

        NativeObject.call(210, this.self, index, divisor);
    }

}






class SvgRendererBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("SvgRendererBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): SvgRendererBase {
        return new SvgRendererBase(obj);
    }

    static new(): SvgRendererBase {
        let result = NativeObject.static_call(9) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new SvgRendererBase(result);
        }
    }

    public set_target(fbo_id: number, target_w: number, target_h: number): void {
        if (this.self == undefined) {
            throw new Error("SvgRendererBase object already destroy");
        }

        NativeObject.call(45, this.self, fbo_id, target_w, target_h);
    }

    public set_viewport(x: number, y: number, width: number, height: number): void {
        if (this.self == undefined) {
            throw new Error("SvgRendererBase object already destroy");
        }

        NativeObject.call(46, this.self, x, y, width, height);
    }

    public set_clear_color(r: number, g: number, b: number, a: number): void {
        if (this.self == undefined) {
            throw new Error("SvgRendererBase object already destroy");
        }

        NativeObject.call(47, this.self, r, g, b, a);
    }

    public load_svg(data: ArrayBuffer): string | Error {
        if (this.self == undefined) {
            throw new Error("SvgRendererBase object already destroy");
        }

        let result = NativeObject.call(48, this.self, data) as string;
        return result;
    }

    public draw_once(scene_ptr: string): string | Error {
        if (this.self == undefined) {
            throw new Error("SvgRendererBase object already destroy");
        }

        let result = NativeObject.call(49, this.self, scene_ptr) as string;
        return result;
    }

    public destroy_scene(scene_ptr: string): void {
        if (this.self == undefined) {
            throw new Error("SvgRendererBase object already destroy");
        }

        NativeObject.call(50, this.self, scene_ptr);
    }

}










class WebGLRenderingContext {
    private context: WebGLRenderingContextBase;
    private texture_type: WEBGLCompressedTextureS3TC;
    canvas: object;
    drawingBufferWidth: number;
    drawingBufferHeight: number;

    private constructor(obj: object, width: number, height: number) {
        let context = WebGLRenderingContextBase.from(obj);
        this.context = context;
        let canvas = {
            width,
            height,
        }

        this.drawingBufferWidth = width;
        this.drawingBufferHeight = height;

        this.canvas = canvas;
        this.texture_type = null;
    }

    static from(obj: object, width: number, height: number): WebGLRenderingContext {
        return new WebGLRenderingContext(obj, width, height);
    }

    public viewport(x: number, y: number, width: number, height: number): void {
        this.context.viewport(x, y, width, height);
    }

    public clearColor(r: number, g: number, b: number, a: number): void {
        this.context.clear_color(r, g, b, a);
    }

    public clear(mask: number): void {
        this.context.clear(mask);
    }

    public createBuffer(): WebGLBuffer {
        let buffer = this.context.create_buffer();
        return new WebGLBuffer(buffer);
    }
    public bindBuffer(target: number, buffer: WebGLBuffer): void {
        if (!buffer) {
            this.context.bind_buffer(target, 0);
        } else {
            this.context.bind_buffer(target, buffer.bufferPtr);
        }

    }

    /**
     * buffer_type: 0 为 gl::ARRAY_BUFFER
     * draw_type: 0 为 gl::STATIC_DRAW
     */
    public bufferData(target: number, data: Float32Array | Uint16Array | ArrayBuffer | number, usage: number): void {
        if (typeof (data) === typeof 0) {
            let buffer = this.getArrayBuffer(new Float32Array(0));
            let size = data as number;
            this.context.buffer_data(target, size, buffer, usage);
        } else if (data instanceof ArrayBuffer) {
            let buffer = (data as ArrayBuffer);
            let size = buffer.byteLength;
            this.context.buffer_data(target, size, buffer, usage);
        } else {
            let buffer = this.getArrayBuffer(data as Float32Array | Uint16Array);
            let size = buffer.byteLength;
            this.context.buffer_data(target, size, buffer, usage);
        }

    }



    public createShader(shader_type: number): WebGLShader {
        let shader = this.context.create_shader(shader_type);
        return new WebGLShader(shader);
    }

    public shaderSource(shader: WebGLShader, shader_code: string): void {
        this.context.shader_source(shader.shaderPtr, shader_code);
    }

    public compileShader(shader: WebGLShader): void {
        this.context.compile_shader(shader.shaderPtr);
    }

    public attachShader(program: WebGLProgram, shader: WebGLShader): void {
        this.context.attach_shader(program.programPtr, shader.shaderPtr);
    }

    public createProgram(): WebGLProgram {
        let program = this.context.create_program();
        return new WebGLProgram(program);
    }

    public linkProgram(program: WebGLProgram): void {
        this.context.link_program(program.programPtr);
    }

    public useProgram(program: WebGLProgram): void {
        if (program) {
            this.context.use_program(program.programPtr);
        } else {
            this.context.use_program(0);
        }

    }

    public getAttribLocation(program: WebGLProgram, key: string): number {
        return this.context.get_attrib_location(program.programPtr, key);
    }

    public enableVertexAttribArray(pos: number): void {
        this.context.enable_vertex_attrib_array(pos);
    }

    public vertexAttribPointer(index: number, size: number, gl_type: number, normalized: boolean, stride: number, pointer: number): void {
        if (normalized) {
            this.context.vertex_attrib_pointer(index, size, gl_type, this.TRUE, stride, pointer);
        } else {
            this.context.vertex_attrib_pointer(index, size, gl_type, this.FALSE, stride, pointer);
        }
    }

    public enable(state: number): void {
        this.context.enable(state);
    }

    public drawArrays(mode: number, first: number, count: number): void {
        this.context.draw_arrays(mode, first, count);
    }

    public drawElements(mode: number, count: number, buffer_type: number, offset: number): void {
        this.context.draw_elements(mode, count, buffer_type, offset);
    }

    public swapBuffer(): void {
        this.context.swap_buffer();
    }

    public activeTexture(texture: number): void {
        this.context.active_texture(texture);
    }

    public bindAttribLocation(program: WebGLProgram, index: number, name: string): void {
        this.context.bind_attrib_location(program.programPtr, index, name);
    }

    public bindFramebuffer(target: number, framebuffer: WebGLFramebuffer): void {
        if (!framebuffer) {
            this.context.bind_framebuffer(target, 0);
        } else {
            this.context.bind_framebuffer(target, framebuffer.framebufferPtr);
        }

    }

    public bindRenderbuffer(target: number, renderbuffer: WebGLRenderbuffer): void {
        if (!renderbuffer) {
            this.context.bind_renderbuffer(target, 0);
        } else {
            this.context.bind_renderbuffer(target, renderbuffer.renderbufferPtr);
        }
    }

    public bindTexture(target: number, texture: WebGLTexture): void {
        if (!texture) {
            this.context.bind_texture(target, 0);
        } else {
            this.context.bind_texture(target, texture.texturePtr);
        }

    }

    public blendColor(r: number, g: number, b: number, a: number): void {
        this.context.blend_color(r, g, b, a);
    }

    public blendEquation(mode: number): void {
        this.context.blend_equation(mode);
    }

    public blendEquationSeparate(mode_rgb: number, mode_alpha: number): void {
        this.context.blend_equation_separate(mode_rgb, mode_alpha);
    }

    public blendFunc(sfactor: number, dfactor: number): void {
        this.context.blend_func(sfactor, dfactor);
    }

    public blendFuncSeparate(src_rgb: number, dst_rgb: number, src_alpha: number, dst_alpha: number): void {
        this.context.blend_func_separate(src_rgb, dst_rgb, src_alpha, dst_alpha);
    }

    public bufferSubData(target: number, offset: number, src_data: Float32Array | Uint16Array | ArrayBuffer): void {
        let buffer: ArrayBuffer;
        if (src_data instanceof ArrayBuffer) {
            buffer = (src_data as ArrayBuffer);
        } else {
            buffer = this.getArrayBuffer(src_data as Float32Array | Uint16Array);
        }
        this.context.buffer_sub_data(target, offset, buffer);
    }

    public checkFramebufferStatus(target: number): void {
        this.context.check_framebuffer_status(target);
    }

    public clearDepth(depth: number): void {
        this.context.clear_depthf(depth);
    }

    public clearStencil(s: number): void {
        this.context.clear_stencil(s);
    }

    public colorMask(r: boolean, g: boolean, b: boolean, a: boolean): void {
        this.context.color_mask(r, g, b, a);
    }

    public copyTexImage2D(target: number, level: number, internalformat: number, x: number, y: number, width: number, height: number, border: number): void {
        this.context.copy_tex_image_2d(target, level, internalformat, x, y, width, height, border);
    }

    public copyTexSubImage2D(target: number, level: number, xoffset: number, yoffset: number, x: number, y: number, width: number, height: number): void {
        this.context.copy_tex_sub_image_2d(target, level, xoffset, yoffset, x, y, width, height);
    }

    public cullFace(mode: number): void {
        this.context.cull_face(mode);
    }

    public deleteBuffer(buffers: WebGLBuffer): void {
        if(!buffers){
            return;
        }
        this.context.delete_buffers(buffers.bufferPtr);
    }

    public deleteFramebuffer(framebuffers: WebGLFramebuffer): void {
        this.context.delete_framebuffers(framebuffers.framebufferPtr);
    }

    public deleteProgram(program: WebGLProgram): void {
        this.context.delete_program(program.programPtr);
    }

    public deleteRenderbuffer(renderbuffers: WebGLRenderbuffer): void {
        this.context.delete_renderbuffers(renderbuffers.renderbufferPtr);
    }

    public deleteShader(shader: WebGLShader): void {
        this.context.delete_shader(shader.shaderPtr);
    }

    public deleteTexture(textures: WebGLTexture): void {
        this.context.delete_textures(textures.texturePtr);
    }

    public depthFunc(func: number): void {
        this.context.depth_func(func);
    }

    public depthMask(flag: boolean): void {
        this.context.depth_mask(flag);
    }

    public depthRange(z_near: number, z_far: number): void {
        this.context.depth_range(z_near, z_far);
    }

    public detachShader(program: WebGLProgram, shader: WebGLShader): void {
        this.context.detach_shader(program.programPtr, shader.shaderPtr);
    }

    public disable(cap: number): void {
        this.context.disable(cap);
    }

    public disableVertexAttribArray(index: number): void {
        this.context.disable_vertex_attrib_array(index);
    }

    public finish(): void {
        this.context.finish();
    }

    public flush(): void {
        this.context.flush();
    }

    public framebufferRenderbuffer(target: number, attachment: number, renderbuffertarget: number, renderbuffer: WebGLRenderbuffer): void {
        if (!!renderbuffer) {
            this.context.framebuffer_renderbuffer(target, attachment, renderbuffertarget, renderbuffer.renderbufferPtr);
        } else {
            this.context.framebuffer_renderbuffer(target, attachment, renderbuffertarget, 0);
        }
    }

    public framebufferTexture2D(target: number, attachment: number, textarget: number, texture: WebGLTexture, level: number): void {
        this.context.framebuffer_texture_2d(target, attachment, textarget, texture.texturePtr, level);
    }

    public frontFace(mode: number): void {
        this.context.front_face(mode);
    }

    public createFramebuffer(): WebGLFramebuffer {
        let result = this.context.create_framebuffer();
        return new WebGLFramebuffer(result);
    }

    public createRenderbuffer(): WebGLRenderbuffer {
        let buffer = this.context.create_render_buffer();
        return new WebGLRenderbuffer(buffer);
    }

    public createTexture(): WebGLTexture {
        let tex = this.context.create_texture();
        return new WebGLTexture(tex);
    }

    public compressedTexImage2D(
        target: number,
        level: number,
        internalformat: number,
        width: number,
        height: number,
        border: number,
        pixels: Uint8Array | Uint16Array | Float32Array | null): void {

        let p: ArrayBuffer;
        if (!pixels) {
            p = new Float32Array(0).buffer;
        } else {
            p = this.getArrayBuffer(pixels);
        }

        this.context.compressed_tex_image_2d(target, level, internalformat, width, height, border, p);
    }

    public compressedTexSubImage2D(
        target: number,
        level: number,
        xoffset: number,
        yoffset: number,
        width: number,
        height: number,
        format: number,
        pixels: Uint8Array | Uint16Array | Float32Array | null): void {

        let p: ArrayBuffer;
        if (!pixels) {
            p = new Float32Array(0).buffer;
        } else {
            p = this.getArrayBuffer(pixels);
        }

        this.context.compressed_tex_sub_image_2d(target, level, xoffset, yoffset, width, height, format, p);
    }

    public generateMipmap(target: number): void {
        this.context.generate_mipmap(target);
    }

    public getProgramParameter(program: WebGLProgram, index: number): number {
        let result = this.context.get_program_parameter(program.programPtr, index);
        return result;
    }

    public getActiveAttrib(program: WebGLProgram, index: number): object {
        let obj = this.context.get_active_attrib(program.programPtr, index);

        return new WebGLActiveInfo(obj);
    }

    public getActiveUniform(program: WebGLProgram, index: number): object {
        let obj = this.context.get_active_uniform(program.programPtr, index);

        return new WebGLActiveInfo(obj);
    }

    public getAttachedShaders(program: WebGLProgram): object {
        let result = this.context.get_attached_shaders(program.programPtr);
        return result;
    }

    public getBufferParameteriv(target: number, pname: number): number {
        let result = this.context.get_buffer_parameteriv(target, pname);
        return result;
    }

    public getError(): number {
        let result = this.context.get_error();
        return result;
    }

    public getFramebufferAttachmentParameteriv(target: number, attachment: number, pname: number): number {
        let result = this.context.get_framebuffer_attachment_parameteriv(target, attachment, pname);
        return result;
    }

    public getProgramInfoLog(program: WebGLProgram): string {
        let result = this.context.get_program_info_log(program.programPtr) as string;

        return result;
    }

    public getRenderbufferParameteriv(target: number, pname: number): number {
        let result = this.context.get_renderbuffer_parameteriv(target, pname);
        return result;
    }

    public getShaderInfoLog(shader: WebGLShader): string {
        let result = this.context.get_shader_info_log(shader.shaderPtr) as string;
        return result;
    }

    public getShaderPrecisionFormat(shader_type: number, precision_type: number): object {
        let obj = this.context.get_shader_precision_format(shader_type, precision_type) as object;
        return new WebGLShaderPrecisionFormat(obj);
    }

    public getShaderSource(shader: WebGLShader): string {
        let result = this.context.get_shader_source(shader.shaderPtr) as string;
        return result;
    }

    public getShaderParameter(shader: WebGLShader, pname: number): number {
        let result = this.context.get_shader_parameter(shader.shaderPtr, pname);
        return result;
    }

    public getTexParameter(target: number, pname: number): number {
        let result = this.context.get_tex_parameter(target, pname);
        return result;
    }

    public getUniform(program: WebGLProgram, location: number): number {
        let result = this.context.get_uniform(program.programPtr, location);
        return result;
    }

    public getUniformLocation(program: WebGLProgram, name: string): WebGLUniformLocaton | null {
        let result = this.context.get_uniform_location(program.programPtr, name);
        if (result < 0) {
            return null;
        } else {
            return new WebGLUniformLocaton(result);
        }
    }

    public getVertexAttrib(index: number, pname: number): number {
        let result = this.context.get_vertex_attrib(index, pname);
        return result;
    }

    public getVertexAttribPointerv(index: number, pname: number): number {
        let result = this.context.get_vertex_attrib_pointerv(index, pname);
        return result;
    }

    public hint(target: number, mode: number): void {
        this.context.hint(target, mode);
    }

    public isBuffer(buffer: WebGLBuffer): boolean {
        let result = this.context.is_buffer(buffer.bufferPtr);
        return result;
    }

    public isEnabled(cap: number): boolean {
        let result = this.context.is_enabled(cap);
        return result;
    }

    public isFramebuffer(framebuffer: WebGLFramebuffer): boolean {
        let result = this.context.is_framebuffer(framebuffer.framebufferPtr);
        return result;
    }

    public isProgram(program: WebGLProgram): boolean {
        let result = this.context.is_program(program.programPtr);
        return result;
    }

    public isRenderbuffer(renderbuffer: WebGLRenderbuffer): boolean {
        let result = this.context.is_renderbuffer(renderbuffer.renderbufferPtr);
        return result;
    }

    public isShader(shader: WebGLShader): boolean {
        let result = this.context.is_shader(shader.shaderPtr);
        return result;
    }

    public isTexture(texture: WebGLTexture): boolean {
        let result = this.context.is_texture(texture.texturePtr);
        return result;
    }

    public lineWidth(width: number): void {
        this.context.line_width(width);
    }

    // !TODO: 暂不支持
    // UNPACK_PREMULTIPLY_ALPHA_WEBGL
    // UNPACK_COLORSPACE_CONVERSION_WEBGL
    public pixelStorei(pname: number, param: number | boolean): void {
        if (pname !== this.UNPACK_PREMULTIPLY_ALPHA_WEBGL
            && pname !== this.UNPACK_COLORSPACE_CONVERSION_WEBGL
        ) {
            if (typeof param === typeof 0) {
                this.context.pixel_storei(pname, param as number);
            } else if (typeof param === typeof true) {
                if (param) {
                    this.context.pixel_storei(pname, 1);
                } else {
                    this.context.pixel_storei(pname, 0);
                }
            }

        }
    }

    public polygonOffset(factor: number, units: number): void {
        this.context.polygon_offset(factor, units);
    }

    public readPixels(x: number, y: number, width: number, height: number, format: number, gl_type: number, pixels: ArrayBuffer): void {
        // let p = this.getArrayBuffer(pixels);
        pixels = this.context.read_pixels(x, y, width, height, format, gl_type);
    }

    public renderbufferStorage(target: number, internal_format: number, width: number, height: number): void {
        this.context.renderbuffer_storage(target, internal_format, width, height);
    }

    public sampleCoverage(value: number, invert: number): void {
        this.context.sample_coverage(value, invert);
    }

    public scissor(x: number, y: number, width: number, height: number): void {
        this.context.scissor(x, y, width, height);
    }

    public stencilFunc(func: number, gl_ref: number, mask: number): void {
        this.context.stencil_func(func, gl_ref, mask);
    }

    public stencilFuncSeparate(face: number, func: number, gl_ref: number, mask: number): void {
        this.context.stencil_func_separate(face, func, gl_ref, mask);
    }

    public stencilMask(mask: number): void {
        this.context.stencil_mask(mask);
    }

    public stencilMaskSeparate(face: number, mask: number): void {
        this.context.stencil_mask_separate(face, mask);
    }

    public stencilOp(fail: number, zfail: number, zpass: number): void {
        this.context.stencil_op(fail, zfail, zpass);
    }

    public stencilOpSeparate(face: number, fail: number, zfail: number, zpass: number): void {
        this.context.stencil_op_separate(face, fail, zfail, zpass);
    }

    public texImage2D(
        target: number,
        level: number,
        internalformat: number,
        widthOrFormat: number,
        heightOrGLType: number,
        borderOrImage: number | object,
        format?: number,
        GLType?: number,
        pixels?: Uint8Array | Uint16Array | Float32Array | Int16Array | Int32Array | Int8Array | Uint32Array | Float64Array): void {

        let width: number;
        let height: number;
        let border: number;
        let mFormat: number;
        let mGLType: number;
        let buffer: ArrayBuffer;
        if (format && GLType) {
            if (typeof widthOrFormat === typeof 0 && typeof heightOrGLType === typeof 0) {
                width = widthOrFormat;
                height = heightOrGLType;
                border = borderOrImage as number;
                mFormat = format;
                mGLType = GLType;

                if (!pixels) {
                    buffer = new Uint8Array([0]).buffer;
                } else {
                    buffer = this.getArrayBuffer(pixels as Uint8Array | Uint16Array | Float32Array | Int16Array | Int32Array | Int8Array | Uint32Array | Float64Array);
                }
                this.context.tex_image_2d(target, level, internalformat, width, height, border, mFormat, mGLType, buffer);
            }
        } else {
            let obj = borderOrImage as object;
            // 离屏处理
            if (obj["contextName"] === "webgl" && obj["id"] > 0) {
                width = obj["width"];
                height = obj["height"];
                border = 0;
                mFormat = widthOrFormat;
                mGLType = heightOrGLType;
                let context = obj["context"];

                let contextBase = context["context"];
                let buffer = contextBase.read_pixels(0, 0, width, height, mFormat, mGLType);
                this.context.tex_image_2d(target, level, internalformat, width, height, border, mFormat, mGLType, buffer);

            } else if (obj instanceof HTMLVideoElement) {
                let frame = obj.getCurrentFrame();
                if (frame) {
                    buffer = frame['frame'];
                    if (!!buffer) {
                        width = frame["width"];
                        height = frame["height"];
                        border = 0;
                        mFormat = widthOrFormat;
                        mGLType = heightOrGLType;
                        this.context.tex_image_2d_of_ptr(target, level, internalformat, width, height, border, mFormat, mGLType, buffer);
                    }
                }
            } else {
                width = obj["width"];
                height = obj["height"];
                border = 0;
                mFormat = widthOrFormat;
                mGLType = heightOrGLType;
                buffer = obj['buffer'];

                this.context.tex_image_2d(target, level, internalformat, width, height, border, mFormat, mGLType, buffer);
            }


        }
    }

    public texParameterf(target: number, pname: number, param: number): void {
        this.context.tex_parameter_f(target, pname, param);
    }

    public texParameteri(target: number, pname: number, param: number): void {
        this.context.tex_parameter_i(target, pname, param);
    }

    public texSubImage2D(
        target: number,
        level: number,
        xoffset: number,
        yoffset: number,
        widthOrFormat: number,
        heightOrGLType: number,
        formatOrObj: number | object,
        GLType?: number,
        pixels?: Uint8Array | Uint16Array | Float32Array | Int16Array | Int32Array | Int8Array | Uint32Array | Float64Array): void {

        let width: number;
        let height: number;
        let mFormat: number;
        let mGLType: number;
        let buffer: ArrayBuffer;

        if (GLType && pixels) {
            if (typeof widthOrFormat === typeof 0 && typeof heightOrGLType === typeof 0) {
                width = widthOrFormat;
                height = heightOrGLType;
                mFormat = formatOrObj as number;
                mGLType = GLType;

                if (!pixels) {
                    buffer = new Uint8Array([0]).buffer;
                } else {
                    buffer = this.getArrayBuffer(pixels as Uint8Array | Uint16Array | Float32Array | Int16Array | Int32Array | Int8Array | Uint32Array | Float64Array);
                }
                this.context.tex_sub_image_2d(target, level, xoffset, yoffset, width, height, mFormat, mGLType, buffer);
            }
        } else {
            let obj = formatOrObj as object;
            width = obj["width"];
            height = obj["height"];
            mFormat = widthOrFormat;
            mGLType = heightOrGLType;
            buffer = obj['buffer'];

            this.context.tex_sub_image_2d(target, level, xoffset, yoffset, width, height, mFormat, mGLType, buffer);
        }
    }

    public uniform1i(location: WebGLUniformLocaton, v0: number): void {
        this.context.uniform1i(location.loc, v0);
    }

    public uniform1f(location: WebGLUniformLocaton, v0: number): void {
        this.context.uniform1f(location.loc, v0);
    }

    public uniform2i(location: WebGLUniformLocaton, v0: number, v1: number): void {
        this.context.uniform2i(location.loc, v0, v1);
    }

    public uniform2f(location: WebGLUniformLocaton, v0: number, v1: number): void {
        this.context.uniform2f(location.loc, v0, v1);
    }

    public uniform3i(location: WebGLUniformLocaton, v0: number, v1: number, v2: number): void {
        this.context.uniform3i(location.loc, v0, v1, v2);
    }

    public uniform3f(location: WebGLUniformLocaton, v0: number, v1: number, v2: number): void {
        this.context.uniform3f(location.loc, v0, v1, v2);
    }

    public uniform4i(location: WebGLUniformLocaton, v0: number, v1: number, v2: number, v3: number): void {
        this.context.uniform4i(location.loc, v0, v1, v2, v3);
    }

    public uniform4f(location: WebGLUniformLocaton, v0: number, v1: number, v2: number, v3: number): void {
        this.context.uniform4f(location.loc, v0, v1, v2, v3);
    }

    public uniform1iv(location: WebGLUniformLocaton, value: Int32Array): void {
        this.context.uniform1iv(location.loc, value.buffer);
    }

    public uniform1fv(location: WebGLUniformLocaton, value: Float32Array | number[]): void {
        if (value instanceof Array) {
            let buf = new Float32Array(value);
            this.context.uniform1fv(location.loc, buf.buffer);
        } else {
            let buffer = this.getArrayBuffer(value);
            this.context.uniform1fv(location.loc, buffer);
        }
    }

    public uniform2iv(location: WebGLUniformLocaton, value: Int32Array): void {
        this.context.uniform2iv(location.loc, value.buffer);
    }

    public uniform2fv(location: WebGLUniformLocaton, value: Float32Array | Array<number>): void {
        if (value instanceof Array) {
            let buf = new Float32Array(value);
            this.context.uniform2fv(location.loc, buf.buffer);
        } else {
            let buf = this.getArrayBuffer(value);
            this.context.uniform2fv(location.loc, buf);
        }
    }

    public uniform3iv(location: WebGLUniformLocaton, value: Int32Array): void {
        this.context.uniform3iv(location.loc, value.buffer);
    }

    public uniform3fv(location: WebGLUniformLocaton, value: Float32Array | Array<number>): void {
        if (value instanceof Array) {
            let buf = new Float32Array(value);
            this.context.uniform3fv(location.loc, buf.buffer);
        } else {
            let buf = this.getArrayBuffer(value);
            this.context.uniform3fv(location.loc, buf);
        }
    }

    public uniform4iv(location: WebGLUniformLocaton, value: Int32Array): void {
        this.context.uniform4iv(location.loc, value.buffer);
    }

    public uniform4fv(location: WebGLUniformLocaton, value: Float32Array | Array<number>): void {
        if (value instanceof Array) {
            let buf = new Float32Array(value);
            this.context.uniform4fv(location.loc, buf.buffer);
        } else {
            let buf = this.getArrayBuffer(value);
            this.context.uniform4fv(location.loc, buf);
        }
    }

    public uniformMatrix2fv(location: WebGLUniformLocaton, transpose: boolean, value: Float32Array): void {
        let buf = this.getArrayBuffer(value);

        if (transpose) {
            this.context.uniform_matrix2fv(location.loc, this.TRUE, buf);
        } else {
            this.context.uniform_matrix2fv(location.loc, this.FALSE, buf);
        }
    }

    public uniformMatrix3fv(location: WebGLUniformLocaton, transpose: boolean, value: Float32Array): void {
        let buf = this.getArrayBuffer(value);
        if (transpose) {
            this.context.uniform_matrix3fv(location.loc, this.TRUE, buf);
        } else {
            this.context.uniform_matrix3fv(location.loc, this.FALSE, buf);
        }
    }

    public uniformMatrix4fv(location: WebGLUniformLocaton, transpose: boolean, value: Float32Array): void {
        let buf = this.getArrayBuffer(value);
        this.context.uniform_matrix4fv(location.loc, transpose, buf);
    }

    public validateProgram(program: WebGLProgram): void {
        this.context.validate_program(program.programPtr);
    }

    public vertexAttrib1f(index: number, v0: number): void {
        this.context.vertex_attrib1f(index, v0);
    }

    public vertexAttrib2f(index: number, v0: number, v1: number): void {
        this.context.vertex_attrib2f(index, v0, v1);
    }

    public vertexAttrib3f(index: number, v0: number, v1: number, v2: number): void {
        this.context.vertex_attrib3f(index, v0, v1, v2);
    }

    public vertexAttrib4f(index: number, v0: number, v1: number, v2: number, v3: number): void {
        this.context.vertex_attrib4f(index, v0, v1, v2, v3);
    }

    public vertexAttrib1fv(index: number, value: Float32Array): void {
        let buf = this.getArrayBuffer(value);
        this.context.vertex_attrib1fv(index, buf);
    }

    public vertexAttrib2fv(index: number, value: Float32Array): void {
        let buf = this.getArrayBuffer(value);
        this.context.vertex_attrib2fv(index, buf);
    }

    public vertexAttrib3fv(index: number, value: Float32Array): void {
        let buf = this.getArrayBuffer(value);
        this.context.vertex_attrib3fv(index, buf);
    }

    public vertexAttrib4fv(index: number, value: Float32Array): void {
        let buf = this.getArrayBuffer(value);
        this.context.vertex_attrib4fv(index, buf);
    }

    public getSupportedExtensions(): string[] {
        let extensions = this.context.get_supported_extensions();

        let v = [];

        let compressedTextureType = ['GL_ANGLE_texture_compression_dxt1', 'GL_ANGLE_texture_compression_dxt3', 'GL_ANGLE_texture_compression_dxt5', 'GL_KHR_texture_compression_astc_ldr'];
        for (let i = 0; i < compressedTextureType.length; i++) {
            if (extensions.indexOf(compressedTextureType[i]) !== -1) {
                v.push("WEBGL_compressed_texture_s3tc");
                break;
            }
        }

        if (extensions.indexOf("GL_ANGLE_instanced_arrays") !== -1) {
            v.push("OES_vertex_array_object");
        }

        if (extensions.indexOf("GL_ANGLE_instanced_arrays") !== -1) {
            v.push("ANGLE_instanced_arrays");
        }

        let e_array = extensions.split(' ');

        for (let i = 0; i < e_array.length; i++) {
            if (e_array[i].indexOf("compress") !== -1) {
                if (e_array[i].indexOf("astc") !== -1) {
                    v.push("WEBGL_compressed_texture_astc");
                    break;
                }
            }
        }

        let e = this.context.get_supported_extensions();

        return v;
    }

    public getExtension(extensionName: string): WEBGLCompressedTextureS3TC | WebGLVertexArrayObjectOES | ANGLEInstancedArrays | WebGLCompressedTextureASTC | OESStandardDerivatives | null {
        if (extensionName === "WEBGL_compressed_texture_s3tc") {
            if (!this.texture_type) {
                let obj = this.context.get_compressed_texture_extension(extensionName);
                this.texture_type = new WEBGLCompressedTextureS3TC(obj);
            }
            return this.texture_type;
        } else if (extensionName === "OES_vertex_array_object") {
            let obj = this.context.get_vertex_array_object_extension(extensionName);
            if (!obj) {
                return null;
            }
            return new WebGLVertexArrayObjectOES(obj);
        } else if (extensionName === "ANGLE_instanced_arrays") {
            let obj = this.context.get_instanced_arrays_extension(extensionName);

            if (!obj) {
                return null;
            }
            return new ANGLEInstancedArrays(obj);
        } else if (extensionName === "WEBGL_compressed_texture_astc") {
            let extension = this.context.get_supported_extensions().split(' ');
            for (let i = 0; i < extension.length; i++) {
                // console.log("extension[i] = " + extension[i]);
                if (extension[i].indexOf("compressed") !== -1 && extension[i].indexOf("astc") !== -1) {
                    return new WebGLCompressedTextureASTC();
                }
            }
            return null;

        } else if (extensionName === "OES_standard_derivatives") {
            let extension = this.context.get_supported_extensions().split(' ');
            for (let i = 0; i < extension.length; i++) {
                // console.log("extension[i] = " + extension[i]);
                if (extension[i].indexOf("GL_ARB_derivative_control") !== -1 || extension[i].indexOf("GL_OES_standard_derivatives") !== -1) {
                    return new OESStandardDerivatives();
                }
            }
            return null;
        } else if (extensionName === "EXT_shader_texture_lod") {
            let extension = this.context.get_supported_extensions().split(' ');
            for (let i = 0; i < extension.length; i++) {
                // console.log("extension[i] = " + extension[i]);
                if (extension[i].indexOf("GL_EXT_texture_lod") !== -1) {
                    return new EXTShaderTextureLOD();
                }
            }
            return null;
        } else {
            return null;
        }
    }

    public getParameter(pname: number): number | string | null {
        if (pname === this.VENDOR ||
            pname === this.RENDERER ||
            pname === this.VERSION ||
            pname === this.SHADING_LANGUAGE_VERSION) {
            return this.context.get_string(pname);
        } else if (pname === this.MAX_COMBINED_TEXTURE_IMAGE_UNITS ||
            pname === this.MAX_CUBE_MAP_TEXTURE_SIZE ||
            pname === this.MAX_FRAGMENT_UNIFORM_VECTORS ||
            pname === this.MAX_RENDERBUFFER_SIZE ||
            pname === this.ARRAY_BUFFER_BINDING ||
            pname === this.MAX_TEXTURE_IMAGE_UNITS ||
            pname === this.MAX_TEXTURE_SIZE ||
            pname === this.MAX_VARYING_VECTORS ||
            pname === this.MAX_VERTEX_ATTRIBS ||
            pname === this.MAX_VERTEX_TEXTURE_IMAGE_UNITS ||
            pname === this.MAX_VERTEX_UNIFORM_VECTORS ||
            pname === this.MAX_VIEWPORT_DIMS) {
            return this.context.get_integerv(pname);
        } else {
            return null;
        }



        // if (!this.texture_type) {
        //     let v: number[];
        //     v.push(this.texture_type.COMPRESSED_RGBA_S3TC_DXT1_EXT);
        //     v.push(this.texture_type.COMPRESSED_RGBA_S3TC_DXT3_EXT);
        //     v.push(this.texture_type.COMPRESSED_RGBA_S3TC_DXT5_EXT);
        //     return v;
        // } else { return null; }
    }

    private getArrayBuffer(buffer: Uint8Array | Uint16Array | Float32Array | Int16Array | Int32Array | Int8Array | Uint32Array | Float64Array) {
        if (ArrayBuffer.isView(buffer)) {
            let view = buffer;
            if (view.byteLength === view.buffer.byteLength) {
                return view.buffer;
            } else {
                view = view.slice();
                return view.buffer;
            }
        }
        return buffer;
    }

    public bindSurface(): void {
        this.context.bind_surface();
    }

    public unbindSurface(version: string): void {
        this.context.unbind_surface(version);
    }

    public destoryContext(): void {
        this.context.destory_context();
    }

    static DEPTH_BUFFER_BIT = 0x00000100;
    static STENCIL_BUFFER_BIT = 0x00000400;
    static COLOR_BUFFER_BIT = 0x00004000;
    static FALSE = 0;
    static TRUE = 1;
    static POINTS = 0x0000;
    static LINES = 0x0001;
    static LINE_LOOP = 0x0002;
    static LINE_STRIP = 0x0003;
    static TRIANGLES = 0x0004;
    static TRIANGLE_STRIP = 0x0005;
    static TRIANGLE_FAN = 0x0006;
    static ZERO = 0;
    static ONE = 1;
    static SRC_COLOR = 0x0300;
    static ONE_MINUS_SRC_COLOR = 0x0301;
    static SRC_ALPHA = 0x0302;
    static ONE_MINUS_SRC_ALPHA = 0x0303;
    static DST_ALPHA = 0x0304;
    static ONE_MINUS_DST_ALPHA = 0x0305;
    static DST_COLOR = 0x0306;
    static ONE_MINUS_DST_COLOR = 0x0307;
    static SRC_ALPHA_SATURATE = 0x0308;
    static FUNC_ADD = 0x8006;
    static BLEND_EQUATION = 0x8009;
    static BLEND_EQUATION_RGB = 0x8009;
    static BLEND_EQUATION_ALPHA = 0x883D;
    static FUNC_SUBTRACT = 0x800A;
    static FUNC_REVERSE_SUBTRACT = 0x800B;
    static BLEND_DST_RGB = 0x80C8;
    static BLEND_SRC_RGB = 0x80C9;
    static BLEND_DST_ALPHA = 0x80CA;
    static BLEND_SRC_ALPHA = 0x80CB;
    static CONSTANT_COLOR = 0x8001;
    static ONE_MINUS_CONSTANT_COLOR = 0x8002;
    static CONSTANT_ALPHA = 0x8003;
    static ONE_MINUS_CONSTANT_ALPHA = 0x8004;
    static BLEND_COLOR = 0x8005;
    static ARRAY_BUFFER = 0x8892;
    static ELEMENT_ARRAY_BUFFER = 0x8893;
    static ARRAY_BUFFER_BINDING = 0x8894;
    static ELEMENT_ARRAY_BUFFER_BINDING = 0x8895;
    static STREAM_DRAW = 0x88E0;
    static STATIC_DRAW = 0x88E4;
    static DYNAMIC_DRAW = 0x88E8;
    static BUFFER_SIZE = 0x8764;
    static BUFFER_USAGE = 0x8765;
    static CURRENT_VERTEX_ATTRIB = 0x8626;
    static FRONT = 0x0404;
    static BACK = 0x0405;
    static FRONT_AND_BACK = 0x0408;
    static TEXTURE_2D = 0x0DE1;
    static CULL_FACE = 0x0B44;
    static BLEND = 0x0BE2;
    static DITHER = 0x0BD0;
    static STENCIL_TEST = 0x0B90;
    static DEPTH_TEST = 0x0B71;
    static SCISSOR_TEST = 0x0C11;
    static POLYGON_OFFSET_FILL = 0x8037;
    static SAMPLE_ALPHA_TO_COVERAGE = 0x809E;
    static SAMPLE_COVERAGE = 0x80A0;
    static NO_ERROR = 0;
    static INVALID_ENUM = 0x0500;
    static INVALID_VALUE = 0x0501;
    static INVALID_OPERATION = 0x0502;
    static OUT_OF_MEMORY = 0x0505;
    static CW = 0x0900;
    static CCW = 0x0901;
    static LINE_WIDTH = 0x0B21;
    static ALIASED_POINT_SIZE_RANGE = 0x846D;
    static ALIASED_LINE_WIDTH_RANGE = 0x846E;
    static CULL_FACE_MODE = 0x0B45;
    static FRONT_FACE = 0x0B46;
    static DEPTH_RANGE = 0x0B70;
    static DEPTH_WRITEMASK = 0x0B72;
    static DEPTH_CLEAR_VALUE = 0x0B73;
    static DEPTH_FUNC = 0x0B74;
    static STENCIL_CLEAR_VALUE = 0x0B91;
    static STENCIL_FUNC = 0x0B92;
    static STENCIL_FAIL = 0x0B94;
    static STENCIL_PASS_DEPTH_FAIL = 0x0B95;
    static STENCIL_PASS_DEPTH_PASS = 0x0B96;
    static STENCIL_REF = 0x0B97;
    static STENCIL_VALUE_MASK = 0x0B93;
    static STENCIL_WRITEMASK = 0x0B98;
    static STENCIL_BACK_FUNC = 0x8800;
    static STENCIL_BACK_FAIL = 0x8801;
    static STENCIL_BACK_PASS_DEPTH_FAIL = 0x8802;
    static STENCIL_BACK_PASS_DEPTH_PASS = 0x8803;
    static STENCIL_BACK_REF = 0x8CA3;
    static STENCIL_BACK_VALUE_MASK = 0x8CA4;
    static STENCIL_BACK_WRITEMASK = 0x8CA5;
    static VIEWPORT = 0x0BA2;
    static SCISSOR_BOX = 0x0C10;
    static COLOR_CLEAR_VALUE = 0x0C22;
    static COLOR_WRITEMASK = 0x0C23;
    static UNPACK_ALIGNMENT = 0x0CF5;
    static PACK_ALIGNMENT = 0x0D05;
    static MAX_TEXTURE_SIZE = 0x0D33;
    static MAX_VIEWPORT_DIMS = 0x0D3A;
    static SUBPIXEL_BITS = 0x0D50;
    static RED_BITS = 0x0D52;
    static GREEN_BITS = 0x0D53;
    static BLUE_BITS = 0x0D54;
    static ALPHA_BITS = 0x0D55;
    static DEPTH_BITS = 0x0D56;
    static STENCIL_BITS = 0x0D57;
    static POLYGON_OFFSET_UNITS = 0x2A00;
    static POLYGON_OFFSET_FACTOR = 0x8038;
    static TEXTURE_BINDING_2D = 0x8069;
    static SAMPLE_BUFFERS = 0x80A8;
    static SAMPLES = 0x80A9;
    static SAMPLE_COVERAGE_VALUE = 0x80AA;
    static SAMPLE_COVERAGE_INVERT = 0x80AB;
    static NUM_COMPRESSED_TEXTURE_FORMATS = 0x86A2;
    static COMPRESSED_TEXTURE_FORMATS = 0x86A3;
    static DONT_CARE = 0x1100;
    static FASTEST = 0x1101;
    static NICEST = 0x1102;
    static GENERATE_MIPMAP_HINT = 0x8192;
    static BYTE = 0x1400;
    static UNSIGNED_BYTE = 0x1401;
    static SHORT = 0x1402;
    static UNSIGNED_SHORT = 0x1403;
    static INT = 0x1404;
    static UNSIGNED_INT = 0x1405;
    static FLOAT = 0x1406;
    static FIXED = 0x140C;
    static DEPTH_COMPONENT = 0x1902;
    static ALPHA = 0x1906;
    static RGB = 0x1907;
    static RGBA = 0x1908;
    static LUMINANCE = 0x1909;
    static LUMINANCE_ALPHA = 0x190A;
    static UNSIGNED_SHORT_4_4_4_4 = 0x8033;
    static UNSIGNED_SHORT_5_5_5_1 = 0x8034;
    static UNSIGNED_SHORT_5_6_5 = 0x8363;
    static FRAGMENT_SHADER = 0x8B30;
    static VERTEX_SHADER = 0x8B31;
    static MAX_VERTEX_ATTRIBS = 0x8869;
    static MAX_VERTEX_UNIFORM_VECTORS = 0x8DFB;
    static MAX_VARYING_VECTORS = 0x8DFC;
    static MAX_COMBINED_TEXTURE_IMAGE_UNITS = 0x8B4D;
    static MAX_VERTEX_TEXTURE_IMAGE_UNITS = 0x8B4C;
    static MAX_TEXTURE_IMAGE_UNITS = 0x8872;
    static MAX_FRAGMENT_UNIFORM_VECTORS = 0x8DFD;
    static SHADER_TYPE = 0x8B4F;
    static DELETE_STATUS = 0x8B80;
    static LINK_STATUS = 0x8B82;
    static VALIDATE_STATUS = 0x8B83;
    static ATTACHED_SHADERS = 0x8B85;
    static ACTIVE_UNIFORMS = 0x8B86;
    static ACTIVE_UNIFORM_MAX_LENGTH = 0x8B87;
    static ACTIVE_ATTRIBUTES = 0x8B89;
    static ACTIVE_ATTRIBUTE_MAX_LENGTH = 0x8B8A;
    static SHADING_LANGUAGE_VERSION = 0x8B8C;
    static CURRENT_PROGRAM = 0x8B8D;
    static NEVER = 0x0200;
    static LESS = 0x0201;
    static EQUAL = 0x0202;
    static LEQUAL = 0x0203;
    static GREATER = 0x0204;
    static NOTEQUAL = 0x0205;
    static GEQUAL = 0x0206;
    static ALWAYS = 0x0207;
    static KEEP = 0x1E00;
    static REPLACE = 0x1E01;
    static INCR = 0x1E02;
    static DECR = 0x1E03;
    static INVERT = 0x150A;
    static INCR_WRAP = 0x8507;
    static DECR_WRAP = 0x8508;
    static VENDOR = 0x1F00;
    static RENDERER = 0x1F01;
    static VERSION = 0x1F02;
    static EXTENSIONS = 0x1F03;
    static NEAREST = 0x2600;
    static LINEAR = 0x2601;
    static NEAREST_MIPMAP_NEAREST = 0x2700;
    static LINEAR_MIPMAP_NEAREST = 0x2701;
    static NEAREST_MIPMAP_LINEAR = 0x2702;
    static LINEAR_MIPMAP_LINEAR = 0x2703;
    static TEXTURE_MAG_FILTER = 0x2800;
    static TEXTURE_MIN_FILTER = 0x2801;
    static TEXTURE_WRAP_S = 0x2802;
    static TEXTURE_WRAP_T = 0x2803;
    static TEXTURE = 0x1702;
    static TEXTURE_CUBE_MAP = 0x8513;
    static TEXTURE_BINDING_CUBE_MAP = 0x8514;
    static TEXTURE_CUBE_MAP_POSITIVE_X = 0x8515;
    static TEXTURE_CUBE_MAP_NEGATIVE_X = 0x8516;
    static TEXTURE_CUBE_MAP_POSITIVE_Y = 0x8517;
    static TEXTURE_CUBE_MAP_NEGATIVE_Y = 0x8518;
    static TEXTURE_CUBE_MAP_POSITIVE_Z = 0x8519;
    static TEXTURE_CUBE_MAP_NEGATIVE_Z = 0x851A;
    static MAX_CUBE_MAP_TEXTURE_SIZE = 0x851C;
    static TEXTURE0 = 0x84C0;
    static TEXTURE1 = 0x84C1;
    static TEXTURE2 = 0x84C2;
    static TEXTURE3 = 0x84C3;
    static TEXTURE4 = 0x84C4;
    static TEXTURE5 = 0x84C5;
    static TEXTURE6 = 0x84C6;
    static TEXTURE7 = 0x84C7;
    static TEXTURE8 = 0x84C8;
    static TEXTURE9 = 0x84C9;
    static TEXTURE10 = 0x84CA;
    static TEXTURE11 = 0x84CB;
    static TEXTURE12 = 0x84CC;
    static TEXTURE13 = 0x84CD;
    static TEXTURE14 = 0x84CE;
    static TEXTURE15 = 0x84CF;
    static TEXTURE16 = 0x84D0;
    static TEXTURE17 = 0x84D1;
    static TEXTURE18 = 0x84D2;
    static TEXTURE19 = 0x84D3;
    static TEXTURE20 = 0x84D4;
    static TEXTURE21 = 0x84D5;
    static TEXTURE22 = 0x84D6;
    static TEXTURE23 = 0x84D7;
    static TEXTURE24 = 0x84D8;
    static TEXTURE25 = 0x84D9;
    static TEXTURE26 = 0x84DA;
    static TEXTURE27 = 0x84DB;
    static TEXTURE28 = 0x84DC;
    static TEXTURE29 = 0x84DD;
    static TEXTURE30 = 0x84DE;
    static TEXTURE31 = 0x84DF;
    static ACTIVE_TEXTURE = 0x84E0;
    static REPEAT = 0x2901;
    static CLAMP_TO_EDGE = 0x812F;
    static MIRRORED_REPEAT = 0x8370;
    static FLOAT_VEC2 = 0x8B50;
    static FLOAT_VEC3 = 0x8B51;
    static FLOAT_VEC4 = 0x8B52;
    static INT_VEC2 = 0x8B53;
    static INT_VEC3 = 0x8B54;
    static INT_VEC4 = 0x8B55;
    static BOOL = 0x8B56;
    static BOOL_VEC2 = 0x8B57;
    static BOOL_VEC3 = 0x8B58;
    static BOOL_VEC4 = 0x8B59;
    static FLOAT_MAT2 = 0x8B5A;
    static FLOAT_MAT3 = 0x8B5B;
    static FLOAT_MAT4 = 0x8B5C;
    static SAMPLER_2D = 0x8B5E;
    static SAMPLER_CUBE = 0x8B60;
    static VERTEX_ATTRIB_ARRAY_ENABLED = 0x8622;
    static VERTEX_ATTRIB_ARRAY_SIZE = 0x8623;
    static VERTEX_ATTRIB_ARRAY_STRIDE = 0x8624;
    static VERTEX_ATTRIB_ARRAY_TYPE = 0x8625;
    static VERTEX_ATTRIB_ARRAY_NORMALIZED = 0x886A;
    static VERTEX_ATTRIB_ARRAY_POINTER = 0x8645;
    static VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = 0x889F;
    static IMPLEMENTATION_COLOR_READ_TYPE = 0x8B9A;
    static IMPLEMENTATION_COLOR_READ_FORMAT = 0x8B9B;
    static COMPILE_STATUS = 0x8B81;
    static INFO_LOG_LENGTH = 0x8B84;
    static SHADER_SOURCE_LENGTH = 0x8B88;
    static SHADER_COMPILER = 0x8DFA;
    static SHADER_BINARY_FORMATS = 0x8DF8;
    static NUM_SHADER_BINARY_FORMATS = 0x8DF9;
    static LOW_FLOAT = 0x8DF0;
    static MEDIUM_FLOAT = 0x8DF1;
    static HIGH_FLOAT = 0x8DF2;
    static LOW_INT = 0x8DF3;
    static MEDIUM_INT = 0x8DF4;
    static HIGH_INT = 0x8DF5;
    static FRAMEBUFFER = 0x8D40;
    static RENDERBUFFER = 0x8D41;
    static RGBA4 = 0x8056;
    static RGB5_A1 = 0x8057;
    static RGB565 = 0x8D62;
    static DEPTH_COMPONENT16 = 0x81A5;
    static STENCIL_INDEX8 = 0x8D48;
    static RENDERBUFFER_WIDTH = 0x8D42;
    static RENDERBUFFER_HEIGHT = 0x8D43;
    static RENDERBUFFER_INTERNAL_FORMAT = 0x8D44;
    static RENDERBUFFER_RED_SIZE = 0x8D50;
    static RENDERBUFFER_GREEN_SIZE = 0x8D51;
    static RENDERBUFFER_BLUE_SIZE = 0x8D52;
    static RENDERBUFFER_ALPHA_SIZE = 0x8D53;
    static RENDERBUFFER_DEPTH_SIZE = 0x8D54;
    static RENDERBUFFER_STENCIL_SIZE = 0x8D55;
    static FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = 0x8CD0;
    static FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = 0x8CD1;
    static FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = 0x8CD2;
    static FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = 0x8CD3;
    static COLOR_ATTACHMENT0 = 0x8CE0;
    static DEPTH_ATTACHMENT = 0x8D00;
    static STENCIL_ATTACHMENT = 0x8D20;
    static NONE = 0;
    static FRAMEBUFFER_COMPLETE = 0x8CD5;
    static FRAMEBUFFER_INCOMPLETE_ATTACHMENT = 0x8CD6;
    static FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = 0x8CD7;
    static FRAMEBUFFER_INCOMPLETE_DIMENSIONS = 0x8CD9;
    static FRAMEBUFFER_UNSUPPORTED = 0x8CDD;
    static FRAMEBUFFER_BINDING = 0x8CA6;
    static RENDERBUFFER_BINDING = 0x8CA7;
    static MAX_RENDERBUFFER_SIZE = 0x84E8;
    static INVALID_FRAMEBUFFER_OPERATION = 0x0506;
    static MAX_SAMPLES = 0x8D57;
    static UNPACK_FLIP_Y_WEBGL = 0x9240;
    static UNPACK_PREMULTIPLY_ALPHA_WEBGL = 0x9241;
    static UNPACK_COLORSPACE_CONVERSION_WEBGL = 0x9243;

    public DEPTH_BUFFER_BIT = 0x00000100;
    public STENCIL_BUFFER_BIT = 0x00000400;
    public COLOR_BUFFER_BIT = 0x00004000;
    public FALSE = 0;
    public TRUE = 1;
    public POINTS = 0x0000;
    public LINES = 0x0001;
    public LINE_LOOP = 0x0002;
    public LINE_STRIP = 0x0003;
    public TRIANGLES = 0x0004;
    public TRIANGLE_STRIP = 0x0005;
    public TRIANGLE_FAN = 0x0006;
    public ZERO = 0;
    public ONE = 1;
    public SRC_COLOR = 0x0300;
    public ONE_MINUS_SRC_COLOR = 0x0301;
    public SRC_ALPHA = 0x0302;
    public ONE_MINUS_SRC_ALPHA = 0x0303;
    public DST_ALPHA = 0x0304;
    public ONE_MINUS_DST_ALPHA = 0x0305;
    public DST_COLOR = 0x0306;
    public ONE_MINUS_DST_COLOR = 0x0307;
    public SRC_ALPHA_SATURATE = 0x0308;
    public FUNC_ADD = 0x8006;
    public BLEND_EQUATION = 0x8009;
    public BLEND_EQUATION_RGB = 0x8009;
    public BLEND_EQUATION_ALPHA = 0x883D;
    public FUNC_SUBTRACT = 0x800A;
    public FUNC_REVERSE_SUBTRACT = 0x800B;
    public BLEND_DST_RGB = 0x80C8;
    public BLEND_SRC_RGB = 0x80C9;
    public BLEND_DST_ALPHA = 0x80CA;
    public BLEND_SRC_ALPHA = 0x80CB;
    public CONSTANT_COLOR = 0x8001;
    public ONE_MINUS_CONSTANT_COLOR = 0x8002;
    public CONSTANT_ALPHA = 0x8003;
    public ONE_MINUS_CONSTANT_ALPHA = 0x8004;
    public BLEND_COLOR = 0x8005;
    public ARRAY_BUFFER = 0x8892;
    public ELEMENT_ARRAY_BUFFER = 0x8893;
    public ARRAY_BUFFER_BINDING = 0x8894;
    public ELEMENT_ARRAY_BUFFER_BINDING = 0x8895;
    public STREAM_DRAW = 0x88E0;
    public STATIC_DRAW = 0x88E4;
    public DYNAMIC_DRAW = 0x88E8;
    public BUFFER_SIZE = 0x8764;
    public BUFFER_USAGE = 0x8765;
    public CURRENT_VERTEX_ATTRIB = 0x8626;
    public FRONT = 0x0404;
    public BACK = 0x0405;
    public FRONT_AND_BACK = 0x0408;
    public TEXTURE_2D = 0x0DE1;
    public CULL_FACE = 0x0B44;
    public BLEND = 0x0BE2;
    public DITHER = 0x0BD0;
    public STENCIL_TEST = 0x0B90;
    public DEPTH_TEST = 0x0B71;
    public SCISSOR_TEST = 0x0C11;
    public POLYGON_OFFSET_FILL = 0x8037;
    public SAMPLE_ALPHA_TO_COVERAGE = 0x809E;
    public SAMPLE_COVERAGE = 0x80A0;
    public NO_ERROR = 0;
    public INVALID_ENUM = 0x0500;
    public INVALID_VALUE = 0x0501;
    public INVALID_OPERATION = 0x0502;
    public OUT_OF_MEMORY = 0x0505;
    public CW = 0x0900;
    public CCW = 0x0901;
    public LINE_WIDTH = 0x0B21;
    public ALIASED_POINT_SIZE_RANGE = 0x846D;
    public ALIASED_LINE_WIDTH_RANGE = 0x846E;
    public CULL_FACE_MODE = 0x0B45;
    public FRONT_FACE = 0x0B46;
    public DEPTH_RANGE = 0x0B70;
    public DEPTH_WRITEMASK = 0x0B72;
    public DEPTH_CLEAR_VALUE = 0x0B73;
    public DEPTH_FUNC = 0x0B74;
    public STENCIL_CLEAR_VALUE = 0x0B91;
    public STENCIL_FUNC = 0x0B92;
    public STENCIL_FAIL = 0x0B94;
    public STENCIL_PASS_DEPTH_FAIL = 0x0B95;
    public STENCIL_PASS_DEPTH_PASS = 0x0B96;
    public STENCIL_REF = 0x0B97;
    public STENCIL_VALUE_MASK = 0x0B93;
    public STENCIL_WRITEMASK = 0x0B98;
    public STENCIL_BACK_FUNC = 0x8800;
    public STENCIL_BACK_FAIL = 0x8801;
    public STENCIL_BACK_PASS_DEPTH_FAIL = 0x8802;
    public STENCIL_BACK_PASS_DEPTH_PASS = 0x8803;
    public STENCIL_BACK_REF = 0x8CA3;
    public STENCIL_BACK_VALUE_MASK = 0x8CA4;
    public STENCIL_BACK_WRITEMASK = 0x8CA5;
    public VIEWPORT = 0x0BA2;
    public SCISSOR_BOX = 0x0C10;
    public COLOR_CLEAR_VALUE = 0x0C22;
    public COLOR_WRITEMASK = 0x0C23;
    public UNPACK_ALIGNMENT = 0x0CF5;
    public PACK_ALIGNMENT = 0x0D05;
    public MAX_TEXTURE_SIZE = 0x0D33;
    public MAX_VIEWPORT_DIMS = 0x0D3A;
    public SUBPIXEL_BITS = 0x0D50;
    public RED_BITS = 0x0D52;
    public GREEN_BITS = 0x0D53;
    public BLUE_BITS = 0x0D54;
    public ALPHA_BITS = 0x0D55;
    public DEPTH_BITS = 0x0D56;
    public STENCIL_BITS = 0x0D57;
    public POLYGON_OFFSET_UNITS = 0x2A00;
    public POLYGON_OFFSET_FACTOR = 0x8038;
    public TEXTURE_BINDING_2D = 0x8069;
    public SAMPLE_BUFFERS = 0x80A8;
    public SAMPLES = 0x80A9;
    public SAMPLE_COVERAGE_VALUE = 0x80AA;
    public SAMPLE_COVERAGE_INVERT = 0x80AB;
    public NUM_COMPRESSED_TEXTURE_FORMATS = 0x86A2;
    public COMPRESSED_TEXTURE_FORMATS = 0x86A3;
    public DONT_CARE = 0x1100;
    public FASTEST = 0x1101;
    public NICEST = 0x1102;
    public GENERATE_MIPMAP_HINT = 0x8192;
    public BYTE = 0x1400;
    public UNSIGNED_BYTE = 0x1401;
    public SHORT = 0x1402;
    public UNSIGNED_SHORT = 0x1403;
    public INT = 0x1404;
    public UNSIGNED_INT = 0x1405;
    public FLOAT = 0x1406;
    public FIXED = 0x140C;
    public DEPTH_COMPONENT = 0x1902;
    public ALPHA = 0x1906;
    public RGB = 0x1907;
    public RGBA = 0x1908;
    public LUMINANCE = 0x1909;
    public LUMINANCE_ALPHA = 0x190A;
    public UNSIGNED_SHORT_4_4_4_4 = 0x8033;
    public UNSIGNED_SHORT_5_5_5_1 = 0x8034;
    public UNSIGNED_SHORT_5_6_5 = 0x8363;
    public FRAGMENT_SHADER = 0x8B30;
    public VERTEX_SHADER = 0x8B31;
    public MAX_VERTEX_ATTRIBS = 0x8869;
    public MAX_VERTEX_UNIFORM_VECTORS = 0x8DFB;
    public MAX_VARYING_VECTORS = 0x8DFC;
    public MAX_COMBINED_TEXTURE_IMAGE_UNITS = 0x8B4D;
    public MAX_VERTEX_TEXTURE_IMAGE_UNITS = 0x8B4C;
    public MAX_TEXTURE_IMAGE_UNITS = 0x8872;
    public MAX_FRAGMENT_UNIFORM_VECTORS = 0x8DFD;
    public SHADER_TYPE = 0x8B4F;
    public DELETE_STATUS = 0x8B80;
    public LINK_STATUS = 0x8B82;
    public VALIDATE_STATUS = 0x8B83;
    public ATTACHED_SHADERS = 0x8B85;
    public ACTIVE_UNIFORMS = 0x8B86;
    public ACTIVE_UNIFORM_MAX_LENGTH = 0x8B87;
    public ACTIVE_ATTRIBUTES = 0x8B89;
    public ACTIVE_ATTRIBUTE_MAX_LENGTH = 0x8B8A;
    public SHADING_LANGUAGE_VERSION = 0x8B8C;
    public CURRENT_PROGRAM = 0x8B8D;
    public NEVER = 0x0200;
    public LESS = 0x0201;
    public EQUAL = 0x0202;
    public LEQUAL = 0x0203;
    public GREATER = 0x0204;
    public NOTEQUAL = 0x0205;
    public GEQUAL = 0x0206;
    public ALWAYS = 0x0207;
    public KEEP = 0x1E00;
    public REPLACE = 0x1E01;
    public INCR = 0x1E02;
    public DECR = 0x1E03;
    public INVERT = 0x150A;
    public INCR_WRAP = 0x8507;
    public DECR_WRAP = 0x8508;
    public VENDOR = 0x1F00;
    public RENDERER = 0x1F01;
    public VERSION = 0x1F02;
    public EXTENSIONS = 0x1F03;
    public NEAREST = 0x2600;
    public LINEAR = 0x2601;
    public NEAREST_MIPMAP_NEAREST = 0x2700;
    public LINEAR_MIPMAP_NEAREST = 0x2701;
    public NEAREST_MIPMAP_LINEAR = 0x2702;
    public LINEAR_MIPMAP_LINEAR = 0x2703;
    public TEXTURE_MAG_FILTER = 0x2800;
    public TEXTURE_MIN_FILTER = 0x2801;
    public TEXTURE_WRAP_S = 0x2802;
    public TEXTURE_WRAP_T = 0x2803;
    public TEXTURE = 0x1702;
    public TEXTURE_CUBE_MAP = 0x8513;
    public TEXTURE_BINDING_CUBE_MAP = 0x8514;
    public TEXTURE_CUBE_MAP_POSITIVE_X = 0x8515;
    public TEXTURE_CUBE_MAP_NEGATIVE_X = 0x8516;
    public TEXTURE_CUBE_MAP_POSITIVE_Y = 0x8517;
    public TEXTURE_CUBE_MAP_NEGATIVE_Y = 0x8518;
    public TEXTURE_CUBE_MAP_POSITIVE_Z = 0x8519;
    public TEXTURE_CUBE_MAP_NEGATIVE_Z = 0x851A;
    public MAX_CUBE_MAP_TEXTURE_SIZE = 0x851C;
    public TEXTURE0 = 0x84C0;
    public TEXTURE1 = 0x84C1;
    public TEXTURE2 = 0x84C2;
    public TEXTURE3 = 0x84C3;
    public TEXTURE4 = 0x84C4;
    public TEXTURE5 = 0x84C5;
    public TEXTURE6 = 0x84C6;
    public TEXTURE7 = 0x84C7;
    public TEXTURE8 = 0x84C8;
    public TEXTURE9 = 0x84C9;
    public TEXTURE10 = 0x84CA;
    public TEXTURE11 = 0x84CB;
    public TEXTURE12 = 0x84CC;
    public TEXTURE13 = 0x84CD;
    public TEXTURE14 = 0x84CE;
    public TEXTURE15 = 0x84CF;
    public TEXTURE16 = 0x84D0;
    public TEXTURE17 = 0x84D1;
    public TEXTURE18 = 0x84D2;
    public TEXTURE19 = 0x84D3;
    public TEXTURE20 = 0x84D4;
    public TEXTURE21 = 0x84D5;
    public TEXTURE22 = 0x84D6;
    public TEXTURE23 = 0x84D7;
    public TEXTURE24 = 0x84D8;
    public TEXTURE25 = 0x84D9;
    public TEXTURE26 = 0x84DA;
    public TEXTURE27 = 0x84DB;
    public TEXTURE28 = 0x84DC;
    public TEXTURE29 = 0x84DD;
    public TEXTURE30 = 0x84DE;
    public TEXTURE31 = 0x84DF;
    public ACTIVE_TEXTURE = 0x84E0;
    public REPEAT = 0x2901;
    public CLAMP_TO_EDGE = 0x812F;
    public MIRRORED_REPEAT = 0x8370;
    public FLOAT_VEC2 = 0x8B50;
    public FLOAT_VEC3 = 0x8B51;
    public FLOAT_VEC4 = 0x8B52;
    public INT_VEC2 = 0x8B53;
    public INT_VEC3 = 0x8B54;
    public INT_VEC4 = 0x8B55;
    public BOOL = 0x8B56;
    public BOOL_VEC2 = 0x8B57;
    public BOOL_VEC3 = 0x8B58;
    public BOOL_VEC4 = 0x8B59;
    public FLOAT_MAT2 = 0x8B5A;
    public FLOAT_MAT3 = 0x8B5B;
    public FLOAT_MAT4 = 0x8B5C;
    public SAMPLER_2D = 0x8B5E;
    public SAMPLER_CUBE = 0x8B60;
    public VERTEX_ATTRIB_ARRAY_ENABLED = 0x8622;
    public VERTEX_ATTRIB_ARRAY_SIZE = 0x8623;
    public VERTEX_ATTRIB_ARRAY_STRIDE = 0x8624;
    public VERTEX_ATTRIB_ARRAY_TYPE = 0x8625;
    public VERTEX_ATTRIB_ARRAY_NORMALIZED = 0x886A;
    public VERTEX_ATTRIB_ARRAY_POINTER = 0x8645;
    public VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = 0x889F;
    public IMPLEMENTATION_COLOR_READ_TYPE = 0x8B9A;
    public IMPLEMENTATION_COLOR_READ_FORMAT = 0x8B9B;
    public COMPILE_STATUS = 0x8B81;
    public INFO_LOG_LENGTH = 0x8B84;
    public SHADER_SOURCE_LENGTH = 0x8B88;
    public SHADER_COMPILER = 0x8DFA;
    public SHADER_BINARY_FORMATS = 0x8DF8;
    public NUM_SHADER_BINARY_FORMATS = 0x8DF9;
    public LOW_FLOAT = 0x8DF0;
    public MEDIUM_FLOAT = 0x8DF1;
    public HIGH_FLOAT = 0x8DF2;
    public LOW_INT = 0x8DF3;
    public MEDIUM_INT = 0x8DF4;
    public HIGH_INT = 0x8DF5;
    public FRAMEBUFFER = 0x8D40;
    public RENDERBUFFER = 0x8D41;
    public RGBA4 = 0x8056;
    public RGB5_A1 = 0x8057;
    public RGB565 = 0x8D62;
    public DEPTH_COMPONENT16 = 0x81A5;
    public STENCIL_INDEX8 = 0x8D48;
    public RENDERBUFFER_WIDTH = 0x8D42;
    public RENDERBUFFER_HEIGHT = 0x8D43;
    public RENDERBUFFER_INTERNAL_FORMAT = 0x8D44;
    public RENDERBUFFER_RED_SIZE = 0x8D50;
    public RENDERBUFFER_GREEN_SIZE = 0x8D51;
    public RENDERBUFFER_BLUE_SIZE = 0x8D52;
    public RENDERBUFFER_ALPHA_SIZE = 0x8D53;
    public RENDERBUFFER_DEPTH_SIZE = 0x8D54;
    public RENDERBUFFER_STENCIL_SIZE = 0x8D55;
    public FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = 0x8CD0;
    public FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = 0x8CD1;
    public FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = 0x8CD2;
    public FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = 0x8CD3;
    public COLOR_ATTACHMENT0 = 0x8CE0;
    public DEPTH_ATTACHMENT = 0x8D00;
    public STENCIL_ATTACHMENT = 0x8D20;
    public NONE = 0;
    public FRAMEBUFFER_COMPLETE = 0x8CD5;
    public FRAMEBUFFER_INCOMPLETE_ATTACHMENT = 0x8CD6;
    public FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = 0x8CD7;
    public FRAMEBUFFER_INCOMPLETE_DIMENSIONS = 0x8CD9;
    public FRAMEBUFFER_UNSUPPORTED = 0x8CDD;
    public FRAMEBUFFER_BINDING = 0x8CA6;
    public RENDERBUFFER_BINDING = 0x8CA7;
    public MAX_RENDERBUFFER_SIZE = 0x84E8;
    public INVALID_FRAMEBUFFER_OPERATION = 0x0506;
    public MAX_SAMPLES = 0x8D57;
    public UNPACK_FLIP_Y_WEBGL = 0x9240;
    public UNPACK_PREMULTIPLY_ALPHA_WEBGL = 0x9241
    public UNPACK_COLORSPACE_CONVERSION_WEBGL = 0x9243;

}

class WebGLShader {
    shaderPtr: number

    constructor(shaderPtr: number) {
        this.shaderPtr = shaderPtr;
    }
}

class WebGLBuffer {
    bufferPtr: number
    constructor(bufferPtr: number) {
        this.bufferPtr = bufferPtr;
    }
}

class WebGLProgram {
    programPtr: number

    constructor(programPtr: number) {
        this.programPtr = programPtr;
    }
}

class WebGLRenderbuffer {
    renderbufferPtr: number

    constructor(renderbufferPtr: number) {
        this.renderbufferPtr = renderbufferPtr;
    }
}

class WebGLFramebuffer {
    framebufferPtr: number

    constructor(framebufferPtr: number) {
        this.framebufferPtr = framebufferPtr;
    }
}

class WebGLTexture {
    texturePtr: number

    constructor(texturePtr: number) {
        this.texturePtr = texturePtr;
    }
}

class WebGLUniformLocaton {
    loc: number

    constructor(loc: number) {
        this.loc = loc;
    }
}

class WebGLActiveInfo {
    name: string;
    size: number;
    type: number;

    constructor(obj: object) {
        let object = WebGLActiveInfoBase.from(obj);

        this.name = object.get_name();
        this.size = object.get_size();
        this.type = object.get_type();
    }
}

class WebGLShaderPrecisionFormat {
    range_max: number;
    range_min: number;
    precision: number;

    constructor(obj: object) {
        let object = WebGLShaderPrecisionFormatBase.from(obj);
        this.range_max = object.get_range_max();
        this.range_min - object.get_rangr_min();
        this.precision = object.get_precision();
    }
}

class WebGLVertexArrayObjectOES {
    readonly VERTEX_ARRAY_BINDING_OES: number;
    base: OESVertexArrayObjectBase

    constructor(obj: object) {
        let base = OESVertexArrayObjectBase.from(obj);

        this.VERTEX_ARRAY_BINDING_OES = 34229;
        this.base = base;
    }

    public bindVertexArrayOES(arrayObject: number | null): void {
        if (arrayObject) {
            this.base.bind_vertex_array_oes(arrayObject);
        } else {
            this.base.bind_vertex_array_oes(0);
        }
    }

    public createVertexArrayOES(): number | null {
        let v = this.base.create_vertex_array_oes();
        if (v) {
            return v;
        } else {
            return null;
        }
    }
    public deleteVertexArrayOES(arrayObject: number | null): void {
        if (arrayObject) {
            this.base.delete_vertex_array_oes(arrayObject);
        }
    }
    public isVertexArrayOES(arrayObject: number | null): GLboolean {
        if (arrayObject) {
            return this.base.is_vertex_array_oes(arrayObject);
        }
    }

}

class WEBGLCompressedTextureS3TC {
    readonly COMPRESSED_RGBA_S3TC_DXT1_EXT: number;
    readonly COMPRESSED_RGBA_S3TC_DXT3_EXT: number;
    readonly COMPRESSED_RGBA_S3TC_DXT5_EXT: number;

    constructor(obj: object) {
        let v = WebGLCompressedTextureS3TCBase.from(obj);

        if (v.get_rgba_dxt1()) {
            this.COMPRESSED_RGBA_S3TC_DXT1_EXT = 33777;
        } else {
            this.COMPRESSED_RGBA_S3TC_DXT1_EXT = -1;
        }

        if (v.get_rgba_dxt3()) {
            this.COMPRESSED_RGBA_S3TC_DXT3_EXT = 33778;
        } else {
            this.COMPRESSED_RGBA_S3TC_DXT3_EXT = -1;
        }

        if (v.get_rgba_dxt5()) {
            this.COMPRESSED_RGBA_S3TC_DXT5_EXT = 33779;
        } else {
            this.COMPRESSED_RGBA_S3TC_DXT5_EXT = -1;
        }
    }
}

class WebGLCompressedTextureASTC {
    readonly COMPRESSED_RGBA_ASTC_8x8_KHR = 37815;
    readonly COMPRESSED_RGBA_ASTC_4x4_KHR = 37808;
}

class ANGLEInstancedArrays {
    readonly VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE: number;
    base: ANGLEInstancedArraysBase;
    constructor(obj: object) {
        let base = ANGLEInstancedArraysBase.from(obj);
        this.base = base;
        this.VERTEX_ATTRIB_ARRAY_DIVISOR_ANGLE = 35070;
    }
    public drawArraysInstancedANGLE(mode: number, first: number, count: number, primcount: number): void {
        this.base.draw_arrays_instanced_angle(mode, first, count, primcount);
    }
    public drawElementsInstancedANGLE(mode: number, count: number, type: number, offset: number, primcount: number): void {
        this.base.draw_elements_instanced_angle(mode, count, type, offset, primcount);
    }
    public vertexAttribDivisorANGLE(index: number, divisor: number): void {
        this.base.vertex_attrib_divisor_angle(index, divisor);
    }

}

class OESStandardDerivatives {
    constructor() { }
}


class EXTShaderTextureLOD {
    constructor() { }
}


class SvgRenderer {
    base: SvgRendererBase;

    constructor() {
        this.base = SvgRendererBase.new();
    }

    public setTarget(fbo: WebGLFramebuffer, width: number, height: number): void {
        this.base.set_target(fbo.framebufferPtr, width, height);
    }
    public setViewport(x: number, y: number, width?: number, height?: number): void {
        if (!width || !height) {
            this.base.set_viewport(x, y, -1, -1);
        } else {
            this.base.set_viewport(x, y, width, height);
        }
    }
    public setClearColor(r: number, g: number, b: number, a: number): void {
        this.base.set_clear_color(r, g, b, a);
    }

    public loadSvg(buffer: ArrayBuffer): string {
        let scene = this.base.load_svg(buffer);
        if (scene instanceof Error) {
            throw scene;
        } else {
            return scene;
        }
    }

    public drawOnce(scene: string): string | Error {
        return this.base.draw_once(scene);
    }

    public destroyScene(scene: string) {
        this.base.destroy_scene(scene);
    }

}

const g = globalThis as any;
g.WebGLRenderingContext = WebGLRenderingContext;
g.SvgRenderer = SvgRenderer;


