/**
 * requestAnimationFrame, cancelAnimationFrame
 */

const rafMod = globalThis._$pi_native.raf;
delete globalThis._$pi_native.raf;
// const swapBufferMod = globalThis._$pi_native.swapBuffer;
// delete globalThis._$pi_native.swapBuffer;
let globalFaqID = 0;
let is_exit = false;
const faqMap = new Map();
let webglContext = undefined;
let currentContext = undefined;
/**
 * 在下次进行重绘时执行的函数
 * @param cb 回调函数
 * @return 底层的ID，用于取消下一次重绘
 */
export const requestAnimationFrame = (cb) => {
    // console.log("---------- js requestAnimationFrame, cb = ", cb);
    if (!cb) {
        throw new Error("requestAnimationFrame failed!! cb is null!!!");
    }

    let id = ++globalFaqID;
    faqMap.set(id, cb);

    // 底层渲染循环开头调用这个函数；
    const onDraw = () => {
        let now = performance.now();
        let ids = [...faqMap.keys()];

        for (let id of ids) {
            try {
                let cb = faqMap.get(id);
                if (cb) {
                    cb(now);
                    faqMap.delete(id);
                }
            } catch (e) {
                console.error("raf cb catch exception err = " + e);
            }
        }
        if (webglContext) {
            webglContext.swapBuffer();
        }
    };

    let isGL = true;
    if (currentContext) {
        isGL = false;
    }
    let obj = { onDraw, isGL };
    // 只调用一次rafMod.requestAnimationFrame触发底层的swapbuffer
    // 底层每隔16毫秒会调用一次ondraw和swapbuffer// console.log("faqMap.size = " + faqMap.size, "is_exit = " + is_exit + "cb =  " + cb);
    if (faqMap.size === 1 && !is_exit) {
        rafMod.requestAnimationFrame(obj);
        is_exit = true;
    }
    return id;
};
/**
 * 取消一个先前通过调用 requestAnimationFrame()方法添加到计划中的动画帧请求
 * @requestID requestID 调用 requestAnimationFrame()方法时返回的ID.
 */
const cancelAnimationFrame = (requestID) => {
    if (requestID) {
        // console.log("cancelAnimationFrame = " + requestID);
        faqMap.delete(requestID);
    }
};
globalThis.cancelAnimationFrame = cancelAnimationFrame;
globalThis.requestAnimationFrame = requestAnimationFrame;
globalThis.__setFaqContext = (context) => {
    webglContext = context;
    globalThis.__setFaqContext = undefined;
};

globalThis.__setCurrentSurface = (surface) => {
    currentContext = surface;
};

