declare var NativeObject;
declare var _$cid;



export async function load_font(pid: number, key: string, buffer: ArrayBuffer): Promise<undefined> {
    let result = NativeObject.async_static_call(1, pid, key, buffer) as Promise<Promise<undefined>>;
    return await result;
}

class GlobalMetrics {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GlobalMetrics already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GlobalMetrics {
        return new GlobalMetrics(obj);
    }

    public get_ascender(): number {
        if (this.self == undefined) {
            throw new Error("GlobalMetrics object already destroy");
        }

        let result = NativeObject.call(27, this.self) as number;
        return result;
    }

    public get_descender(): number {
        if (this.self == undefined) {
            throw new Error("GlobalMetrics object already destroy");
        }

        let result = NativeObject.call(28, this.self) as number;
        return result;
    }

    public get_height(): number {
        if (this.self == undefined) {
            throw new Error("GlobalMetrics object already destroy");
        }

        let result = NativeObject.call(29, this.self) as number;
        return result;
    }

    public get_max_advance(): number {
        if (this.self == undefined) {
            throw new Error("GlobalMetrics object already destroy");
        }

        let result = NativeObject.call(30, this.self) as number;
        return result;
    }

}

class CanvasRenderingContext2DBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): CanvasRenderingContext2DBase {
        return new CanvasRenderingContext2DBase(obj);
    }

    /**
     * 加载系统字体
     */
    public set_font_form_system(_family_name: string): string | Error {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        let result = NativeObject.call(31, this.self, _family_name) as string;
        return result;
    }

    /**
     * 设置描边宽度
     */
    public set_stroker_width(width: number): void {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        NativeObject.call(32, this.self, width);
    }

    /**
     * 获取字符串的布局信息
     */
    public measure_text(text: string): number {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        let result = NativeObject.call(33, this.self, text) as number;
        return result;
    }

    /**
     * 解码高层传下来的参数，并设置相应的状态
     * 参数参考：https://developer.mozilla.org/zh-CN/docs/Web/CSS/font
     */
    public set_font(font: string): string | Error {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        let result = NativeObject.call(34, this.self, font) as string;
        return result;
    }

    public set_stroke_style(color: string): void {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        NativeObject.call(35, this.self, color);
    }

    public set_fill_style(color: string): void {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        NativeObject.call(36, this.self, color);
    }

    /**
     * 获取一组与字形无关的布局信息
     */
    public get_global_metrics(): object {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        let result = NativeObject.call(37, this.self) as object;
        return result;
    }

    public fill_text(text: string, x_pos: number, y_pos: number): string | Error {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }
        try {
            let result = NativeObject.call(38, this.self, text, x_pos, y_pos) as string;
            return result;
        } catch (e) {
            console.warn(`fill_text ${text} failed!! reason: ${e}!!!`)
        }
    }

    public storker_text(text: string, x_pos: number, y_pos: number): string | Error {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        try {
            let result = NativeObject.call(39, this.self, text, x_pos, y_pos) as string;
            return result;
        } catch (e) {
            console.warn(`storker_text ${text} failed!! reason: ${e}!!!`)
        }
    }

    public set_shadow_color(color: string): void {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        NativeObject.call(40, this.self, color);
    }

    public set_shadow_offset_x(offset_x: number): void {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        NativeObject.call(41, this.self, offset_x);
    }

    public set_shadow_offset_y(offset_y: number): void {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        NativeObject.call(42, this.self, offset_y);
    }

    public set_shadow_blur(level: number): void {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        NativeObject.call(43, this.self, level);
    }

    public fill_rect(x: number, y: number, width: number, height: number): void {
        if (this.self == undefined) {
            throw new Error("CanvasRenderingContext2DBase object already destroy");
        }

        NativeObject.call(44, this.self, x, y, width, height);
    }

}





class CanvasRenderingContext2D {
    face: CanvasRenderingContext2DBase;

    constructor(base: object) {
        this.face = CanvasRenderingContext2DBase.from(base);
    }

    static from(obj: object): CanvasRenderingContext2D {
        return new CanvasRenderingContext2D(obj);
    }

    public destroy() {
        this.face.destroy();
    }

    set font(font: string) {
        // console.log(`font = ${font}`);
        this.face.set_font(font);
    }

    set fillStyle(color: string) {
        let str = color.replace(/\s+/g, "");
        // console.log(`str = ${str}`);
        this.face.set_fill_style(str);
    }

    set strokeStyle(color: string) {
        let str = color.replace(/\s+/g, "");
        this.face.set_stroke_style(str);
    }

    set shadowColor(color: string) {
        let str = color.replace(/\s+/g, "");
        this.face.set_shadow_color(str);
    }
    // 将阴影向右移动15px，向上移动10px
    set shadowOffsetX(offsetX: number) {
        this.face.set_shadow_offset_x(offsetX);
    }
    set shadowOffsetY(offsetY: number) {
        this.face.set_shadow_offset_y(offsetY);
    }
    // 轻微模糊阴影
    set shadowBlur(level: number) {
        this.face.set_shadow_blur(level);
    }

    set textAlign(align: string) {
    }

    set textBaseline(pos: string) {

    }


    public fillRect(x: number, y: number, width: number, height: number) {
        this.face.fill_rect(x, y, width, height);
    }

    public fillText(text: string, x: number, y: number) {
        this.face.fill_text(text, x, y);
    }

    public strokeText(text: string, x: number, y: number) {
        this.face.storker_text(text, x, y);
    }

    public measureText(text: string): object {
        let width = this.face.measure_text(text);
        return { width };
    }

    set lineWidth(width: number) {
        this.face.set_stroker_width(width);
    }
}

class FontFace {
    key: string;
    path: string | ArrayBuffer;
    descriptors: object

    constructor(key: string, path: string | ArrayBuffer, descriptors?: object) {
        if (path instanceof String) {
            this.path = path.replace(/\s+/g, "");;
        } else {
            this.path = path;
        }
        this.key = key;
        this.descriptors = descriptors;
    }

    public async load(): Promise<string> {
        let buffer;
        if (this.path instanceof ArrayBuffer) {
            buffer = this.path;
        } else {
            const blobUrl = 'blob:';
            const pathStart = 'url(';
            const pathEnd = ')';
            if (this.path.indexOf(blobUrl) !== -1) {
                let url = this.path.substring(this.path.indexOf(blobUrl), this.path.length - 1);
                let blob = globalThis.BolbMap.get(url);

                if (!blob) {
                    // console.log(`font path is invalid!! path = ${url}`);
                    return Promise.reject(`font path is invalid!! path = ${this.path}`);
                }

                buffer = blob.buffer.slice();
                // if (blob.buffer instanceof Uint8Array) {
                //     buffer = blob.buffer.slice();
                // }else if (blob.buffer instanceof ArrayBuffer){
                //     buffer = blob.buffer.slice();
                // }

            } else if (this.path.indexOf(pathStart) !== -1) {
                let start = this.path.indexOf(pathStart) + pathStart.length;
                if (this.path.indexOf(pathEnd, start) !== -1) {
                    let end = this.path.indexOf(pathEnd, start);
                    let url = this.path.substring(start, end);

                    const { readFile } = globalThis.getFileSystemManager();
                    buffer = await readFile(url);
                } else {
                    // console.log(`font path is invalid!! path = ${this.path}`);
                    return Promise.reject(`font path is invalid!! path = ${this.path}`);
                }
            }
            else if (this.path.indexOf("http:") !== -1 || this.path.indexOf("https:") !== -1) {
                let index = this.path.indexOf("http:");
                if (index === -1) {
                    index = this.path.indexOf("https:");
                }
                let url = this.path.substring(index, this.path.length - 2);
                console.log("======== path is http, path = " + this.path + "; url = " + url);
                let http = new XMLHttpRequest();
                http.onload = () => {
                    console.log(" ====== request font file succeed!!");
                    buffer = http.response;
                }
                http.onerror = () => {
                    console.log(" ====== request font file filed!!");
                }
                http.open("GET", url);

                http.send();
            }
            else {
                // console.log(`font path is invalid!! path = ${this.path}`);
                return Promise.reject(`font path is invalid!! path = ${this.path}`);
            }
        }
        let pid: number = globalThis.ESProcess.pid;
        await load_font(pid, this.key, buffer);
    }
}


let g = globalThis as any;
g.CanvasRenderingContext2D = CanvasRenderingContext2D;
g.FontFace = FontFace;


