
// TODO ResponsePointer 的 释放


const ERROR_TIMEOUT = -14;

interface HttpMod {

    // 终止任务
    abortRequest(abortPointer: string);

    destroyResponsePointer(responsePointer: string);

    /**
     * @return {string} 用于终止底层任务的指针；
     */
    httpRequest(
        url: string,

        headers: Object,

        body: string | ArrayBuffer | FormData,

        timeout: number,

        method: string,

        responseType: string,

        version: number,

        callback: {
            onloadstart: (status: number, statusText: string, headers: Object) => void,
            onload: (data: ArrayBuffer | string | null, text: string) => void,
            onerror: (code: number, reason: string) => void,
            onprogress: (total: number, loaded: number) => void,
            onabort: () => void,
        }
    ): string;
}

const httpMod: HttpMod = globalThis._$pi_native.Http;
delete globalThis._$pi_native.Http;

export class FormData {

    private _data = []; // [ [key, value, fileName], ... ]

    public _type = "application/x-www-form-urlencoded"; // "multipart/form-data"

    append(key: string, value: string | ArrayBuffer, fileName?: string) {

        if (typeof key !== typeof "" || !key) {
            throw new Error(`invalid key = ${key}`);
        }

        if (fileName) {
            if (typeof fileName !== typeof "" || !(value instanceof ArrayBuffer)) {
                throw new Error("value and fileName not match, file must have binary value");
            } else {
                this._type = "multipart/form-data";
            }
        } else {
            fileName = undefined;
            if (typeof value !== typeof "") {
                throw new Error("value must string type when fileName isn't set");
            }
        }

        this._data.push([key, value, fileName]);
    }
}

class XMLHttpRequest {

    static UNSEND = 0; // 代理被创建，但尚未调用 open() 方法。

    static OPENDED = 1; // open() 方法已经被调用。

    static HEADERS_RECEIVED = 2; // send() 方法已经被调用，并且头部和状态已经可获得。

    static LOADING = 3; // 下载中； responseText 属性已经包含部分数据。

    static DONE = 4; // 下载操作已完成。

    // ====================== 只读 属性

    response: ArrayBuffer | string | JSON = null; // 取决于 responseType 的值

    responseText: string = null; // 取决于 responseType 的值

    status: number = 0; // 请求的响应状态。

    statusText: string = ""; // 完整的响应状态文本（例如，"200 OK"）

    readyState: number = 0; // 代表请求的状态码

    // 上传 进度，不可用
    upload: XMLHttpRequestUpload = null;

    // ====================== 可 修改属性

    responseType: string = "arraybuffer"; //  "arraybuffer", "json", "text"

    // 默认：10分钟；
    timeout = 10 * 60 * 1000; // 请求的最大请求时间（毫秒），若超出该时间，请求会自动终止。

    // 不可用
    withCredentials = false;

    private _isRealRequst = false;
    private _url = "";
    private _method = "GET";
    // 默认http 版本为1.0;
    private _version = 1.1;
    private _headers = {};
    private _respPointer: string = null;
    private _abortPointer: string = null;

    private _progressCb = null;
    private _loadCb = null;
    private _loadstartCb = null;
    private _loadendCb = null;
    private _errorCb = null;
    private _timeoutCb = null;
    private _abortCb = null;
    private _stateChangeCb = null;

    // 适配浏览器的addEventListener， removeEventListener函数
    private _progressEventCb = new Set<(total: number, loaded: number) => void>();
    private _loadEventCb = new Set<(data: ArrayBuffer | string) => void>();
    private _loadstartEventCb = new Set<(status: number, statusText: string, headers: Object) => void>();
    private _loadendEventCb = new Set<() => void>();
    private _errorEventCb = new Set<(error: object) => void>();
    private _timeoutEventCb = new Set<(error: object) => void>();
    private _abortEventCb = new Set<() => void>();
    private _stateChangeEventCb = new Set<() => void>();

    // ====================== 方法

    private _changeReadyState(old: number, replace: number, isCheck = true) {

        if (isCheck && this.readyState !== old) {
            throw new Error(`invalid ready state, expect = ${old}, now = ${replace}`);
        }

        if (this.readyState !== replace) {
            this.readyState = replace;
            this._stateChangeCb && this._stateChangeCb();

            for (let cb of this._stateChangeEventCb) {
                cb();
            }
        }
    }

    // 序列化（serialized）的响应 URL
    // TODO: 304 重定向 需要處理，這時候返回的 并不是原始的url
    get responseURL(): string {
        return this._url;
    }


    // 终止请求
    abort() {
        if (this._abortPointer) {
            httpMod.abortRequest(this._abortPointer);
            this._abortPointer = null;
        }
    }

    getResponseHeader(name: string): string {
        if (!name || typeof name !== typeof "") {
            throw new Error(`invalid name = ${name}`);
        }

        if (this.readyState < XMLHttpRequest.HEADERS_RECEIVED) {
            throw new Error("invalid readyState");
        }

        return this._headers[name];
    }

    getAllResponseHeaders() {
        if (this.readyState < XMLHttpRequest.HEADERS_RECEIVED) {
            throw new Error("invalid readyState");
        }
        return Object.assign(this._headers);
    }

    // 初始化请求
    open(method: string, url: string, async = true, username: string = null, password: string = null) {

        if (!url) {
            throw new Error("invalid url");
        }

        this._changeReadyState(XMLHttpRequest.UNSEND, XMLHttpRequest.OPENDED);

        this._method = method || "GET";
        this._url = url;
    }

    // TODO: 覆写由服务器返回的 MIME 类型。
    overrideMimeType(mime: string): void {

    }

    // 显示设置http 版本; 版本参数只能是 0.9, 1.0, 1.1, 2.0, 3.0 其中之一
    setVersion(version: number): void {
        if (version === 0.9 || version === 1.0 || version === 1.1 || version === 2.0 || version === 3.0) {
            this._version = version;
        } else {
            throw new Error("version arg is invalid, only support 0.9, 1.0, 1.1, 2.0 or 3.0");
        }
    }

    // 发送请求。
    // 如果请求是异步的（默认），那么该方法将在请求发送后立即返回
    send(body?: ArrayBuffer | string | FormData): void {

        // TODO BufferView的處理；
        if (ArrayBuffer.isView(body)) {
        }

        this._changeReadyState(XMLHttpRequest.OPENDED, XMLHttpRequest.OPENDED);

        this._isRealRequst = true;

        const onloadstart = (status: number, statusText: string, headers: Object) => {
            this._changeReadyState(XMLHttpRequest.HEADERS_RECEIVED, XMLHttpRequest.HEADERS_RECEIVED, false);

            this.status = status;
            this.statusText = statusText;
            this._headers = headers;

            this._loadstartCb && this._loadstartCb();
            for (let cb of this._loadstartEventCb) {
                cb(status, statusText, headers);
            }
        }

        const loadendCb = () => {
            if (this._abortPointer) {
                httpMod.abortRequest(this._abortPointer);
                this._abortPointer = null;
            }

            this._loadendCb && this._loadendCb();

            for (let cb of this._loadendEventCb) {
                cb();
            }
        }

        const onload = (response: ArrayBuffer | string, responseText: string) => {
            // this.status = 200;

            if (this.responseType === "json" && typeof response === typeof "") {
                try {
                    this.response = JSON.parse(response as string);
                } catch (e) {
                    // TODO Error
                }
            } else {
                this.response = response;
            }
            this.responseText = responseText;

            // console.log(`response = ${this.response}, responseText = ${this.responseText}`);

            this._changeReadyState(XMLHttpRequest.DONE, XMLHttpRequest.DONE, false);

            this._loadCb && this._loadCb();

            for (let cb of this._loadEventCb) {
                cb(response);
            }

            loadendCb();
        }

        const onerror = (code: number, message: string) => {
            this._changeReadyState(XMLHttpRequest.DONE, XMLHttpRequest.DONE, false);

            if (code === ERROR_TIMEOUT) {
                this._timeoutCb && this._timeoutCb();

                for (let cb of this._timeoutEventCb) {
                    cb({});
                }
            } else {
                this._errorCb && this._errorCb({ code, message });
                for (let cb of this._errorEventCb) {
                    cb({ code, message });
                }
            }

            loadendCb();

        };

        const onprogress = (total: number, loaded: number) => {

            if (this.readyState === XMLHttpRequest.HEADERS_RECEIVED) {
                this._changeReadyState(XMLHttpRequest.LOADING, XMLHttpRequest.LOADING, false);
            }

            this._progressCb && this._progressCb({ total, loaded });

            for (let cb of this._progressEventCb) {
                cb(total, loaded);
            }
        };

        const onabort = () => {
            if (this.readyState === XMLHttpRequest.HEADERS_RECEIVED) {
                this._changeReadyState(XMLHttpRequest.DONE, XMLHttpRequest.DONE, false);
            }

            this._abortCb && this._abortCb();

            for (let cb of this._abortEventCb) {
                cb();
            }
        };

        let cb = {
            onloadstart, onload, onerror, onprogress, onabort
        };

        if (body instanceof FormData) {
            // node服务端 会自动加上数据类型；不需要在请求头中显式设置
        } else if (body) {
            let contentType = this._headers["Content-Type"];
            if (contentType === "application/json" && typeof body !== typeof "") {
                try {
                    body = JSON.stringify(body);
                } catch (e) {
                    throw new Error("json object invalid");
                }
            }
            if (!this._headers["Content-Type"]) {
                this._headers["Content-Type"] = typeof body === typeof "" ? "text/plain" : "application/octet-stream";
            }
        }

        this._abortPointer = httpMod.httpRequest(
            this._url,
            this._headers,
            body,
            this.timeout,
            this._method,
            this.responseType,
            this._version,
            cb);
    }

    // 设置请求头
    setRequestHeader(name: string, value: string): void {
        if (this.readyState < XMLHttpRequest.OPENDED) {
            throw new Error("invalid readyState");
        }

        if (!name) {
            return;
        }

        if (!value) {
            delete this._headers[name];
            return;
        }

        if (typeof value !== typeof "" || typeof name !== typeof "") {
            throw new Error("invalid name or value,  not string");
        }

        this._headers[name] = value;
    }

    public addEventListener(event: String, cb: (this: XMLHttpRequest) => void, options?: boolean) {
        if (event.indexOf("loadend") !== -1) {
            this._loadendEventCb.add(cb);
        } else if (event.indexOf("readystatechange") !== -1) {
            this._stateChangeEventCb.add(cb);
        } else if (event.indexOf("abort") !== -1) {
            this._abortEventCb.add(cb);
        } else if (event.indexOf("error") !== -1) {
            this._errorEventCb.add(cb);
        } else if (event.indexOf("load") !== -1) {
            this._loadEventCb.add(cb);
        } else if (event.indexOf("loadstart") !== -1) {
            this._loadstartEventCb.add(cb);
        } else if (event.indexOf("progress") !== -1) {
            this._progressEventCb.add(cb);
        } else if (event.indexOf("timeout") !== -1) {
            this._timeoutEventCb.add(cb);
        } else {

        }
    }
    public removeEventListener(event: String, cb: (this: XMLHttpRequest) => void, options?: boolean) {
        if (event.indexOf("loadend") !== -1) {
            this._loadendEventCb.delete(cb);
        } else if (event.indexOf("readystatechange") !== -1) {
            this._stateChangeEventCb.delete(cb);
        } else if (event.indexOf("abort") !== -1) {
            this._abortEventCb.delete(cb);
        } else if (event.indexOf("error") !== -1) {
            this._errorEventCb.delete(cb);
        } else if (event.indexOf("load") !== -1) {
            this._loadEventCb.delete(cb);
        } else if (event.indexOf("loadstart") !== -1) {
            this._loadstartEventCb.delete(cb);
        } else if (event.indexOf("progress") !== -1) {
            this._progressEventCb.delete(cb);
        } else if (event.indexOf("timeout") !== -1) {
            this._timeoutEventCb.delete(cb);
        } else {

        }
    }

    // ====================== 设置回调函数

    // readyState 属性发生变化时，事件
    set onreadystatechange(cb: (this: XMLHttpRequest) => void) {
        this._stateChangeCb = cb ? cb.bind(this) : null;
    }

    // 注册：请求 终止 时候的回调函数
    set onabort(cb: (this: XMLHttpRequest) => void) {
        this._abortCb = cb ? cb.bind(this) : null;
    }

    // 注册：请求 失败 时候的回调函数
    set onerror(cb: (this: XMLHttpRequest, ev: { code: number, message: string }) => void) {
        this._errorCb = cb ? cb.bind(this) : null;
    }

    // 注册：http 正常完成时候 时候的回调函数
    set onload(cb: (this: XMLHttpRequest) => void) {
        this._loadCb = cb ? cb.bind(this) : null;
    }

    // 注册：开始传送数据 时候的回调函数
    set onloadstart(cb: (this: XMLHttpRequest) => void) {
        this._loadstartCb = cb ? cb.bind(this) : null;
    }

    // 注册：请求完成，不管成功失败都会回调
    set onloadend(cb: (this: XMLHttpRequest) => void) {
        this._loadendCb = cb ? cb.bind(this) : null;
    }

    // 进度回调
    set onprogress(cb: (this: XMLHttpRequest, ev: { total: number, loaded: number }) => void) {
        this._progressCb = cb ? cb.bind(this) : null;
    }

    // 注册：超时 时候的回调函数
    set ontimeout(cb: (this: XMLHttpRequest) => void) {
        this._timeoutCb = cb ? cb.bind(this) : null;
    }
}

let g = globalThis as any;
g.FormData = FormData;
g.XMLHttpRequest = XMLHttpRequest;


