


// 和rust交互的接口
interface QuicMod {
    open(rootCert: ArrayBuffer): string;
    connect(clientPtr: string, remote: string, hostname: string, timeout?: number): Promise<string>;
    send(connectPtr: string, buf: ArrayBuffer | string): void;
    receive(connectPtr: string, encoding?: string): Promise<ArrayBuffer | string>;
    close(connectPtr: string): Promise<string>;
    destroyClient(senderPointer: string): void;
}

const quicMod: QuicMod = globalThis._$pi_native.QuicMod;
delete globalThis._$pi_native.QuicMod;

export class QuicClient {
    private client: string | undefined;

    constructor(rootCert: ArrayBuffer) {
        try {
            this.client = quicMod.open(rootCert);
        } catch (error) {
            throw error;
        }
    }

    connect(remote: string, hostname: string, timeout?: number) {
        return new Promise((resolve, reject) => {
            if (!this.client) {
                throw Error("QuicConnection send failed!! reason: client is destroy!!")
            }

            quicMod.connect(this.client, remote, hostname, timeout).then((connection) => {
                let quicConnection = new QuicConnection(connection);
                resolve(quicConnection);
            }).catch((err) => {
                reject(err);
            });
        });

    }

    destory() {
        if (!this.client) {
            throw Error("QuicConnection send failed!! reason: client is destroy!!")
        }

        let client = this.client;
        this.client = undefined;
        quicMod.destroyClient(client,);
    }
}

class QuicConnection {
    private connection: string | undefined;

    constructor(connectionPtr) {
        this.connection = connectionPtr
    }

    send(buf: ArrayBuffer | string) {
        try {
            if (!this.connection) {
                throw Error("QuicConnection send failed!! reason: connection is closed!!")
            }
            quicMod.send(this.connection, buf);
        } catch (error) {
            throw error;
        }
    }

    receive(encoding?: string) {
        return new Promise((resolve, reject) => {
            try {
                if (!this.connection) {
                    throw Error("QuicConnection send failed!! reason: connection is closed!!")
                }
                quicMod.receive(this.connection, encoding).then((resp) => {
                    resolve(resp);
                });
            } catch (error) {
                reject(error);
            }
        });

    }

    close() {
        if (!this.connection) {
            throw Error("QuicConnection send failed!! reason: connection is closed!!")
        }

        let connection = this.connection;
        this.connection = undefined;
        return new Promise((resolve, reject) => {
            quicMod.close(connection).then((_) => {
                resolve(null);
            }).catch((err) => {
                reject(err);
            });
        });


    }
}

const g = globalThis as any;
g.QuicClient = QuicClient;

