/**
 * 定时器：setInterval, clearInterval
 */

const emptyFunc = () => { };

let globalIntervalID = 0;
const intervalIDSet = new Set<number>();
const _claerTimeout = globalThis.clearTimeout;

/**
 * 设置：间隔定时器
 * @param handler 回调函数；
 * @param ms 间隔时间，单位：毫秒；
 * @return 大于0的数字，表示定时器id，用于取消定时器
 */
export const setInterval = (handler: TimerHandler, ms: number): number => {
    if (typeof ms !== typeof 0 || ms < 0) {
        throw new Error(`setInterval failed, the second parameter ms invalid, ms = ${ms}`);
    }

    if (typeof handler !== typeof emptyFunc) {
        throw new Error(`setInterval failed, the first parameter cb isn't function`);
    }

    const currId = ++globalIntervalID;
    intervalIDSet.add(currId);

    const _impl = () => {
        setTimeout(() => {
            // 时间到还没取消，就运行回调；
            if (intervalIDSet.has(currId)) {
                (handler as Function)();
                _impl();
            }
        }, ms);
    }

    _impl();
    return currId;
}

/**
 * 取消：间隔定时器
 * @param intervalID setInterval的返回值，大于0的数字，表示定时器ID；
 */
const clearInterval = (intervalID: number) => {
    intervalIDSet.delete(intervalID);
};

globalThis.clearTimeout = (intervalID: number) => {
    if (typeof intervalID === typeof 0) {
        _claerTimeout(intervalID)
    }
}

globalThis.setInterval = setInterval;
globalThis.clearInterval = clearInterval;
