
interface VideoMod {
    // 传入图片的原始二进制数据，返回rgba8的二进制数据
    decodeVideo(path: string, callback: {
        onAudioReadly: (ptr: string) => void,
        onVideoReadly: (rate: number, frames: number, duration: number, ptr: string, width: number, height: number, format: string) => void,
    }): Promise<ArrayBuffer>;
    nextFrame(ptr: string, callback: { onData: (frame_buffer: ArrayBuffer, pts: number, frames: number) => void }): object;
    play(ptr: string): void;
    pause(ptr: string): void;
    skip(audio_ptr: string, video_ptr: string, time: number): object;
    speed(audio_ptr: string, video_ptr: string, speed: number): number;
    loop(ptr: string, is_loop: boolean): object;
    destroy(audioPtr: string, videoPtr: string,): void;
    crashFrames(audioPtr: string, videoPtr: string,): number;
    volume(audioPtr: string, videoPtr: string, volume: number): void;
}
declare var NativeObject;
declare var _$cid;
const videoMod: VideoMod = globalThis._$pi_native.Video;
delete globalThis._$pi_native.Video;

// const TimeupdateCb = new Set<(object: Object) => void>();
// const PlayingCb = new Set<(object: Object) => void>();
// const CanplaythroughCb = new Set<(object: Object) => void>();
// const EndCb = new Set<(object: Object) => void>();

enum State {
    Initing = 0,
    Inited = 1,
    Playing = 2,
    Played = 3,
    Pauseing = 4,
    Pauseed = 5,
}

class Frame {
    _data: ArrayBuffer;
    pts: number;
}

export class Video {
    private _src = "";

    private TimeupdateCb = new Set<(object: Object) => void>();
    private PlayingCb = new Set<(object: Object) => void>();
    private CanplaythroughCb = new Set<(object: Object) => void>();
    private EndCb = new Set<(object: Object) => void>();

    private _rate: number = undefined;
    private _frames: number = undefined;
    private _duration: number = undefined;
    private _player: string = undefined;
    private _state: State = State.Initing;
    private _rustState: State = State.Initing;
    private _loop: boolean = false;
    private _muted: boolean = false;
    private _muted2: boolean = false;
    private begin = 0;
    private _frameIndex = 0;
    // 视频每帧间隔时间
    private _fpsTime = 0;

    private _lastFrame;
    private _width;
    private _height;
    private _videoPtr;

    public HAVE_NOTHING = 0;
    public HAVE_METADATA = 1;
    public HAVE_CURRENT_DATA = 2;
    public HAVE_FUTURE_DATA = 3;
    public HAVE_ENOUGH_DATA = 4;

    private videoData = [];

    private audioData: [ArrayBuffer];
    // 返回一个无符号整数值，该值指示资源的固有高度（以 CSS 像素为单位），如果尚无可用的媒体，则返回 0。
    public videoHeight: number = 0;

    // 返回一个无符号整数值，该值指示资源的固有宽度（以 CSS 像素为单位），如果尚无可用的媒体，则返回
    public videoWidth: number = 0;

    // 获得可用音频轨道的数量
    public readonly audioTracks: number;
    // private crash_frame = 0;
    start = (_index: number) => {
        return this._lastFrame.pts;
    }
    end = (_index: number) => {
        return this._lastFrame.pts / 1000 + videoMod.crashFrames(this._player, this._videoPtr) / this._rate;
    }
    // 返回一个只读 TimeRanges 对象 返回媒体已缓冲的区域
    public readonly buffered = {start: this.start, end: this.end, length: 1};

    // 属性包含所选媒体源的绝对URL路径。例如当程序会根据用户显示器分辨率选择不同媒体文件时会用到这个属性，显然它是只读的。
    public readonly currentSrc: number;

    // 该属性指示媒体元素的音频输出是否应默认静音。此属性没有动态效果。要使音频输出静音和取消静音，请使用该muted属性。
    public readonly defaultMuted: number;

    // 属性指示媒体的默认播放速率。
    // 1.0是“正常速度”，低于该值1.0会使媒体播放比正常速度慢，较高的值使其播放速度更快。
    public readonly defaultPlaybackRate: number;

    // 示元素媒体 的duration长度（以秒为单位）。
    // 一个双精度浮点值，指示媒体的持续时间（以秒为单位）。
    // 如果没有可用的媒体数据，NaN则返回该值。
    // 如果元素的媒体没有已知的持续时间（例如实时媒体流），则duration值为+Infinity。
    public readonly duration: number;

    // 表示包含的媒体是否已完成播放。
    public readonly ended: boolean;

    // 描述媒体元素上最近发生的错误或未发生错误 的对象null。
    public readonly error: MediaError;

    // 指示通过网络获取媒体的当前状态。
    // 参考：https://developer.mozilla.org/en-US/docs/Web/API/HTMLMediaElement/networkState
    public readonly networkState: number;

    // true已暂停false且未暂停。
    public paused: boolean;

    // 确定浏览器是否应调整音频的音高以补偿通过设置对播放速率所做的更改
    public readonly preservesPitch: boolean;

    // 指示媒体的就绪状态。
    // 参考：https://developer.mozilla.org/en-US/docs/Web/API/HTMLMediaElement/readyState
    public readyState: number = 0;

    // 该对象列出了TextTrack 表示媒体元素的文本轨道的所有对象，其顺序与文本轨道列表中的顺序相同
    // 参考：https://developer.mozilla.org/en-US/docs/Web/API/HTMLMediaElement/textTracks
    public readonly textTracks: TextTrackList;

    // 它表示当player下载到足够可以播放的媒体文件时，是否自动播放视频。
    // 注意: 一些版本的Chrome只支持autostart, 不支持autoplay
    public autoplay: boolean;

    // 负责控制播放的媒体(视频或音频)的控制条是否显示。
    public controls: boolean;

    // 会以秒为单位返回当前媒体元素的播放时间。设置这个属性会改变媒体元素当前播放位置。
    private _currentTime = 0;

    // 该属性控制媒体元素在到达末尾时是否应重新开始。
    set loop(isLoop: boolean) {
        if (this._loop !== isLoop) {
            this._loop = isLoop;
            videoMod.loop(this._player, isLoop)
        }
    }

    // true表示静音和false不静音。
    set muted(v: boolean){
        this._muted = v;
    }

    // 属性设置播放媒体的速率。这用于实现用户控制快进、慢动作等。正常播放速率乘以该值得到当前速率，因此值为 1.0 表示正常速度。
    // 如果playbackRate为负，则不向后播放媒体。
    // 当快进或慢动作超出有用范围时，音频会静音（例如，Gecko 会将超出范围的声音静音0.25到4.0）。
    public playbackRate: boolean;

    // 设置播放媒体的音量。
    // 双精度值必须介于 0 和 1 之间，其中 0 表示有效静音，而 1 是最大可能值。
    public volume: number;
    public format: string;

    private frame_onload: boolean;
    private frame_onend = false;
    private is_destory: boolean;

    get src(): string {
        return this._src;
    }
    private videoRestart = false;
    private audioRestart = false;

    private cb = (frame: ArrayBuffer, pts: number, frames: number) => {
        // console.log("frames:", frames, "; pts: ", this._lastFrame?.pts);
        // this.crash_frame = frames;
        if (this.is_destory) {
            videoMod.destroy(this._player, this._videoPtr)
            this._player = undefined;
            this._videoPtr = undefined;
            return;
        }
        if (this._lastFrame && pts < this._lastFrame.pts) {
            this.videoRestart = true;
            if (!this._loop) {
                this.frame_onend = true;
                return;
            }

            // NativeObject.release(_$cid, frame);
        }

        this.frame_onload = true;


        this._lastFrame = {
            frame,
            pts,
        };

        for (let cb of this.TimeupdateCb) {
            cb({});
        }

        for (let cb of this.PlayingCb) {
            cb({});
        }

        for (let cb of this.CanplaythroughCb) {
            cb({});
        }
    }

    is_destroy = false;
    set src(url: string) {
        if (url === "") {
            videoMod.pause(this._player);
            if (this._videoPtr && this._player && this.frame_onload) {
                videoMod.destroy(this._player, this._videoPtr)
                this._player = undefined;
                this._videoPtr = undefined;

            } else {
                this.is_destory = true;
            }
            // 释放帧数据
            if (this._lastFrame && this._lastFrame.buffer) {
                if (globalThis.navigator.userAgent.indexOf("PI_WINDOWS") >= 0) {
                    NativeObject.release(_$cid, this._lastFrame.frame);
                }
                this._lastFrame = undefined;
            }
            return;
        }
        this._src = url;

        let videoReadly = false;
        let audioReadly = false;
        const onVideoReadly = (rate: number, frames: number, duration: number, ptr: string, width: number, height: number, format: string) => {
            this.videoWidth = width;
            this.videoHeight = height;
            this.format = format;
            if (this._state < State.Inited) {
                this._rustState = State.Inited;
            }
            console.log("duration: ", duration, " frames: ", frames);
            this.readyState = 4;

            if (!this._rate) {
                this._rate = frames / duration;
            }
            this._fpsTime = 1000 / this._rate;

            if (!this._frames) {
                this._frames = frames;
            }

            if (!this._duration) {
                this._duration = duration;
            }

            if (!this._videoPtr) {
                this._videoPtr = ptr;
            }

            if (!this._width) {
                this._width = width;
            }

            if (!this._height) {
                this._height = height;
            }
            videoReadly = true;

            if (videoReadly && audioReadly && this._videoPtr) {
                this.frame_onload = false;
                videoMod.nextFrame(this._videoPtr, { onData: this.cb });

                videoReadly = false;
                audioReadly = false;
            }

        }

        const onAudioReadly = (ptr: string) => {
            this._player = ptr;
            audioReadly = true;
            if (videoReadly && audioReadly) {
                this.frame_onload = false;
                videoMod.nextFrame(this._videoPtr, { onData: this.cb });


                videoReadly = false;
                audioReadly = false;
            }
        }

        videoMod.decodeVideo(url, { onVideoReadly, onAudioReadly });
    }

    // 事件在资源未完全加载时触发，但不是错误的结果。
    set onabort(cb: (this: Video) => void) { }

    // 当用户代理可以播放媒体时触发该canplay事件，但估计没有加载足够的数据来播放媒体直到其结束，而无需停止以进一步缓冲内容。
    set oncanplay(cb: (this: Video) => void) { }

    // 当用户代理可以播放媒体时触发该canplaythrough事件，并估计已经加载了足够的数据来播放媒体直到其结束，而不必停止进一步缓冲内容。
    set oncanplaythrough(cb: (this: Video) => void) {
        this.CanplaythroughCb.add(cb);
    }

    // 当属性更新duration时触发该事件; 
    set ondurationchange(cb: (this: Video) => void) { }

    // 当媒体变空时触发该事件；
    set onemptied(cb: (this: Video) => void) { }

    // 当由于到达媒体末尾或没有更多可用数据而停止播放或流式传输时会触发该事件
    set onended(cb: (this: Video) => void) {
        this.EndCb.add(cb);
    }

    // 由于错误（例如，网络连接问题）而无法加载资源时会触发该事件。
    set onerror(cb: (this: Video) => void) { }

    // 当媒体当前播放位置的帧完成加载时触发该事件；通常是第一帧。
    set onloadeddata(cb: (this: Video) => void) { }

    // 加载元数据时触发该事件。
    set onloadedmetadata(cb: (this: Video) => void) { }

    // 当浏览器开始加载资源时触发该事件。
    set onloadstart(cb: (this: Video) => void) { }

    // 当处理暂停活动的请求并且活动已进入其暂停状态时发送该事件，
    // 最常见的是在通过调用元素的pause()方法暂停媒体之后。
    set onpause(cb: (this: Video) => void) { }

    // 当属性由于方法或属性而从 更改为时play，将触发该事件
    set onplay(cb: (this: Video) => void) { }

    // 件在第一次开始播放后以及重新启动时触发
    set onplaying(cb: (this: Video) => void) { }

    // 当浏览器加载资源时，该progress事件会定期触发。
    set onprogress(cb: (this: Video) => void) { }

    // 当播放速率改变时触发该事件
    set onratechange(cb: (this: Video) => void) { }

    // seeked当查找操作完成、当前播放位置已更改且布尔seeking属性更改为false时，将触发该事件。
    set onseeked(cb: (this: Video) => void) { }

    // 当搜索操作开始时触发该seeking事件，这意味着布尔seeking属性已更改为true并且媒体正在搜索新位置。
    set onseeking(cb: (this: Video) => void) { }

    // stalled当用户代理尝试获取媒体数据时触发该事件，但数据出乎意料地没有出现。
    set onstalled(cb: (this: Video) => void) { }

    // 当属性指示的时间已更新timeupdate时，将触发该事件。currentTime
    set ontimeupdate(cb: (this: Video) => void) { }

    // volumechange当音量发生变化时触发该事件。
    set onvolumechange(cb: (this: Video) => void) { }

    // waiting当由于暂时缺少数据而停止播放时触发该事件。
    set onwaiting(cb: (this: Video) => void) { }

    get currentTime(): number {
        return this._currentTime;
    }

    set currentTime(time: number) {
        this._currentTime = time;
        // 用户设置了currentTime 属性
        console.log("========= 用户设置了currentTime 属性, this.currentTime: " + this._currentTime + "; this._duration: " + this._duration);
        if (this._currentTime >= this._duration) {
            this._state = State.Pauseing;
            videoMod.pause(this._player);
        }
    }

    public captureStream(): MediaStream {
        return new MediaStream();
    }

    public canPlayType(sourceType: string): boolean {
        return false;
    }

    public fastSeek(time: number) { }

    public load() { }

    public pause() {
        this._state = State.Pauseing;
    }

    public play() {
        this._state = State.Playing;
    }
    private lastAudioPts = -1;
    public getCurrentFrame(): object {
        if (this._muted != this._muted2 && this._player && this._videoPtr){
            if (this._muted){
                videoMod.volume(this._player, this._videoPtr, 0);
            }else{
                videoMod.volume(this._player, this._videoPtr, 1);
            }
            this._muted2 = this._muted;
        }
        // let begin = performance.now();
        if (this._state > State.Inited && this._state < State.Pauseing) {
            if (this._player && this._state === State.Playing) {
                // console.log("========= play audio: player: " + this._player);
                videoMod.play(this._player);
                this._state = State.Played;
                this.begin = performance.now();
            }

            if (this._player && this._videoPtr) {
                let audio_pts = videoMod.speed(this._player, this._videoPtr, this._lastFrame.pts);
                if (audio_pts < this.lastAudioPts) {
                    this.audioRestart = true;
                }
                this.lastAudioPts = audio_pts;

                // console.log("audio_pts: ", audio_pts, " video pts: ", this._lastFrame.pts, " this.audioRestart: ", this.audioRestart, " this.videoRestart: ", this.videoRestart);
                if (audio_pts > this._lastFrame.pts || this.audioRestart) {
                    // 正常播放完毕
                    if (this.frame_onend) {
                        // console.log("========= 正常播放完毕");
                        this.paused = true;
                        videoMod.pause(this._player);
                        this.EndCb.forEach(cb => {
                            cb({});
                        });
                    } else {
                        if (this.frame_onload) {
                            this.frame_onload = false;
                            videoMod.nextFrame(this._videoPtr, { onData: this.cb });
                        }
                        // console.log("nextFrame: " ,  performance.now() - begin);

                    }
                    if (this.audioRestart && this.videoRestart) {
                        this.audioRestart = false;
                        this.videoRestart = false;
                    }

                }

            }

        } else if (this._state > State.Played && this._state <= State.Pauseed) {
            if (this._player && this._state === State.Pauseing) {
                // console.log("========= pause audio: player: " + this._player);
                videoMod.pause(this._player);
                this._state = State.Pauseed;
            }
            // console.log("time5: " ,  performance.now() - begin);
        }


        return { frame: this._lastFrame.frame, width: this._width, height: this._height };
    }

    public addEventListener(event: string, cb: (this: Video) => void, options?: boolean) {
        switch (event) {
            case "playing":
                this.PlayingCb.add(cb);
                break;
            case "timeupdate":
                this.TimeupdateCb.add(cb);
                break;
            default:
                break;
        }
    }

    public removeEventListener(event: string, cb: (this: Video) => void, options?: boolean) {
        switch (event) {
            case "playing":
                this.PlayingCb.delete(cb);
                break;
            case "timeupdate":
                this.TimeupdateCb.delete(cb);
                break;
            default:
                break;
        }
    }

    public setAttribute(key: string, v: any) {
        this[key] = v;
    }
}

let g = globalThis as any;
g.HTMLVideoElement = Video;
