declare var NativeObject;
declare var _$cid;

export async function request_adapter(options_ptr: number): Promise<object> {
    let result = NativeObject.async_static_call(3, options_ptr) as Promise<object>;
    return await result;
}

function render(rpass: number, render_pipeline: number): void {
    NativeObject.static_call(33, rpass, render_pipeline);
}

class GPURequestAdapterOptionsBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPURequestAdapterOptionsBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPURequestAdapterOptionsBase {
        return new GPURequestAdapterOptionsBase(obj);
    }

    static new(): GPURequestAdapterOptionsBase {
        let result = NativeObject.static_call(9) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPURequestAdapterOptionsBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPURequestAdapterOptionsBase object already destroy");
        }

        let result = NativeObject.call(205, this.self) as number;
        return result;
    }

    public set_power_preference(power_preference: string): void {
        if (this.self == undefined) {
            throw new Error("GPURequestAdapterOptionsBase object already destroy");
        }

        NativeObject.call(206, this.self, power_preference);
    }

    public set_force_fallback_adapter(force_fallback_adapter: boolean): void {
        if (this.self == undefined) {
            throw new Error("GPURequestAdapterOptionsBase object already destroy");
        }

        NativeObject.call(207, this.self, force_fallback_adapter);
    }

    public power_preference(): string {
        if (this.self == undefined) {
            throw new Error("GPURequestAdapterOptionsBase object already destroy");
        }

        let result = NativeObject.call(208, this.self) as string;
        return result;
    }

    public force_fallback_adapter(): boolean {
        if (this.self == undefined) {
            throw new Error("GPURequestAdapterOptionsBase object already destroy");
        }

        let result = NativeObject.call(209, this.self) as boolean;
        return result;
    }

}

class GPUCanvasConfiguration {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUCanvasConfiguration already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUCanvasConfiguration {
        return new GPUCanvasConfiguration(obj);
    }

    static new(device_ptr: number, format: string, width: number, height: number): GPUCanvasConfiguration {
        let result = NativeObject.static_call(10, device_ptr, format, width, height) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUCanvasConfiguration(result);
        }
    }

    public set_usage(usage: number): string | Error {
        if (this.self == undefined) {
            throw new Error("GPUCanvasConfiguration object already destroy");
        }

        let result = NativeObject.call(210, this.self, usage) as string;
        return result;
    }

    public set_format(format: string): void {
        if (this.self == undefined) {
            throw new Error("GPUCanvasConfiguration object already destroy");
        }

        NativeObject.call(211, this.self, format);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUCanvasConfiguration object already destroy");
        }

        let result = NativeObject.call(212, this.self) as number;
        return result;
    }

}

class GPUCanvasContextBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUCanvasContextBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUCanvasContextBase {
        return new GPUCanvasContextBase(obj);
    }

    public configure(configuration_ptr: number): void {
        if (this.self == undefined) {
            throw new Error("GPUCanvasContextBase object already destroy");
        }

        NativeObject.call(213, this.self, configuration_ptr);
    }

    public unconfigure(): void {
        if (this.self == undefined) {
            throw new Error("GPUCanvasContextBase object already destroy");
        }

        NativeObject.call(214, this.self);
    }

    public get_preferred_format(adapter_ptr: number): string | undefined {
        if (this.self == undefined) {
            throw new Error("GPUCanvasContextBase object already destroy");
        }

        let result = NativeObject.call(215, this.self, adapter_ptr) as string;
        return result;
    }

    public get_current_texture(): object | Error {
        if (this.self == undefined) {
            throw new Error("GPUCanvasContextBase object already destroy");
        }

        let result = NativeObject.call(216, this.self) as object;
        return result;
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUCanvasContextBase object already destroy");
        }

        let result = NativeObject.call(217, this.self) as number;
        return result;
    }

}

class GPUTextureViewDescriptorBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUTextureViewDescriptorBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUTextureViewDescriptorBase {
        return new GPUTextureViewDescriptorBase(obj);
    }

    static new(): GPUTextureViewDescriptorBase {
        let result = NativeObject.static_call(11) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUTextureViewDescriptorBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUTextureViewDescriptorBase object already destroy");
        }

        let result = NativeObject.call(218, this.self) as number;
        return result;
    }

}

class GPUTextureViewBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUTextureViewBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUTextureViewBase {
        return new GPUTextureViewBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUTextureViewBase object already destroy");
        }

        let result = NativeObject.call(219, this.self) as number;
        return result;
    }

}

class GPUTextureBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUTextureBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUTextureBase {
        return new GPUTextureBase(obj);
    }

    public create_view(format: string, dimension: string, aspect: string, base_mip_level: number, mip_level_count: number, base_array_layer: number, array_layer_count: number): object | Error {
        if (this.self == undefined) {
            throw new Error("GPUTextureBase object already destroy");
        }

        let result = NativeObject.call(220, this.self, format, dimension, aspect, base_mip_level, mip_level_count, base_array_layer, array_layer_count) as object;
        return result;
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUTextureBase object already destroy");
        }

        let result = NativeObject.call(221, this.self) as number;
        return result;
    }

}

class GPUSurfaceTextureBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUSurfaceTextureBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUSurfaceTextureBase {
        return new GPUSurfaceTextureBase(obj);
    }

    public create_view(format: string, dimension: string, aspect: string, base_mip_level: number, mip_level_count: number, base_array_layer: number, array_layer_count: number): object | Error {
        if (this.self == undefined) {
            throw new Error("GPUSurfaceTextureBase object already destroy");
        }

        let result = NativeObject.call(222, this.self, format, dimension, aspect, base_mip_level, mip_level_count, base_array_layer, array_layer_count) as object;
        return result;
    }

    public present(): void {
        if (this.self == undefined) {
            throw new Error("GPUSurfaceTextureBase object already destroy");
        }

        NativeObject.call(223, this.self);
    }

    public ptr(): number | undefined {
        if (this.self == undefined) {
            throw new Error("GPUSurfaceTextureBase object already destroy");
        }

        let result = NativeObject.call(224, this.self) as number;
        return result;
    }

}

class GPUSupportedFeaturesBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUSupportedFeaturesBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUSupportedFeaturesBase {
        return new GPUSupportedFeaturesBase(obj);
    }

    static new(): GPUSupportedFeaturesBase {
        let result = NativeObject.static_call(12) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUSupportedFeaturesBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedFeaturesBase object already destroy");
        }

        let result = NativeObject.call(225, this.self) as number;
        return result;
    }

    public set_features(features: string): void {
        if (this.self == undefined) {
            throw new Error("GPUSupportedFeaturesBase object already destroy");
        }

        NativeObject.call(226, this.self, features);
    }

    public features(): string {
        if (this.self == undefined) {
            throw new Error("GPUSupportedFeaturesBase object already destroy");
        }

        let result = NativeObject.call(227, this.self) as string;
        return result;
    }

}

class GPUSupportedLimitsBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUSupportedLimitsBase {
        return new GPUSupportedLimitsBase(obj);
    }

    static new(): GPUSupportedLimitsBase {
        let result = NativeObject.static_call(13) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUSupportedLimitsBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(228, this.self) as number;
        return result;
    }

    public set_max_dynamic_uniform_buffers_per_pipeline_layout(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(229, this.self, value) as number;
        return result;
    }

    public set_max_dynamic_storage_buffers_per_pipeline_layout(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(230, this.self, value) as number;
        return result;
    }

    public set_max_sampled_textures_per_shader_stage(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(231, this.self, value) as number;
        return result;
    }

    public set_max_samplers_per_shader_stage(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(232, this.self, value) as number;
        return result;
    }

    public set_max_storage_buffers_per_shader_stage(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(233, this.self, value) as number;
        return result;
    }

    public set_max_storage_textures_per_shader_stage(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(234, this.self, value) as number;
        return result;
    }

    public set_max_uniform_buffers_per_shader_stage(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(235, this.self, value) as number;
        return result;
    }

    public set_max_uniform_buffer_binding_size(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(236, this.self, value) as number;
        return result;
    }

    public set_max_storage_buffer_binding_size(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(237, this.self, value) as number;
        return result;
    }

    public set_min_uniform_buffer_offset_alignment(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(238, this.self, value) as number;
        return result;
    }

    public set_min_storage_buffer_offset_alignment(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(239, this.self, value) as number;
        return result;
    }

    public set_max_vertex_buffers(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(240, this.self, value) as number;
        return result;
    }

    public set_max_vertex_attributes(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(241, this.self, value) as number;
        return result;
    }

    public set_max_vertex_buffer_array_stride(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(242, this.self, value) as number;
        return result;
    }

    public set_max_inter_stage_shader_components(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(243, this.self, value) as number;
        return result;
    }

    public set_max_compute_workgroup_storage_size(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(244, this.self, value) as number;
        return result;
    }

    public set_max_compute_invocations_per_workgroup(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(245, this.self, value) as number;
        return result;
    }

    public set_max_compute_workgroup_size_x(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(246, this.self, value) as number;
        return result;
    }

    public set_max_compute_workgroup_size_y(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(247, this.self, value) as number;
        return result;
    }

    public set_max_compute_workgroup_size_z(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(248, this.self, value) as number;
        return result;
    }

    public set_max_compute_workgroups_per_dimension(value: number): number {
        if (this.self == undefined) {
            throw new Error("GPUSupportedLimitsBase object already destroy");
        }

        let result = NativeObject.call(249, this.self, value) as number;
        return result;
    }

}

class GPUDeviceDescriptorBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUDeviceDescriptorBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUDeviceDescriptorBase {
        return new GPUDeviceDescriptorBase(obj);
    }

    static new(): GPUDeviceDescriptorBase {
        let result = NativeObject.static_call(14) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUDeviceDescriptorBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUDeviceDescriptorBase object already destroy");
        }

        let result = NativeObject.call(250, this.self) as number;
        return result;
    }

    public set_limits(limits_ptr: number): void {
        if (this.self == undefined) {
            throw new Error("GPUDeviceDescriptorBase object already destroy");
        }

        NativeObject.call(251, this.self, limits_ptr);
    }

    public set_features(features_ptr: number): void {
        if (this.self == undefined) {
            throw new Error("GPUDeviceDescriptorBase object already destroy");
        }

        NativeObject.call(252, this.self, features_ptr);
    }

}

class AdapterBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("AdapterBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): AdapterBase {
        return new AdapterBase(obj);
    }

    public async request_device(desc_ptr: number): Promise<object> {
        if (this.self == undefined) {
            throw new Error("AdapterBase object already destroy");
        }

        let result = NativeObject.async_call(0, this.self, desc_ptr) as Promise<object>;
        return await result;
    }

    public name(): string {
        if (this.self == undefined) {
            throw new Error("AdapterBase object already destroy");
        }

        let result = NativeObject.call(253, this.self) as string;
        return result;
    }

    public features(): string {
        if (this.self == undefined) {
            throw new Error("AdapterBase object already destroy");
        }

        let result = NativeObject.call(254, this.self) as string;
        return result;
    }

    public linits(): object {
        if (this.self == undefined) {
            throw new Error("AdapterBase object already destroy");
        }

        let result = NativeObject.call(255, this.self) as object;
        return result;
    }

    public is_fallback_adapter(): boolean {
        if (this.self == undefined) {
            throw new Error("AdapterBase object already destroy");
        }

        let result = NativeObject.call(256, this.self) as boolean;
        return result;
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("AdapterBase object already destroy");
        }

        let result = NativeObject.call(257, this.self) as number;
        return result;
    }

}

class ShaderModuleBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("ShaderModuleBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): ShaderModuleBase {
        return new ShaderModuleBase(obj);
    }

    public compilation_info(): void {
        if (this.self == undefined) {
            throw new Error("ShaderModuleBase object already destroy");
        }

        NativeObject.call(258, this.self);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("ShaderModuleBase object already destroy");
        }

        let result = NativeObject.call(259, this.self) as number;
        return result;
    }

}

class GPUPipelineLayoutBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUPipelineLayoutBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUPipelineLayoutBase {
        return new GPUPipelineLayoutBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUPipelineLayoutBase object already destroy");
        }

        let result = NativeObject.call(260, this.self) as number;
        return result;
    }

}

class GPUCommandEncoderDescriptorBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderDescriptorBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUCommandEncoderDescriptorBase {
        return new GPUCommandEncoderDescriptorBase(obj);
    }

    static new(): GPUCommandEncoderDescriptorBase {
        let result = NativeObject.static_call(15) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUCommandEncoderDescriptorBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderDescriptorBase object already destroy");
        }

        let result = NativeObject.call(261, this.self) as number;
        return result;
    }

}

class GPURenderPipelineBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPURenderPipelineBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPURenderPipelineBase {
        return new GPURenderPipelineBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPURenderPipelineBase object already destroy");
        }

        let result = NativeObject.call(262, this.self) as number;
        return result;
    }

    public get_bind_group_layout(index: number): object {
        if (this.self == undefined) {
            throw new Error("GPURenderPipelineBase object already destroy");
        }

        let result = NativeObject.call(263, this.self, index) as object;
        return result;
    }

}

class GPUVertexBufferLayoutBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUVertexBufferLayoutBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUVertexBufferLayoutBase {
        return new GPUVertexBufferLayoutBase(obj);
    }

    static new(array_stride: number, step_mode_str: string, attribute_ptrs: string): GPUVertexBufferLayoutBase {
        let result = NativeObject.static_call(16, array_stride, step_mode_str, attribute_ptrs) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUVertexBufferLayoutBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUVertexBufferLayoutBase object already destroy");
        }

        let result = NativeObject.call(264, this.self) as number;
        return result;
    }

}

class GPUVertexStateBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUVertexStateBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUVertexStateBase {
        return new GPUVertexStateBase(obj);
    }

    static new(module_ptr: number, entry_point: string, buffers_ptr: string): GPUVertexStateBase {
        let result = NativeObject.static_call(17, module_ptr, entry_point, buffers_ptr) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUVertexStateBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUVertexStateBase object already destroy");
        }

        let result = NativeObject.call(265, this.self) as number;
        return result;
    }

}

class GPUStencilFaceStateBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUStencilFaceStateBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUStencilFaceStateBase {
        return new GPUStencilFaceStateBase(obj);
    }

    static new(): GPUStencilFaceStateBase {
        let result = NativeObject.static_call(18) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUStencilFaceStateBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUStencilFaceStateBase object already destroy");
        }

        let result = NativeObject.call(266, this.self) as number;
        return result;
    }

    public set_compare(compare: string): void {
        if (this.self == undefined) {
            throw new Error("GPUStencilFaceStateBase object already destroy");
        }

        NativeObject.call(267, this.self, compare);
    }

    public set_fail_op(fail_op: string): void {
        if (this.self == undefined) {
            throw new Error("GPUStencilFaceStateBase object already destroy");
        }

        NativeObject.call(268, this.self, fail_op);
    }

    public set_depth_fail_op(depth_fail_op: string): void {
        if (this.self == undefined) {
            throw new Error("GPUStencilFaceStateBase object already destroy");
        }

        NativeObject.call(269, this.self, depth_fail_op);
    }

    public set_pass_op(pass_op: string): void {
        if (this.self == undefined) {
            throw new Error("GPUStencilFaceStateBase object already destroy");
        }

        NativeObject.call(270, this.self, pass_op);
    }

}

class GPUDepthStencilStateBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUDepthStencilStateBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUDepthStencilStateBase {
        return new GPUDepthStencilStateBase(obj);
    }

    static new(format: string, depth_write_enabled: boolean, depth_compare: string, stencil_front_ptr: number, stencil_back_ptr: number, stencil_read_mask: number, stencil_write_mask: number, depth_bias: number, depth_bias_slope_scale: number, depth_bias_clamp: number): GPUDepthStencilStateBase {
        let result = NativeObject.static_call(19, format, depth_write_enabled, depth_compare, stencil_front_ptr, stencil_back_ptr, stencil_read_mask, stencil_write_mask, depth_bias, depth_bias_slope_scale, depth_bias_clamp) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUDepthStencilStateBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUDepthStencilStateBase object already destroy");
        }

        let result = NativeObject.call(271, this.self) as number;
        return result;
    }

}

class GPUMultisampleStateBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUMultisampleStateBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUMultisampleStateBase {
        return new GPUMultisampleStateBase(obj);
    }

    static new(count: number, mask: number, alpha_to_coverage_enabled: boolean): GPUMultisampleStateBase {
        let result = NativeObject.static_call(20, count, mask, alpha_to_coverage_enabled) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUMultisampleStateBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUMultisampleStateBase object already destroy");
        }

        let result = NativeObject.call(272, this.self) as number;
        return result;
    }

    public set_count(count: number): number {
        if (this.self == undefined) {
            throw new Error("GPUMultisampleStateBase object already destroy");
        }

        let result = NativeObject.call(273, this.self, count) as number;
        return result;
    }

    public set_mask(mask: number): number {
        if (this.self == undefined) {
            throw new Error("GPUMultisampleStateBase object already destroy");
        }

        let result = NativeObject.call(274, this.self, mask) as number;
        return result;
    }

    public set_alpha_to_coverage_enabled(alpha_to_coverage_enabled: boolean): boolean {
        if (this.self == undefined) {
            throw new Error("GPUMultisampleStateBase object already destroy");
        }

        let result = NativeObject.call(275, this.self, alpha_to_coverage_enabled) as boolean;
        return result;
    }

}

class GPUFragmentStateBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUFragmentStateBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUFragmentStateBase {
        return new GPUFragmentStateBase(obj);
    }

    static new(module_ptr: number, entry_point: string, target_ptrs: string): GPUFragmentStateBase {
        let result = NativeObject.static_call(21, module_ptr, entry_point, target_ptrs) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUFragmentStateBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUFragmentStateBase object already destroy");
        }

        let result = NativeObject.call(276, this.self) as number;
        return result;
    }

}

class GPURenderPipelineDescriptorBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPURenderPipelineDescriptorBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPURenderPipelineDescriptorBase {
        return new GPURenderPipelineDescriptorBase(obj);
    }

    static new(vertex_ptr: number, primitive_ptr: number, depth_stencil_ptr: number, multisample_ptr: number, fragment_ptr: number): GPURenderPipelineDescriptorBase {
        let result = NativeObject.static_call(22, vertex_ptr, primitive_ptr, depth_stencil_ptr, multisample_ptr, fragment_ptr) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPURenderPipelineDescriptorBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPURenderPipelineDescriptorBase object already destroy");
        }

        let result = NativeObject.call(277, this.self) as number;
        return result;
    }

}

class GPUQueueBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUQueueBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUQueueBase {
        return new GPUQueueBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUQueueBase object already destroy");
        }

        let result = NativeObject.call(278, this.self) as number;
        return result;
    }

    public submit(command_buffer_ids: string): void {
        if (this.self == undefined) {
            throw new Error("GPUQueueBase object already destroy");
        }

        NativeObject.call(279, this.self, command_buffer_ids);
    }

    public async on_submitted_work_done(): Promise<undefined> {
        if (this.self == undefined) {
            throw new Error("GPUQueueBase object already destroy");
        }

        let result = NativeObject.async_call(1, this.self) as Promise<Promise<undefined>>;
        return await result;
    }

    public write_buffer(buffer_ptr: number, buffer_offset: number, data: ArrayBuffer, data_offset: number, size: number): void {
        if (this.self == undefined) {
            throw new Error("GPUQueueBase object already destroy");
        }

        NativeObject.call(280, this.self, buffer_ptr, buffer_offset, data, data_offset, size);
    }

    public write_texture(texture: number, mip_level: number, origin_x: number, origin_y: number, origin_z: number, aspect: string, data: ArrayBuffer, offset: number, bytes_per_row: number, rows_per_image: number, width: number, height: number, depth_or_array_layers: number): void {
        if (this.self == undefined) {
            throw new Error("GPUQueueBase object already destroy");
        }

        NativeObject.call(281, this.self, texture, mip_level, origin_x, origin_y, origin_z, aspect, data, offset, bytes_per_row, rows_per_image, width, height, depth_or_array_layers);
    }

    public copy_external_image_to_texture(source_ptr: number, destination_ptr: number, copy_size_ptr: number): void {
        if (this.self == undefined) {
            throw new Error("GPUQueueBase object already destroy");
        }

        NativeObject.call(282, this.self, source_ptr, destination_ptr, copy_size_ptr);
    }

}

class GPUSamplerBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUSamplerBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUSamplerBase {
        return new GPUSamplerBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUSamplerBase object already destroy");
        }

        let result = NativeObject.call(283, this.self) as number;
        return result;
    }

}

class GPUBindGroupBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUBindGroupBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUBindGroupBase {
        return new GPUBindGroupBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUBindGroupBase object already destroy");
        }

        let result = NativeObject.call(284, this.self) as number;
        return result;
    }

}

class GPUBindGroupLayoutBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUBindGroupLayoutBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUBindGroupLayoutBase {
        return new GPUBindGroupLayoutBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUBindGroupLayoutBase object already destroy");
        }

        let result = NativeObject.call(285, this.self) as number;
        return result;
    }

}

class GPUComputePipelineBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUComputePipelineBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUComputePipelineBase {
        return new GPUComputePipelineBase(obj);
    }

}

class GPURenderBundleEncoderBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPURenderBundleEncoderBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPURenderBundleEncoderBase {
        return new GPURenderBundleEncoderBase(obj);
    }

}

class GPUDeviceBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUDeviceBase {
        return new GPUDeviceBase(obj);
    }

    public features(): object {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(286, this.self) as object;
        return result;
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(287, this.self) as number;
        return result;
    }

    public limits(): object {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(288, this.self) as object;
        return result;
    }

    public queue(): object | undefined {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(289, this.self) as object;
        return result;
    }

    public create_buffer(desc_ptr: number): object {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(290, this.self, desc_ptr) as object;
        return result;
    }

    public create_texture(desc_ptr: number): object {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(291, this.self, desc_ptr) as object;
        return result;
    }

    public create_sampler(desc_ptr: number): object {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(292, this.self, desc_ptr) as object;
        return result;
    }

    public import_external_texture(): void {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        NativeObject.call(293, this.self);
    }

    public create_bind_group_layout(args: string, r_types: string): object | Error {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(294, this.self, args, r_types) as object;
        return result;
    }

    public create_pipeline_layout(ptrs: string): object {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(295, this.self, ptrs) as object;
        return result;
    }

    public create_bind_group(layout: number, entries: string, entries_type: string): object | Error {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(296, this.self, layout, entries, entries_type) as object;
        return result;
    }

    public create_shader_module(shader_code: string): object {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(297, this.self, shader_code) as object;
        return result;
    }

    public create_compute_pipeline(desc_ptr: number): object {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(298, this.self, desc_ptr) as object;
        return result;
    }

    public create_render_pipeline(layout: number, vertex_shader_module: number, vertex_entry_point: string, vertex_buffers: string, fragment_shader_module: number, fragment_entry_point: string, fragment_target: string, depth_stencil: number, multisample: number, topology: string, strip_index_format: string, front_face: string, cull_mode: string, unclipped_depth: boolean): object | Error {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(299, this.self, layout, vertex_shader_module, vertex_entry_point, vertex_buffers, fragment_shader_module, fragment_entry_point, fragment_target, depth_stencil, multisample, topology, strip_index_format, front_face, cull_mode, unclipped_depth) as object;
        return result;
    }

    public async create_compute_pipeline_async(): Promise<undefined> {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.async_call(2, this.self) as Promise<Promise<undefined>>;
        return await result;
    }

    public async create_render_pipeline_async(): Promise<undefined> {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.async_call(3, this.self) as Promise<Promise<undefined>>;
        return await result;
    }

    public create_command_encoder(desc_ptr: number): object {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(300, this.self, desc_ptr) as object;
        return result;
    }

    public create_query_set(desc_ptr: number): object {
        if (this.self == undefined) {
            throw new Error("GPUDeviceBase object already destroy");
        }

        let result = NativeObject.call(301, this.self, desc_ptr) as object;
        return result;
    }

}

class GPURenderPassDescriptorBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPURenderPassDescriptorBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPURenderPassDescriptorBase {
        return new GPURenderPassDescriptorBase(obj);
    }

    static new(): GPURenderPassDescriptorBase {
        let result = NativeObject.static_call(23) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPURenderPassDescriptorBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPURenderPassDescriptorBase object already destroy");
        }

        let result = NativeObject.call(302, this.self) as number;
        return result;
    }

}

class GPURenderPassEncoderBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPURenderPassEncoderBase {
        return new GPURenderPassEncoderBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        let result = NativeObject.call(303, this.self) as number;
        return result;
    }

    public set_viewport(x: number, y: number, width: number, height: number, min_depth: number, max_depth: number): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(304, this.self, x, y, width, height, min_depth, max_depth);
    }

    public set_scissor_rect(x: number, y: number, width: number, height: number): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(305, this.self, x, y, width, height);
    }

    public set_blend_constant(color_ptr: number): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(306, this.self, color_ptr);
    }

    public set_stencil_reference(reference: number): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(307, this.self, reference);
    }

    public begin_occlusion_query(query_index: number): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(308, this.self, query_index);
    }

    public end_occlusion_query(): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(309, this.self);
    }

    public execute_bundles(): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(310, this.self);
    }

    public end(): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(311, this.self);
    }

    public set_pipeline(pipeline_ptr: number): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(312, this.self, pipeline_ptr);
    }

    public set_vertex_buffer(slot: number, buffer_ptr: number, offset: number, size: number): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(313, this.self, slot, buffer_ptr, offset, size);
    }

    public draw(vertex_count: number, instance_count: number, first_vertex: number, first_instance: number): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(314, this.self, vertex_count, instance_count, first_vertex, first_instance);
    }

    public set_index_buffer(buffer: number, index_format: string, offset: number, size: number): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(315, this.self, buffer, index_format, offset, size);
    }

    public set_bind_group(index: number, bind_group: number, offsets: ArrayBuffer): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(316, this.self, index, bind_group, offsets);
    }

    public draw_indexed(index_count: number, instance_count: number, first_index: number, base_vertex: number, first_instance: number): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(317, this.self, index_count, instance_count, first_index, base_vertex, first_instance);
    }

    public end_pass(): void {
        if (this.self == undefined) {
            throw new Error("GPURenderPassEncoderBase object already destroy");
        }

        NativeObject.call(318, this.self);
    }

}

class GPUComputePassEncoderBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUComputePassEncoderBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUComputePassEncoderBase {
        return new GPUComputePassEncoderBase(obj);
    }

}

class GPUComputePassDescriptorBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUComputePassDescriptorBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUComputePassDescriptorBase {
        return new GPUComputePassDescriptorBase(obj);
    }

    static new(): GPUComputePassDescriptorBase {
        let result = NativeObject.static_call(24) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUComputePassDescriptorBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUComputePassDescriptorBase object already destroy");
        }

        let result = NativeObject.call(319, this.self) as number;
        return result;
    }

}

class GPUBufferBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUBufferBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUBufferBase {
        return new GPUBufferBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUBufferBase object already destroy");
        }

        let result = NativeObject.call(320, this.self) as number;
        return result;
    }

    public async map_async(mode: number, offset: number, size: number): Promise<string> {
        if (this.self == undefined) {
            throw new Error("GPUBufferBase object already destroy");
        }

        let result = NativeObject.async_call(4, this.self, mode, offset, size) as Promise<string>;
        return await result;
    }

    public mapped_range(offset: number, size: number): ArrayBuffer {
        if (this.self == undefined) {
            throw new Error("GPUBufferBase object already destroy");
        }

        let result = NativeObject.call(321, this.self, offset, size) as ArrayBuffer;
        return result;
    }

    public unmap(): void {
        if (this.self == undefined) {
            throw new Error("GPUBufferBase object already destroy");
        }

        NativeObject.call(322, this.self);
    }

}

class GPUImageCopyBufferBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUImageCopyBufferBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUImageCopyBufferBase {
        return new GPUImageCopyBufferBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUImageCopyBufferBase object already destroy");
        }

        let result = NativeObject.call(323, this.self) as number;
        return result;
    }

}

class GPUImageCopyTextureBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUImageCopyTextureBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUImageCopyTextureBase {
        return new GPUImageCopyTextureBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUImageCopyTextureBase object already destroy");
        }

        let result = NativeObject.call(324, this.self) as number;
        return result;
    }

}

class GPUExtent3DBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUExtent3DBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUExtent3DBase {
        return new GPUExtent3DBase(obj);
    }

    static new(width: number, height: number, depth_or_array_layers: number): GPUExtent3DBase {
        let result = NativeObject.static_call(25, width, height, depth_or_array_layers) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUExtent3DBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUExtent3DBase object already destroy");
        }

        let result = NativeObject.call(325, this.self) as number;
        return result;
    }

}

class GPUQuerySetBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUQuerySetBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUQuerySetBase {
        return new GPUQuerySetBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUQuerySetBase object already destroy");
        }

        let result = NativeObject.call(326, this.self) as number;
        return result;
    }

}

class GPUCommandBufferBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUCommandBufferBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUCommandBufferBase {
        return new GPUCommandBufferBase(obj);
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUCommandBufferBase object already destroy");
        }

        let result = NativeObject.call(327, this.self) as number;
        return result;
    }

}

class GPUCommandEncoderBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUCommandEncoderBase {
        return new GPUCommandEncoderBase(obj);
    }

    public begin_render_pass(color_attachments_ptrs: string, depth_stencil_attachment_ptr: number): object | undefined {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        let result = NativeObject.call(328, this.self, color_attachments_ptrs, depth_stencil_attachment_ptr) as object;
        return result;
    }

    public begin_compute_pass(desc_ptr: number): object | undefined {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        let result = NativeObject.call(329, this.self, desc_ptr) as object;
        return result;
    }

    public copy_buffer_to_buffer(source_ptr: number, source_offset: number, destination_ptr: number, destination_offset: number, copy_size: number): void {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        NativeObject.call(330, this.self, source_ptr, source_offset, destination_ptr, destination_offset, copy_size);
    }

    public copy_buffer_to_texture(source_ptr: number, destination_ptr: number, copy_size_ptr: number): void {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        NativeObject.call(331, this.self, source_ptr, destination_ptr, copy_size_ptr);
    }

    public copy_texture_to_buffer(source_ptr: number, destination_ptr: number, copy_size_ptr: number): void {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        NativeObject.call(332, this.self, source_ptr, destination_ptr, copy_size_ptr);
    }

    public copy_texture_to_texture(source_ptr: number, destination_ptr: number, copy_size_ptr: number): void {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        NativeObject.call(333, this.self, source_ptr, destination_ptr, copy_size_ptr);
    }

    public clear_buffer(buffer_ptr: number, offset: number, size: number): void {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        NativeObject.call(334, this.self, buffer_ptr, offset, size);
    }

    public write_timestamp(query_set_ptr: number, query_index: number): void {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        NativeObject.call(335, this.self, query_set_ptr, query_index);
    }

    public resolve_query_set(query_set_ptr: number, first_query: number, query_count: number, destination_ptr: number, destination_offset: number): void {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        NativeObject.call(336, this.self, query_set_ptr, first_query, query_count, destination_ptr, destination_offset);
    }

    public finish(): number | undefined {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        let result = NativeObject.call(337, this.self) as number;
        return result;
    }

    public ptr(): number | undefined {
        if (this.self == undefined) {
            throw new Error("GPUCommandEncoderBase object already destroy");
        }

        let result = NativeObject.call(338, this.self) as number;
        return result;
    }

}

class BufferDescriptorBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("BufferDescriptorBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): BufferDescriptorBase {
        return new BufferDescriptorBase(obj);
    }

    static new(size: number, usage: number, mapped_at_creation: boolean): BufferDescriptorBase | Error {
        let result = NativeObject.static_call(26, size, usage, mapped_at_creation) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new BufferDescriptorBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("BufferDescriptorBase object already destroy");
        }

        let result = NativeObject.call(339, this.self) as number;
        return result;
    }

}

class TextureDescriptorBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("TextureDescriptorBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): TextureDescriptorBase {
        return new TextureDescriptorBase(obj);
    }

    static new(size_ptr: number, mip_level_count: number, sample_count: number, dimension: string, format: string, usage: number): TextureDescriptorBase | undefined {
        let result = NativeObject.static_call(27, size_ptr, mip_level_count, sample_count, dimension, format, usage) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new TextureDescriptorBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("TextureDescriptorBase object already destroy");
        }

        let result = NativeObject.call(340, this.self) as number;
        return result;
    }

}

class GPUVertexAttributeBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUVertexAttributeBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUVertexAttributeBase {
        return new GPUVertexAttributeBase(obj);
    }

    static new(format: string, offset: number, shader_location: number): GPUVertexAttributeBase {
        let result = NativeObject.static_call(28, format, offset, shader_location) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUVertexAttributeBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUVertexAttributeBase object already destroy");
        }

        let result = NativeObject.call(341, this.self) as number;
        return result;
    }

}

class GPUColorTargetStateBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUColorTargetStateBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUColorTargetStateBase {
        return new GPUColorTargetStateBase(obj);
    }

    static new(format: string, blend: number, write_mask: number): GPUColorTargetStateBase {
        let result = NativeObject.static_call(29, format, blend, write_mask) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUColorTargetStateBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUColorTargetStateBase object already destroy");
        }

        let result = NativeObject.call(342, this.self) as number;
        return result;
    }

}

class GPURenderPassColorAttachmentBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPURenderPassColorAttachmentBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPURenderPassColorAttachmentBase {
        return new GPURenderPassColorAttachmentBase(obj);
    }

    static new(view_ptr: number, resolve_target_ptr: number, r: number, g: number, b: number, a: number, store: boolean): GPURenderPassColorAttachmentBase {
        let result = NativeObject.static_call(30, view_ptr, resolve_target_ptr, r, g, b, a, store) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPURenderPassColorAttachmentBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPURenderPassColorAttachmentBase object already destroy");
        }

        let result = NativeObject.call(343, this.self) as number;
        return result;
    }

}

class GPURenderPassDepthStencilAttachmentBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPURenderPassDepthStencilAttachmentBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPURenderPassDepthStencilAttachmentBase {
        return new GPURenderPassDepthStencilAttachmentBase(obj);
    }

    static new(view_ptr: number, depth_load_value: number, depth_store_op: string, stencil_load_value: number, stencil_store_op: string): GPURenderPassDepthStencilAttachmentBase {
        let result = NativeObject.static_call(31, view_ptr, depth_load_value, depth_store_op, stencil_load_value, stencil_store_op) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPURenderPassDepthStencilAttachmentBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPURenderPassDepthStencilAttachmentBase object already destroy");
        }

        let result = NativeObject.call(344, this.self) as number;
        return result;
    }

}

class GPUSamplerDescriptorBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("GPUSamplerDescriptorBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): GPUSamplerDescriptorBase {
        return new GPUSamplerDescriptorBase(obj);
    }

    static new(address_mode_u: string, address_mode_v: string, address_mode_w: string, mag_filter: string, min_filter: string, mipmap_filter: string, lod_min_clamp: number, lod_max_clamp: number, compare: string, anisotropy_clamp: number): GPUSamplerDescriptorBase | Error {
        let result = NativeObject.static_call(32, address_mode_u, address_mode_v, address_mode_w, mag_filter, min_filter, mipmap_filter, lod_min_clamp, lod_max_clamp, compare, anisotropy_clamp) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new GPUSamplerDescriptorBase(result);
        }
    }

    public ptr(): number {
        if (this.self == undefined) {
            throw new Error("GPUSamplerDescriptorBase object already destroy");
        }

        let result = NativeObject.call(345, this.self) as number;
        return result;
    }

}

















































async function requestAdapter(requestDescriptor?: object) {
    let powerPreference = "low-power";
    let forceFallbackAdapter = false;
    if (!!requestDescriptor) {
        let powerPreferenceInner: string = requestDescriptor["powerPreference"];
        if (!!powerPreferenceInner) {
            powerPreference = powerPreferenceInner;
        }
        let forceFallbackAdapterInner: boolean = requestDescriptor["forceFallbackAdapter"];
        if (!!forceFallbackAdapterInner) {
            forceFallbackAdapter = forceFallbackAdapterInner;
        }
    }
    let options = GPURequestAdapterOptionsBase.new();
    options.set_power_preference(powerPreference);
    options.set_force_fallback_adapter(forceFallbackAdapter);

    let adapterBase = await request_adapter(options.ptr());

    return new GPUAdapter(adapterBase);
}


class GPUAdapter {
    private _adapterBase: AdapterBase;

    constructor(adapterBase: object) {
        this._adapterBase = AdapterBase.from(adapterBase);
    }
    // Promise<GPUDevice> requestDevice(optional GPUDeviceDescriptor descriptor = {});
    public async requestDevice(requestDescriptor?: object): Promise<GPUDevice> {
        let features = GPUSupportedFeaturesBase.new();
        let limits = GPUSupportedLimitsBase.new();
        if (!!requestDescriptor) {
            let featuresInner: string[] = requestDescriptor["requiredFeatures"];
            if (!!featuresInner) {
                featuresInner.forEach(element => {
                    features.set_features(element);
                });
            }

            let limitsInner: object = requestDescriptor["requiredLimits"];
            if (!!limitsInner) {
                limits = setLimits(limits, limitsInner);
            }
        }
        // console.log("=========== 1111");
        let desc = GPUDeviceDescriptorBase.new();
        // console.log("=========== 22222");
        desc.set_features(features.ptr());
        // console.log("=========== 33333");
        desc.set_limits(limits.ptr());

        let device = await this._adapterBase.request_device(desc.ptr());
        return new GPUDevice(device);
    }

    public ptr() {
        return this._adapterBase.ptr();
    }
};


class GPUDevice {
    private _deviceBase: GPUDeviceBase;
    private _queue: GPUQueue = undefined;

    constructor(device: object) {
        this._deviceBase = GPUDeviceBase.from(device);
    }

    public addEventListener(event: String, cb: (this: GPUDevice) => void, options?: boolean) {

    }

    public ptr() {
        return this._deviceBase.ptr();
    }

    // [SameObject] readonly attribute GPUSupportedFeatures features;
    get features() {
        let f = GPUSupportedFeaturesBase.from(this._deviceBase.features()).features().split(" ");

        return new Set(f);
    }

    // [SameObject] readonly attribute GPUSupportedLimits limits;
    get limits() {
        return GPUSupportedLimitsBase.from(this._deviceBase.limits());
    }

    // [SameObject] readonly attribute GPUQueue queue;
    get queue() {
        if (!this._queue) {
            this._queue = new GPUQueue(this._deviceBase.queue());
        }
        return this._queue;
    }

    // undefined destroy();
    public destroy() {
        this._deviceBase.destroy();
    }

    // GPUBuffer createBuffer(GPUBufferDescriptor descriptor);
    public createBuffer(descriptor: object) {
        let mappedAtCreation = descriptor["mappedAtCreation"];
        let size = descriptor["size"]
        let usage = descriptor["usage"]
        if (!size) {
            throw new Error("createBuffer failed!! size id undefined!!!!")
        }

        if (!usage) {
            throw new Error("createBuffer failed!! usage id undefined!!!!")
        }

        if (typeof mappedAtCreation !== typeof true) {
            mappedAtCreation = false;
        }

        let desc = BufferDescriptorBase.new(size, usage, mappedAtCreation);
        if (typeof desc === typeof Error) {
            throw desc;
        }
        let buffer = this._deviceBase.create_buffer((desc as BufferDescriptorBase).ptr());
        return new GPUBuffer(buffer);
    }

    // GPUTexture createTexture(GPUTextureDescriptor descriptor);
    public createTexture(descriptor: object) {
        let size = descriptor["size"];
        let mipLevelCount = descriptor["mipLevelCount "];
        let sampleCount = descriptor["sampleCount"];
        let dimension = descriptor["dimension "];
        let format = descriptor["format"];
        let usage = descriptor["usage"];

        let extent3D;
        if (!size) {
            throw new Error("createTexture failed!! size is undefined!!!!")
        } else {

            let width = size["width"] !== undefined ? size["width"] : size[0];
            if (typeof width !== typeof 0) {
                throw new Error("createTexture failed!! width of size is not number!!!!")
            }

            let height = size["height"] !== undefined ? size["height"] : size[1];
            if (typeof height !== typeof 0) {
                height = 1;
            }

            let depthOrArrayLayers = size["depthOrArrayLayers"];
            if (!depthOrArrayLayers) {
                depthOrArrayLayers = 1;
            }

            extent3D = GPUExtent3DBase.new(width, height, depthOrArrayLayers);
        }

        if (!mipLevelCount) {
            mipLevelCount = 1;
        }

        if (sampleCount === undefined) {
            sampleCount = 1;
        }

        if (!dimension) {
            dimension = "2d";
        }

        if (!format) {
            throw new Error("createTexture failed!! format is undefined!!!!")
        }

        if (!usage) {
            throw new Error("createTexture failed!! usage is undefined!!!!")
        }

        let desc = TextureDescriptorBase.new(extent3D.ptr(), mipLevelCount, sampleCount, dimension, format, usage);
        let texture = this._deviceBase.create_texture(desc.ptr());
        return new GPUTexture(texture);
    }

    // GPUSampler createSampler(optional GPUSamplerDescriptor descriptor = {});
    public createSampler(descriptor: object) {
        let addressModeU = descriptor["addressModeU"];
        if (!addressModeU) {
            addressModeU = "clamp-to-edge";
        }

        let addressModeV = descriptor["addressModeV"];
        if (!addressModeV) {
            addressModeV = "clamp-to-edge";
        }

        let addressModeW = descriptor["addressModeW"];
        if (!addressModeW) {
            addressModeW = "clamp-to-edge";
        }

        let magFilter = descriptor["magFilter"];
        if (!magFilter) {
            magFilter = "nearest";
        }

        let minFilter = descriptor["minFilter"];
        if (!minFilter) {
            minFilter = "nearest";
        }

        let mipmapFilter = descriptor["mipmapFilter"];
        if (!mipmapFilter) {
            mipmapFilter = "nearest";
        }

        let lodMinClamp = descriptor["lodMinClamp"];
        if (typeof lodMinClamp !== typeof 0) {
            lodMinClamp = 0;
        }

        let lodMaxClamp = descriptor["lodMaxClamp"];
        if (typeof lodMaxClamp !== typeof 0) {
            lodMaxClamp = 32;
        }

        let compare = descriptor["descriptor"];
        if (!compare) {
            compare = "";
        }

        let maxAnisotropy = descriptor["descriptor"];
        if (typeof maxAnisotropy !== typeof 0) {
            maxAnisotropy = 1;
        }

        let desc = GPUSamplerDescriptorBase.new(addressModeU, addressModeV, addressModeW, magFilter, minFilter, mipmapFilter, lodMinClamp, lodMaxClamp, compare, maxAnisotropy);

        if (typeof desc === typeof Error) {
            throw Error;
        }

        let sampler = this._deviceBase.create_sampler((desc as GPUSamplerDescriptorBase).ptr());

        return new GPUSampler(sampler);
    }
    // GPUExternalTexture importExternalTexture(GPUExternalTextureDescriptor descriptor);

    // GPUBindGroupLayout createBindGroupLayout(GPUBindGroupLayoutDescriptor descriptor);
    public createBindGroupLayout(descriptor: object) {
        let ty = "";
        let args = ""
        descriptor["entries"].forEach((element) => {
            let binding = element["binding"];
            if (typeof binding !== typeof 0) {
                throw new Error("createBindGroupLayout failed!!! binding not number!!!")
            }
            args = args + binding.toString() + " ";

            let visibility = element["visibility"];
            if (typeof visibility !== typeof 0) {
                throw new Error("createBindGroupLayout failed!!! visibility not number!!!")
            }
            args = args + visibility.toString() + " ";

            if (element["buffer"]) {
                ty = ty + "buffer" + " ";

                let buffer = element["buffer"];

                let bufferType = buffer["type"];
                if (typeof bufferType !== typeof "") {
                    bufferType = "uniform"
                }
                args = args + bufferType + " ";

                let hasDynamicOffset = 0;
                if (typeof buffer["hasDynamicOffset"] === typeof true) {
                    if (buffer["hasDynamicOffset"]) {
                        hasDynamicOffset = 1;
                    }
                }
                args = args + hasDynamicOffset.toString() + " ";

                let minBindingSize = buffer["minBindingSize"];
                if (typeof minBindingSize !== typeof 0) {
                    minBindingSize = 0;
                }
                args = args + minBindingSize.toString() + " ";

            } else if (element["sampler"]) {
                ty = ty + "sampler" + " ";

                let sampler = element["sampler"];

                let type = sampler["type"];
                if (typeof type !== typeof "") {
                    type = "filtering"
                }
                args = args + type + " ";
            } else if (element["texture"]) {
                ty = ty + "texture" + " ";

                let texture = element["texture"];

                let sampleType = texture["sampleType"];
                if (typeof sampleType !== typeof "") {
                    sampleType = "float";
                }
                args = args + sampleType + " ";

                let viewDimension = texture["viewDimension"];
                if (typeof viewDimension !== typeof "") {
                    viewDimension = "2d";
                }
                args = args + viewDimension + " ";

                let multisampled = 0;
                if (texture["multisampled"] === true) {
                    multisampled = 1;
                }
                args = args + multisampled.toString() + " ";
            } else if (element["storageTexture"]) {
                ty = ty + "storageTexture" + " ";

                let storageTexture = element["storageTexture"];

                let access = storageTexture["access"];
                if (typeof access !== typeof "") {
                    access = "write-only";
                }
                args = args + access + " ";

                let format = storageTexture["format"];
                if (typeof format !== typeof 0) {
                    throw new Error("createBindGroupLayout failed!!! storageTexture format not number!!!")
                }
                args = args + format + " ";

                let viewDimension = storageTexture["viewDimension"];
                if (typeof viewDimension !== typeof "") {
                    viewDimension = "2d";
                }
                args = args + viewDimension + " ";
            } else {
                throw new Error("createBindGroupLayout failed!!! dest not buffer, sampler, texture or storageTexture");
            }
        });

        let group = this._deviceBase.create_bind_group_layout(args, ty);
        if (typeof group === typeof Error) {
            throw group;
        }
        return new GPUBindGroupLayout(group);
    }
    // GPUPipelineLayout createPipelineLayout(GPUPipelineLayoutDescriptor descriptor);
    public createPipelineLayout(descriptor: object) {
        let bindGroupLayouts: any[] = descriptor["bindGroupLayouts"];
        if (!bindGroupLayouts) {
            throw new Error("createPipelineLayout failed!! bindGroupLayouts is undefined!!!!")
        }

        let ptrs = "";
        bindGroupLayouts.forEach(element => {
            ptrs = ptrs + element.ptr() + " ";
        });

        let pipeline = this._deviceBase.create_pipeline_layout(ptrs);
        return new GPUPipelineLayout(pipeline);
    }

    // GPUBindGroup createBindGroup(GPUBindGroupDescriptor descriptor);
    public createBindGroup(descriptor: object) {
        let layout: GPUBindGroupLayout = descriptor["layout"];
        if (!layout) {
            throw new Error("createBindGroup failed!! layout is undefined!!!!")
        }

        let entries = "";
        let entries_type = ""
        descriptor["entries"].forEach((element) => {
            entries = entries + element["binding"].toString() + " ";

            if (element["resource"] instanceof GPUTextureView) {
                entries_type = entries_type + "texture_view" + " ";
                entries = entries + element["resource"].ptr() + " ";
            } else if (element["resource"] instanceof GPUSampler) {
                entries_type = entries_type + "sampler" + " ";
                entries = entries + element["resource"].ptr() + " ";
            } else if (element["resource"].buffer instanceof GPUBuffer) {
                entries_type = entries_type + "buffer" + " ";
                entries = entries + element["resource"].buffer.ptr() + " ";

                let offset = 0;
                if (typeof element["resource"].offset === typeof 0) {
                    offset = element["resource"].offset;
                }
                entries = entries + offset.toString() + " ";

                let size = 0;
                if (typeof element["resource"].size === typeof 0) {
                    offset = element["resource"].size;
                }
                entries = entries + size.toString() + " ";
            } else {
                throw new Error("createBindGroup failed!! resource: " + element["resource"] + " type invalid!!!!")
            }
        });

        let group = this._deviceBase.create_bind_group(layout.ptr(), entries, entries_type);
        return new GPUBindGroup(group);
    }

    // GPUShaderModule createShaderModule(GPUShaderModuleDescriptor descriptor);
    public createShaderModule(descriptor: object) {
        let code = descriptor["code"];
        if (typeof code !== typeof "") {
            throw new Error("createShaderModule failed!! code is not string!!!!")
        }
        return new GPUShaderModule(this._deviceBase.create_shader_module(code));
    }
    // GPUComputePipeline createComputePipeline(GPUComputePipelineDescriptor descriptor);
    // GPURenderPipeline createRenderPipeline(GPURenderPipelineDescriptor descriptor);
    public createRenderPipeline(descriptor: object) {
        let layout_temp: GPUPipelineLayout = descriptor["layout"];
        let vertex = descriptor["vertex"];
        let primitive = descriptor["primitive"];
        let depthStencil = descriptor["depthStencil"];
        let multisample = descriptor["multisample"];
        let fragment = descriptor["fragment"];

        let layout = 0;
        if (!!layout_temp) {
            layout = layout_temp.ptr()
        };

        if (!vertex) {
            throw new Error("createShaderModule failed!! vertex is undefined!!!!")
        }

        let vertexModule: GPUShaderModule = vertex["module"];
        if (!vertexModule) {
            throw new Error("createShaderModule failed!! vertex is undefined!!!!")
        }

        let vertexEntryPoint = vertex["entryPoint"]
        if (!vertexEntryPoint) {
            throw new Error("createShaderModule failed!! vertex is undefined!!!!")
        }

        let buffers: [] = vertex["buffers"]
        if (!buffers) {
            buffers = [];
        }

        let layout_ptrs = "";
        let attributes_temp = "";
        let attributesBase = [];
        let layoutBase = [];

        buffers.forEach((element) => {
            let arrayStride: number = element["arrayStride"];
            if (!arrayStride) {
                throw new Error("createShaderModule failed!! arrayStride is undefined!!!!")
            }

            let stepMode: string = element["stepMode"];
            if (!stepMode) {
                stepMode = "vertex";
            }

            let attributes: [] = element["attributes"];

            attributes.forEach(item => {
                let format = item["format"];
                let offset = item["offset"];
                let shaderLocation = item["shaderLocation"];

                let attribute = GPUVertexAttributeBase.new(format, offset, shaderLocation);
                attributesBase.push(attribute);
                attributes_temp = attributes_temp + attribute.ptr().toString() + " ";
            });

            let layout = GPUVertexBufferLayoutBase.new(arrayStride, stepMode, attributes_temp);
            layoutBase.push(layout);
            layout_ptrs = layout_ptrs + layout.ptr().toString() + " ";
        });

        let fragmentModule: GPUShaderModule = fragment["module"];
        if (!fragmentModule) {
            throw new Error("createShaderModule failed!! module of fragment is undefined!!!!")
        }

        let fragmentEntryPoint: string = fragment["entryPoint"]
        if (!vertexEntryPoint) {
            throw new Error("createShaderModule failed!! entryPoint is undefined!!!!")
        }

        let targets: [] = fragment["targets"];
        let targetBase = [];
        let targetPtr = "";
        targets.forEach(element => {
            let format: string = element["format"];
            if (!format) {
                throw new Error("createShaderModule failed!! format is undefined!!!!")
            }

            let writeMask: number = element["writeMask "];
            if (!writeMask) {
                writeMask = 0xF;
            }
            let target = GPUColorTargetStateBase.new(format, 0, writeMask);
            targetBase.push(target);
            targetPtr = targetPtr + target.ptr().toString() + " ";
        });
        // console.log("========== createRenderPipeline2");


        let stencilState = undefined;
        if (depthStencil) {
            let depthWriteEnabled: boolean = depthStencil["depthWriteEnabled"];
            if (!depthWriteEnabled) {
                depthWriteEnabled = false;
            }

            let depthCompare: string = depthStencil["depthCompare"];
            if (!depthCompare) {
                depthCompare = "always";
            }
            // console.log("========== createRenderPipeline2");
            let state = GPUStencilFaceStateBase.new();

            let stencilReadMask: number = depthStencil["stencilReadMask"];
            if (!stencilReadMask) {
                stencilReadMask = 0xFFFFFFFF;
            }

            let stencilWriteMask: number = depthStencil["stencilWriteMask"];
            if (!stencilWriteMask) {
                stencilWriteMask = 0xFFFFFFFF;
            }

            let depthBias: number = depthStencil["depthBias"];
            if (!depthBias) {
                depthBias = 0;
            }

            let depthBiasSlopeScale: number = depthStencil["depthBiasSlopeScale"];
            if (!depthBiasSlopeScale) {
                depthBiasSlopeScale = 0;
            }

            let depthBiasClamp: number = depthStencil["depthBiasClamp"];
            if (!depthBiasClamp) {
                depthBiasClamp = 0;
            }
            // console.log("========== createRenderPipeline3");

            stencilState = GPUDepthStencilStateBase.new(
                depthStencil["format"],
                depthWriteEnabled,
                depthCompare,
                state.ptr(),
                state.ptr(),
                stencilReadMask,
                stencilWriteMask,
                depthBias,
                depthBiasSlopeScale,
                depthBiasClamp);

        }

        let stencilStatePtr = 0;
        if (stencilState) {
            stencilStatePtr = stencilState.ptr();
        }

        let count = 1;
        let mask = 0xFFFFFFFF;
        let alphaToCoverageEnabled = false;

        if (multisample) {
            if (typeof multisample["count"] === typeof 0) {
                count = multisample["count"];
            }
            if (typeof multisample["mask"] === typeof 0) {
                mask = multisample["mask"];
            }
            if (typeof multisample["alphaToCoverageEnabled"] === typeof true) {
                alphaToCoverageEnabled = multisample["alphaToCoverageEnabled"];
            }
        }

        let vertexModulePtr = vertexModule.ptr();
        let fragmentModulePtr = fragmentModule.ptr();
        // console.log("========== createRenderPipeline4");
        let sampleState = GPUMultisampleStateBase.new(count, mask, alphaToCoverageEnabled);
        let sampleStatePtr = sampleState.ptr();
        // console.log("========== createRenderPipeline5; vertexEntryPoint = " + vertexEntryPoint + "fragmentEntryPoint = " + fragmentEntryPoint);

        if (!primitive) {
            throw new Error("createRenderPipeline failed!!! reason: primitive is undefined")
        }
        // console.log("======== ")
        let topology = "triangle-list";
        let stripIndexFormat = primitive['stripIndexFormat']
        let frontFace = "ccw";
        let cullMode = "none";
        let unclippedDepth = false;

        if (typeof primitive['topology'] === typeof "") {
            topology = primitive['topology'];
        }

        if (typeof stripIndexFormat !== typeof "") {
            stripIndexFormat = "none"
        }

        if (typeof primitive['frontFace'] === typeof "") {
            frontFace = primitive['frontFace'];
        }

        if (typeof primitive['cullMode'] === typeof "") {
            cullMode = primitive['cullMode'];
        }

        if (typeof primitive['unclippedDepth'] === typeof true) {
            unclippedDepth = primitive['unclippedDepth'];
        }

        let render = this._deviceBase.create_render_pipeline(
            layout,
            vertexModulePtr,
            vertexEntryPoint,
            layout_ptrs,
            fragmentModulePtr,
            fragmentEntryPoint,
            targetPtr,
            stencilStatePtr,
            sampleStatePtr,
            topology,
            stripIndexFormat,
            frontFace,
            cullMode,
            unclippedDepth);
        // console.log("========== createRenderPipeline6");
        return new GPURenderPipeline(render);
    }

    // Promise<GPUComputePipeline> createComputePipelineAsync(GPUComputePipelineDescriptor descriptor);
    // Promise<GPURenderPipeline> createRenderPipelineAsync(GPURenderPipelineDescriptor descriptor);

    // GPUCommandEncoder createCommandEncoder(optional GPUCommandEncoderDescriptor descriptor = {});
    public createCommandEncoder(descriptor: object) {
        let desc = GPUCommandEncoderDescriptorBase.new();
        let commandEncoder = this._deviceBase.create_command_encoder(desc.ptr());
        return new GPUCommandEncoder(commandEncoder);
    }
    // GPURenderBundleEncoder createRenderBundleEncoder(GPURenderBundleEncoderDescriptor descriptor);

    // GPUQuerySet createQuerySet(GPUQuerySetDescriptor descriptor);
};

class GPUQueue {
    private _queueBase: GPUQueueBase;

    constructor(queue: object) {
        this._queueBase = GPUQueueBase.from(queue);
    }

    public submit(ids: [number]) {
        let id_str = ""
        ids.forEach(element => {
            id_str = id_str + element.toString() + " ";
        });
        this._queueBase.submit(id_str)
    }
    public writeBuffer(buffer: GPUBuffer, buffer_offset: number, data: ArrayBuffer, data_offset: number, size: number) {
        console.log("======== writeBuffer");
        this._queueBase.write_buffer(buffer.ptr(), buffer_offset, data, data_offset, size);
    }

    public writeTexture(destination: GPUTexture, data: ArrayBuffer, dataLayout, size): void {
        let texture = destination["texture"];
        if (!texture) {
            throw new Error("writeTexture failed!! texture of destination is undefined!!!!")
        }

        let mipLevel = 0;
        if (typeof destination["mipLevel"] === typeof 0) {
            mipLevel = destination["mipLevel"];
        }

        let originX = 0;
        let originY = 0;
        let originZ = 0;
        let origin = destination["origin"];
        if (origin) {
            let x = origin["x"] !== undefined ? origin["x"] : origin[0];
            if (typeof x === typeof 0) {
                originX = x;
            }

            let y = origin["y"] !== undefined ? origin["y"] : origin[1];
            if (typeof x === typeof 0) {
                originY = y;
            }

            let z = origin["z"] !== undefined ? origin["z"] : origin[2];
            if (typeof x === typeof 0) {
                originZ = z;
            }
        }

        let aspect = "all";
        if (typeof destination["aspect"] === typeof "") {
            aspect = destination["aspect"];
        }

        console.log("======= writeTexture!!! destination = " + JSON.stringify(destination));
        debugger;
        let offset = dataLayout["offset"];
        if (typeof offset !== typeof 0) {
            offset = 0;
        }

        let bytesPerRow = dataLayout["bytesPerRow"];
        if (typeof bytesPerRow !== typeof 0) {
            throw new Error("writeTexture failed!! bytesPerRow of dataLayout is not number!!!!")
        }

        let rowsPerImage = dataLayout["rowsPerImage"];
        if (typeof rowsPerImage !== typeof 0) {
            throw new Error("writeTexture failed!! rowsPerImage of dataLayout is not number!!!!")
        }

        let width = size["width"] !== undefined ? size["width"] : size[0];
        if (typeof width !== typeof 0) {
            throw new Error("writeTexture failed!! width of size is not number!!!!")
        }

        let height = size["height"] !== undefined ? size["height"] : size[1];
        if (typeof height !== typeof 0) {
            height = 1;
        }

        let depthOrArrayLayers = size["depthOrArrayLayers"];
        if (!depthOrArrayLayers) {
            depthOrArrayLayers = 1;
        }

        this._queueBase.write_texture(texture.ptr(), mipLevel, originX, originY, originZ, aspect, data, offset, bytesPerRow, rowsPerImage, width, height, depthOrArrayLayers);

    }


    public copyExternalImageToTexture(source: object, destination: object, size: object) {
        let img = source["source"];
        if (!img) {
            throw new Error("copyExternalImageToTexture failed!! source is undefined!!!!")
        }

        let texture: GPUTexture = destination["texture"];
        if (!texture) {
            throw new Error("copyExternalImageToTexture failed!! texture is undefined!!!!")
        }

        let width = size["width"] !== undefined ? size["width"] : size[0];
        if (typeof width !== typeof 0) {
            throw new Error("copyExternalImageToTexture failed!! width of size is not number!!!!")
        }

        let height = size["height"] !== undefined ? size["height"] : size[1];
        if (typeof height !== typeof 0) {
            height = 1;
        }

        let depthOrArrayLayers = size["depthOrArrayLayers"];
        if (!depthOrArrayLayers) {
            depthOrArrayLayers = 1;
        }


        // this._queueBase.write_texture(texture.ptr(), img.buffer,);
    }
}

class GPUBuffer {
    private _bufferBase: GPUBufferBase;

    constructor(buffer: object) {
        this._bufferBase = GPUBufferBase.from(buffer);
    }

    public getMappedRange(offset?: number, size?: number) {
        let offsetImpl = 0;
        if (typeof offset == typeof 0) {
            offsetImpl = offset;
        }

        let sizeImpl = 0;
        if (typeof size == typeof 0) {
            sizeImpl = size;
        }

        return this._bufferBase.mapped_range(offsetImpl, sizeImpl);
    }

    public unmap() {
        this._bufferBase.unmap();
    }

    public destroy() {
        this._bufferBase.destroy();
    }

    public ptr() {
        return this._bufferBase.ptr();
    }
}

class GPUTextureView {
    private _textureViewBase: GPUTextureViewBase;

    constructor(texture: object) {
        this._textureViewBase = GPUTextureViewBase.from(texture);
    }

    public ptr() {
        return this._textureViewBase.ptr();
    }
}

class GPUSurfaceTexture {
    private _surface: GPUSurfaceTextureBase

    constructor(surface: object) {
        this._surface = GPUSurfaceTextureBase.from(surface);
    }

    public createView(descriptor = {}) {
        // console.log("============ js createView");
        let format = "none"
        if (typeof descriptor["format"] === typeof "") {
            format = descriptor["format"]
        }

        let dimension = "none";
        if (typeof descriptor["dimension"] === typeof "") {
            dimension = descriptor["dimension"]
        }

        let aspect = "all";
        if (typeof descriptor["aspect"] === typeof "") {
            aspect = descriptor["aspect"]
        }

        let baseMipLevel = 0;
        if (typeof descriptor["baseMipLevel"] === typeof 0) {
            baseMipLevel = descriptor["baseMipLevel"]
        }

        let mipLevelCount = -1;
        if (typeof descriptor["mipLevelCount"] === typeof 0) {
            mipLevelCount = descriptor["mipLevelCount"]
        }

        let baseArrayLayer = 0;
        if (typeof descriptor["baseArrayLayer"] === typeof 0) {
            baseArrayLayer = descriptor["baseArrayLayer"]
        }

        let arrayLayerCount = -1;
        if (typeof descriptor["arrayLayerCount"] === typeof 0) {
            arrayLayerCount = descriptor["arrayLayerCount"]
        }


        let view = this._surface.create_view(format, dimension, aspect, baseMipLevel, mipLevelCount, baseArrayLayer, arrayLayerCount);
        if (typeof view === typeof Error) {
            throw view;
        }
        return new GPUTextureView(view);
    }

    public present() {
        this._surface.present()
    }

    public ptr() {
        return this._surface.ptr()
    }
}

class GPUTexture {
    private _textureBase: GPUTextureBase;

    constructor(texture: object) {
        this._textureBase = GPUTextureBase.from(texture);
    }

    public createView(descriptor = {}) {
        let format = "none"
        if (typeof descriptor["format"] === typeof "") {
            format = descriptor["format"]
        }

        let dimension = "none";
        if (typeof descriptor["dimension"] === typeof "") {
            dimension = descriptor["dimension"]
        }

        let aspect = "all";
        if (typeof descriptor["aspect"] === typeof "") {
            aspect = descriptor["aspect"]
        }

        let baseMipLevel = 0;
        if (typeof descriptor["baseMipLevel"] === typeof 0) {
            baseMipLevel = descriptor["baseMipLevel"]
        }

        let mipLevelCount = -1;
        if (typeof descriptor["mipLevelCount"] === typeof 0) {
            mipLevelCount = descriptor["mipLevelCount"]
        }

        let baseArrayLayer = 0;
        if (typeof descriptor["baseArrayLayer"] === typeof 0) {
            baseArrayLayer = descriptor["baseArrayLayer"]
        }

        let arrayLayerCount = -1;
        if (typeof descriptor["arrayLayerCount"] === typeof 0) {
            arrayLayerCount = descriptor["arrayLayerCount"]
        }


        let view = this._textureBase.create_view(format, dimension, aspect, baseMipLevel, mipLevelCount, baseArrayLayer, arrayLayerCount);
        if (typeof view === typeof Error) {
            throw view;
        }
        return new GPUTextureView(view);
    }
    public destroy() {
        this._textureBase.destroy();
    }

    public ptr() {
        return this._textureBase.ptr();
    }
}

class GPUPipelineLayout {
    private _pipelineBase: GPUPipelineLayoutBase;

    constructor(pipeline: object) {
        this._pipelineBase = GPUPipelineLayoutBase.from(pipeline);
    }

    public ptr(): number {
        return this._pipelineBase.ptr();
    }
}

class GPUShaderModule {
    private _moduleBase: ShaderModuleBase;

    constructor(module: object) {
        this._moduleBase = ShaderModuleBase.from(module);
    }

    public compilationInfo() {
        this._moduleBase.compilation_info();
    }

    public ptr(): number {
        return this._moduleBase.ptr();
    }
}

class GPURenderPipeline {
    private _renderBase: GPURenderPipelineBase;

    constructor(render: object) {
        this._renderBase = GPURenderPipelineBase.from(render);
    }

    public getBindGroupLayout(index: number) {
        let group = this._renderBase.get_bind_group_layout(index);
        return new GPUBindGroupLayout(group);
    }

    public ptr() {
        return this._renderBase.ptr();
    }
}
class GPUCommandEncoder {
    private _command: GPUCommandEncoderBase;

    constructor(command: object) {
        this._command = GPUCommandEncoderBase.from(command);
    }


    public beginRenderPass(renderPassDesc: object) {
        // console.log("========== beginRenderPass 1");

        let attachments = [];
        let attachment_ptrs = "";
        renderPassDesc["colorAttachments"].forEach(element => {
            let view: GPUTextureView = element["view"];

            let color: 4[] = element["loadValue"] !== undefined ? element["loadValue"] : element["clearValue"];
            // console.log("======= color[0]: " + color[0] + "; color[r]: " + color["r"]);

            let r = color[0] !== undefined ? color[0] : color["r"];
            if (typeof r !== typeof 0) {
                throw new Error("beginRenderPass failed!! color.r is not number!!!!")
            }

            let g = color[1] !== undefined ? color[1] : color["g"];
            if (typeof g !== typeof 0) {
                throw new Error("beginRenderPass failed!! color.g is not number!!!!")
            }

            let b = color[2] !== undefined ? color[2] : color["b"];
            if (typeof b !== typeof 0) {
                throw new Error("beginRenderPass failed!! color.b is not number!!!!")
            }

            let a = color[3] !== undefined ? color[3] : color["a"];
            if (typeof a !== typeof 0) {
                throw new Error("beginRenderPass failed!! color.a is not number!!!!")
            }

            let resolveTargetPtr = 0;
            if (element["resolveTarget"]) {
                resolveTargetPtr = element["resolveTarget"].ptr();
            }

            let store = true;
            if (element["storeOp"] === "discard") {
                store = false;
            }


            // console.log("========== beginRenderPass 2, view.ptr() = ", view.ptr());
            let attachment = GPURenderPassColorAttachmentBase.new(view.ptr(), resolveTargetPtr, r, g, b, a, store);
            // console.log("========== beginRenderPass 3");
            attachments.push(attachment);
            attachment_ptrs = attachment_ptrs + attachment.ptr().toString() + " ";
        });

        let depth = undefined;
        let depthStencilAttachment = renderPassDesc["depthStencilAttachment"];
        if (depthStencilAttachment) {
            let view: GPUTextureView = depthStencilAttachment["view"];

            let depthLoadValue = depthStencilAttachment["depthLoadValue"] !== undefined ? depthStencilAttachment["depthLoadValue"] : depthStencilAttachment["depthClearValue"];
            if (typeof depthLoadValue !== typeof 0) {
                depthLoadValue = 0;
            }
            let depthStoreOp = depthStencilAttachment["depthStoreOp"];

            let stencilLoadValue = depthStencilAttachment["stencilLoadValue"] !== undefined ? depthStencilAttachment["stencilLoadValue"] : depthStencilAttachment["stencilClearValue"];
            if (typeof stencilLoadValue !== typeof 0) {
                stencilLoadValue = 0;
            }

            let stencilStoreOp = depthStencilAttachment["stencilStoreOp"];

            // console.log("========== beginRenderPass 33");
            depth = GPURenderPassDepthStencilAttachmentBase.new(view.ptr(), depthLoadValue, depthStoreOp, stencilLoadValue, stencilStoreOp);
            // console.log("========== beginRenderPass 333");
        }

        let depthPtr = 0;
        if (depth) {
            depthPtr = depth.ptr();
        }

        // console.log("========== beginRenderPass 4");
        let pass = this._command.begin_render_pass(attachment_ptrs, depthPtr);
        // console.log("========== beginRenderPass 5");
        return new GPURenderPassEncoder(pass);
    }

    public finish() {
        return this._command.finish();
    }

    public ptr() {
        return this._command.ptr();
    }
}

class GPURenderPassEncoder {
    private _renderPass: GPURenderPassEncoderBase;

    constructor(encoder: object) {
        this._renderPass = GPURenderPassEncoderBase.from(encoder);
    }

    public ptr() {
        return this._renderPass.ptr();
    }

    public setPipeline(renderPipeline: GPURenderPipeline) {
        this._renderPass.set_pipeline(renderPipeline.ptr())
    }

    public setViewport(x: number, y: number, width: number, height: number, minDepth: number, maxDepth: number) {
        this._renderPass.set_viewport(x, y, width, height, minDepth, maxDepth);
    }

    public setIndexBuffer(buffer: GPUBuffer, indexFormat: GPUIndexFormat, offset = 0, size = -1) {
        // console.log("========= setIndexBuffer! offset = " + offset + "; size = " + size + "; indexFormat = " + indexFormat.toString());

        this._renderPass.set_index_buffer(buffer.ptr(), indexFormat.toString(), offset, size);
    }

    public setBindGroup(index: number, bindGroup: GPUBindGroup, dynamicOffsetsData?: Uint32Array, dynamicOffsetsDataStart?: number, dynamicOffsetsDataLength?: number) {
        let data: ArrayBufferLike;
        if (dynamicOffsetsData) {
            data = dynamicOffsetsData.slice(dynamicOffsetsDataStart, dynamicOffsetsDataLength).buffer;
        } else {
            data = new Uint32Array([]).buffer
        }
        this._renderPass.set_bind_group(index, bindGroup.ptr(), data)
    }

    public drawIndexed(indexCount: number, instanceCount = 1, firstIndex = 0, baseVertex = 0, firstInstance = 0) {
        this._renderPass.draw_indexed(indexCount, instanceCount, firstIndex, baseVertex, firstInstance);
    }

    public setVertexBuffer(slot: number, buffer: GPUBuffer, offset?: number, size?: number) {
        let offsetTemp = 0;
        if (!!offset) {
            offsetTemp = offset;
        }

        let sizeTemp = 0;
        if (!!size) {
            sizeTemp = size;
        }

        this._renderPass.set_vertex_buffer(slot, buffer.ptr(), offsetTemp, sizeTemp)
    }

    public draw(vertexCount: number, instanceCount?: number,
        firstVertex?: number, firstInstance?: number) {
        let instanceCountTemp = 1;
        if (!!instanceCount) {
            instanceCountTemp = instanceCount
        }

        let firstVertexTemp = 0;
        if (!!firstVertex) {
            firstVertexTemp = firstVertex
        }

        let firstInstanceTemp = 0;
        if (!!firstInstance) {
            firstInstanceTemp = firstInstance
        }

        this._renderPass.draw(vertexCount, instanceCountTemp, firstVertexTemp, firstInstanceTemp);
    }

    public endPass() {
        this._renderPass.end_pass();
    }

    public end() {
        this._renderPass.end_pass();
    }
}

class GPUCanvasContext {
    private _context: GPUCanvasContextBase;
    private _width: number;
    private _height: number;

    constructor(context: object, width: number, height: number) {
        this._context = GPUCanvasContextBase.from(context);
        this._width = width;
        this._height = height;
    }

    public configure(configuration: object) {
        let device: GPUDevice = configuration["device"];
        let format: string = configuration["format"];


        let desc = GPUCanvasConfiguration.new(device.ptr(), format, this._width, this._height);
        this._context.configure(desc.ptr());
    }

    public getPreferredFormat(adapter: GPUAdapter) {
        return this._context.get_preferred_format(adapter.ptr())
    }

    public getCurrentTexture() {

        let texture = this._context.get_current_texture()
        if (texture instanceof Error) {
            throw texture;
        }

        let surface = new GPUSurfaceTexture(texture);
        globalThis.__setCurrentSurface(surface);
        return surface;
    }

    public ptr() {
        console.log("====== GPUCanvasContext: ptr");
        return this._context.ptr();
    }
}

class GPUBindGroupLayout {
    private _groupLayout: GPUBindGroupLayoutBase

    constructor(group: object) {
        this._groupLayout = GPUBindGroupLayoutBase.from(group);
    }

    public ptr() {
        return this._groupLayout.ptr();
    }
}

class GPUSampler {
    private _samplerBase: GPUSamplerBase;

    constructor(sampler: object) {
        this._samplerBase = GPUSamplerBase.from(sampler);
    }
    public ptr() {
        return this._samplerBase.ptr();
    }
}

class GPUImageCopyTexture {
    private _destination: GPUImageCopyTextureBase

    constructor(destination: object) {
        this._destination = GPUImageCopyTextureBase.from(destination);
    }

    public ptr() {
        return this._destination.ptr();
    }
}

class GPUBindGroup {
    private _groupBase: GPUBindGroupBase;

    constructor(group: object) {
        this._groupBase = GPUBindGroupBase.from(group);
    }

    public ptr() {
        return this._groupBase.ptr();
    }
}


function setLimits(limits: GPUSupportedLimitsBase, limitsInner: object): GPUSupportedLimitsBase {
    let tempValue = limitsInner["maxDynamicUniformBuffersPerPipelineLayout"];
    if (!!tempValue) {
        limits.set_max_dynamic_uniform_buffers_per_pipeline_layout(tempValue);
    }

    tempValue = limitsInner["maxDynamicStorageBuffersPerPipelineLayout"];
    if (!!tempValue) {
        limits.set_max_dynamic_storage_buffers_per_pipeline_layout(tempValue);
    }

    tempValue = limitsInner["maxSampledTexturesPerShaderStage"];
    if (!!tempValue) {
        limits.set_max_sampled_textures_per_shader_stage(tempValue);
    }

    tempValue = limitsInner["maxSamplersPerShaderStage"];
    if (!!tempValue) {
        limits.set_max_samplers_per_shader_stage(tempValue);
    }

    tempValue = limitsInner["maxStorageBuffersPerShaderStage"];
    if (!!tempValue) {
        limits.set_max_storage_buffers_per_shader_stage(tempValue);
    }

    tempValue = limitsInner["maxStorageTexturesPerShaderStage"];
    if (!!tempValue) {
        limits.set_max_storage_textures_per_shader_stage(tempValue);
    }

    tempValue = limitsInner["maxUniformBuffersPerShaderStage"];
    if (!!tempValue) {
        limits.set_max_uniform_buffers_per_shader_stage(tempValue);
    }

    tempValue = limitsInner["maxUniformBufferBindingSize"];
    if (!!tempValue) {
        limits.set_max_uniform_buffer_binding_size(tempValue);
    }

    tempValue = limitsInner["maxStorageBufferBindingSize"];
    if (!!tempValue) {
        limits.set_max_storage_buffer_binding_size(tempValue);
    }

    tempValue = limitsInner["minUniformBufferOffsetAlignment"];
    if (!!tempValue) {
        limits.set_min_uniform_buffer_offset_alignment(tempValue);
    }

    tempValue = limitsInner["minStorageBufferOffsetAlignment"];
    if (!!tempValue) {
        limits.set_min_storage_buffer_offset_alignment(tempValue);
    }

    tempValue = limitsInner["maxVertexBuffers"];
    if (!!tempValue) {
        limits.set_max_vertex_buffers(tempValue);
    }

    tempValue = limitsInner["maxVertexAttributes"];
    if (!!tempValue) {
        limits.set_max_vertex_attributes(tempValue);
    }

    tempValue = limitsInner["maxVertexBufferArrayStride"];
    if (!!tempValue) {
        limits.set_max_vertex_buffer_array_stride(tempValue);
    }

    tempValue = limitsInner["maxInterStageShaderComponents"];
    if (!!tempValue) {
        limits.set_max_inter_stage_shader_components(tempValue);
    }

    tempValue = limitsInner["maxComputeWorkgroupStorageSize"];
    if (!!tempValue) {
        limits.set_max_compute_workgroup_storage_size(tempValue);
    }

    tempValue = limitsInner["maxComputeInvocationsPerWorkgroup"];
    if (!!tempValue) {
        limits.set_max_compute_invocations_per_workgroup(tempValue);
    }

    tempValue = limitsInner["maxComputeWorkgroupSizeX"];
    if (!!tempValue) {
        limits.set_max_compute_workgroup_size_x(tempValue);
    }

    tempValue = limitsInner["maxComputeWorkgroupSizeY"];
    if (!!tempValue) {
        limits.set_max_compute_workgroup_size_y(tempValue);
    }

    tempValue = limitsInner["maxComputeWorkgroupSizeZ"];
    if (!!tempValue) {
        limits.set_max_compute_workgroup_size_z(tempValue);
    }

    tempValue = limitsInner["maxComputeWorkgroupsPerDimension"];
    if (!!tempValue) {
        limits.set_max_compute_workgroups_per_dimension(tempValue);
    }


    return limits;
}





const g = globalThis as any;
g.GPUCanvasContext = GPUCanvasContext;
g.navigator.gpu = { requestAdapter };

class GPUBufferUsage {
    static MAP_READ = 0x0001;
    static MAP_WRITE = 0x0002;
    static COPY_SRC = 0x0004;
    static COPY_DST = 0x0008;
    static INDEX = 0x0010;
    static VERTEX = 0x0020;
    static UNIFORM = 0x0040;
    static STORAGE = 0x0080;
    static INDIRECT = 0x0100;
    static QUERY_RESOLVE = 0x0200;
};
g.GPUBufferUsage = GPUBufferUsage;

class GPUTextureUsage {
    static COPY_SRC = 0x01;
    static COPY_DST = 0x02;
    static TEXTURE_BINDING = 0x04;
    static STORAGE_BINDING = 0x08;
    static RENDER_ATTACHMENT = 0x10;
};
g.GPUTextureUsage = GPUTextureUsage;

class GPUColorWrite {
    static RED = 0x1;
    static GREEN = 0x2;
    static BLUE = 0x4;
    static ALPHA = 0x8;
    static ALL = 0xF;
};
g.GPUColorWrite = GPUColorWrite;

class GPUShaderStage {
    static VERTEX = 0x1;
    static FRAGMENT = 0x2;
    static COMPUTE = 0x4;
}
g.GPUShaderStage = GPUShaderStage;

enum GPUIndexFormat {
    "uint16",
    "uint32",
};
g.GPUIndexFormat = GPUIndexFormat;

g.render = render;
    // g.createImageBitmap = (image, options)=>{
    //     return Promise.resolve(image);
    // }




