declare var NativeObject;
declare var _$cid;


/**
* web shell 管理
*/
export class WebShellMgrBase {
    /**
     * 本地对象
     */
    private self: object;

    /**
     * 类的私有构造方法
     */
    private constructor(self: object) {
        this.self = self;
        if (NativeObject.registry != undefined) {
            NativeObject.registry.register(self, [self]);
        }
    }

    /**
     * 获取本地对象方法
     */
    public get_self() {
        return this.self;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase already destroy");
        }

        NativeObject.release(_$cid, this.self);
        this.self = undefined;
    }

    /**
     * 从指定本地对象构建当前类方法，此方法是不安全的，使用错误的本地对象将会导致调用时异常
     */
    static from(obj: object): WebShellMgrBase {
        return new WebShellMgrBase(obj);
    }

    static new(pid: number): WebShellMgrBase {
        let result = NativeObject.static_call(10, pid) as object;
        if (result instanceof Error) {
            throw result;
        } else {
            return new WebShellMgrBase(result);
        }
    }

    /**
     * 创建web_shell
     */
    public async crate_web_shell(): Promise<number> {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = NativeObject.async_call(0, this.self) as Promise<number>;
        return await result;
    }

    /**
     * 用户输入
     */
    public client_input(pid: number, s: string): boolean | Error {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = NativeObject.call(211, this.self, pid, s) as boolean;
        return result;
    }

    /**
     * 获取输出
     */
    public get_out(pid: number): string | Error {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = NativeObject.call(212, this.self, pid) as string;
        return result;
    }

    /**
     * 写入输出
     */
    public set_out(pid: number, s: string): boolean | Error {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = NativeObject.call(213, this.self, pid, s) as boolean;
        return result;
    }

    /**
     * 关闭webshell
     */
    public close(pid: number): boolean | Error {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = NativeObject.call(214, this.self, pid) as boolean;
        return result;
    }

    /**
     * 获取所有webshell
     */
    public get_web_shell_all(): number[] {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = NativeObject.call(215, this.self) as number[];
        return result;
    }

}

class WebShellMgr {
    /**
     * 本地对象
     */
    private self: WebShellMgrBase | undefined;


    constructor() {
        let pid: number = globalThis.ESProcess.pid;
        this.self = WebShellMgrBase.new(pid);
    }

    /**
     * 创建web_shell
     */
    public async crateWebShell(): Promise<number> {
        if (this.self == undefined) {
            throw new Error("WebShellMgr object already destroy");
        }

        let result = this.self.crate_web_shell();
        return await result;
    }

    /**
     * 用户输入
     */
    public clientInput(pid: number, s: string): boolean | Error {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = this.self.client_input(pid, s);
        return result;
    }

    /**
     * 获取输出
     */
    public getOut(pid: number): string | Error {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = this.self.get_out(pid);
        return result;
    }

    /**
     * 写入输出
     */
    public setOut(pid: number, s: string): boolean | Error {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = this.self.set_out(pid, s);
        return result;
    }

    /**
     * 关闭webshell
     */
    public close(pid: number): boolean | Error {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = this.self.close(pid);
        return result;
    }

    /**
     * 获取所有webshell
     */
    public getWebShellAll(): number[] {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase object already destroy");
        }

        let result = this.self.get_web_shell_all();
        return result;
    }

    /**
     * 释放本地对象的方法
     */
    public destroy() {
        if (this.self == undefined) {
            throw new Error("WebShellMgrBase already destroy");
        }
        this.self.destroy();
        this.self = undefined;
    }

}

const g = globalThis as any;
g.WebShellMgr = WebShellMgr;

